;;; shen-elisp.el --- An implementation of the Shen programming language  -*- lexical-binding: t -*-

;; Copyright (C) 2015-2016  Aditya Siram

;; Author: Aditya Siram <aditya.siram@gmail.com>
;; Homepage: https://github.com/deech/shen-elisp
;; License: BSD 3-Clause License
;;   http://opensource.org/licenses/BSD-3-Clause

;;; Commentary:

;; This is an implemenatation of the Shen programming language in
;; Elisp. The end goal is to provide:
;;
;; 1. An easy way to play with Shen with no other installation
;;    hassle (assuming you use Emacs).
;; 2. A first-class development experience when writing Shen.
;;    The idea is that an editor that understands the code can
;;    be much more helpful than one that does not. To this end
;;    the roadmap involves a full gamut of source code
;;    introspection and debugging tools.

;;; Code:

(require 'shen-primitives)
(setq max-lisp-eval-depth 60000)
(setq max-specpdl-size 13000)

(defun shen/shen\.shen nil
  (shen/do
   (shen/shen\.credits)
   (shen/shen\.loop)))
(shen/set 'shen\.*continue-repl-loop* 'true)
(defun shen/exit
    (V3800)
  (shen/set 'shen\.*continue-repl-loop* 'false))
(defun shen/shen\.loop nil
  (cl-flet
      ((tail-trampoline nil
                        (shen/do
                         (shen/shen\.initialise_environment)
                         (shen/do
                          (shen/shen\.prompt)
                          (shen/do
                           (shen/trap-error
                            (shen/shen\.read-evaluate-print)
                            (shen/lambda E
                                         (shen/pr
                                          (shen/error-to-string E)
                                          (shen/stoutput))))
                           (shen/if
                            (shen/value 'shen\.*continue-repl-loop*)
                            (vector
                             (list))
                            'exit))))))
    (let
        ((result
          (funcall #'tail-trampoline)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.credits nil
  (shen/do
   (shen/shen\.prhush "\nShen, copyright (C) 2010-2015 Mark Tarver\n"
                      (shen/stoutput))
   (shen/do
    (shen/shen\.prhush
     (shen/cn "www.shenlanguage.org, "
              (shen/shen\.app
               (shen/value '*version*)
               "\n" 'shen\.a))
     (shen/stoutput))
    (shen/do
     (shen/shen\.prhush
      (shen/cn "running under "
               (shen/shen\.app
                (shen/value '*language*)
                (shen/cn ", implementation: "
                         (shen/shen\.app
                          (shen/value '*implementation*)
                          "" 'shen\.a))
                'shen\.a))
      (shen/stoutput))
     (shen/shen\.prhush
      (shen/cn "\nport "
               (shen/shen\.app
                (shen/value '*port*)
                (shen/cn " ported by "
                         (shen/shen\.app
                          (shen/value '*porters*)
                          "\n" 'shen\.a))
                'shen\.a))
      (shen/stoutput))))))
(defun shen/shen\.initialise_environment nil
  (shen/shen\.multiple-set
   (list 'shen\.*call* 0 'shen\.*infs* 0 'shen\.*process-counter* 0 'shen\.*catch* 0)))
(defun shen/shen\.multiple-set
    (V3802)
  (cl-flet
      ((tail-trampoline
        (V3802)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3802))
          nil)
         ((shen/and
           (shen/cons\? V3802)
           (shen/cons\?
            (nthcdr 1 V3802)))
          (shen/do
           (shen/set
            (shen/hd V3802)
            (shen/hd
             (nthcdr 1 V3802)))
           (vector
            (list
             (nthcdr 2 V3802)))))
         (shen/true
          (shen/shen\.f_error 'shen\.multiple-set)))))
    (let
        ((result
          (funcall #'tail-trampoline V3802)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/destroy
    (V3804)
  (shen/declare V3804 'symbol))
(shen/set 'shen\.*history* nil)
(defun shen/shen\.read-evaluate-print nil
  (shen/let Lineread
            (shen/shen\.toplineread)
            (shen/let History
                      (shen/value 'shen\.*history*)
                      (shen/let NewLineread
                                (shen/shen\.retrieve-from-history-if-needed Lineread History)
                                (shen/let NewHistory
                                          (shen/shen\.update_history NewLineread History)
                                          (shen/let Parsed
                                                    (shen/fst NewLineread)
                                                    (shen/shen\.toplevel Parsed)))))))
(defun shen/shen\.retrieve-from-history-if-needed
    (V3816 V3817)
  (cl-flet
      ((tail-trampoline
        (V3816 V3817)
        (shen/cond
         ((shen/and
           (shen/tuple\? V3816)
           (shen/and
            (shen/cons\?
             (shen/snd V3816))
            (shen/element\?
             (shen/hd
              (shen/snd V3816))
             (list
              (shen/shen\.space)
              (shen/shen\.newline)))))
          (vector
           (list
            (shen/@p
             (shen/fst V3816)
             (nthcdr 1
                     (shen/snd V3816)))
            V3817)))
         ((shen/and
           (shen/tuple\? V3816)
           (shen/and
            (shen/cons\?
             (shen/snd V3816))
            (shen/and
             (shen/cons\?
              (nthcdr 1
                      (shen/snd V3816)))
             (shen/and
              (shen/internal/predicate->shen
               (null
                (nthcdr 2
                        (shen/snd V3816))))
              (shen/and
               (shen/cons\? V3817)
               (shen/and
                (shen/=
                 (shen/hd
                  (shen/snd V3816))
                 (shen/shen\.exclamation))
                (shen/=
                 (shen/hd
                  (nthcdr 1
                          (shen/snd V3816)))
                 (shen/shen\.exclamation))))))))
          (shen/let PastPrint
                    (shen/shen\.prbytes
                     (shen/snd
                      (shen/hd V3817)))
                    (shen/hd V3817)))
         ((shen/and
           (shen/tuple\? V3816)
           (shen/and
            (shen/cons\?
             (shen/snd V3816))
            (shen/=
             (shen/hd
              (shen/snd V3816))
             (shen/shen\.exclamation))))
          (shen/let Key\?
                    (shen/shen\.make-key
                     (nthcdr 1
                             (shen/snd V3816))
                     V3817)
                    (shen/let Find
                              (shen/head
                               (shen/shen\.find-past-inputs Key\? V3817))
                              (shen/let PastPrint
                                        (shen/shen\.prbytes
                                         (shen/snd Find))
                                        Find))))
         ((shen/and
           (shen/tuple\? V3816)
           (shen/and
            (shen/cons\?
             (shen/snd V3816))
            (shen/and
             (shen/internal/predicate->shen
              (null
               (nthcdr 1
                       (shen/snd V3816))))
             (shen/=
              (shen/hd
               (shen/snd V3816))
              (shen/shen\.percent)))))
          (shen/do
           (shen/shen\.print-past-inputs
            (shen/lambda X 'true)
            (shen/reverse V3817)
            0)
           (shen/abort)))
         ((shen/and
           (shen/tuple\? V3816)
           (shen/and
            (shen/cons\?
             (shen/snd V3816))
            (shen/=
             (shen/hd
              (shen/snd V3816))
             (shen/shen\.percent))))
          (shen/let Key\?
                    (shen/shen\.make-key
                     (nthcdr 1
                             (shen/snd V3816))
                     V3817)
                    (shen/let Pastprint
                              (shen/shen\.print-past-inputs Key\?
                                                            (shen/reverse V3817)
                                                            0)
                              (shen/abort))))
         (shen/true V3816))))
    (let
        ((result
          (funcall #'tail-trampoline V3816 V3817)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.percent nil 37)
(defun shen/shen\.exclamation nil 33)
(defun shen/shen\.prbytes
    (V3819)
  (shen/do
   (shen/for-each
    (shen/lambda Byte
                 (shen/pr
                  (shen/n->string Byte)
                  (shen/stoutput)))
    V3819)
   (shen/nl 1)))
(defun shen/shen\.update_history
    (V3822 V3823)
  (shen/set 'shen\.*history*
            (append
             (list V3822)
             V3823)))
(defun shen/shen\.toplineread nil
  (shen/shen\.toplineread_loop
   (shen/read-char-code
    (shen/stinput))
   nil))
(defun shen/shen\.toplineread_loop
    (V3827 V3828)
  (cl-flet
      ((tail-trampoline
        (V3827 V3828)
        (shen/cond
         ((shen/and
           (shen/= -1 V3827)
           (shen/internal/predicate->shen
            (null V3828)))
          (shen/exit 0))
         ((shen/= V3827
                  (shen/shen\.hat))
          (shen/simple-error "line read aborted"))
         ((shen/element\? V3827
                          (list
                           (shen/shen\.newline)
                           (shen/shen\.carriage-return)))
          (shen/let Line
                    (shen/compile
                     (shen/lambda X
                                  (shen/shen\.<st_input> X))
                     V3828
                     (shen/lambda E 'shen\.nextline))
                    (shen/let It
                              (shen/shen\.record-it V3828)
                              (shen/if
                               (shen/or
                                (shen/= Line 'shen\.nextline)
                                (shen/empty\? Line))
                               (vector
                                (list
                                 (shen/read-char-code
                                  (shen/stinput))
                                 (shen/append V3828
                                              (list V3827))))
                               (shen/@p Line V3828)))))
         (shen/true
          (vector
           (list
            (shen/read-char-code
             (shen/stinput))
            (shen/if
             (shen/= V3827 -1)
             V3828
             (shen/append V3828
                          (list V3827)))))))))
    (let
        ((result
          (funcall #'tail-trampoline V3827 V3828)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.hat nil 94)
(defun shen/shen\.newline nil 10)
(defun shen/shen\.carriage-return nil 13)
(defun shen/tc
    (V3834)
  (shen/cond
   ((shen/= '+ V3834)
    (shen/set 'shen\.*tc* 'true))
   ((shen/= '- V3834)
    (shen/set 'shen\.*tc* 'false))
   (shen/true
    (shen/simple-error "tc expects a + or -"))))
(defun shen/shen\.prompt nil
  (shen/if
   (shen/value 'shen\.*tc*)
   (shen/shen\.prhush
    (shen/cn "\n\n("
             (shen/shen\.app
              (shen/length
               (shen/value 'shen\.*history*))
              "+) " 'shen\.a))
    (shen/stoutput))
   (shen/shen\.prhush
    (shen/cn "\n\n("
             (shen/shen\.app
              (shen/length
               (shen/value 'shen\.*history*))
              "-) " 'shen\.a))
    (shen/stoutput))))
(defun shen/shen\.toplevel
    (V3836)
  (shen/shen\.toplevel_evaluate V3836
                                (shen/value 'shen\.*tc*)))
(defun shen/shen\.find-past-inputs
    (V3839 V3840)
  (shen/let F
            (shen/shen\.find V3839 V3840)
            (shen/if
             (shen/empty\? F)
             (shen/simple-error "input not found\n")
             F)))
(defun shen/shen\.make-key
    (V3843 V3844)
  (shen/let Atom
            (shen/hd
             (shen/compile
              (shen/lambda X
                           (shen/shen\.<st_input> X))
              V3843
              (shen/lambda E
                           (shen/if
                            (shen/cons\? E)
                            (shen/simple-error
                             (shen/cn "parse error here: "
                                      (shen/shen\.app E "\n" 'shen\.s)))
                            (shen/simple-error "parse error\n")))))
            (shen/if
             (shen/integer\? Atom)
             (shen/lambda X
                          (shen/= X
                                  (shen/nth
                                   (1+ Atom)
                                   (shen/reverse V3844))))
             (shen/lambda X
                          (shen/shen\.prefix\? V3843
                                               (shen/shen\.trim-gubbins
                                                (shen/snd X)))))))
(defun shen/shen\.trim-gubbins
    (V3846)
  (cl-flet
      ((tail-trampoline
        (V3846)
        (shen/cond
         ((shen/and
           (shen/cons\? V3846)
           (shen/=
            (shen/hd V3846)
            (shen/shen\.space)))
          (vector
           (list
            (nthcdr 1 V3846))))
         ((shen/and
           (shen/cons\? V3846)
           (shen/=
            (shen/hd V3846)
            (shen/shen\.newline)))
          (vector
           (list
            (nthcdr 1 V3846))))
         ((shen/and
           (shen/cons\? V3846)
           (shen/=
            (shen/hd V3846)
            (shen/shen\.carriage-return)))
          (vector
           (list
            (nthcdr 1 V3846))))
         ((shen/and
           (shen/cons\? V3846)
           (shen/=
            (shen/hd V3846)
            (shen/shen\.tab)))
          (vector
           (list
            (nthcdr 1 V3846))))
         ((shen/and
           (shen/cons\? V3846)
           (shen/=
            (shen/hd V3846)
            (shen/shen\.left-round)))
          (vector
           (list
            (nthcdr 1 V3846))))
         (shen/true V3846))))
    (let
        ((result
          (funcall #'tail-trampoline V3846)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.space nil 32)
(defun shen/shen\.tab nil 9)
(defun shen/shen\.left-round nil 40)
(defun shen/shen\.find
    (V3855 V3856)
  (cl-flet
      ((tail-trampoline
        (V3855 V3856)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3856))
          nil)
         ((shen/and
           (shen/cons\? V3856)
           (shen/internal/apply-higher-order-function V3855
                                                      (list
                                                       (shen/hd V3856))))
          (append
           (list
            (shen/hd V3856))
           (shen/shen\.find V3855
                            (nthcdr 1 V3856))))
         ((shen/cons\? V3856)
          (vector
           (list V3855
                 (nthcdr 1 V3856))))
         (shen/true
          (shen/shen\.f_error 'shen\.find)))))
    (let
        ((result
          (funcall #'tail-trampoline V3855 V3856)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.prefix\?
    (V3870 V3871)
  (cl-flet
      ((tail-trampoline
        (V3870 V3871)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3870))
          'true)
         ((shen/and
           (shen/cons\? V3870)
           (shen/and
            (shen/cons\? V3871)
            (shen/=
             (shen/hd V3871)
             (shen/hd V3870))))
          (vector
           (list
            (nthcdr 1 V3870)
            (nthcdr 1 V3871))))
         (shen/true 'false))))
    (let
        ((result
          (funcall #'tail-trampoline V3870 V3871)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.print-past-inputs
    (V3883 V3884 V3885)
  (cl-flet
      ((tail-trampoline
        (V3883 V3884 V3885)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3884))
          '_)
         ((shen/and
           (shen/cons\? V3884)
           (shen/not
            (shen/internal/apply-higher-order-function V3883
                                                       (list
                                                        (shen/hd V3884)))))
          (vector
           (list V3883
                 (nthcdr 1 V3884)
                 (1+ V3885))))
         ((shen/and
           (shen/cons\? V3884)
           (shen/tuple\?
            (shen/hd V3884)))
          (shen/do
           (shen/shen\.prhush
            (shen/shen\.app V3885 ". " 'shen\.a)
            (shen/stoutput))
           (shen/do
            (shen/shen\.prbytes
             (shen/snd
              (shen/hd V3884)))
            (vector
             (list V3883
                   (nthcdr 1 V3884)
                   (1+ V3885))))))
         (shen/true
          (shen/shen\.f_error 'shen\.print-past-inputs)))))
    (let
        ((result
          (funcall #'tail-trampoline V3883 V3884 V3885)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.toplevel_evaluate
    (V3888 V3889)
  (cl-flet
      ((tail-trampoline
        (V3888 V3889)
        (shen/cond
         ((shen/and
           (shen/cons\? V3888)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V3888))
            (shen/and
             (shen/= ':
                     (shen/hd
                      (nthcdr 1 V3888)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V3888))
              (shen/and
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V3888)))
               (shen/= 'true V3889))))))
          (shen/shen\.typecheck-and-evaluate
           (shen/hd V3888)
           (shen/hd
            (nthcdr 2 V3888))))
         ((shen/and
           (shen/cons\? V3888)
           (shen/cons\?
            (nthcdr 1 V3888)))
          (shen/do
           (shen/shen\.toplevel_evaluate
            (list
             (shen/hd V3888))
            V3889)
           (shen/do
            (shen/nl 1)
            (vector
             (list
              (nthcdr 1 V3888)
              V3889)))))
         ((shen/and
           (shen/cons\? V3888)
           (shen/and
            (shen/internal/predicate->shen
             (null
              (nthcdr 1 V3888)))
            (shen/= 'true V3889)))
          (shen/shen\.typecheck-and-evaluate
           (shen/hd V3888)
           (shen/gensym 'A)))
         ((shen/and
           (shen/cons\? V3888)
           (shen/and
            (shen/internal/predicate->shen
             (null
              (nthcdr 1 V3888)))
            (shen/= 'false V3889)))
          (shen/let Eval
                    (shen/shen\.eval-without-macros
                     (shen/hd V3888))
                    (shen/print Eval)))
         (shen/true
          (shen/shen\.f_error 'shen\.toplevel_evaluate)))))
    (let
        ((result
          (funcall #'tail-trampoline V3888 V3889)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.typecheck-and-evaluate
    (V3892 V3893)
  (shen/let Typecheck
            (shen/shen\.typecheck V3892 V3893)
            (shen/if
             (shen/= Typecheck 'false)
             (shen/simple-error "type error\n")
             (shen/let Eval
                       (shen/shen\.eval-without-macros V3892)
                       (shen/let Type
                                 (shen/shen\.pretty-type Typecheck)
                                 (shen/shen\.prhush
                                  (shen/shen\.app Eval
                                                  (shen/cn " : "
                                                           (shen/shen\.app Type "" 'shen\.r))
                                                  'shen\.s)
                                  (shen/stoutput)))))))
(defun shen/shen\.pretty-type
    (V3895)
  (shen/shen\.mult_subst
   (shen/value 'shen\.*alphabet*)
   (shen/shen\.extract-pvars V3895)
   V3895))
(defun shen/shen\.extract-pvars
    (V3901)
  (shen/cond
   ((shen/shen\.pvar\? V3901)
    (list V3901))
   ((shen/cons\? V3901)
    (shen/union
     (shen/shen\.extract-pvars
      (shen/hd V3901))
     (shen/shen\.extract-pvars
      (nthcdr 1 V3901))))
   (shen/true nil)))
(defun shen/shen\.mult_subst
    (V3909 V3910 V3911)
  (cl-flet
      ((tail-trampoline
        (V3909 V3910 V3911)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3909))
          V3911)
         ((shen/internal/predicate->shen
           (null V3910))
          V3911)
         ((shen/and
           (shen/cons\? V3909)
           (shen/cons\? V3910))
          (vector
           (list
            (nthcdr 1 V3909)
            (nthcdr 1 V3910)
            (shen/subst
             (shen/hd V3909)
             (shen/hd V3910)
             V3911))))
         (shen/true
          (shen/shen\.f_error 'shen\.mult_subst)))))
    (let
        ((result
          (funcall #'tail-trampoline V3909 V3910 V3911)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen->kl
    (V1193 V1194)
  (shen/compile
   (shen/lambda X
                (shen/shen\.<define> X))
   (append
    (list V1193)
    V1194)
   (shen/lambda X
                (shen/shen-syntax-error V1193 X))))
(defun shen/shen-syntax-error
    (V1201 V1202)
  (shen/cond
   ((shen/cons\? V1202)
    (shen/simple-error
     (shen/cn "syntax error in "
              (shen/shen\.app V1201
                              (shen/cn " here:\n\n "
                                       (shen/shen\.app
                                        (shen/shen\.next-50 50
                                                            (shen/hd V1202))
                                        "\n" 'shen\.a))
                              'shen\.a))))
   (shen/true
    (shen/simple-error
     (shen/cn "syntax error in "
              (shen/shen\.app V1201 "\n" 'shen\.a))))))
(defun shen/shen\.<define>
    (V1204)
  (shen/let YaccParse
            (shen/let Parse_shen\.<name>
                      (shen/shen\.<name> V1204)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<name>))
                       (shen/let Parse_shen\.<signature>
                                 (shen/shen\.<signature> Parse_shen\.<name>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<signature>))
                                  (shen/let Parse_shen\.<rules>
                                            (shen/shen\.<rules> Parse_shen\.<signature>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<rules>))
                                             (shen/shen\.pair
                                              (shen/hd Parse_shen\.<rules>)
                                              (shen/shen\.compile_to_machine_code
                                               (shen/shen\.hdtl Parse_shen\.<name>)
                                               (shen/shen\.hdtl Parse_shen\.<rules>)))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<name>
                       (shen/shen\.<name> V1204)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<name>))
                        (shen/let Parse_shen\.<rules>
                                  (shen/shen\.<rules> Parse_shen\.<name>)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<rules>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<rules>)
                                    (shen/shen\.compile_to_machine_code
                                     (shen/shen\.hdtl Parse_shen\.<name>)
                                     (shen/shen\.hdtl Parse_shen\.<rules>)))
                                   (shen/fail)))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<name>
    (V1206)
  (shen/if
   (shen/cons\?
    (shen/hd V1206))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V1206))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V1206))
                (shen/shen\.hdtl V1206)))
              (shen/if
               (shen/and
                (shen/symbol\? Parse_X)
                (shen/not
                 (shen/shen\.sysfunc\? Parse_X)))
               Parse_X
               (shen/simple-error
                (shen/shen\.app Parse_X " is not a legitimate function name.\n" 'shen\.a)))))
   (shen/fail)))
(defun shen/shen\.sysfunc\?
    (V1208)
  (shen/element\? V1208
                  (shen/get
                   (shen/intern "shen")
                   'shen\.external-symbols
                   (shen/value '*property-vector*))))
(defun shen/shen\.<signature>
    (V1210)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V1210))
    (shen/= '{
            (shen/hd
             (shen/hd V1210))))
   (shen/let Parse_shen\.<signature-help>
             (shen/shen\.<signature-help>
              (shen/shen\.pair
               (nthcdr 1
                       (shen/hd V1210))
               (shen/shen\.hdtl V1210)))
             (shen/if
              (shen/not
               (shen/=
                (shen/fail)
                Parse_shen\.<signature-help>))
              (shen/if
               (shen/and
                (shen/cons\?
                 (shen/hd Parse_shen\.<signature-help>))
                (shen/= '}
                        (shen/hd
                         (shen/hd Parse_shen\.<signature-help>))))
               (shen/shen\.pair
                (shen/hd
                 (shen/shen\.pair
                  (nthcdr 1
                          (shen/hd Parse_shen\.<signature-help>))
                  (shen/shen\.hdtl Parse_shen\.<signature-help>)))
                (shen/shen\.demodulate
                 (shen/shen\.curry-type
                  (shen/shen\.hdtl Parse_shen\.<signature-help>))))
               (shen/fail))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.curry-type
    (V1212)
  (cl-flet
      ((tail-trampoline
        (V1212)
        (shen/cond
         ((shen/and
           (shen/cons\? V1212)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1212))
            (shen/and
             (shen/= '-->
                     (shen/hd
                      (nthcdr 1 V1212)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1212))
              (shen/and
               (shen/cons\?
                (nthcdr 3 V1212))
               (shen/= '-->
                       (shen/hd
                        (nthcdr 3 V1212))))))))
          (vector
           (list
            (list
             (shen/hd V1212)
             '-->
             (nthcdr 2 V1212)))))
         ((shen/and
           (shen/cons\? V1212)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1212))
            (shen/and
             (shen/= '*
                     (shen/hd
                      (nthcdr 1 V1212)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1212))
              (shen/and
               (shen/cons\?
                (nthcdr 3 V1212))
               (shen/= '*
                       (shen/hd
                        (nthcdr 3 V1212))))))))
          (vector
           (list
            (list
             (shen/hd V1212)
             '*
             (nthcdr 2 V1212)))))
         ((shen/cons\? V1212)
          (shen/map
           (shen/lambda Z
                        (shen/shen\.curry-type Z))
           V1212))
         (shen/true V1212))))
    (let
        ((result
          (funcall #'tail-trampoline V1212)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.<signature-help>
    (V1214)
  (shen/let YaccParse
            (shen/if
             (shen/cons\?
              (shen/hd V1214))
             (shen/let Parse_X
                       (shen/hd
                        (shen/hd V1214))
                       (shen/let Parse_shen\.<signature-help>
                                 (shen/shen\.<signature-help>
                                  (shen/shen\.pair
                                   (nthcdr 1
                                           (shen/hd V1214))
                                   (shen/shen\.hdtl V1214)))
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<signature-help>))
                                  (shen/if
                                   (shen/not
                                    (shen/element\? Parse_X
                                                    (list '{ '})))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<signature-help>)
                                    (append
                                     (list Parse_X)
                                     (shen/shen\.hdtl Parse_shen\.<signature-help>)))
                                   (shen/fail))
                                  (shen/fail))))
             (shen/fail))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V1214)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<rules>
    (V1216)
  (shen/let YaccParse
            (shen/let Parse_shen\.<rule>
                      (shen/shen\.<rule> V1216)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<rule>))
                       (shen/let Parse_shen\.<rules>
                                 (shen/shen\.<rules> Parse_shen\.<rule>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<rules>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<rules>)
                                   (append
                                    (list
                                     (shen/shen\.linearise
                                      (shen/shen\.hdtl Parse_shen\.<rule>)))
                                    (shen/shen\.hdtl Parse_shen\.<rules>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<rule>
                       (shen/shen\.<rule> V1216)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<rule>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<rule>)
                         (list
                          (shen/shen\.linearise
                           (shen/shen\.hdtl Parse_shen\.<rule>))))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<rule>
    (V1218)
  (shen/let YaccParse
            (shen/let Parse_shen\.<patterns>
                      (shen/shen\.<patterns> V1218)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<patterns>))
                       (shen/if
                        (shen/and
                         (shen/cons\?
                          (shen/hd Parse_shen\.<patterns>))
                         (shen/= '->
                                 (shen/hd
                                  (shen/hd Parse_shen\.<patterns>))))
                        (shen/let Parse_shen\.<action>
                                  (shen/shen\.<action>
                                   (shen/shen\.pair
                                    (nthcdr 1
                                            (shen/hd Parse_shen\.<patterns>))
                                    (shen/shen\.hdtl Parse_shen\.<patterns>)))
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<action>))
                                   (shen/if
                                    (shen/and
                                     (shen/cons\?
                                      (shen/hd Parse_shen\.<action>))
                                     (shen/= 'where
                                             (shen/hd
                                              (shen/hd Parse_shen\.<action>))))
                                    (shen/let Parse_shen\.<guard>
                                              (shen/shen\.<guard>
                                               (shen/shen\.pair
                                                (nthcdr 1
                                                        (shen/hd Parse_shen\.<action>))
                                                (shen/shen\.hdtl Parse_shen\.<action>)))
                                              (shen/if
                                               (shen/not
                                                (shen/=
                                                 (shen/fail)
                                                 Parse_shen\.<guard>))
                                               (shen/shen\.pair
                                                (shen/hd Parse_shen\.<guard>)
                                                (list
                                                 (shen/shen\.hdtl Parse_shen\.<patterns>)
                                                 (list 'where
                                                       (shen/shen\.hdtl Parse_shen\.<guard>)
                                                       (shen/shen\.hdtl Parse_shen\.<action>))))
                                               (shen/fail)))
                                    (shen/fail))
                                   (shen/fail)))
                        (shen/fail))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<patterns>
                                 (shen/shen\.<patterns> V1218)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<patterns>))
                                  (shen/if
                                   (shen/and
                                    (shen/cons\?
                                     (shen/hd Parse_shen\.<patterns>))
                                    (shen/= '->
                                            (shen/hd
                                             (shen/hd Parse_shen\.<patterns>))))
                                   (shen/let Parse_shen\.<action>
                                             (shen/shen\.<action>
                                              (shen/shen\.pair
                                               (nthcdr 1
                                                       (shen/hd Parse_shen\.<patterns>))
                                               (shen/shen\.hdtl Parse_shen\.<patterns>)))
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<action>))
                                              (shen/shen\.pair
                                               (shen/hd Parse_shen\.<action>)
                                               (list
                                                (shen/shen\.hdtl Parse_shen\.<patterns>)
                                                (shen/shen\.hdtl Parse_shen\.<action>)))
                                              (shen/fail)))
                                   (shen/fail))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let YaccParse
                                  (shen/let Parse_shen\.<patterns>
                                            (shen/shen\.<patterns> V1218)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<patterns>))
                                             (shen/if
                                              (shen/and
                                               (shen/cons\?
                                                (shen/hd Parse_shen\.<patterns>))
                                               (shen/= '<-
                                                       (shen/hd
                                                        (shen/hd Parse_shen\.<patterns>))))
                                              (shen/let Parse_shen\.<action>
                                                        (shen/shen\.<action>
                                                         (shen/shen\.pair
                                                          (nthcdr 1
                                                                  (shen/hd Parse_shen\.<patterns>))
                                                          (shen/shen\.hdtl Parse_shen\.<patterns>)))
                                                        (shen/if
                                                         (shen/not
                                                          (shen/=
                                                           (shen/fail)
                                                           Parse_shen\.<action>))
                                                         (shen/if
                                                          (shen/and
                                                           (shen/cons\?
                                                            (shen/hd Parse_shen\.<action>))
                                                           (shen/= 'where
                                                                   (shen/hd
                                                                    (shen/hd Parse_shen\.<action>))))
                                                          (shen/let Parse_shen\.<guard>
                                                                    (shen/shen\.<guard>
                                                                     (shen/shen\.pair
                                                                      (nthcdr 1
                                                                              (shen/hd Parse_shen\.<action>))
                                                                      (shen/shen\.hdtl Parse_shen\.<action>)))
                                                                    (shen/if
                                                                     (shen/not
                                                                      (shen/=
                                                                       (shen/fail)
                                                                       Parse_shen\.<guard>))
                                                                     (shen/shen\.pair
                                                                      (shen/hd Parse_shen\.<guard>)
                                                                      (list
                                                                       (shen/shen\.hdtl Parse_shen\.<patterns>)
                                                                       (list 'where
                                                                             (shen/shen\.hdtl Parse_shen\.<guard>)
                                                                             (list 'shen\.choicepoint!
                                                                                   (shen/shen\.hdtl Parse_shen\.<action>)))))
                                                                     (shen/fail)))
                                                          (shen/fail))
                                                         (shen/fail)))
                                              (shen/fail))
                                             (shen/fail)))
                                  (shen/if
                                   (shen/= YaccParse
                                           (shen/fail))
                                   (shen/let Parse_shen\.<patterns>
                                             (shen/shen\.<patterns> V1218)
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<patterns>))
                                              (shen/if
                                               (shen/and
                                                (shen/cons\?
                                                 (shen/hd Parse_shen\.<patterns>))
                                                (shen/= '<-
                                                        (shen/hd
                                                         (shen/hd Parse_shen\.<patterns>))))
                                               (shen/let Parse_shen\.<action>
                                                         (shen/shen\.<action>
                                                          (shen/shen\.pair
                                                           (nthcdr 1
                                                                   (shen/hd Parse_shen\.<patterns>))
                                                           (shen/shen\.hdtl Parse_shen\.<patterns>)))
                                                         (shen/if
                                                          (shen/not
                                                           (shen/=
                                                            (shen/fail)
                                                            Parse_shen\.<action>))
                                                          (shen/shen\.pair
                                                           (shen/hd Parse_shen\.<action>)
                                                           (list
                                                            (shen/shen\.hdtl Parse_shen\.<patterns>)
                                                            (list 'shen\.choicepoint!
                                                                  (shen/shen\.hdtl Parse_shen\.<action>))))
                                                          (shen/fail)))
                                               (shen/fail))
                                              (shen/fail)))
                                   YaccParse))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.fail_if
    (V1221 V1222)
  (shen/if
   (shen/internal/apply-higher-order-function V1221
                                              (list V1222))
   (shen/fail)
   V1222))
(defun shen/shen\.succeeds\?
    (V1228)
  (shen/cond
   ((shen/= V1228
            (shen/fail))
    'false)
   (shen/true 'true)))
(defun shen/shen\.<patterns>
    (V1230)
  (shen/let YaccParse
            (shen/let Parse_shen\.<pattern>
                      (shen/shen\.<pattern> V1230)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<pattern>))
                       (shen/let Parse_shen\.<patterns>
                                 (shen/shen\.<patterns> Parse_shen\.<pattern>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<patterns>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<patterns>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<pattern>))
                                    (shen/shen\.hdtl Parse_shen\.<patterns>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V1230)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<pattern>
    (V1237)
  (shen/let YaccParse
            (shen/if
             (shen/and
              (shen/cons\?
               (shen/hd V1237))
              (shen/cons\?
               (shen/hd
                (shen/hd V1237))))
             (shen/if
              (shen/and
               (shen/cons\?
                (shen/hd
                 (shen/shen\.pair
                  (shen/hd
                   (shen/hd V1237))
                  (shen/hd
                   (nthcdr 1 V1237)))))
               (shen/= '@p
                       (shen/hd
                        (shen/hd
                         (shen/shen\.pair
                          (shen/hd
                           (shen/hd V1237))
                          (shen/hd
                           (nthcdr 1 V1237)))))))
              (shen/let Parse_shen\.<pattern1>
                        (shen/shen\.<pattern1>
                         (shen/shen\.pair
                          (nthcdr 1
                                  (shen/hd
                                   (shen/shen\.pair
                                    (shen/hd
                                     (shen/hd V1237))
                                    (shen/hd
                                     (nthcdr 1 V1237)))))
                          (shen/shen\.hdtl
                           (shen/shen\.pair
                            (shen/hd
                             (shen/hd V1237))
                            (shen/hd
                             (nthcdr 1 V1237))))))
                        (shen/if
                         (shen/not
                          (shen/=
                           (shen/fail)
                           Parse_shen\.<pattern1>))
                         (shen/let Parse_shen\.<pattern2>
                                   (shen/shen\.<pattern2> Parse_shen\.<pattern1>)
                                   (shen/if
                                    (shen/not
                                     (shen/=
                                      (shen/fail)
                                      Parse_shen\.<pattern2>))
                                    (shen/shen\.pair
                                     (shen/hd
                                      (shen/shen\.pair
                                       (nthcdr 1
                                               (shen/hd V1237))
                                       (shen/hd
                                        (nthcdr 1 V1237))))
                                     (list '@p
                                           (shen/shen\.hdtl Parse_shen\.<pattern1>)
                                           (shen/shen\.hdtl Parse_shen\.<pattern2>)))
                                    (shen/fail)))
                         (shen/fail)))
              (shen/fail))
             (shen/fail))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/if
                        (shen/and
                         (shen/cons\?
                          (shen/hd V1237))
                         (shen/cons\?
                          (shen/hd
                           (shen/hd V1237))))
                        (shen/if
                         (shen/and
                          (shen/cons\?
                           (shen/hd
                            (shen/shen\.pair
                             (shen/hd
                              (shen/hd V1237))
                             (shen/hd
                              (nthcdr 1 V1237)))))
                          (shen/= 'cons
                                  (shen/hd
                                   (shen/hd
                                    (shen/shen\.pair
                                     (shen/hd
                                      (shen/hd V1237))
                                     (shen/hd
                                      (nthcdr 1 V1237)))))))
                         (shen/let Parse_shen\.<pattern1>
                                   (shen/shen\.<pattern1>
                                    (shen/shen\.pair
                                     (nthcdr 1
                                             (shen/hd
                                              (shen/shen\.pair
                                               (shen/hd
                                                (shen/hd V1237))
                                               (shen/hd
                                                (nthcdr 1 V1237)))))
                                     (shen/shen\.hdtl
                                      (shen/shen\.pair
                                       (shen/hd
                                        (shen/hd V1237))
                                       (shen/hd
                                        (nthcdr 1 V1237))))))
                                   (shen/if
                                    (shen/not
                                     (shen/=
                                      (shen/fail)
                                      Parse_shen\.<pattern1>))
                                    (shen/let Parse_shen\.<pattern2>
                                              (shen/shen\.<pattern2> Parse_shen\.<pattern1>)
                                              (shen/if
                                               (shen/not
                                                (shen/=
                                                 (shen/fail)
                                                 Parse_shen\.<pattern2>))
                                               (shen/shen\.pair
                                                (shen/hd
                                                 (shen/shen\.pair
                                                  (nthcdr 1
                                                          (shen/hd V1237))
                                                  (shen/hd
                                                   (nthcdr 1 V1237))))
                                                (list 'cons
                                                      (shen/shen\.hdtl Parse_shen\.<pattern1>)
                                                      (shen/shen\.hdtl Parse_shen\.<pattern2>)))
                                               (shen/fail)))
                                    (shen/fail)))
                         (shen/fail))
                        (shen/fail))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let YaccParse
                                  (shen/if
                                   (shen/and
                                    (shen/cons\?
                                     (shen/hd V1237))
                                    (shen/cons\?
                                     (shen/hd
                                      (shen/hd V1237))))
                                   (shen/if
                                    (shen/and
                                     (shen/cons\?
                                      (shen/hd
                                       (shen/shen\.pair
                                        (shen/hd
                                         (shen/hd V1237))
                                        (shen/hd
                                         (nthcdr 1 V1237)))))
                                     (shen/= '@v
                                             (shen/hd
                                              (shen/hd
                                               (shen/shen\.pair
                                                (shen/hd
                                                 (shen/hd V1237))
                                                (shen/hd
                                                 (nthcdr 1 V1237)))))))
                                    (shen/let Parse_shen\.<pattern1>
                                              (shen/shen\.<pattern1>
                                               (shen/shen\.pair
                                                (nthcdr 1
                                                        (shen/hd
                                                         (shen/shen\.pair
                                                          (shen/hd
                                                           (shen/hd V1237))
                                                          (shen/hd
                                                           (nthcdr 1 V1237)))))
                                                (shen/shen\.hdtl
                                                 (shen/shen\.pair
                                                  (shen/hd
                                                   (shen/hd V1237))
                                                  (shen/hd
                                                   (nthcdr 1 V1237))))))
                                              (shen/if
                                               (shen/not
                                                (shen/=
                                                 (shen/fail)
                                                 Parse_shen\.<pattern1>))
                                               (shen/let Parse_shen\.<pattern2>
                                                         (shen/shen\.<pattern2> Parse_shen\.<pattern1>)
                                                         (shen/if
                                                          (shen/not
                                                           (shen/=
                                                            (shen/fail)
                                                            Parse_shen\.<pattern2>))
                                                          (shen/shen\.pair
                                                           (shen/hd
                                                            (shen/shen\.pair
                                                             (nthcdr 1
                                                                     (shen/hd V1237))
                                                             (shen/hd
                                                              (nthcdr 1 V1237))))
                                                           (list '@v
                                                                 (shen/shen\.hdtl Parse_shen\.<pattern1>)
                                                                 (shen/shen\.hdtl Parse_shen\.<pattern2>)))
                                                          (shen/fail)))
                                               (shen/fail)))
                                    (shen/fail))
                                   (shen/fail))
                                  (shen/if
                                   (shen/= YaccParse
                                           (shen/fail))
                                   (shen/let YaccParse
                                             (shen/if
                                              (shen/and
                                               (shen/cons\?
                                                (shen/hd V1237))
                                               (shen/cons\?
                                                (shen/hd
                                                 (shen/hd V1237))))
                                              (shen/if
                                               (shen/and
                                                (shen/cons\?
                                                 (shen/hd
                                                  (shen/shen\.pair
                                                   (shen/hd
                                                    (shen/hd V1237))
                                                   (shen/hd
                                                    (nthcdr 1 V1237)))))
                                                (shen/= '@s
                                                        (shen/hd
                                                         (shen/hd
                                                          (shen/shen\.pair
                                                           (shen/hd
                                                            (shen/hd V1237))
                                                           (shen/hd
                                                            (nthcdr 1 V1237)))))))
                                               (shen/let Parse_shen\.<pattern1>
                                                         (shen/shen\.<pattern1>
                                                          (shen/shen\.pair
                                                           (nthcdr 1
                                                                   (shen/hd
                                                                    (shen/shen\.pair
                                                                     (shen/hd
                                                                      (shen/hd V1237))
                                                                     (shen/hd
                                                                      (nthcdr 1 V1237)))))
                                                           (shen/shen\.hdtl
                                                            (shen/shen\.pair
                                                             (shen/hd
                                                              (shen/hd V1237))
                                                             (shen/hd
                                                              (nthcdr 1 V1237))))))
                                                         (shen/if
                                                          (shen/not
                                                           (shen/=
                                                            (shen/fail)
                                                            Parse_shen\.<pattern1>))
                                                          (shen/let Parse_shen\.<pattern2>
                                                                    (shen/shen\.<pattern2> Parse_shen\.<pattern1>)
                                                                    (shen/if
                                                                     (shen/not
                                                                      (shen/=
                                                                       (shen/fail)
                                                                       Parse_shen\.<pattern2>))
                                                                     (shen/shen\.pair
                                                                      (shen/hd
                                                                       (shen/shen\.pair
                                                                        (nthcdr 1
                                                                                (shen/hd V1237))
                                                                        (shen/hd
                                                                         (nthcdr 1 V1237))))
                                                                      (list '@s
                                                                            (shen/shen\.hdtl Parse_shen\.<pattern1>)
                                                                            (shen/shen\.hdtl Parse_shen\.<pattern2>)))
                                                                     (shen/fail)))
                                                          (shen/fail)))
                                               (shen/fail))
                                              (shen/fail))
                                             (shen/if
                                              (shen/= YaccParse
                                                      (shen/fail))
                                              (shen/let YaccParse
                                                        (shen/if
                                                         (shen/and
                                                          (shen/cons\?
                                                           (shen/hd V1237))
                                                          (shen/cons\?
                                                           (shen/hd
                                                            (shen/hd V1237))))
                                                         (shen/if
                                                          (shen/and
                                                           (shen/cons\?
                                                            (shen/hd
                                                             (shen/shen\.pair
                                                              (shen/hd
                                                               (shen/hd V1237))
                                                              (shen/hd
                                                               (nthcdr 1 V1237)))))
                                                           (shen/= 'vector
                                                                   (shen/hd
                                                                    (shen/hd
                                                                     (shen/shen\.pair
                                                                      (shen/hd
                                                                       (shen/hd V1237))
                                                                      (shen/hd
                                                                       (nthcdr 1 V1237)))))))
                                                          (shen/if
                                                           (shen/and
                                                            (shen/cons\?
                                                             (shen/hd
                                                              (shen/shen\.pair
                                                               (nthcdr 1
                                                                       (shen/hd
                                                                        (shen/shen\.pair
                                                                         (shen/hd
                                                                          (shen/hd V1237))
                                                                         (shen/hd
                                                                          (nthcdr 1 V1237)))))
                                                               (shen/shen\.hdtl
                                                                (shen/shen\.pair
                                                                 (shen/hd
                                                                  (shen/hd V1237))
                                                                 (shen/hd
                                                                  (nthcdr 1 V1237)))))))
                                                            (shen/= 0
                                                                    (shen/hd
                                                                     (shen/hd
                                                                      (shen/shen\.pair
                                                                       (nthcdr 1
                                                                               (shen/hd
                                                                                (shen/shen\.pair
                                                                                 (shen/hd
                                                                                  (shen/hd V1237))
                                                                                 (shen/hd
                                                                                  (nthcdr 1 V1237)))))
                                                                       (shen/shen\.hdtl
                                                                        (shen/shen\.pair
                                                                         (shen/hd
                                                                          (shen/hd V1237))
                                                                         (shen/hd
                                                                          (nthcdr 1 V1237)))))))))
                                                           (shen/shen\.pair
                                                            (shen/hd
                                                             (shen/shen\.pair
                                                              (nthcdr 1
                                                                      (shen/hd V1237))
                                                              (shen/hd
                                                               (nthcdr 1 V1237))))
                                                            (list 'vector 0))
                                                           (shen/fail))
                                                          (shen/fail))
                                                         (shen/fail))
                                                        (shen/if
                                                         (shen/= YaccParse
                                                                 (shen/fail))
                                                         (shen/let YaccParse
                                                                   (shen/if
                                                                    (shen/cons\?
                                                                     (shen/hd V1237))
                                                                    (shen/let Parse_X
                                                                              (shen/hd
                                                                               (shen/hd V1237))
                                                                              (shen/if
                                                                               (shen/cons\? Parse_X)
                                                                               (shen/shen\.pair
                                                                                (shen/hd
                                                                                 (shen/shen\.pair
                                                                                  (nthcdr 1
                                                                                          (shen/hd V1237))
                                                                                  (shen/shen\.hdtl V1237)))
                                                                                (shen/shen\.constructor-error Parse_X))
                                                                               (shen/fail)))
                                                                    (shen/fail))
                                                                   (shen/if
                                                                    (shen/= YaccParse
                                                                            (shen/fail))
                                                                    (shen/let Parse_shen\.<simple_pattern>
                                                                              (shen/shen\.<simple_pattern> V1237)
                                                                              (shen/if
                                                                               (shen/not
                                                                                (shen/=
                                                                                 (shen/fail)
                                                                                 Parse_shen\.<simple_pattern>))
                                                                               (shen/shen\.pair
                                                                                (shen/hd Parse_shen\.<simple_pattern>)
                                                                                (shen/shen\.hdtl Parse_shen\.<simple_pattern>))
                                                                               (shen/fail)))
                                                                    YaccParse))
                                                         YaccParse))
                                              YaccParse))
                                   YaccParse))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.constructor-error
    (V1239)
  (shen/simple-error
   (shen/shen\.app V1239 " is not a legitimate constructor\n" 'shen\.a)))
(defun shen/shen\.<simple_pattern>
    (V1241)
  (shen/let YaccParse
            (shen/if
             (shen/cons\?
              (shen/hd V1241))
             (shen/let Parse_X
                       (shen/hd
                        (shen/hd V1241))
                       (shen/if
                        (shen/= Parse_X '_)
                        (shen/shen\.pair
                         (shen/hd
                          (shen/shen\.pair
                           (nthcdr 1
                                   (shen/hd V1241))
                           (shen/shen\.hdtl V1241)))
                         (shen/gensym 'Parse_Y))
                        (shen/fail)))
             (shen/fail))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/if
              (shen/cons\?
               (shen/hd V1241))
              (shen/let Parse_X
                        (shen/hd
                         (shen/hd V1241))
                        (shen/if
                         (shen/not
                          (shen/element\? Parse_X
                                          (list '-> '<-)))
                         (shen/shen\.pair
                          (shen/hd
                           (shen/shen\.pair
                            (nthcdr 1
                                    (shen/hd V1241))
                            (shen/shen\.hdtl V1241)))
                          Parse_X)
                         (shen/fail)))
              (shen/fail))
             YaccParse)))
(defun shen/shen\.<pattern1>
    (V1243)
  (shen/let Parse_shen\.<pattern>
            (shen/shen\.<pattern> V1243)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<pattern>))
             (shen/shen\.pair
              (shen/hd Parse_shen\.<pattern>)
              (shen/shen\.hdtl Parse_shen\.<pattern>))
             (shen/fail))))
(defun shen/shen\.<pattern2>
    (V1245)
  (shen/let Parse_shen\.<pattern>
            (shen/shen\.<pattern> V1245)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<pattern>))
             (shen/shen\.pair
              (shen/hd Parse_shen\.<pattern>)
              (shen/shen\.hdtl Parse_shen\.<pattern>))
             (shen/fail))))
(defun shen/shen\.<action>
    (V1247)
  (shen/if
   (shen/cons\?
    (shen/hd V1247))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V1247))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V1247))
                (shen/shen\.hdtl V1247)))
              Parse_X))
   (shen/fail)))
(defun shen/shen\.<guard>
    (V1249)
  (shen/if
   (shen/cons\?
    (shen/hd V1249))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V1249))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V1249))
                (shen/shen\.hdtl V1249)))
              Parse_X))
   (shen/fail)))
(defun shen/shen\.compile_to_machine_code
    (V1252 V1253)
  (shen/let Lambda+
            (shen/shen\.compile_to_lambda+ V1252 V1253)
            (shen/let KL
                      (shen/shen\.compile_to_kl V1252 Lambda+)
                      (shen/let Record
                                (shen/shen\.record-source V1252 KL)
                                KL))))
(defun shen/shen\.record-source
    (V1258 V1259)
  (shen/cond
   ((shen/value 'shen\.*installing-kl*)
    'shen\.skip)
   (shen/true
    (shen/put V1258 'shen\.source V1259
              (shen/value '*property-vector*)))))
(defun shen/shen\.compile_to_lambda+
    (V1262 V1263)
  (shen/let Arity
            (shen/shen\.aritycheck V1262 V1263)
            (shen/let UpDateSymbolTable
                      (shen/shen\.update-symbol-table V1262 Arity)
                      (shen/let Free
                                (shen/for-each
                                 (shen/lambda Rule
                                              (shen/shen\.free_variable_check V1262 Rule))
                                 V1263)
                                (shen/let Variables
                                          (shen/shen\.parameters Arity)
                                          (shen/let Strip
                                                    (shen/map
                                                     (shen/lambda X
                                                                  (shen/shen\.strip-protect X))
                                                     V1263)
                                                    (shen/let Abstractions
                                                              (shen/map
                                                               (shen/lambda X
                                                                            (shen/shen\.abstract_rule X))
                                                               Strip)
                                                              (shen/let Applications
                                                                        (shen/map
                                                                         (shen/lambda X
                                                                                      (shen/shen\.application_build Variables X))
                                                                         Abstractions)
                                                                        (list Variables Applications)))))))))
(defun shen/shen\.update-symbol-table
    (V1266 V1267)
  (shen/cond
   ((shen/= 0 V1267)
    'shen\.skip)
   (shen/true
    (shen/put V1266 'shen\.lambda-form
              (shen/eval-kl
               (shen/shen\.lambda-form V1266 V1267))
              (shen/value '*property-vector*)))))
(defun shen/shen\.free_variable_check
    (V1270 V1271)
  (shen/cond
   ((shen/and
     (shen/cons\? V1271)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1271))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1271)))))
    (shen/let Bound
              (shen/shen\.extract_vars
               (shen/hd V1271))
              (shen/let Free
                        (shen/shen\.extract_free_vars Bound
                                                      (shen/hd
                                                       (nthcdr 1 V1271)))
                        (shen/shen\.free_variable_warnings V1270 Free))))
   (shen/true
    (shen/shen\.f_error 'shen\.free_variable_check))))
(defun shen/shen\.extract_vars
    (V1273)
  (shen/cond
   ((shen/variable\? V1273)
    (list V1273))
   ((shen/cons\? V1273)
    (shen/union
     (shen/shen\.extract_vars
      (shen/hd V1273))
     (shen/shen\.extract_vars
      (nthcdr 1 V1273))))
   (shen/true nil)))
(defun shen/shen\.extract_free_vars
    (V1285 V1286)
  (cl-flet
      ((tail-trampoline
        (V1285 V1286)
        (shen/cond
         ((shen/and
           (shen/cons\? V1286)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1286))
            (shen/and
             (shen/internal/predicate->shen
              (null
               (nthcdr 2 V1286)))
             (shen/=
              (shen/hd V1286)
              'protect))))
          nil)
         ((shen/and
           (shen/variable\? V1286)
           (shen/not
            (shen/element\? V1286 V1285)))
          (list V1286))
         ((shen/and
           (shen/cons\? V1286)
           (shen/and
            (shen/= 'lambda
                    (shen/hd V1286))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1286))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1286))
              (shen/internal/predicate->shen
               (null
                (nthcdr 3 V1286)))))))
          (vector
           (list
            (append
             (list
              (shen/hd
               (nthcdr 1 V1286)))
             V1285)
            (shen/hd
             (nthcdr 2 V1286)))))
         ((shen/and
           (shen/cons\? V1286)
           (shen/and
            (shen/= 'let
                    (shen/hd V1286))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1286))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1286))
              (shen/and
               (shen/cons\?
                (nthcdr 3 V1286))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 4 V1286))))))))
          (shen/union
           (shen/shen\.extract_free_vars V1285
                                         (shen/hd
                                          (nthcdr 2 V1286)))
           (shen/shen\.extract_free_vars
            (append
             (list
              (shen/hd
               (nthcdr 1 V1286)))
             V1285)
            (shen/hd
             (nthcdr 3 V1286)))))
         ((shen/cons\? V1286)
          (shen/union
           (shen/shen\.extract_free_vars V1285
                                         (shen/hd V1286))
           (shen/shen\.extract_free_vars V1285
                                         (nthcdr 1 V1286))))
         (shen/true nil))))
    (let
        ((result
          (funcall #'tail-trampoline V1285 V1286)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.free_variable_warnings
    (V1291 V1292)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1292))
    '_)
   (shen/true
    (shen/simple-error
     (shen/cn "error: the following variables are free in "
              (shen/shen\.app V1291
                              (shen/cn ": "
                                       (shen/shen\.app
                                        (shen/shen\.list_variables V1292)
                                        "" 'shen\.a))
                              'shen\.a))))))
(defun shen/shen\.list_variables
    (V1294)
  (shen/cond
   ((shen/and
     (shen/cons\? V1294)
     (shen/internal/predicate->shen
      (null
       (nthcdr 1 V1294))))
    (shen/cn
     (shen/str
      (shen/hd V1294))
     "."))
   ((shen/cons\? V1294)
    (shen/cn
     (shen/str
      (shen/hd V1294))
     (shen/cn ", "
              (shen/shen\.list_variables
               (nthcdr 1 V1294)))))
   (shen/true
    (shen/shen\.f_error 'shen\.list_variables))))
(defun shen/shen\.strip-protect
    (V1296)
  (cl-flet
      ((tail-trampoline
        (V1296)
        (shen/cond
         ((shen/and
           (shen/cons\? V1296)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1296))
            (shen/and
             (shen/internal/predicate->shen
              (null
               (nthcdr 2 V1296)))
             (shen/=
              (shen/hd V1296)
              'protect))))
          (vector
           (list
            (shen/hd
             (nthcdr 1 V1296)))))
         ((shen/cons\? V1296)
          (shen/map
           (shen/lambda Z
                        (shen/shen\.strip-protect Z))
           V1296))
         (shen/true V1296))))
    (let
        ((result
          (funcall #'tail-trampoline V1296)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.linearise
    (V1298)
  (shen/cond
   ((shen/and
     (shen/cons\? V1298)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1298))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1298)))))
    (shen/shen\.linearise_help
     (shen/shen\.flatten
      (shen/hd V1298))
     (shen/hd V1298)
     (shen/hd
      (nthcdr 1 V1298))))
   (shen/true
    (shen/shen\.f_error 'shen\.linearise))))
(defun shen/shen\.flatten
    (V1300)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1300))
    nil)
   ((shen/cons\? V1300)
    (shen/append
     (shen/shen\.flatten
      (shen/hd V1300))
     (shen/shen\.flatten
      (nthcdr 1 V1300))))
   (shen/true
    (list V1300))))
(defun shen/shen\.linearise_help
    (V1304 V1305 V1306)
  (cl-flet
      ((tail-trampoline
        (V1304 V1305 V1306)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1304))
          (list V1305 V1306))
         ((shen/cons\? V1304)
          (shen/if
           (shen/and
            (shen/variable\?
             (shen/hd V1304))
            (shen/element\?
             (shen/hd V1304)
             (nthcdr 1 V1304)))
           (shen/let Var
                     (shen/gensym
                      (shen/hd V1304))
                     (shen/let NewAction
                               (list 'where
                                     (list '=
                                           (shen/hd V1304)
                                           Var)
                                     V1306)
                               (shen/let NewPatts
                                         (shen/shen\.linearise_X
                                          (shen/hd V1304)
                                          Var V1305)
                                         (vector
                                          (list
                                           (nthcdr 1 V1304)
                                           NewPatts NewAction)))))
           (vector
            (list
             (nthcdr 1 V1304)
             V1305 V1306))))
         (shen/true
          (shen/shen\.f_error 'shen\.linearise_help)))))
    (let
        ((result
          (funcall #'tail-trampoline V1304 V1305 V1306)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.linearise_X
    (V1319 V1320 V1321)
  (shen/cond
   ((shen/= V1321 V1319)
    V1320)
   ((shen/cons\? V1321)
    (shen/let L
              (shen/shen\.linearise_X V1319 V1320
                                      (shen/hd V1321))
              (shen/if
               (shen/= L
                       (shen/hd V1321))
               (append
                (list
                 (shen/hd V1321))
                (shen/shen\.linearise_X V1319 V1320
                                        (nthcdr 1 V1321)))
               (append
                (list L)
                (nthcdr 1 V1321)))))
   (shen/true V1321)))
(defun shen/shen\.aritycheck
    (V1324 V1325)
  (cl-flet
      ((tail-trampoline
        (V1324 V1325)
        (shen/cond
         ((shen/and
           (shen/cons\? V1325)
           (shen/and
            (shen/cons\?
             (shen/hd V1325))
            (shen/and
             (shen/cons\?
              (nthcdr 1
                      (shen/hd V1325)))
             (shen/and
              (shen/internal/predicate->shen
               (null
                (nthcdr 2
                        (shen/hd V1325))))
              (shen/internal/predicate->shen
               (null
                (nthcdr 1 V1325)))))))
          (shen/do
           (shen/shen\.aritycheck-action
            (shen/hd
             (nthcdr 1
                     (shen/hd V1325))))
           (shen/shen\.aritycheck-name V1324
                                       (shen/arity V1324)
                                       (shen/length
                                        (shen/hd
                                         (shen/hd V1325))))))
         ((shen/and
           (shen/cons\? V1325)
           (shen/and
            (shen/cons\?
             (shen/hd V1325))
            (shen/and
             (shen/cons\?
              (nthcdr 1
                      (shen/hd V1325)))
             (shen/and
              (shen/internal/predicate->shen
               (null
                (nthcdr 2
                        (shen/hd V1325))))
              (shen/and
               (shen/cons\?
                (nthcdr 1 V1325))
               (shen/and
                (shen/cons\?
                 (shen/hd
                  (nthcdr 1 V1325)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1 V1325))))
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 2
                           (shen/hd
                            (nthcdr 1 V1325))))))))))))
          (shen/if
           (shen/=
            (shen/length
             (shen/hd
              (shen/hd V1325)))
            (shen/length
             (shen/hd
              (shen/hd
               (nthcdr 1 V1325)))))
           (shen/do
            (shen/shen\.aritycheck-action
             (shen/hd
              (nthcdr 1
                      (shen/hd V1325))))
            (vector
             (list V1324
                   (nthcdr 1 V1325))))
           (shen/simple-error
            (shen/cn "arity error in "
                     (shen/shen\.app V1324 "\n" 'shen\.a)))))
         (shen/true
          (shen/shen\.f_error 'shen\.aritycheck)))))
    (let
        ((result
          (funcall #'tail-trampoline V1324 V1325)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.aritycheck-name
    (V1338 V1339 V1340)
  (shen/cond
   ((shen/= -1 V1339)
    V1340)
   ((shen/= V1340 V1339)
    V1340)
   (shen/true
    (shen/do
     (shen/shen\.prhush
      (shen/cn "\nwarning: changing the arity of "
               (shen/shen\.app V1338 " can cause errors.\n" 'shen\.a))
      (shen/stoutput))
     V1340))))
(defun shen/shen\.aritycheck-action
    (V1346)
  (shen/cond
   ((shen/cons\? V1346)
    (shen/do
     (shen/shen\.aah
      (shen/hd V1346)
      (nthcdr 1 V1346))
     (shen/for-each
      (shen/lambda Y
                   (shen/shen\.aritycheck-action Y))
      V1346)))
   (shen/true 'shen\.skip)))
(defun shen/shen\.aah
    (V1349 V1350)
  (shen/let Arity
            (shen/arity V1349)
            (shen/let Len
                      (shen/length V1350)
                      (shen/if
                       (shen/and
                        (shen/> Arity -1)
                        (shen/> Len Arity))
                       (shen/shen\.prhush
                        (shen/cn "warning: "
                                 (shen/shen\.app V1349
                                                 (shen/cn " might not like "
                                                          (shen/shen\.app Len
                                                                          (shen/cn " argument"
                                                                                   (shen/shen\.app
                                                                                    (shen/if
                                                                                     (shen/> Len 1)
                                                                                     "s" "")
                                                                                    ".\n" 'shen\.a))
                                                                          'shen\.a))
                                                 'shen\.a))
                        (shen/stoutput))
                       'shen\.skip))))
(defun shen/shen\.abstract_rule
    (V1352)
  (shen/cond
   ((shen/and
     (shen/cons\? V1352)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1352))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1352)))))
    (shen/shen\.abstraction_build
     (shen/hd V1352)
     (shen/hd
      (nthcdr 1 V1352))))
   (shen/true
    (shen/shen\.f_error 'shen\.abstract_rule))))
(defun shen/shen\.abstraction_build
    (V1355 V1356)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1355))
    V1356)
   ((shen/cons\? V1355)
    (list '/\.
          (shen/hd V1355)
          (shen/shen\.abstraction_build
           (nthcdr 1 V1355)
           V1356)))
   (shen/true
    (shen/shen\.f_error 'shen\.abstraction_build))))
(defun shen/shen\.parameters
    (V1358)
  (shen/cond
   ((shen/= 0 V1358)
    nil)
   (shen/true
    (append
     (list
      (shen/gensym 'V))
     (shen/shen\.parameters
      (shen/- V1358 1))))))
(defun shen/shen\.application_build
    (V1361 V1362)
  (cl-flet
      ((tail-trampoline
        (V1361 V1362)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1361))
          V1362)
         ((shen/cons\? V1361)
          (vector
           (list
            (nthcdr 1 V1361)
            (list V1362
                  (shen/hd V1361)))))
         (shen/true
          (shen/shen\.f_error 'shen\.application_build)))))
    (let
        ((result
          (funcall #'tail-trampoline V1361 V1362)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.compile_to_kl
    (V1365 V1366)
  (shen/cond
   ((shen/and
     (shen/cons\? V1366)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1366))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1366)))))
    (shen/let Arity
              (shen/shen\.store-arity V1365
                                      (shen/length
                                       (shen/hd V1366)))
              (shen/let Reduce
                        (shen/map
                         (shen/lambda X
                                      (shen/shen\.reduce X))
                         (shen/hd
                          (nthcdr 1 V1366)))
                        (shen/let CondExpression
                                  (shen/shen\.cond-expression V1365
                                                              (shen/hd V1366)
                                                              Reduce)
                                  (shen/let TypeTable
                                            (shen/if
                                             (shen/value 'shen\.*optimise*)
                                             (shen/shen\.typextable
                                              (shen/shen\.get-type V1365)
                                              (shen/hd V1366))
                                             'shen\.skip)
                                            (shen/let TypedCondExpression
                                                      (shen/if
                                                       (shen/value 'shen\.*optimise*)
                                                       (shen/shen\.assign-types
                                                        (shen/hd V1366)
                                                        TypeTable CondExpression)
                                                       CondExpression)
                                                      (list 'defun V1365
                                                            (shen/hd V1366)
                                                            TypedCondExpression)))))))
   (shen/true
    (shen/shen\.f_error 'shen\.compile_to_kl))))
(defun shen/shen\.get-type
    (V1372)
  (shen/cond
   ((shen/cons\? V1372)
    'shen\.skip)
   (shen/true
    (shen/let FType
              (shen/assoc V1372
                          (shen/value 'shen\.*signedfuncs*))
              (shen/if
               (shen/empty\? FType)
               'shen\.skip
               (nthcdr 1 FType))))))
(defun shen/shen\.typextable
    (V1383 V1384)
  (cl-flet
      ((tail-trampoline
        (V1383 V1384)
        (shen/cond
         ((shen/and
           (shen/cons\? V1383)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1383))
            (shen/and
             (shen/= '-->
                     (shen/hd
                      (nthcdr 1 V1383)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1383))
              (shen/and
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1383)))
               (shen/cons\? V1384))))))
          (shen/if
           (shen/variable\?
            (shen/hd V1383))
           (vector
            (list
             (shen/hd
              (nthcdr 2 V1383))
             (nthcdr 1 V1384)))
           (append
            (list
             (append
              (list
               (shen/hd V1384))
              (shen/hd V1383)))
            (shen/shen\.typextable
             (shen/hd
              (nthcdr 2 V1383))
             (nthcdr 1 V1384)))))
         (shen/true nil))))
    (let
        ((result
          (funcall #'tail-trampoline V1383 V1384)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.assign-types
    (V1388 V1389 V1390)
  (shen/cond
   ((shen/and
     (shen/cons\? V1390)
     (shen/and
      (shen/= 'let
              (shen/hd V1390))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1390))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1390))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V1390))
         (shen/internal/predicate->shen
          (null
           (nthcdr 4 V1390))))))))
    (list 'let
          (shen/hd
           (nthcdr 1 V1390))
          (shen/shen\.assign-types V1388 V1389
                                   (shen/hd
                                    (nthcdr 2 V1390)))
          (shen/shen\.assign-types
           (append
            (list
             (shen/hd
              (nthcdr 1 V1390)))
            V1388)
           V1389
           (shen/hd
            (nthcdr 3 V1390)))))
   ((shen/and
     (shen/cons\? V1390)
     (shen/and
      (shen/= 'lambda
              (shen/hd V1390))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1390))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1390))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1390)))))))
    (list 'lambda
          (shen/hd
           (nthcdr 1 V1390))
          (shen/shen\.assign-types
           (append
            (list
             (shen/hd
              (nthcdr 1 V1390)))
            V1388)
           V1389
           (shen/hd
            (nthcdr 2 V1390)))))
   ((shen/and
     (shen/cons\? V1390)
     (shen/= 'cond
             (shen/hd V1390)))
    (append
     (list 'cond)
     (shen/map
      (shen/lambda Y
                   (list
                    (shen/shen\.assign-types V1388 V1389
                                             (shen/hd Y))
                    (shen/shen\.assign-types V1388 V1389
                                             (shen/hd
                                              (nthcdr 1 Y)))))
      (nthcdr 1 V1390))))
   ((shen/cons\? V1390)
    (shen/let NewTable
              (shen/shen\.typextable
               (shen/shen\.get-type
                (shen/hd V1390))
               (nthcdr 1 V1390))
              (append
               (list
                (shen/hd V1390))
               (shen/map
                (shen/lambda Y
                             (shen/shen\.assign-types V1388
                                                      (shen/append V1389 NewTable)
                                                      Y))
                (nthcdr 1 V1390)))))
   (shen/true
    (shen/let AtomType
              (shen/assoc V1390 V1389)
              (shen/if
               (shen/cons\? AtomType)
               (list 'type V1390
                     (nthcdr 1 AtomType))
               (shen/if
                (shen/element\? V1390 V1388)
                V1390
                (shen/shen\.atom-type V1390)))))))
(defun shen/shen\.atom-type
    (V1392)
  (shen/if
   (shen/string\? V1392)
   (list 'type V1392 'string)
   (shen/if
    (shen/number\? V1392)
    (list 'type V1392 'number)
    (shen/if
     (shen/boolean\? V1392)
     (list 'type V1392 'boolean)
     (shen/if
      (shen/symbol\? V1392)
      (list 'type V1392 'symbol)
      V1392)))))
(defun shen/shen\.store-arity
    (V1397 V1398)
  (shen/cond
   ((shen/value 'shen\.*installing-kl*)
    'shen\.skip)
   (shen/true
    (shen/put V1397 'arity V1398
              (shen/value '*property-vector*)))))
(defun shen/shen\.reduce
    (V1400)
  (shen/do
   (shen/set 'shen\.*teststack* nil)
   (shen/let Result
             (shen/shen\.reduce_help V1400)
             (list
              (append
               (list ': 'shen\.tests)
               (shen/reverse
                (shen/value 'shen\.*teststack*)))
              Result))))
(defun shen/shen\.reduce_help
    (V1402)
  (cl-flet
      ((tail-trampoline
        (V1402)
        (shen/cond
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (shen/hd V1402))
            (shen/and
             (shen/= '/\.
                     (shen/hd
                      (shen/hd V1402)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1402)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1402))))
               (shen/and
                (shen/= 'cons
                        (shen/hd
                         (shen/hd
                          (nthcdr 1
                                  (shen/hd V1402)))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1402)))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 2
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd V1402)))))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 3
                             (shen/hd
                              (nthcdr 1
                                      (shen/hd V1402))))))
                   (shen/and
                    (shen/cons\?
                     (nthcdr 2
                             (shen/hd V1402)))
                    (shen/and
                     (shen/internal/predicate->shen
                      (null
                       (nthcdr 3
                               (shen/hd V1402))))
                     (shen/and
                      (shen/cons\?
                       (nthcdr 1 V1402))
                      (shen/internal/predicate->shen
                       (null
                        (nthcdr 2 V1402)))))))))))))))
          (shen/do
           (shen/shen\.add_test
            (append
             (list 'cons\?)
             (nthcdr 1 V1402)))
           (shen/let Abstraction
                     (list '/\.
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd
                                     (nthcdr 1
                                             (shen/hd V1402)))))
                           (list '/\.
                                 (shen/hd
                                  (nthcdr 2
                                          (shen/hd
                                           (nthcdr 1
                                                   (shen/hd V1402)))))
                                 (shen/shen\.ebr
                                  (shen/hd
                                   (nthcdr 1 V1402))
                                  (shen/hd
                                   (nthcdr 1
                                           (shen/hd V1402)))
                                  (shen/hd
                                   (nthcdr 2
                                           (shen/hd V1402))))))
                     (shen/let Application
                               (list
                                (list Abstraction
                                      (append
                                       (list 'hd)
                                       (nthcdr 1 V1402)))
                                (append
                                 (list 'tl)
                                 (nthcdr 1 V1402)))
                               (vector
                                (list Application))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (shen/hd V1402))
            (shen/and
             (shen/= '/\.
                     (shen/hd
                      (shen/hd V1402)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1402)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1402))))
               (shen/and
                (shen/= '@p
                        (shen/hd
                         (shen/hd
                          (nthcdr 1
                                  (shen/hd V1402)))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1402)))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 2
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd V1402)))))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 3
                             (shen/hd
                              (nthcdr 1
                                      (shen/hd V1402))))))
                   (shen/and
                    (shen/cons\?
                     (nthcdr 2
                             (shen/hd V1402)))
                    (shen/and
                     (shen/internal/predicate->shen
                      (null
                       (nthcdr 3
                               (shen/hd V1402))))
                     (shen/and
                      (shen/cons\?
                       (nthcdr 1 V1402))
                      (shen/internal/predicate->shen
                       (null
                        (nthcdr 2 V1402)))))))))))))))
          (shen/do
           (shen/shen\.add_test
            (append
             (list 'tuple\?)
             (nthcdr 1 V1402)))
           (shen/let Abstraction
                     (list '/\.
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd
                                     (nthcdr 1
                                             (shen/hd V1402)))))
                           (list '/\.
                                 (shen/hd
                                  (nthcdr 2
                                          (shen/hd
                                           (nthcdr 1
                                                   (shen/hd V1402)))))
                                 (shen/shen\.ebr
                                  (shen/hd
                                   (nthcdr 1 V1402))
                                  (shen/hd
                                   (nthcdr 1
                                           (shen/hd V1402)))
                                  (shen/hd
                                   (nthcdr 2
                                           (shen/hd V1402))))))
                     (shen/let Application
                               (list
                                (list Abstraction
                                      (append
                                       (list 'fst)
                                       (nthcdr 1 V1402)))
                                (append
                                 (list 'snd)
                                 (nthcdr 1 V1402)))
                               (vector
                                (list Application))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (shen/hd V1402))
            (shen/and
             (shen/= '/\.
                     (shen/hd
                      (shen/hd V1402)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1402)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1402))))
               (shen/and
                (shen/= '@v
                        (shen/hd
                         (shen/hd
                          (nthcdr 1
                                  (shen/hd V1402)))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1402)))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 2
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd V1402)))))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 3
                             (shen/hd
                              (nthcdr 1
                                      (shen/hd V1402))))))
                   (shen/and
                    (shen/cons\?
                     (nthcdr 2
                             (shen/hd V1402)))
                    (shen/and
                     (shen/internal/predicate->shen
                      (null
                       (nthcdr 3
                               (shen/hd V1402))))
                     (shen/and
                      (shen/cons\?
                       (nthcdr 1 V1402))
                      (shen/internal/predicate->shen
                       (null
                        (nthcdr 2 V1402)))))))))))))))
          (shen/do
           (shen/shen\.add_test
            (append
             (list 'shen\.+vector\?)
             (nthcdr 1 V1402)))
           (shen/let Abstraction
                     (list '/\.
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd
                                     (nthcdr 1
                                             (shen/hd V1402)))))
                           (list '/\.
                                 (shen/hd
                                  (nthcdr 2
                                          (shen/hd
                                           (nthcdr 1
                                                   (shen/hd V1402)))))
                                 (shen/shen\.ebr
                                  (shen/hd
                                   (nthcdr 1 V1402))
                                  (shen/hd
                                   (nthcdr 1
                                           (shen/hd V1402)))
                                  (shen/hd
                                   (nthcdr 2
                                           (shen/hd V1402))))))
                     (shen/let Application
                               (list
                                (list Abstraction
                                      (append
                                       (list 'hdv)
                                       (nthcdr 1 V1402)))
                                (append
                                 (list 'tlv)
                                 (nthcdr 1 V1402)))
                               (vector
                                (list Application))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (shen/hd V1402))
            (shen/and
             (shen/= '/\.
                     (shen/hd
                      (shen/hd V1402)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1402)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1402))))
               (shen/and
                (shen/= '@s
                        (shen/hd
                         (shen/hd
                          (nthcdr 1
                                  (shen/hd V1402)))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1402)))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 2
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd V1402)))))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 3
                             (shen/hd
                              (nthcdr 1
                                      (shen/hd V1402))))))
                   (shen/and
                    (shen/cons\?
                     (nthcdr 2
                             (shen/hd V1402)))
                    (shen/and
                     (shen/internal/predicate->shen
                      (null
                       (nthcdr 3
                               (shen/hd V1402))))
                     (shen/and
                      (shen/cons\?
                       (nthcdr 1 V1402))
                      (shen/internal/predicate->shen
                       (null
                        (nthcdr 2 V1402)))))))))))))))
          (shen/do
           (shen/shen\.add_test
            (append
             (list 'shen\.+string\?)
             (nthcdr 1 V1402)))
           (shen/let Abstraction
                     (list '/\.
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd
                                     (nthcdr 1
                                             (shen/hd V1402)))))
                           (list '/\.
                                 (shen/hd
                                  (nthcdr 2
                                          (shen/hd
                                           (nthcdr 1
                                                   (shen/hd V1402)))))
                                 (shen/shen\.ebr
                                  (shen/hd
                                   (nthcdr 1 V1402))
                                  (shen/hd
                                   (nthcdr 1
                                           (shen/hd V1402)))
                                  (shen/hd
                                   (nthcdr 2
                                           (shen/hd V1402))))))
                     (shen/let Application
                               (list
                                (list Abstraction
                                      (list 'pos
                                            (shen/hd
                                             (nthcdr 1 V1402))
                                            0))
                                (append
                                 (list 'tlstr)
                                 (nthcdr 1 V1402)))
                               (vector
                                (list Application))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (shen/hd V1402))
            (shen/and
             (shen/= '/\.
                     (shen/hd
                      (shen/hd V1402)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1402)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1402)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1402))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1402))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 2 V1402)))
                  (shen/not
                   (shen/variable\?
                    (shen/hd
                     (nthcdr 1
                             (shen/hd V1402)))))))))))))
          (shen/do
           (shen/shen\.add_test
            (append
             (list '=
                   (shen/hd
                    (nthcdr 1
                            (shen/hd V1402))))
             (nthcdr 1 V1402)))
           (vector
            (list
             (shen/hd
              (nthcdr 2
                      (shen/hd V1402)))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (shen/hd V1402))
            (shen/and
             (shen/= '/\.
                     (shen/hd
                      (shen/hd V1402)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1402)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1402)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1402))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1402))
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 2 V1402))))))))))
          (vector
           (list
            (shen/shen\.ebr
             (shen/hd
              (nthcdr 1 V1402))
             (shen/hd
              (nthcdr 1
                      (shen/hd V1402)))
             (shen/hd
              (nthcdr 2
                      (shen/hd V1402)))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/= 'where
                    (shen/hd V1402))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1402))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1402))
              (shen/internal/predicate->shen
               (null
                (nthcdr 3 V1402)))))))
          (shen/do
           (shen/shen\.add_test
            (shen/hd
             (nthcdr 1 V1402)))
           (vector
            (list
             (shen/hd
              (nthcdr 2 V1402))))))
         ((shen/and
           (shen/cons\? V1402)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1402))
            (shen/internal/predicate->shen
             (null
              (nthcdr 2 V1402)))))
          (shen/let Z
                    (shen/shen\.reduce_help
                     (shen/hd V1402))
                    (shen/if
                     (shen/=
                      (shen/hd V1402)
                      Z)
                     V1402
                     (vector
                      (list
                       (append
                        (list Z)
                        (nthcdr 1 V1402)))))))
         (shen/true V1402))))
    (let
        ((result
          (funcall #'tail-trampoline V1402)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.+string\?
    (V1404)
  (shen/cond
   ((shen/= "" V1404)
    'false)
   (shen/true
    (shen/string\? V1404))))
(defun shen/shen\.+vector\?
    (V1406)
  (shen/and
   (shen/absvector\? V1406)
   (shen/>
    (shen/<-address V1406 0)
    0)))
(defun shen/shen\.ebr
    (V1420 V1421 V1422)
  (shen/cond
   ((shen/= V1422 V1421)
    V1420)
   ((shen/and
     (shen/cons\? V1422)
     (shen/and
      (shen/= '/\.
              (shen/hd V1422))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1422))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1422))
        (shen/and
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V1422)))
         (shen/>
          (shen/occurrences V1421
                            (shen/hd
                             (nthcdr 1 V1422)))
          0))))))
    V1422)
   ((shen/and
     (shen/cons\? V1422)
     (shen/and
      (shen/= 'lambda
              (shen/hd V1422))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1422))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1422))
        (shen/and
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V1422)))
         (shen/>
          (shen/occurrences V1421
                            (shen/hd
                             (nthcdr 1 V1422)))
          0))))))
    V1422)
   ((shen/and
     (shen/cons\? V1422)
     (shen/and
      (shen/= 'let
              (shen/hd V1422))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1422))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1422))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V1422))
         (shen/and
          (shen/internal/predicate->shen
           (null
            (nthcdr 4 V1422)))
          (shen/=
           (shen/hd
            (nthcdr 1 V1422))
           V1421)))))))
    (append
     (list 'let
           (shen/hd
            (nthcdr 1 V1422))
           (shen/shen\.ebr V1420
                           (shen/hd
                            (nthcdr 1 V1422))
                           (shen/hd
                            (nthcdr 2 V1422))))
     (nthcdr 3 V1422)))
   ((shen/cons\? V1422)
    (append
     (list
      (shen/shen\.ebr V1420 V1421
                      (shen/hd V1422)))
     (shen/shen\.ebr V1420 V1421
                     (nthcdr 1 V1422))))
   (shen/true V1422)))
(defun shen/shen\.add_test
    (V1424)
  (shen/set 'shen\.*teststack*
            (append
             (list V1424)
             (shen/value 'shen\.*teststack*))))
(defun shen/shen\.cond-expression
    (V1428 V1429 V1430)
  (shen/let Err
            (shen/shen\.err-condition V1428)
            (shen/let Cases
                      (shen/shen\.case-form V1430 Err)
                      (shen/let EncodeChoices
                                (shen/shen\.encode-choices Cases V1428)
                                (shen/shen\.cond-form EncodeChoices)))))
(defun shen/shen\.cond-form
    (V1434)
  (shen/cond
   ((shen/and
     (shen/cons\? V1434)
     (shen/and
      (shen/cons\?
       (shen/hd V1434))
      (shen/and
       (shen/= 'true
               (shen/hd
                (shen/hd V1434)))
       (shen/and
        (shen/cons\?
         (nthcdr 1
                 (shen/hd V1434)))
        (shen/internal/predicate->shen
         (null
          (nthcdr 2
                  (shen/hd V1434))))))))
    (shen/hd
     (nthcdr 1
             (shen/hd V1434))))
   (shen/true
    (append
     (list 'cond)
     V1434))))
(defun shen/shen\.encode-choices
    (V1439 V1440)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1439))
    nil)
   ((shen/and
     (shen/cons\? V1439)
     (shen/and
      (shen/cons\?
       (shen/hd V1439))
      (shen/and
       (shen/= 'true
               (shen/hd
                (shen/hd V1439)))
       (shen/and
        (shen/cons\?
         (nthcdr 1
                 (shen/hd V1439)))
        (shen/and
         (shen/cons\?
          (shen/hd
           (nthcdr 1
                   (shen/hd V1439))))
         (shen/and
          (shen/= 'shen\.choicepoint!
                  (shen/hd
                   (shen/hd
                    (nthcdr 1
                            (shen/hd V1439)))))
          (shen/and
           (shen/cons\?
            (nthcdr 1
                    (shen/hd
                     (nthcdr 1
                             (shen/hd V1439)))))
           (shen/and
            (shen/internal/predicate->shen
             (null
              (nthcdr 2
                      (shen/hd
                       (nthcdr 1
                               (shen/hd V1439))))))
            (shen/and
             (shen/internal/predicate->shen
              (null
               (nthcdr 2
                       (shen/hd V1439))))
             (shen/internal/predicate->shen
              (null
               (nthcdr 1 V1439))))))))))))
    (list
     (list 'true
           (list 'let 'Result
                 (shen/hd
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1439)))))
                 (list 'if
                       (list '= 'Result
                             (list 'fail))
                       (shen/if
                        (shen/value 'shen\.*installing-kl*)
                        (list 'shen\.sys-error V1440)
                        (list 'shen\.f_error V1440))
                       'Result)))))
   ((shen/and
     (shen/cons\? V1439)
     (shen/and
      (shen/cons\?
       (shen/hd V1439))
      (shen/and
       (shen/= 'true
               (shen/hd
                (shen/hd V1439)))
       (shen/and
        (shen/cons\?
         (nthcdr 1
                 (shen/hd V1439)))
        (shen/and
         (shen/cons\?
          (shen/hd
           (nthcdr 1
                   (shen/hd V1439))))
         (shen/and
          (shen/= 'shen\.choicepoint!
                  (shen/hd
                   (shen/hd
                    (nthcdr 1
                            (shen/hd V1439)))))
          (shen/and
           (shen/cons\?
            (nthcdr 1
                    (shen/hd
                     (nthcdr 1
                             (shen/hd V1439)))))
           (shen/and
            (shen/internal/predicate->shen
             (null
              (nthcdr 2
                      (shen/hd
                       (nthcdr 1
                               (shen/hd V1439))))))
            (shen/internal/predicate->shen
             (null
              (nthcdr 2
                      (shen/hd V1439))))))))))))
    (list
     (list 'true
           (list 'let 'Result
                 (shen/hd
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1439)))))
                 (list 'if
                       (list '= 'Result
                             (list 'fail))
                       (shen/shen\.cond-form
                        (shen/shen\.encode-choices
                         (nthcdr 1 V1439)
                         V1440))
                       'Result)))))
   ((shen/and
     (shen/cons\? V1439)
     (shen/and
      (shen/cons\?
       (shen/hd V1439))
      (shen/and
       (shen/cons\?
        (nthcdr 1
                (shen/hd V1439)))
       (shen/and
        (shen/cons\?
         (shen/hd
          (nthcdr 1
                  (shen/hd V1439))))
        (shen/and
         (shen/= 'shen\.choicepoint!
                 (shen/hd
                  (shen/hd
                   (nthcdr 1
                           (shen/hd V1439)))))
         (shen/and
          (shen/cons\?
           (nthcdr 1
                   (shen/hd
                    (nthcdr 1
                            (shen/hd V1439)))))
          (shen/and
           (shen/internal/predicate->shen
            (null
             (nthcdr 2
                     (shen/hd
                      (nthcdr 1
                              (shen/hd V1439))))))
           (shen/internal/predicate->shen
            (null
             (nthcdr 2
                     (shen/hd V1439)))))))))))
    (list
     (list 'true
           (list 'let 'Freeze
                 (list 'freeze
                       (shen/shen\.cond-form
                        (shen/shen\.encode-choices
                         (nthcdr 1 V1439)
                         V1440)))
                 (list 'if
                       (shen/hd
                        (shen/hd V1439))
                       (list 'let 'Result
                             (shen/hd
                              (nthcdr 1
                                      (shen/hd
                                       (nthcdr 1
                                               (shen/hd V1439)))))
                             (list 'if
                                   (list '= 'Result
                                         (list 'fail))
                                   (list 'thaw 'Freeze)
                                   'Result))
                       (list 'thaw 'Freeze))))))
   ((shen/and
     (shen/cons\? V1439)
     (shen/and
      (shen/cons\?
       (shen/hd V1439))
      (shen/and
       (shen/cons\?
        (nthcdr 1
                (shen/hd V1439)))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2
                 (shen/hd V1439)))))))
    (append
     (list
      (shen/hd V1439))
     (shen/shen\.encode-choices
      (nthcdr 1 V1439)
      V1440)))
   (shen/true
    (shen/shen\.f_error 'shen\.encode-choices))))
(defun shen/shen\.case-form
    (V1447 V1448)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1447))
    (list V1448))
   ((shen/and
     (shen/cons\? V1447)
     (shen/and
      (shen/cons\?
       (shen/hd V1447))
      (shen/and
       (shen/cons\?
        (shen/hd
         (shen/hd V1447)))
       (shen/and
        (shen/= ':
                (shen/hd
                 (shen/hd
                  (shen/hd V1447))))
        (shen/and
         (shen/cons\?
          (nthcdr 1
                  (shen/hd
                   (shen/hd V1447))))
         (shen/and
          (shen/= 'shen\.tests
                  (shen/hd
                   (nthcdr 1
                           (shen/hd
                            (shen/hd V1447)))))
          (shen/and
           (shen/internal/predicate->shen
            (null
             (nthcdr 2
                     (shen/hd
                      (shen/hd V1447)))))
           (shen/and
            (shen/cons\?
             (nthcdr 1
                     (shen/hd V1447)))
            (shen/and
             (shen/cons\?
              (shen/hd
               (nthcdr 1
                       (shen/hd V1447))))
             (shen/and
              (shen/= 'shen\.choicepoint!
                      (shen/hd
                       (shen/hd
                        (nthcdr 1
                                (shen/hd V1447)))))
              (shen/and
               (shen/cons\?
                (nthcdr 1
                        (shen/hd
                         (nthcdr 1
                                 (shen/hd V1447)))))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 2
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1447))))))
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 2
                          (shen/hd V1447))))))))))))))))
    (append
     (list
      (append
       (list 'true)
       (nthcdr 1
               (shen/hd V1447))))
     (shen/shen\.case-form
      (nthcdr 1 V1447)
      V1448)))
   ((shen/and
     (shen/cons\? V1447)
     (shen/and
      (shen/cons\?
       (shen/hd V1447))
      (shen/and
       (shen/cons\?
        (shen/hd
         (shen/hd V1447)))
       (shen/and
        (shen/= ':
                (shen/hd
                 (shen/hd
                  (shen/hd V1447))))
        (shen/and
         (shen/cons\?
          (nthcdr 1
                  (shen/hd
                   (shen/hd V1447))))
         (shen/and
          (shen/= 'shen\.tests
                  (shen/hd
                   (nthcdr 1
                           (shen/hd
                            (shen/hd V1447)))))
          (shen/and
           (shen/internal/predicate->shen
            (null
             (nthcdr 2
                     (shen/hd
                      (shen/hd V1447)))))
           (shen/and
            (shen/cons\?
             (nthcdr 1
                     (shen/hd V1447)))
            (shen/internal/predicate->shen
             (null
              (nthcdr 2
                      (shen/hd V1447))))))))))))
    (list
     (append
      (list 'true)
      (nthcdr 1
              (shen/hd V1447)))))
   ((shen/and
     (shen/cons\? V1447)
     (shen/and
      (shen/cons\?
       (shen/hd V1447))
      (shen/and
       (shen/cons\?
        (shen/hd
         (shen/hd V1447)))
       (shen/and
        (shen/= ':
                (shen/hd
                 (shen/hd
                  (shen/hd V1447))))
        (shen/and
         (shen/cons\?
          (nthcdr 1
                  (shen/hd
                   (shen/hd V1447))))
         (shen/and
          (shen/= 'shen\.tests
                  (shen/hd
                   (nthcdr 1
                           (shen/hd
                            (shen/hd V1447)))))
          (shen/and
           (shen/cons\?
            (nthcdr 1
                    (shen/hd V1447)))
           (shen/internal/predicate->shen
            (null
             (nthcdr 2
                     (shen/hd V1447)))))))))))
    (append
     (list
      (append
       (list
        (shen/shen\.embed-and
         (nthcdr 2
                 (shen/hd
                  (shen/hd V1447)))))
       (nthcdr 1
               (shen/hd V1447))))
     (shen/shen\.case-form
      (nthcdr 1 V1447)
      V1448)))
   (shen/true
    (shen/shen\.f_error 'shen\.case-form))))
(defun shen/shen\.embed-and
    (V1450)
  (shen/cond
   ((shen/and
     (shen/cons\? V1450)
     (shen/internal/predicate->shen
      (null
       (nthcdr 1 V1450))))
    (shen/hd V1450))
   ((shen/cons\? V1450)
    (list 'and
          (shen/hd V1450)
          (shen/shen\.embed-and
           (nthcdr 1 V1450))))
   (shen/true
    (shen/shen\.f_error 'shen\.embed-and))))
(defun shen/shen\.err-condition
    (V1452)
  (list 'true
        (list 'shen\.f_error V1452)))
(defun shen/shen\.sys-error
    (V1454)
  (shen/simple-error
   (shen/cn "system function "
            (shen/shen\.app V1454 ": unexpected argument\n" 'shen\.a))))
(defun shen/thaw
    (V2632)
  (shen/internal/apply-higher-order-function V2632
                                             (list)))
(defun shen/eval
    (V2634)
  (shen/let Macroexpand
            (shen/shen\.walk
             (shen/lambda Y
                          (shen/macroexpand Y))
             V2634)
            (shen/if
             (shen/shen\.packaged\? Macroexpand)
             (shen/map
              (shen/lambda Z
                           (shen/shen\.eval-without-macros Z))
              (shen/shen\.package-contents Macroexpand))
             (shen/shen\.eval-without-macros Macroexpand))))
(defun shen/shen\.eval-without-macros
    (V2636)
  (shen/eval-kl
   (shen/shen\.elim-def
    (shen/shen\.proc-input+ V2636))))
(defun shen/shen\.proc-input+
    (V2638)
  (shen/cond
   ((shen/and
     (shen/cons\? V2638)
     (shen/and
      (shen/= 'input+
              (shen/hd V2638))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2638))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V2638))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V2638)))))))
    (append
     (list 'input+
           (shen/shen\.rcons_form
            (shen/hd
             (nthcdr 1 V2638))))
     (nthcdr 2 V2638)))
   ((shen/and
     (shen/cons\? V2638)
     (shen/and
      (shen/= 'shen\.read+
              (shen/hd V2638))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2638))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V2638))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V2638)))))))
    (append
     (list 'shen\.read+
           (shen/shen\.rcons_form
            (shen/hd
             (nthcdr 1 V2638))))
     (nthcdr 2 V2638)))
   ((shen/cons\? V2638)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.proc-input+ Z))
     V2638))
   (shen/true V2638)))
(defun shen/shen\.elim-def
    (V2640)
  (cl-flet
      ((tail-trampoline
        (V2640)
        (shen/cond
         ((shen/and
           (shen/cons\? V2640)
           (shen/and
            (shen/= 'define
                    (shen/hd V2640))
            (shen/cons\?
             (nthcdr 1 V2640))))
          (shen/shen->kl
           (shen/hd
            (nthcdr 1 V2640))
           (nthcdr 2 V2640)))
         ((shen/and
           (shen/cons\? V2640)
           (shen/and
            (shen/= 'defmacro
                    (shen/hd V2640))
            (shen/cons\?
             (nthcdr 1 V2640))))
          (shen/let Default
                    (list 'X '-> 'X)
                    (shen/let Def
                              (shen/shen\.elim-def
                               (append
                                (list 'define
                                      (shen/hd
                                       (nthcdr 1 V2640)))
                                (shen/append
                                 (nthcdr 2 V2640)
                                 Default)))
                              (shen/let MacroAdd
                                        (shen/shen\.add-macro
                                         (shen/hd
                                          (nthcdr 1 V2640)))
                                        Def))))
         ((shen/and
           (shen/cons\? V2640)
           (shen/and
            (shen/= 'defcc
                    (shen/hd V2640))
            (shen/cons\?
             (nthcdr 1 V2640))))
          (vector
           (list
            (shen/shen\.yacc V2640))))
         ((shen/cons\? V2640)
          (shen/map
           (shen/lambda Z
                        (shen/shen\.elim-def Z))
           V2640))
         (shen/true V2640))))
    (let
        ((result
          (funcall #'tail-trampoline V2640)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.add-macro
    (V2642)
  (shen/let MacroReg
            (shen/value 'shen\.*macroreg*)
            (shen/let NewMacroReg
                      (shen/set 'shen\.*macroreg*
                                (shen/adjoin V2642
                                             (shen/value 'shen\.*macroreg*)))
                      (shen/if
                       (shen/= MacroReg NewMacroReg)
                       'shen\.skip
                       (shen/set '*macros*
                                 (append
                                  (list
                                   (shen/function V2642))
                                  (shen/value '*macros*)))))))
(defun shen/shen\.packaged\?
    (V2650)
  (shen/cond
   ((shen/and
     (shen/cons\? V2650)
     (shen/and
      (shen/= 'package
              (shen/hd V2650))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2650))
       (shen/cons\?
        (nthcdr 2 V2650)))))
    'true)
   (shen/true 'false)))
(defun shen/external
    (V2652)
  (shen/get/or V2652 'shen\.external-symbols
               (shen/freeze
                (shen/simple-error
                 (shen/cn "package "
                          (shen/shen\.app V2652 " has not been used.\n" 'shen\.a))))
               (shen/value '*property-vector*)))
(defun shen/internal
    (V2654)
  (shen/get/or V2654 'shen\.internal-symbols
               (shen/freeze
                (shen/simple-error
                 (shen/cn "package "
                          (shen/shen\.app V2654 " has not been used.\n" 'shen\.a))))
               (shen/value '*property-vector*)))
(defun shen/shen\.package-contents
    (V2658)
  (shen/cond
   ((shen/and
     (shen/cons\? V2658)
     (shen/and
      (shen/= 'package
              (shen/hd V2658))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2658))
       (shen/and
        (shen/= 'null
                (shen/hd
                 (nthcdr 1 V2658)))
        (shen/cons\?
         (nthcdr 2 V2658))))))
    (nthcdr 3 V2658))
   ((shen/and
     (shen/cons\? V2658)
     (shen/and
      (shen/= 'package
              (shen/hd V2658))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2658))
       (shen/cons\?
        (nthcdr 2 V2658)))))
    (shen/shen\.packageh
     (shen/hd
      (nthcdr 1 V2658))
     (shen/hd
      (nthcdr 2 V2658))
     (nthcdr 3 V2658)))
   (shen/true
    (shen/shen\.f_error 'shen\.package-contents))))
(defun shen/shen\.walk
    (V2661 V2662)
  (shen/cond
   ((shen/cons\? V2662)
    (shen/internal/apply-higher-order-function V2661
                                               (list
                                                (shen/map
                                                 (shen/lambda Z
                                                              (shen/shen\.walk V2661 Z))
                                                 V2662))))
   (shen/true
    (shen/internal/apply-higher-order-function V2661
                                               (list V2662)))))
(defun shen/compile
    (V2666 V2667 V2668)
  (shen/let O
            (shen/internal/apply-higher-order-function V2666
                                                       (list
                                                        (list V2667 nil)))
            (shen/if
             (shen/or
              (shen/=
               (shen/fail)
               O)
              (shen/not
               (shen/empty\?
                (shen/hd O))))
             (shen/internal/apply-higher-order-function V2668
                                                        (list O))
             (shen/shen\.hdtl O))))
(defun shen/fail-if
    (V2671 V2672)
  (shen/if
   (shen/internal/apply-higher-order-function V2671
                                              (list V2672))
   (shen/fail)
   V2672))
(defun shen/@s
    (V2675 V2676)
  (shen/cn V2675 V2676))
(defun shen/tc\? nil
  (shen/value 'shen\.*tc*))
(defun shen/ps
    (V2678)
  (shen/get/or V2678 'shen\.source
               (shen/freeze
                (shen/simple-error
                 (shen/shen\.app V2678 " not found.\n" 'shen\.a)))
               (shen/value '*property-vector*)))
(defun shen/stinput nil
  (shen/value '*stinput*))
(defun shen/<-address/or
    (Vector N Or)
  (let
      ((Value
        (shen/<-address Vector N)))
    (if
        (not Value)
        (shen/thaw Or)
      Value)))
(defun shen/value/or
    (V2687 V2688)
  (shen/trap-error
   (shen/value V2687)
   (shen/lambda E
                (shen/thaw V2688))))
(defun shen/vector
    (V2690)
  (shen/let Vector
            (shen/absvector
             (1+ V2690))
            (shen/let ZeroStamp
                      (shen/address-> Vector 0 V2690)
                      (shen/let Standard
                                (shen/if
                                 (shen/= V2690 0)
                                 ZeroStamp
                                 (shen/shen\.fillvector ZeroStamp 1 V2690
                                                        (shen/fail)))
                                Standard))))
(defun shen/shen\.fillvector
    (V2696 V2697 V2698 V2699)
  (cl-flet
      ((tail-trampoline
        (V2696 V2697 V2698 V2699)
        (shen/cond
         ((shen/= V2698 V2697)
          (shen/address-> V2696 V2698 V2699))
         (shen/true
          (vector
           (list
            (shen/address-> V2696 V2697 V2699)
            (1+ V2697)
            V2698 V2699))))))
    (let
        ((result
          (funcall #'tail-trampoline V2696 V2697 V2698 V2699)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/vector\?
    (V2701)
  (shen/and
   (shen/absvector\? V2701)
   (shen/let X
             (shen/<-address/or V2701 0
                                (shen/freeze -1))
             (shen/and
              (shen/number\? X)
              (shen/>= X 0)))))
(defun shen/vector->
    (V2705 V2706 V2707)
  (shen/if
   (shen/= V2706 0)
   (shen/simple-error "cannot access 0th element of a vector\n")
   (shen/address-> V2705 V2706 V2707)))
(defun shen/<-vector
    (V2710 V2711)
  (shen/if
   (shen/= V2711 0)
   (shen/simple-error "cannot access 0th element of a vector\n")
   (shen/let VectorElement
             (shen/<-address V2710 V2711)
             (shen/if
              (shen/= VectorElement
                      (shen/fail))
              (shen/simple-error "vector element not found\n")
              VectorElement))))
(defun shen/<-vector/or
    (V2715 V2716 V2717)
  (shen/if
   (shen/= V2716 0)
   (shen/simple-error "cannot access 0th element of a vector\n")
   (shen/let VectorElement
             (shen/<-address/or V2715 V2716 V2717)
             (shen/if
              (shen/= VectorElement
                      (shen/fail))
              (shen/thaw V2717)
              VectorElement))))
(defun shen/shen\.posint\?
    (V2719)
  (shen/and
   (shen/integer\? V2719)
   (shen/>= V2719 0)))
(defun shen/limit
    (V2721)
  (shen/<-address V2721 0))
(defun shen/symbol\?
    (V2723)
  (shen/cond
   ((shen/or
     (shen/boolean\? V2723)
     (shen/or
      (shen/number\? V2723)
      (shen/string\? V2723)))
    'false)
   (shen/true
    (shen/trap-error
     (shen/let String
               (shen/str V2723)
               (shen/shen\.analyse-symbol\? String))
     (shen/lambda E 'false)))))
(defun shen/shen\.analyse-symbol\?
    (V2725)
  (shen/cond
   ((shen/= "" V2725)
    'false)
   ((shen/shen\.+string\? V2725)
    (shen/and
     (shen/shen\.alpha\?
      (shen/pos V2725 0))
     (shen/shen\.alphanums\?
      (shen/tlstr V2725))))
   (shen/true
    (shen/shen\.f_error 'shen\.analyse-symbol\?))))
(defun shen/shen\.alpha\?
    (V2727)
  (shen/element\? V2727
                  (list "A" "B" "C" "D" "E" "F" "G" "H" "I" "J" "K" "L" "M" "N" "O" "P" "Q" "R" "S" "T" "U" "V" "W" "X" "Y" "Z" "a" "b" "c" "d" "e" "f" "g" "h" "i" "j" "k" "l" "m" "n" "o" "p" "q" "r" "s" "t" "u" "v" "w" "x" "y" "z" "=" "*" "/" "+" "-" "_" "?" "$" "!" "@" "~" ">" "<" "&" "%" "{" "}" ":" ";" "`" "#" "'" ".")))
(defun shen/shen\.alphanums\?
    (V2729)
  (shen/cond
   ((shen/= "" V2729)
    'true)
   ((shen/shen\.+string\? V2729)
    (shen/and
     (shen/shen\.alphanum\?
      (shen/pos V2729 0))
     (shen/shen\.alphanums\?
      (shen/tlstr V2729))))
   (shen/true
    (shen/shen\.f_error 'shen\.alphanums\?))))
(defun shen/shen\.alphanum\?
    (V2731)
  (shen/or
   (shen/shen\.alpha\? V2731)
   (shen/shen\.digit\? V2731)))
(defun shen/shen\.digit\?
    (V2733)
  (shen/element\? V2733
                  (list "1" "2" "3" "4" "5" "6" "7" "8" "9" "0")))
(defun shen/variable\?
    (V2735)
  (shen/cond
   ((shen/or
     (shen/boolean\? V2735)
     (shen/or
      (shen/number\? V2735)
      (shen/string\? V2735)))
    'false)
   (shen/true
    (shen/trap-error
     (shen/let String
               (shen/str V2735)
               (shen/shen\.analyse-variable\? String))
     (shen/lambda E 'false)))))
(defun shen/shen\.analyse-variable\?
    (V2737)
  (shen/cond
   ((shen/shen\.+string\? V2737)
    (shen/and
     (shen/shen\.uppercase\?
      (shen/pos V2737 0))
     (shen/shen\.alphanums\?
      (shen/tlstr V2737))))
   (shen/true
    (shen/shen\.f_error 'shen\.analyse-variable\?))))
(defun shen/shen\.uppercase\?
    (V2739)
  (shen/element\? V2739
                  (list "A" "B" "C" "D" "E" "F" "G" "H" "I" "J" "K" "L" "M" "N" "O" "P" "Q" "R" "S" "T" "U" "V" "W" "X" "Y" "Z")))
(defun shen/gensym
    (V2741)
  (shen/concat V2741
               (shen/set 'shen\.*gensym*
                         (1+
                          (shen/value 'shen\.*gensym*)))))
(defun shen/concat
    (V2744 V2745)
  (shen/intern
   (shen/cn
    (shen/str V2744)
    (shen/str V2745))))
(defun shen/@p
    (V2748 V2749)
  (shen/let Vector
            (shen/absvector 3)
            (shen/let Tag
                      (shen/address-> Vector 0 'shen\.tuple)
                      (shen/let Fst
                                (shen/address-> Vector 1 V2748)
                                (shen/let Snd
                                          (shen/address-> Vector 2 V2749)
                                          Vector)))))
(defun shen/fst
    (V2751)
  (shen/<-address V2751 1))
(defun shen/snd
    (V2753)
  (shen/<-address V2753 2))
(defun shen/tuple\?
    (V2755)
  (shen/and
   (shen/absvector\? V2755)
   (shen/= 'shen\.tuple
           (shen/<-address/or V2755 0
                              (shen/freeze 'shen\.not-tuple)))))
(defun shen/append
    (Xs Ys)
  (append Xs Ys))
(defun shen/@v
    (V2762 V2763)
  (shen/let Limit
            (shen/limit V2763)
            (shen/let NewVector
                      (shen/vector
                       (1+ Limit))
                      (shen/let X+NewVector
                                (shen/vector-> NewVector 1 V2762)
                                (shen/if
                                 (shen/= Limit 0)
                                 X+NewVector
                                 (shen/shen\.@v-help V2763 1 Limit X+NewVector))))))
(defun shen/shen\.@v-help
    (V2769 V2770 V2771 V2772)
  (cl-flet
      ((tail-trampoline
        (V2769 V2770 V2771 V2772)
        (shen/cond
         ((shen/= V2771 V2770)
          (shen/shen\.copyfromvector V2769 V2772 V2771
                                     (1+ V2771)))
         (shen/true
          (vector
           (list V2769
                 (1+ V2770)
                 V2771
                 (shen/shen\.copyfromvector V2769 V2772 V2770
                                            (1+ V2770))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2769 V2770 V2771 V2772)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.copyfromvector
    (V2777 V2778 V2779 V2780)
  (shen/trap-error
   (shen/vector-> V2778 V2780
                  (shen/<-vector V2777 V2779))
   (shen/lambda E V2778)))
(defun shen/hdv
    (V2782)
  (shen/<-vector/or V2782 1
                    (shen/freeze
                     (shen/simple-error
                      (shen/cn "hdv needs a non-empty vector as an argument; not "
                               (shen/shen\.app V2782 "\n" 'shen\.s))))))
(defun shen/tlv
    (V2784)
  (shen/let Limit
            (shen/limit V2784)
            (shen/if
             (shen/= Limit 0)
             (shen/simple-error "cannot take the tail of the empty vector\n")
             (shen/if
              (shen/= Limit 1)
              (shen/vector 0)
              (shen/let NewVector
                        (shen/vector
                         (shen/- Limit 1))
                        (shen/shen\.tlv-help V2784 2 Limit
                                             (shen/vector
                                              (shen/- Limit 1))))))))
(defun shen/shen\.tlv-help
    (V2790 V2791 V2792 V2793)
  (cl-flet
      ((tail-trampoline
        (V2790 V2791 V2792 V2793)
        (shen/cond
         ((shen/= V2792 V2791)
          (shen/shen\.copyfromvector V2790 V2793 V2792
                                     (shen/- V2792 1)))
         (shen/true
          (vector
           (list V2790
                 (1+ V2791)
                 V2792
                 (shen/shen\.copyfromvector V2790 V2793 V2791
                                            (shen/- V2791 1))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2790 V2791 V2792 V2793)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/assoc
    (V2805 V2806)
  (cl-flet
      ((tail-trampoline
        (V2805 V2806)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2806))
          nil)
         ((shen/and
           (shen/cons\? V2806)
           (shen/and
            (shen/cons\?
             (shen/hd V2806))
            (shen/=
             (shen/hd
              (shen/hd V2806))
             V2805)))
          (shen/hd V2806))
         ((shen/cons\? V2806)
          (vector
           (list V2805
                 (nthcdr 1 V2806))))
         (shen/true
          (shen/shen\.f_error 'assoc)))))
    (let
        ((result
          (funcall #'tail-trampoline V2805 V2806)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/boolean\?
    (V2812)
  (shen/cond
   ((shen/= 'true V2812)
    'true)
   ((shen/= 'false V2812)
    'true)
   (shen/true 'false)))
(defun shen/nl
    (V2814)
  (cl-flet
      ((tail-trampoline
        (V2814)
        (shen/cond
         ((shen/= 0 V2814)
          0)
         (shen/true
          (shen/do
           (shen/shen\.prhush "\n"
                              (shen/stoutput))
           (vector
            (list
             (shen/- V2814 1))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2814)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/difference
    (V2819 V2820)
  (cl-flet
      ((tail-trampoline
        (V2819 V2820)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2819))
          nil)
         ((shen/cons\? V2819)
          (shen/if
           (shen/element\?
            (shen/hd V2819)
            V2820)
           (vector
            (list
             (nthcdr 1 V2819)
             V2820))
           (append
            (list
             (shen/hd V2819))
            (shen/difference
             (nthcdr 1 V2819)
             V2820))))
         (shen/true
          (shen/shen\.f_error 'difference)))))
    (let
        ((result
          (funcall #'tail-trampoline V2819 V2820)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/do
    (V2823 V2824)
  V2824)
(defun shen/element\?
    (Element Xs)
  (let
      ((SearchList Xs)
       (Found nil)
       (Length
        (length Xs))
       (Current 0))
    (while
        (and
         (not Found)
         SearchList)
      (setq Found
            (shen/internal/= Element
                             (pop SearchList))))
    (shen/internal/predicate->shen Found)))
(defun shen/empty\?
    (V2843)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2843))
    'true)
   (shen/true 'false)))
(defun shen/fix
    (V2846 V2847)
  (shen/shen\.fix-help V2846 V2847
                       (shen/internal/apply-higher-order-function V2846
                                                                  (list V2847))))
(defun shen/shen\.fix-help
    (V2858 V2859 V2860)
  (cl-flet
      ((tail-trampoline
        (V2858 V2859 V2860)
        (shen/cond
         ((shen/= V2860 V2859)
          V2860)
         (shen/true
          (vector
           (list V2858 V2860
                 (shen/internal/apply-higher-order-function V2858
                                                            (list V2860))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2858 V2859 V2860)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/dict
    (Size)
  (let
      ((Dict
        (shen/absvector 4))
       (Contents
        (shen/absvector Size)))
    (progn
      (shen/address-> Dict 0 'dictionary)
      (shen/address-> Dict 1 Size)
      (shen/address-> Dict 2 0)
      (shen/address-> Dict 3 Contents)
      Dict)))
(defun shen/dict\?
    (V2864)
  (shen/and
   (shen/absvector\? V2864)
   (shen/=
    (shen/<-address/or V2864 0
                       (shen/freeze 'shen\.not-dictionary))
    'shen\.dictionary)))
(defun shen/shen\.dict-capacity
    (V2866)
  (shen/<-address V2866 1))
(defun shen/dict-count
    (V2868)
  (shen/<-address V2868 2))
(defun shen/shen\.dict-count->
    (V2871 V2872)
  (shen/address-> V2871 2 V2872))
(defun shen/shen\.<-dict-bucket
    (V2875 V2876)
  (shen/<-address V2875
                  (shen/+ 3 V2876)))
(defun shen/shen\.dict-bucket->
    (V2880 V2881 V2882)
  (shen/address-> V2880
                  (shen/+ 3 V2881)
                  V2882))
(defun shen/shen\.set-key-entry-value
    (V2889 V2890 V2891)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2891))
    (list
     (append
      (list V2889)
      V2890)))
   ((shen/and
     (shen/cons\? V2891)
     (shen/and
      (shen/cons\?
       (shen/hd V2891))
      (shen/=
       (shen/hd
        (shen/hd V2891))
       V2889)))
    (append
     (list
      (append
       (list
        (shen/hd
         (shen/hd V2891)))
       V2890))
     (nthcdr 1 V2891)))
   ((shen/cons\? V2891)
    (append
     (list
      (shen/hd V2891))
     (shen/shen\.set-key-entry-value V2889 V2890
                                     (nthcdr 1 V2891))))
   (shen/true
    (shen/shen\.f_error 'shen\.set-key-entry-value))))
(defun shen/shen\.remove-key-entry-value
    (V2897 V2898)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2898))
    nil)
   ((shen/and
     (shen/cons\? V2898)
     (shen/and
      (shen/cons\?
       (shen/hd V2898))
      (shen/=
       (shen/hd
        (shen/hd V2898))
       V2897)))
    (nthcdr 1 V2898))
   ((shen/cons\? V2898)
    (append
     (list
      (shen/hd V2898))
     (shen/shen\.remove-key-entry-value V2897
                                        (nthcdr 1 V2898))))
   (shen/true
    (shen/shen\.f_error 'shen\.remove-key-entry-value))))
(defun shen/shen\.dict-update-count
    (V2902 V2903 V2904)
  (shen/let Diff
            (shen/-
             (shen/length V2904)
             (shen/length V2903))
            (shen/shen\.dict-count-> V2902
                                     (shen/+ Diff
                                             (shen/dict-count V2902)))))
(defun shen/dict->
    (Dict Key Value)
  (let*
      ((Count
        (shen/dict-count Dict))
       (Contents
        (shen/<-address Dict 3))
       (Exists
        (shen/<-address Contents Key)))
    (progn
      (if
          (not Exists)
          (shen/address-> Dict 2
                          (1+ Count)))
      (shen/address-> Contents Key Value))))
(defun shen/<-dict/or
    (Dict Key Or)
  (let*
      ((Contents
        (shen/<-address Dict 3))
       (Existing
        (shen/<-address Contents Key)))
    (if
        (not Existing)
        (shen/thaw Or)
      Existing)))
(defun shen/<-dict
    (Dict Key)
  (let*
      ((Contents
        (shen/<-address Dict 3))
       (Existing
        (shen/<-address Contents Key)))
    (if
        (not Existing)
        (shen/freeze
         (shen/simple-error "value not found"))
      Existing)))
(defun shen/dict-rm
    (Dict Key)
  (let*
      ((Count
        (shen/dict-count Dict))
       (Contents
        (shen/<-address Dict 3))
       (Exists
        (shen/<-address Contents Key)))
    (if
        (not Exists)
        Key
      (progn
        (remhash Key Contents)
        (shen/address-> Dict 2
                        (1- Count))
        Key))))
(defun shen/dict-fold
    (F Dict Acc)
  (let
      ((Contents
        (shen/<-address Dict 3)))
    (progn
      (setq NewAcc Acc)
      (maphash
       (lambda
         (Key Value)
         (setq NewAcc
               (shen/internal/apply-higher-order-function F
                                                          (list Key Value NewAcc))))
       Contents)
      NewAcc)))
(defun shen/shen\.dict-fold-h
    (V2937 V2938 V2939 V2940 V2941)
  (cl-flet
      ((tail-trampoline
        (V2937 V2938 V2939 V2940 V2941)
        (shen/cond
         ((shen/= V2941 V2940)
          V2939)
         (shen/true
          (shen/let B
                    (shen/shen\.<-dict-bucket V2938 V2940)
                    (shen/let Acc
                              (shen/shen\.bucket-fold V2937 B V2939)
                              (vector
                               (list V2937 V2938 Acc
                                     (1+ V2940)
                                     V2941))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2937 V2938 V2939 V2940 V2941)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.bucket-fold
    (V2945 V2946 V2947)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2946))
    V2947)
   ((shen/and
     (shen/cons\? V2946)
     (shen/cons\?
      (shen/hd V2946)))
    (shen/internal/apply-higher-order-function V2945
                                               (list
                                                (shen/hd
                                                 (shen/hd V2946))
                                                (nthcdr 1
                                                        (shen/hd V2946))
                                                (shen/fold-right V2945
                                                                 (nthcdr 1 V2946)
                                                                 V2947))))
   (shen/true
    (shen/shen\.f_error 'shen\.bucket-fold))))
(defun shen/dict-keys
    (Dict)
  (let*
      ((Contents
        (shen/<-address Dict 3)))
    (hash-table-keys Contents)))
(defun shen/dict-values
    (Dict)
  (let*
      ((Contents
        (shen/<-address Dict 3)))
    (hash-table-values Contents)))
(defun shen/put
    (X Pointer Y Dict)
  (let*
      ((Contents
        (shen/<-address Dict 3))
       (X-Contents
        (shen/<-address Contents X)))
    (if X-Contents
        (progn
          (puthash Pointer Y X-Contents)
          Y)
      (progn
        (setq X-Contents
              (shen/absvector 100))
        (puthash X X-Contents Contents)
        (puthash Pointer Y X-Contents)
        Y))))
(defun shen/unput
    (X Pointer Dict)
  (let*
      ((Contents
        (shen/<-address Dict 3))
       (X-Contents
        (shen/<-address Contents X)))
    (progn
      (if X-Contents
          (remhash Pointer X-Contents))
      X)))
(defun shen/get/or
    (X Pointer Or Dict)
  (let*
      ((Contents
        (shen/<-address Dict 3))
       (X-Contents
        (shen/<-address Contents X)))
    (if X-Contents
        (let
            ((Pointer-Contents
              (shen/<-address X-Contents Pointer)))
          (if
              (not Pointer-Contents)
              (shen/thaw Or)
            Pointer-Contents))
      (shen/thaw Or))))
(defun shen/get
    (V2977 V2978 V2979)
  (shen/get/or V2977 V2978
               (shen/freeze
                (shen/simple-error "value not found\n"))
               V2979))
(defun shen/hash
    (N Div)
  (sxhash N))
(defun shen/shen\.mod
    (N Div)
  (mod N Div))
(defun shen/shen\.multiples
    (V2990 V2991)
  (cl-flet
      ((tail-trampoline
        (V2990 V2991)
        (shen/cond
         ((shen/and
           (shen/cons\? V2991)
           (shen/>
            (shen/hd V2991)
            V2990))
          (nthcdr 1 V2991))
         ((shen/cons\? V2991)
          (vector
           (list V2990
                 (append
                  (list
                   (shen/* 2
                           (shen/hd V2991)))
                  V2991))))
         (shen/true
          (shen/shen\.f_error 'shen\.multiples)))))
    (let
        ((result
          (funcall #'tail-trampoline V2990 V2991)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.modh
    (V2996 V2997)
  (cl-flet
      ((tail-trampoline
        (V2996 V2997)
        (shen/cond
         ((shen/= 0 V2996)
          0)
         ((shen/internal/predicate->shen
           (null V2997))
          V2996)
         ((shen/and
           (shen/cons\? V2997)
           (shen/>
            (shen/hd V2997)
            V2996))
          (shen/if
           (shen/empty\?
            (nthcdr 1 V2997))
           V2996
           (vector
            (list V2996
                  (nthcdr 1 V2997)))))
         ((shen/cons\? V2997)
          (vector
           (list
            (shen/- V2996
                    (shen/hd V2997))
            V2997)))
         (shen/true
          (shen/shen\.f_error 'shen\.modh)))))
    (let
        ((result
          (funcall #'tail-trampoline V2996 V2997)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/sum
    (Xs)
  (apply #'+ Xs))
(defun shen/head
    (V3007)
  (shen/cond
   ((shen/cons\? V3007)
    (shen/hd V3007))
   (shen/true
    (shen/simple-error "head expects a non-empty list"))))
(defun shen/tail
    (V3015)
  (shen/cond
   ((shen/cons\? V3015)
    (nthcdr 1 V3015))
   (shen/true
    (shen/simple-error "tail expects a non-empty list"))))
(defun shen/hdstr
    (V3017)
  (shen/pos V3017 0))
(defun shen/intersection
    (V3022 V3023)
  (cl-flet
      ((tail-trampoline
        (V3022 V3023)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3022))
          nil)
         ((shen/cons\? V3022)
          (shen/if
           (shen/element\?
            (shen/hd V3022)
            V3023)
           (append
            (list
             (shen/hd V3022))
            (shen/intersection
             (nthcdr 1 V3022)
             V3023))
           (vector
            (list
             (nthcdr 1 V3022)
             V3023))))
         (shen/true
          (shen/shen\.f_error 'intersection)))))
    (let
        ((result
          (funcall #'tail-trampoline V3022 V3023)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/reverse
    (V3025)
  (shen/shen\.reverse_help V3025 nil))
(defun shen/shen\.reverse_help
    (V3028 V3029)
  (cl-flet
      ((tail-trampoline
        (V3028 V3029)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3028))
          V3029)
         ((shen/cons\? V3028)
          (vector
           (list
            (nthcdr 1 V3028)
            (append
             (list
              (shen/hd V3028))
             V3029))))
         (shen/true
          (shen/shen\.f_error 'shen\.reverse_help)))))
    (let
        ((result
          (funcall #'tail-trampoline V3028 V3029)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/union
    (V3032 V3033)
  (cl-flet
      ((tail-trampoline
        (V3032 V3033)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3032))
          V3033)
         ((shen/cons\? V3032)
          (shen/if
           (shen/element\?
            (shen/hd V3032)
            V3033)
           (vector
            (list
             (nthcdr 1 V3032)
             V3033))
           (append
            (list
             (shen/hd V3032))
            (shen/union
             (nthcdr 1 V3032)
             V3033))))
         (shen/true
          (shen/shen\.f_error 'union)))))
    (let
        ((result
          (funcall #'tail-trampoline V3032 V3033)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/y-or-n\?
    (V3035)
  (cl-flet
      ((tail-trampoline
        (V3035)
        (shen/let Message
                  (shen/shen\.prhush
                   (shen/shen\.proc-nl V3035)
                   (shen/stoutput))
                  (shen/let Y-or-N
                            (shen/shen\.prhush " (y/n) "
                                               (shen/stoutput))
                            (shen/let Input
                                      (shen/shen\.app
                                       (shen/read
                                        (shen/stinput))
                                       "" 'shen\.s)
                                      (shen/if
                                       (shen/= "y" Input)
                                       'true
                                       (shen/if
                                        (shen/= "n" Input)
                                        'false
                                        (shen/do
                                         (shen/shen\.prhush "please answer y or n\n"
                                                            (shen/stoutput))
                                         (vector
                                          (list V3035))))))))))
    (let
        ((result
          (funcall #'tail-trampoline V3035)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/not
    (V3037)
  (shen/if V3037 'false 'true))
(defun shen/subst
    (V3050 V3051 V3052)
  (shen/cond
   ((shen/= V3052 V3051)
    V3050)
   ((shen/cons\? V3052)
    (shen/map
     (shen/lambda W
                  (shen/subst V3050 V3051 W))
     V3052))
   (shen/true V3052)))
(defun shen/explode
    (V3054)
  (shen/shen\.explode-h
   (shen/shen\.app V3054 "" 'shen\.a)))
(defun shen/shen\.explode-h
    (V3056)
  (shen/cond
   ((shen/= "" V3056)
    nil)
   ((shen/shen\.+string\? V3056)
    (append
     (list
      (shen/pos V3056 0))
     (shen/shen\.explode-h
      (shen/tlstr V3056))))
   (shen/true
    (shen/shen\.f_error 'shen\.explode-h))))
(defun shen/cd
    (V3058)
  (shen/set '*home-directory*
            (shen/if
             (shen/= V3058 "")
             ""
             (shen/shen\.app V3058 "/" 'shen\.a))))
(defun shen/for-each
    (V3061 V3062)
  (cl-flet
      ((tail-trampoline
        (V3061 V3062)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3062))
          'true)
         ((shen/cons\? V3062)
          (shen/let _
                    (shen/internal/apply-higher-order-function V3061
                                                               (list
                                                                (shen/hd V3062)))
                    (vector
                     (list V3061
                           (nthcdr 1 V3062)))))
         (shen/true
          (shen/shen\.f_error 'for-each)))))
    (let
        ((result
          (funcall #'tail-trampoline V3061 V3062)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/fold-right
    (V3066 V3067 V3068)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V3067))
    V3068)
   ((shen/cons\? V3067)
    (shen/internal/apply-higher-order-function V3066
                                               (list
                                                (shen/hd V3067)
                                                (shen/fold-right V3066
                                                                 (nthcdr 1 V3067)
                                                                 V3068))))
   (shen/true
    (shen/shen\.f_error 'fold-right))))
(defun shen/fold-left
    (V3072 V3073 V3074)
  (cl-flet
      ((tail-trampoline
        (V3072 V3073 V3074)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3074))
          V3073)
         ((shen/cons\? V3074)
          (vector
           (list V3072
                 (shen/internal/apply-higher-order-function V3072
                                                            (list V3073
                                                                  (shen/hd V3074)))
                 (nthcdr 1 V3074))))
         (shen/true
          (shen/shen\.f_error 'fold-left)))))
    (let
        ((result
          (funcall #'tail-trampoline V3072 V3073 V3074)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/filter
    (V3077 V3078)
  (shen/shen\.filter-h V3077 nil V3078))
(defun shen/shen\.filter-h
    (V3088 V3089 V3090)
  (cl-flet
      ((tail-trampoline
        (V3088 V3089 V3090)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3090))
          (shen/reverse V3089))
         ((shen/and
           (shen/cons\? V3090)
           (shen/internal/apply-higher-order-function V3088
                                                      (list
                                                       (shen/hd V3090))))
          (vector
           (list V3088
                 (append
                  (list
                   (shen/hd V3090))
                  V3089)
                 (nthcdr 1 V3090))))
         ((shen/cons\? V3090)
          (vector
           (list V3088 V3089
                 (nthcdr 1 V3090))))
         (shen/true
          (shen/shen\.f_error 'shen\.filter-h)))))
    (let
        ((result
          (funcall #'tail-trampoline V3088 V3089 V3090)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/map
    (F Xs)
  (mapcar
   (lambda
     (X)
     (shen/internal/apply-higher-order-function F
                                                (list X)))
   Xs))
(defun shen/shen\.map-h
    (V3100 V3101 V3102)
  (cl-flet
      ((tail-trampoline
        (V3100 V3101 V3102)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3101))
          (shen/reverse V3102))
         ((shen/cons\? V3101)
          (vector
           (list V3100
                 (nthcdr 1 V3101)
                 (append
                  (list
                   (shen/internal/apply-higher-order-function V3100
                                                              (list
                                                               (shen/hd V3101))))
                  V3102))))
         (shen/true
          (shen/shen\.f_error 'shen\.map-h)))))
    (let
        ((result
          (funcall #'tail-trampoline V3100 V3101 V3102)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/length
    (V3104)
  (shen/shen\.length-h V3104 0))
(defun shen/shen\.length-h
    (V3107 V3108)
  (cl-flet
      ((tail-trampoline
        (V3107 V3108)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3107))
          V3108)
         (shen/true
          (vector
           (list
            (nthcdr 1 V3107)
            (1+ V3108)))))))
    (let
        ((result
          (funcall #'tail-trampoline V3107 V3108)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/occurrences
    (V3120 V3121)
  (shen/cond
   ((shen/= V3121 V3120)
    1)
   ((shen/cons\? V3121)
    (shen/+
     (shen/occurrences V3120
                       (shen/hd V3121))
     (shen/occurrences V3120
                       (nthcdr 1 V3121))))
   (shen/true 0)))
(defun shen/nth
    (I Xs)
  (nth I Xs))
(defun shen/integer\?
    (N)
  (shen/internal/predicate->shen
   (integerp N)))
(defun shen/shen\.abs
    (V3135)
  (shen/if
   (shen/> V3135 0)
   V3135
   (shen/- 0 V3135)))
(defun shen/shen\.magless
    (V3138 V3139)
  (cl-flet
      ((tail-trampoline
        (V3138 V3139)
        (shen/let Nx2
                  (shen/* V3139 2)
                  (shen/if
                   (shen/> Nx2 V3138)
                   V3139
                   (vector
                    (list V3138 Nx2))))))
    (let
        ((result
          (funcall #'tail-trampoline V3138 V3139)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.integer-test\?
    (V3145 V3146)
  (cl-flet
      ((tail-trampoline
        (V3145 V3146)
        (shen/cond
         ((shen/= 0 V3145)
          'true)
         ((shen/> 1 V3145)
          'false)
         (shen/true
          (shen/let Abs-N
                    (shen/- V3145 V3146)
                    (shen/if
                     (shen/> 0 Abs-N)
                     (shen/integer\? V3145)
                     (vector
                      (list Abs-N V3146))))))))
    (let
        ((result
          (funcall #'tail-trampoline V3145 V3146)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/mapcan
    (V3151 V3152)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V3152))
    nil)
   ((shen/cons\? V3152)
    (shen/append
     (shen/internal/apply-higher-order-function V3151
                                                (list
                                                 (shen/hd V3152)))
     (shen/mapcan V3151
                  (nthcdr 1 V3152))))
   (shen/true
    (shen/shen\.f_error 'mapcan))))
(defun shen/==
    (V3164 V3165)
  (shen/cond
   ((shen/= V3165 V3164)
    'true)
   (shen/true 'false)))
(defun shen/abort nil
  (shen/simple-error ""))
(defun shen/bound\?
    (V3167)
  (shen/and
   (shen/symbol\? V3167)
   (shen/let Val
             (shen/value/or V3167
                            (shen/freeze 'shen\.this-symbol-is-unbound))
             (shen/if
              (shen/= Val 'shen\.this-symbol-is-unbound)
              'false 'true))))
(defun shen/shen\.string->bytes
    (S)
  (string-to-list S))
(defun shen/maxinferences
    (V3171)
  (shen/set 'shen\.*maxinferences* V3171))
(defun shen/inferences nil
  (shen/value 'shen\.*infs*))
(defun shen/protect
    (V3173)
  V3173)
(defun shen/stoutput nil
  (shen/value '*stoutput*))
(defun shen/sterror nil
  (shen/value 'shen\.*sterror*))
(defun shen/string->symbol
    (V3175)
  (shen/let Symbol
            (shen/intern V3175)
            (shen/if
             (shen/symbol\? Symbol)
             Symbol
             (shen/simple-error
              (shen/cn "cannot intern "
                       (shen/shen\.app V3175 " to a symbol" 'shen\.s))))))
(defun shen/optimise
    (V3181)
  (shen/cond
   ((shen/= '+ V3181)
    (shen/set 'shen\.*optimise* 'true))
   ((shen/= '- V3181)
    (shen/set 'shen\.*optimise* 'false))
   (shen/true
    (shen/simple-error "optimise expects a + or a -.\n"))))
(defun shen/os nil
  (shen/value '*os*))
(defun shen/language nil
  (shen/value '*language*))
(defun shen/version nil
  (shen/value '*version*))
(defun shen/port nil
  (shen/value '*port*))
(defun shen/porters nil
  (shen/value '*porters*))
(defun shen/implementation nil
  (shen/value '*implementation*))
(defun shen/release nil
  (shen/value '*release*))
(defun shen/package\?
    (V3183)
  (shen/trap-error
   (shen/do
    (shen/external V3183)
    'true)
   (shen/lambda E 'false)))
(defun shen/function
    (S)
  (shen/shen\.lookup-func
   (shen/internal/unprefix-symbol S)))
(defun shen/shen\.lookup-func
    (V3187)
  (shen/get/or V3187 'shen\.lambda-form
               (shen/freeze
                (shen/simple-error
                 (shen/shen\.app V3187 " has no lambda expansion\n" 'shen\.a)))
               (shen/value '*property-vector*)))
(defun shen/shen\.datatype-error
    (V2440)
  (shen/cond
   ((shen/and
     (shen/cons\? V2440)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2440))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V2440)))))
    (shen/simple-error
     (shen/cn "datatype syntax error here:\n\n "
              (shen/shen\.app
               (shen/shen\.next-50 50
                                   (shen/hd V2440))
               "\n" 'shen\.a))))
   (shen/true
    (shen/shen\.f_error 'shen\.datatype-error))))
(defun shen/shen\.<datatype-rules>
    (V2442)
  (shen/let YaccParse
            (shen/let Parse_shen\.<datatype-rule>
                      (shen/shen\.<datatype-rule> V2442)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<datatype-rule>))
                       (shen/let Parse_shen\.<datatype-rules>
                                 (shen/shen\.<datatype-rules> Parse_shen\.<datatype-rule>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<datatype-rules>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<datatype-rules>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<datatype-rule>))
                                    (shen/shen\.hdtl Parse_shen\.<datatype-rules>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2442)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<datatype-rule>
    (V2444)
  (shen/let YaccParse
            (shen/let Parse_shen\.<side-conditions>
                      (shen/shen\.<side-conditions> V2444)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<side-conditions>))
                       (shen/let Parse_shen\.<premises>
                                 (shen/shen\.<premises> Parse_shen\.<side-conditions>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<premises>))
                                  (shen/let Parse_shen\.<singleunderline>
                                            (shen/shen\.<singleunderline> Parse_shen\.<premises>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<singleunderline>))
                                             (shen/let Parse_shen\.<conclusion>
                                                       (shen/shen\.<conclusion> Parse_shen\.<singleunderline>)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<conclusion>))
                                                        (shen/shen\.pair
                                                         (shen/hd Parse_shen\.<conclusion>)
                                                         (shen/shen\.sequent 'shen\.single
                                                                             (list
                                                                              (shen/shen\.hdtl Parse_shen\.<side-conditions>)
                                                                              (shen/shen\.hdtl Parse_shen\.<premises>)
                                                                              (shen/shen\.hdtl Parse_shen\.<conclusion>))))
                                                        (shen/fail)))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<side-conditions>
                       (shen/shen\.<side-conditions> V2444)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<side-conditions>))
                        (shen/let Parse_shen\.<premises>
                                  (shen/shen\.<premises> Parse_shen\.<side-conditions>)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<premises>))
                                   (shen/let Parse_shen\.<doubleunderline>
                                             (shen/shen\.<doubleunderline> Parse_shen\.<premises>)
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<doubleunderline>))
                                              (shen/let Parse_shen\.<conclusion>
                                                        (shen/shen\.<conclusion> Parse_shen\.<doubleunderline>)
                                                        (shen/if
                                                         (shen/not
                                                          (shen/=
                                                           (shen/fail)
                                                           Parse_shen\.<conclusion>))
                                                         (shen/shen\.pair
                                                          (shen/hd Parse_shen\.<conclusion>)
                                                          (shen/shen\.sequent 'shen\.double
                                                                              (list
                                                                               (shen/shen\.hdtl Parse_shen\.<side-conditions>)
                                                                               (shen/shen\.hdtl Parse_shen\.<premises>)
                                                                               (shen/shen\.hdtl Parse_shen\.<conclusion>))))
                                                         (shen/fail)))
                                              (shen/fail)))
                                   (shen/fail)))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<side-conditions>
    (V2446)
  (shen/let YaccParse
            (shen/let Parse_shen\.<side-condition>
                      (shen/shen\.<side-condition> V2446)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<side-condition>))
                       (shen/let Parse_shen\.<side-conditions>
                                 (shen/shen\.<side-conditions> Parse_shen\.<side-condition>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<side-conditions>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<side-conditions>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<side-condition>))
                                    (shen/shen\.hdtl Parse_shen\.<side-conditions>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2446)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<side-condition>
    (V2448)
  (shen/let YaccParse
            (shen/if
             (shen/and
              (shen/cons\?
               (shen/hd V2448))
              (shen/= 'if
                      (shen/hd
                       (shen/hd V2448))))
             (shen/let Parse_shen\.<expr>
                       (shen/shen\.<expr>
                        (shen/shen\.pair
                         (nthcdr 1
                                 (shen/hd V2448))
                         (shen/shen\.hdtl V2448)))
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<expr>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<expr>)
                         (list 'if
                               (shen/shen\.hdtl Parse_shen\.<expr>)))
                        (shen/fail)))
             (shen/fail))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/if
              (shen/and
               (shen/cons\?
                (shen/hd V2448))
               (shen/= 'let
                       (shen/hd
                        (shen/hd V2448))))
              (shen/let Parse_shen\.<variable\?>
                        (shen/shen\.<variable\?>
                         (shen/shen\.pair
                          (nthcdr 1
                                  (shen/hd V2448))
                          (shen/shen\.hdtl V2448)))
                        (shen/if
                         (shen/not
                          (shen/=
                           (shen/fail)
                           Parse_shen\.<variable\?>))
                         (shen/let Parse_shen\.<expr>
                                   (shen/shen\.<expr> Parse_shen\.<variable\?>)
                                   (shen/if
                                    (shen/not
                                     (shen/=
                                      (shen/fail)
                                      Parse_shen\.<expr>))
                                    (shen/shen\.pair
                                     (shen/hd Parse_shen\.<expr>)
                                     (list 'let
                                           (shen/shen\.hdtl Parse_shen\.<variable\?>)
                                           (shen/shen\.hdtl Parse_shen\.<expr>)))
                                    (shen/fail)))
                         (shen/fail)))
              (shen/fail))
             YaccParse)))
(defun shen/shen\.<variable\?>
    (V2450)
  (shen/if
   (shen/cons\?
    (shen/hd V2450))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2450))
             (shen/if
              (shen/variable\? Parse_X)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2450))
                 (shen/shen\.hdtl V2450)))
               Parse_X)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<expr>
    (V2452)
  (shen/if
   (shen/cons\?
    (shen/hd V2452))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2452))
             (shen/if
              (shen/not
               (shen/or
                (shen/element\? Parse_X
                                (list '>> '\;))
                (shen/or
                 (shen/shen\.singleunderline\? Parse_X)
                 (shen/shen\.doubleunderline\? Parse_X))))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2452))
                 (shen/shen\.hdtl V2452)))
               (shen/shen\.remove-bar Parse_X))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.remove-bar
    (V2454)
  (shen/cond
   ((shen/and
     (shen/cons\? V2454)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2454))
      (shen/and
       (shen/cons\?
        (nthcdr 2 V2454))
       (shen/and
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V2454)))
        (shen/=
         (shen/hd
          (nthcdr 1 V2454))
         'bar!)))))
    (append
     (list
      (shen/hd V2454))
     (shen/hd
      (nthcdr 2 V2454))))
   ((shen/cons\? V2454)
    (append
     (list
      (shen/shen\.remove-bar
       (shen/hd V2454)))
     (shen/shen\.remove-bar
      (nthcdr 1 V2454))))
   (shen/true V2454)))
(defun shen/shen\.<premises>
    (V2456)
  (shen/let YaccParse
            (shen/let Parse_shen\.<premise>
                      (shen/shen\.<premise> V2456)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<premise>))
                       (shen/let Parse_shen\.<semicolon-symbol>
                                 (shen/shen\.<semicolon-symbol> Parse_shen\.<premise>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<semicolon-symbol>))
                                  (shen/let Parse_shen\.<premises>
                                            (shen/shen\.<premises> Parse_shen\.<semicolon-symbol>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<premises>))
                                             (shen/shen\.pair
                                              (shen/hd Parse_shen\.<premises>)
                                              (append
                                               (list
                                                (shen/shen\.hdtl Parse_shen\.<premise>))
                                               (shen/shen\.hdtl Parse_shen\.<premises>)))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2456)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<semicolon-symbol>
    (V2458)
  (shen/if
   (shen/cons\?
    (shen/hd V2458))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2458))
             (shen/if
              (shen/= Parse_X '\;)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2458))
                 (shen/shen\.hdtl V2458)))
               'shen\.skip)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<premise>
    (V2460)
  (shen/let YaccParse
            (shen/if
             (shen/and
              (shen/cons\?
               (shen/hd V2460))
              (shen/= '!
                      (shen/hd
                       (shen/hd V2460))))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V2460))
                (shen/shen\.hdtl V2460)))
              '!)
             (shen/fail))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<formulae>
                                 (shen/shen\.<formulae> V2460)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<formulae>))
                                  (shen/if
                                   (shen/and
                                    (shen/cons\?
                                     (shen/hd Parse_shen\.<formulae>))
                                    (shen/= '>>
                                            (shen/hd
                                             (shen/hd Parse_shen\.<formulae>))))
                                   (shen/let Parse_shen\.<formula>
                                             (shen/shen\.<formula>
                                              (shen/shen\.pair
                                               (nthcdr 1
                                                       (shen/hd Parse_shen\.<formulae>))
                                               (shen/shen\.hdtl Parse_shen\.<formulae>)))
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<formula>))
                                              (shen/shen\.pair
                                               (shen/hd Parse_shen\.<formula>)
                                               (shen/shen\.sequent
                                                (shen/shen\.hdtl Parse_shen\.<formulae>)
                                                (shen/shen\.hdtl Parse_shen\.<formula>)))
                                              (shen/fail)))
                                   (shen/fail))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let Parse_shen\.<formula>
                                  (shen/shen\.<formula> V2460)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<formula>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<formula>)
                                    (shen/shen\.sequent nil
                                                        (shen/shen\.hdtl Parse_shen\.<formula>)))
                                   (shen/fail)))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.<conclusion>
    (V2462)
  (shen/let YaccParse
            (shen/let Parse_shen\.<formulae>
                      (shen/shen\.<formulae> V2462)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<formulae>))
                       (shen/if
                        (shen/and
                         (shen/cons\?
                          (shen/hd Parse_shen\.<formulae>))
                         (shen/= '>>
                                 (shen/hd
                                  (shen/hd Parse_shen\.<formulae>))))
                        (shen/let Parse_shen\.<formula>
                                  (shen/shen\.<formula>
                                   (shen/shen\.pair
                                    (nthcdr 1
                                            (shen/hd Parse_shen\.<formulae>))
                                    (shen/shen\.hdtl Parse_shen\.<formulae>)))
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<formula>))
                                   (shen/let Parse_shen\.<semicolon-symbol>
                                             (shen/shen\.<semicolon-symbol> Parse_shen\.<formula>)
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<semicolon-symbol>))
                                              (shen/shen\.pair
                                               (shen/hd Parse_shen\.<semicolon-symbol>)
                                               (shen/shen\.sequent
                                                (shen/shen\.hdtl Parse_shen\.<formulae>)
                                                (shen/shen\.hdtl Parse_shen\.<formula>)))
                                              (shen/fail)))
                                   (shen/fail)))
                        (shen/fail))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<formula>
                       (shen/shen\.<formula> V2462)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<formula>))
                        (shen/let Parse_shen\.<semicolon-symbol>
                                  (shen/shen\.<semicolon-symbol> Parse_shen\.<formula>)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<semicolon-symbol>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<semicolon-symbol>)
                                    (shen/shen\.sequent nil
                                                        (shen/shen\.hdtl Parse_shen\.<formula>)))
                                   (shen/fail)))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.sequent
    (V2465 V2466)
  (shen/@p V2465 V2466))
(defun shen/shen\.<formulae>
    (V2468)
  (shen/let YaccParse
            (shen/let Parse_shen\.<formula>
                      (shen/shen\.<formula> V2468)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<formula>))
                       (shen/let Parse_shen\.<comma-symbol>
                                 (shen/shen\.<comma-symbol> Parse_shen\.<formula>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<comma-symbol>))
                                  (shen/let Parse_shen\.<formulae>
                                            (shen/shen\.<formulae> Parse_shen\.<comma-symbol>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<formulae>))
                                             (shen/shen\.pair
                                              (shen/hd Parse_shen\.<formulae>)
                                              (append
                                               (list
                                                (shen/shen\.hdtl Parse_shen\.<formula>))
                                               (shen/shen\.hdtl Parse_shen\.<formulae>)))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<formula>
                                 (shen/shen\.<formula> V2468)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<formula>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<formula>)
                                   (list
                                    (shen/shen\.hdtl Parse_shen\.<formula>)))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let Parse_<e>
                                  (shen/<e> V2468)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_<e>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_<e>)
                                    nil)
                                   (shen/fail)))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.<comma-symbol>
    (V2470)
  (shen/if
   (shen/cons\?
    (shen/hd V2470))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2470))
             (shen/if
              (shen/= Parse_X
                      (shen/intern ","))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2470))
                 (shen/shen\.hdtl V2470)))
               'shen\.skip)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<formula>
    (V2472)
  (shen/let YaccParse
            (shen/let Parse_shen\.<expr>
                      (shen/shen\.<expr> V2472)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<expr>))
                       (shen/if
                        (shen/and
                         (shen/cons\?
                          (shen/hd Parse_shen\.<expr>))
                         (shen/= ':
                                 (shen/hd
                                  (shen/hd Parse_shen\.<expr>))))
                        (shen/let Parse_shen\.<type>
                                  (shen/shen\.<type>
                                   (shen/shen\.pair
                                    (nthcdr 1
                                            (shen/hd Parse_shen\.<expr>))
                                    (shen/shen\.hdtl Parse_shen\.<expr>)))
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<type>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<type>)
                                    (list
                                     (shen/shen\.curry
                                      (shen/shen\.hdtl Parse_shen\.<expr>))
                                     ':
                                     (shen/shen\.demodulate
                                      (shen/shen\.hdtl Parse_shen\.<type>))))
                                   (shen/fail)))
                        (shen/fail))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<expr>
                       (shen/shen\.<expr> V2472)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<expr>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<expr>)
                         (shen/shen\.hdtl Parse_shen\.<expr>))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<type>
    (V2474)
  (shen/let Parse_shen\.<expr>
            (shen/shen\.<expr> V2474)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<expr>))
             (shen/shen\.pair
              (shen/hd Parse_shen\.<expr>)
              (shen/shen\.curry-type
               (shen/shen\.hdtl Parse_shen\.<expr>)))
             (shen/fail))))
(defun shen/shen\.<doubleunderline>
    (V2476)
  (shen/if
   (shen/cons\?
    (shen/hd V2476))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2476))
             (shen/if
              (shen/shen\.doubleunderline\? Parse_X)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2476))
                 (shen/shen\.hdtl V2476)))
               Parse_X)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<singleunderline>
    (V2478)
  (shen/if
   (shen/cons\?
    (shen/hd V2478))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2478))
             (shen/if
              (shen/shen\.singleunderline\? Parse_X)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2478))
                 (shen/shen\.hdtl V2478)))
               Parse_X)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.singleunderline\?
    (V2480)
  (shen/and
   (shen/symbol\? V2480)
   (shen/shen\.sh\?
    (shen/str V2480))))
(defun shen/shen\.sh\?
    (V2482)
  (shen/cond
   ((shen/= "_" V2482)
    'true)
   (shen/true
    (shen/and
     (shen/=
      (shen/pos V2482 0)
      "_")
     (shen/shen\.sh\?
      (shen/tlstr V2482))))))
(defun shen/shen\.doubleunderline\?
    (V2484)
  (shen/and
   (shen/symbol\? V2484)
   (shen/shen\.dh\?
    (shen/str V2484))))
(defun shen/shen\.dh\?
    (V2486)
  (shen/cond
   ((shen/= "=" V2486)
    'true)
   (shen/true
    (shen/and
     (shen/=
      (shen/pos V2486 0)
      "=")
     (shen/shen\.dh\?
      (shen/tlstr V2486))))))
(defun shen/shen\.process-datatype
    (V2489 V2490)
  (shen/shen\.remember-datatype
   (shen/shen\.s-prolog
    (shen/shen\.rules->horn-clauses V2489 V2490))))
(defun shen/shen\.remember-datatype
    (V2496)
  (shen/cond
   ((shen/cons\? V2496)
    (shen/do
     (shen/set 'shen\.*datatypes*
               (shen/adjoin
                (shen/hd V2496)
                (shen/value 'shen\.*datatypes*)))
     (shen/do
      (shen/set 'shen\.*alldatatypes*
                (shen/adjoin
                 (shen/hd V2496)
                 (shen/value 'shen\.*alldatatypes*)))
      (shen/hd V2496))))
   (shen/true
    (shen/shen\.f_error 'shen\.remember-datatype))))
(defun shen/shen\.rules->horn-clauses
    (V2501 V2502)
  (cl-flet
      ((tail-trampoline
        (V2501 V2502)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2502))
          nil)
         ((shen/and
           (shen/cons\? V2502)
           (shen/and
            (shen/tuple\?
             (shen/hd V2502))
            (shen/= 'shen\.single
                    (shen/fst
                     (shen/hd V2502)))))
          (append
           (list
            (shen/shen\.rule->horn-clause V2501
                                          (shen/snd
                                           (shen/hd V2502))))
           (shen/shen\.rules->horn-clauses V2501
                                           (nthcdr 1 V2502))))
         ((shen/and
           (shen/cons\? V2502)
           (shen/and
            (shen/tuple\?
             (shen/hd V2502))
            (shen/= 'shen\.double
                    (shen/fst
                     (shen/hd V2502)))))
          (vector
           (list V2501
                 (shen/append
                  (shen/shen\.double->singles
                   (shen/snd
                    (shen/hd V2502)))
                  (nthcdr 1 V2502)))))
         (shen/true
          (shen/shen\.f_error 'shen\.rules->horn-clauses)))))
    (let
        ((result
          (funcall #'tail-trampoline V2501 V2502)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.double->singles
    (V2504)
  (list
   (shen/shen\.right-rule V2504)
   (shen/shen\.left-rule V2504)))
(defun shen/shen\.right-rule
    (V2506)
  (shen/@p 'shen\.single V2506))
(defun shen/shen\.left-rule
    (V2508)
  (shen/cond
   ((shen/and
     (shen/cons\? V2508)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2508))
      (shen/and
       (shen/cons\?
        (nthcdr 2 V2508))
       (shen/and
        (shen/tuple\?
         (shen/hd
          (nthcdr 2 V2508)))
        (shen/and
         (shen/internal/predicate->shen
          (null
           (shen/fst
            (shen/hd
             (nthcdr 2 V2508)))))
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V2508))))))))
    (shen/let Q
              (shen/gensym 'Qv)
              (shen/let NewConclusion
                        (shen/@p
                         (list
                          (shen/snd
                           (shen/hd
                            (nthcdr 2 V2508))))
                         Q)
                        (shen/let NewPremises
                                  (list
                                   (shen/@p
                                    (shen/map
                                     (shen/lambda X
                                                  (shen/shen\.right->left X))
                                     (shen/hd
                                      (nthcdr 1 V2508)))
                                    Q))
                                  (shen/@p 'shen\.single
                                           (list
                                            (shen/hd V2508)
                                            NewPremises NewConclusion))))))
   (shen/true
    (shen/shen\.f_error 'shen\.left-rule))))
(defun shen/shen\.right->left
    (V2514)
  (shen/cond
   ((shen/and
     (shen/tuple\? V2514)
     (shen/internal/predicate->shen
      (null
       (shen/fst V2514))))
    (shen/snd V2514))
   (shen/true
    (shen/simple-error "syntax error with ==========\n"))))
(defun shen/shen\.rule->horn-clause
    (V2517 V2518)
  (shen/cond
   ((shen/and
     (shen/cons\? V2518)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2518))
      (shen/and
       (shen/cons\?
        (nthcdr 2 V2518))
       (shen/and
        (shen/tuple\?
         (shen/hd
          (nthcdr 2 V2518)))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V2518)))))))
    (list
     (shen/shen\.rule->horn-clause-head V2517
                                        (shen/snd
                                         (shen/hd
                                          (nthcdr 2 V2518))))
     ':-
     (shen/shen\.rule->horn-clause-body
      (shen/hd V2518)
      (shen/hd
       (nthcdr 1 V2518))
      (shen/fst
       (shen/hd
        (nthcdr 2 V2518))))))
   (shen/true
    (shen/shen\.f_error 'shen\.rule->horn-clause))))
(defun shen/shen\.rule->horn-clause-head
    (V2521 V2522)
  (list V2521
        (shen/shen\.mode-ify V2522)
        'Context_1957))
(defun shen/shen\.mode-ify
    (V2524)
  (shen/cond
   ((shen/and
     (shen/cons\? V2524)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2524))
      (shen/and
       (shen/= ':
               (shen/hd
                (nthcdr 1 V2524)))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V2524))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V2524)))))))
    (list 'mode
          (list
           (shen/hd V2524)
           ':
           (list 'mode
                 (shen/hd
                  (nthcdr 2 V2524))
                 '+))
          '-))
   (shen/true V2524)))
(defun shen/shen\.rule->horn-clause-body
    (V2528 V2529 V2530)
  (shen/let Variables
            (shen/map
             (shen/lambda X
                          (shen/shen\.extract_vars X))
             V2530)
            (shen/let Predicates
                      (shen/map
                       (shen/lambda X
                                    (shen/gensym 'shen\.cl))
                       V2530)
                      (shen/let SearchLiterals
                                (shen/shen\.construct-search-literals Predicates Variables 'Context_1957 'Context1_1957)
                                (shen/let SearchClauses
                                          (shen/shen\.construct-search-clauses Predicates V2530 Variables)
                                          (shen/let SideLiterals
                                                    (shen/shen\.construct-side-literals V2528)
                                                    (shen/let PremissLiterals
                                                              (shen/map
                                                               (shen/lambda X
                                                                            (shen/shen\.construct-premiss-literal X
                                                                                                                  (shen/empty\? V2530)))
                                                               V2529)
                                                              (shen/append SearchLiterals
                                                                           (shen/append SideLiterals PremissLiterals)))))))))
(defun shen/shen\.construct-search-literals
    (V2539 V2540 V2541 V2542)
  (shen/cond
   ((shen/and
     (shen/internal/predicate->shen
      (null V2539))
     (shen/internal/predicate->shen
      (null V2540)))
    nil)
   (shen/true
    (shen/shen\.csl-help V2539 V2540 V2541 V2542))))
(defun shen/shen\.csl-help
    (V2549 V2550 V2551 V2552)
  (shen/cond
   ((shen/and
     (shen/internal/predicate->shen
      (null V2549))
     (shen/internal/predicate->shen
      (null V2550)))
    (list
     (list 'bind 'ContextOut_1957 V2551)))
   ((shen/and
     (shen/cons\? V2549)
     (shen/cons\? V2550))
    (append
     (list
      (append
       (list
        (shen/hd V2549)
        V2551 V2552)
       (shen/hd V2550)))
     (shen/shen\.csl-help
      (nthcdr 1 V2549)
      (nthcdr 1 V2550)
      V2552
      (shen/gensym 'Context))))
   (shen/true
    (shen/shen\.f_error 'shen\.csl-help))))
(defun shen/shen\.construct-search-clauses
    (V2556 V2557 V2558)
  (cl-flet
      ((tail-trampoline
        (V2556 V2557 V2558)
        (shen/cond
         ((shen/and
           (shen/internal/predicate->shen
            (null V2556))
           (shen/and
            (shen/internal/predicate->shen
             (null V2557))
            (shen/internal/predicate->shen
             (null V2558))))
          'shen\.skip)
         ((shen/and
           (shen/cons\? V2556)
           (shen/and
            (shen/cons\? V2557)
            (shen/cons\? V2558)))
          (shen/do
           (shen/shen\.construct-search-clause
            (shen/hd V2556)
            (shen/hd V2557)
            (shen/hd V2558))
           (vector
            (list
             (nthcdr 1 V2556)
             (nthcdr 1 V2557)
             (nthcdr 1 V2558)))))
         (shen/true
          (shen/shen\.f_error 'shen\.construct-search-clauses)))))
    (let
        ((result
          (funcall #'tail-trampoline V2556 V2557 V2558)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.construct-search-clause
    (V2562 V2563 V2564)
  (shen/shen\.s-prolog
   (list
    (shen/shen\.construct-base-search-clause V2562 V2563 V2564)
    (shen/shen\.construct-recursive-search-clause V2562 V2563 V2564))))
(defun shen/shen\.construct-base-search-clause
    (V2568 V2569 V2570)
  (list
   (append
    (list V2568
          (append
           (list
            (shen/shen\.mode-ify V2569))
           'In_1957)
          'In_1957)
    V2570)
   ':- nil))
(defun shen/shen\.construct-recursive-search-clause
    (V2574 V2575 V2576)
  (list
   (append
    (list V2574
          (append
           (list 'Assumption_1957)
           'Assumptions_1957)
          (append
           (list 'Assumption_1957)
           'Out_1957))
    V2576)
   ':-
   (list
    (append
     (list V2574 'Assumptions_1957 'Out_1957)
     V2576))))
(defun shen/shen\.construct-side-literals
    (V2582)
  (cl-flet
      ((tail-trampoline
        (V2582)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2582))
          nil)
         ((shen/and
           (shen/cons\? V2582)
           (shen/and
            (shen/cons\?
             (shen/hd V2582))
            (shen/and
             (shen/= 'if
                     (shen/hd
                      (shen/hd V2582)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V2582)))
              (shen/internal/predicate->shen
               (null
                (nthcdr 2
                        (shen/hd V2582))))))))
          (append
           (list
            (append
             (list 'when)
             (nthcdr 1
                     (shen/hd V2582))))
           (shen/shen\.construct-side-literals
            (nthcdr 1 V2582))))
         ((shen/and
           (shen/cons\? V2582)
           (shen/and
            (shen/cons\?
             (shen/hd V2582))
            (shen/and
             (shen/= 'let
                     (shen/hd
                      (shen/hd V2582)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V2582)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V2582)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3
                         (shen/hd V2582)))))))))
          (append
           (list
            (append
             (list 'is)
             (nthcdr 1
                     (shen/hd V2582))))
           (shen/shen\.construct-side-literals
            (nthcdr 1 V2582))))
         ((shen/cons\? V2582)
          (vector
           (list
            (nthcdr 1 V2582))))
         (shen/true
          (shen/shen\.f_error 'shen\.construct-side-literals)))))
    (let
        ((result
          (funcall #'tail-trampoline V2582)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.construct-premiss-literal
    (V2589 V2590)
  (shen/cond
   ((shen/tuple\? V2589)
    (list 'shen\.t*
          (shen/shen\.recursive_cons_form
           (shen/snd V2589))
          (shen/shen\.construct-context V2590
                                        (shen/fst V2589))))
   ((shen/= '! V2589)
    (list 'cut 'Throwcontrol))
   (shen/true
    (shen/shen\.f_error 'shen\.construct-premiss-literal))))
(defun shen/shen\.construct-context
    (V2593 V2594)
  (shen/cond
   ((shen/and
     (shen/= 'true V2593)
     (shen/internal/predicate->shen
      (null V2594)))
    'Context_1957)
   ((shen/and
     (shen/= 'false V2593)
     (shen/internal/predicate->shen
      (null V2594)))
    'ContextOut_1957)
   ((shen/cons\? V2594)
    (list 'cons
          (shen/shen\.recursive_cons_form
           (shen/hd V2594))
          (shen/shen\.construct-context V2593
                                        (nthcdr 1 V2594))))
   (shen/true
    (shen/shen\.f_error 'shen\.construct-context))))
(defun shen/shen\.recursive_cons_form
    (V2596)
  (shen/cond
   ((shen/cons\? V2596)
    (list 'cons
          (shen/shen\.recursive_cons_form
           (shen/hd V2596))
          (shen/shen\.recursive_cons_form
           (nthcdr 1 V2596))))
   (shen/true V2596)))
(defun shen/preclude
    (V2598)
  (shen/shen\.preclude-h
   (shen/map
    (shen/lambda X
                 (shen/shen\.intern-type X))
    V2598)))
(defun shen/shen\.preclude-h
    (V2600)
  (shen/let FilterDatatypes
            (shen/set 'shen\.*datatypes*
                      (shen/difference
                       (shen/value 'shen\.*datatypes*)
                       V2600))
            (shen/value 'shen\.*datatypes*)))
(defun shen/include
    (V2602)
  (shen/shen\.include-h
   (shen/map
    (shen/lambda X
                 (shen/shen\.intern-type X))
    V2602)))
(defun shen/shen\.include-h
    (V2604)
  (shen/let ValidTypes
            (shen/intersection V2604
                               (shen/value 'shen\.*alldatatypes*))
            (shen/let NewDatatypes
                      (shen/set 'shen\.*datatypes*
                                (shen/union ValidTypes
                                            (shen/value 'shen\.*datatypes*)))
                      (shen/value 'shen\.*datatypes*))))
(defun shen/preclude-all-but
    (V2606)
  (shen/shen\.preclude-h
   (shen/difference
    (shen/value 'shen\.*alldatatypes*)
    (shen/map
     (shen/lambda X
                  (shen/shen\.intern-type X))
     V2606))))
(defun shen/include-all-but
    (V2608)
  (shen/shen\.include-h
   (shen/difference
    (shen/value 'shen\.*alldatatypes*)
    (shen/map
     (shen/lambda X
                  (shen/shen\.intern-type X))
     V2608))))
(defun shen/shen\.synonyms-help
    (V2614)
  (cl-flet
      ((tail-trampoline
        (V2614)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2614))
          (shen/shen\.update-demodulation-function
           (shen/value 'shen\.*tc*)
           (shen/mapcan
            (shen/lambda X
                         (shen/shen\.demod-rule X))
            (shen/value 'shen\.*synonyms*))))
         ((shen/and
           (shen/cons\? V2614)
           (shen/cons\?
            (nthcdr 1 V2614)))
          (shen/let Vs
                    (shen/difference
                     (shen/shen\.extract_vars
                      (shen/hd
                       (nthcdr 1 V2614)))
                     (shen/shen\.extract_vars
                      (shen/hd V2614)))
                    (shen/if
                     (shen/empty\? Vs)
                     (shen/do
                      (shen/shen\.pushnew
                       (list
                        (shen/hd V2614)
                        (shen/hd
                         (nthcdr 1 V2614)))
                       'shen\.*synonyms*)
                      (vector
                       (list
                        (nthcdr 2 V2614))))
                     (shen/shen\.free_variable_warnings
                      (shen/hd
                       (nthcdr 1 V2614))
                      Vs))))
         (shen/true
          (shen/simple-error "odd number of synonyms\n")))))
    (let
        ((result
          (funcall #'tail-trampoline V2614)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.pushnew
    (V2617 V2618)
  (shen/if
   (shen/element\? V2617
                   (shen/value V2618))
   (shen/value V2618)
   (shen/set V2618
             (append
              (list V2617)
              (shen/value V2618)))))
(defun shen/shen\.demod-rule
    (V2620)
  (shen/cond
   ((shen/and
     (shen/cons\? V2620)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2620))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V2620)))))
    (list
     (shen/shen\.rcons_form
      (shen/hd V2620))
     '->
     (shen/shen\.rcons_form
      (shen/hd
       (nthcdr 1 V2620)))))
   (shen/true
    (shen/shen\.f_error 'shen\.demod-rule))))
(defun shen/shen\.lambda-of-defun
    (V2626)
  (shen/cond
   ((shen/and
     (shen/cons\? V2626)
     (shen/and
      (shen/= 'defun
              (shen/hd V2626))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2626))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V2626))
        (shen/and
         (shen/cons\?
          (shen/hd
           (nthcdr 2 V2626)))
         (shen/and
          (shen/internal/predicate->shen
           (null
            (nthcdr 1
                    (shen/hd
                     (nthcdr 2 V2626)))))
          (shen/and
           (shen/cons\?
            (nthcdr 3 V2626))
           (shen/internal/predicate->shen
            (null
             (nthcdr 4 V2626))))))))))
    (shen/eval
     (append
      (list '/\.
            (shen/hd
             (shen/hd
              (nthcdr 2 V2626))))
      (nthcdr 3 V2626))))
   (shen/true
    (shen/shen\.f_error 'shen\.lambda-of-defun))))
(defun shen/shen\.update-demodulation-function
    (V2629 V2630)
  (shen/do
   (shen/tc '-)
   (shen/do
    (shen/set 'shen\.*demodulation-function*
              (shen/shen\.lambda-of-defun
               (shen/shen\.elim-def
                (append
                 (list 'define 'shen\.demod)
                 (shen/append V2630
                              (shen/shen\.default-rule))))))
    (shen/do
     (shen/if V2629
              (shen/tc '+)
              'shen\.skip)
     'synonyms))))
(defun shen/shen\.default-rule nil
  (list 'X '-> 'X))
(defun shen/shen\.yacc
    (V4140)
  (shen/cond
   ((shen/and
     (shen/cons\? V4140)
     (shen/and
      (shen/= 'defcc
              (shen/hd V4140))
      (shen/cons\?
       (nthcdr 1 V4140))))
    (shen/shen\.yacc->shen
     (shen/hd
      (nthcdr 1 V4140))
     (nthcdr 2 V4140)))
   (shen/true
    (shen/shen\.f_error 'shen\.yacc))))
(defun shen/shen\.yacc->shen
    (V4143 V4144)
  (shen/let CCRules
            (shen/shen\.split_cc_rules 'true V4144 nil)
            (shen/let CCBody
                      (shen/map
                       (shen/lambda X
                                    (shen/shen\.cc_body X))
                       CCRules)
                      (shen/let YaccCases
                                (shen/shen\.yacc_cases CCBody)
                                (list 'define V4143 'Stream '->
                                      (shen/shen\.kill-code YaccCases))))))
(defun shen/shen\.kill-code
    (V4146)
  (shen/cond
   ((shen/>
     (shen/occurrences 'kill V4146)
     0)
    (list 'trap-error V4146
          (list 'lambda 'E
                (list 'shen\.analyse-kill 'E))))
   (shen/true V4146)))
(defun shen/kill nil
  (shen/simple-error "yacc kill"))
(defun shen/shen\.analyse-kill
    (V4148)
  (shen/let String
            (shen/error-to-string V4148)
            (shen/if
             (shen/= String "yacc kill")
             (shen/fail)
             V4148)))
(defun shen/shen\.split_cc_rules
    (V4154 V4155 V4156)
  (cl-flet
      ((tail-trampoline
        (V4154 V4155 V4156)
        (shen/cond
         ((shen/and
           (shen/internal/predicate->shen
            (null V4155))
           (shen/internal/predicate->shen
            (null V4156)))
          nil)
         ((shen/internal/predicate->shen
           (null V4155))
          (list
           (shen/shen\.split_cc_rule V4154
                                     (shen/reverse V4156)
                                     nil)))
         ((shen/and
           (shen/cons\? V4155)
           (shen/= '\;
                   (shen/hd V4155)))
          (append
           (list
            (shen/shen\.split_cc_rule V4154
                                      (shen/reverse V4156)
                                      nil))
           (shen/shen\.split_cc_rules V4154
                                      (nthcdr 1 V4155)
                                      nil)))
         ((shen/cons\? V4155)
          (vector
           (list V4154
                 (nthcdr 1 V4155)
                 (append
                  (list
                   (shen/hd V4155))
                  V4156))))
         (shen/true
          (shen/shen\.f_error 'shen\.split_cc_rules)))))
    (let
        ((result
          (funcall #'tail-trampoline V4154 V4155 V4156)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.split_cc_rule
    (V4164 V4165 V4166)
  (cl-flet
      ((tail-trampoline
        (V4164 V4165 V4166)
        (shen/cond
         ((shen/and
           (shen/cons\? V4165)
           (shen/and
            (shen/= ':=
                    (shen/hd V4165))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V4165))
             (shen/internal/predicate->shen
              (null
               (nthcdr 2 V4165))))))
          (append
           (list
            (shen/reverse V4166))
           (nthcdr 1 V4165)))
         ((shen/and
           (shen/cons\? V4165)
           (shen/and
            (shen/= ':=
                    (shen/hd V4165))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V4165))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V4165))
              (shen/and
               (shen/= 'where
                       (shen/hd
                        (nthcdr 2 V4165)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V4165))
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 4 V4165)))))))))
          (list
           (shen/reverse V4166)
           (list 'where
                 (shen/hd
                  (nthcdr 3 V4165))
                 (shen/hd
                  (nthcdr 1 V4165)))))
         ((shen/internal/predicate->shen
           (null V4165))
          (shen/do
           (shen/shen\.semantic-completion-warning V4164 V4166)
           (vector
            (list V4164
                  (list ':=
                        (shen/shen\.default_semantics
                         (shen/reverse V4166)))
                  V4166))))
         ((shen/cons\? V4165)
          (vector
           (list V4164
                 (nthcdr 1 V4165)
                 (append
                  (list
                   (shen/hd V4165))
                  V4166))))
         (shen/true
          (shen/shen\.f_error 'shen\.split_cc_rule)))))
    (let
        ((result
          (funcall #'tail-trampoline V4164 V4165 V4166)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.semantic-completion-warning
    (V4177 V4178)
  (shen/cond
   ((shen/= 'true V4177)
    (shen/do
     (shen/shen\.prhush "warning: "
                        (shen/stoutput))
     (shen/do
      (shen/for-each
       (shen/lambda X
                    (shen/shen\.prhush
                     (shen/shen\.app X " " 'shen\.a)
                     (shen/stoutput)))
       (shen/reverse V4178))
      (shen/shen\.prhush "has no semantics.\n"
                         (shen/stoutput)))))
   (shen/true 'shen\.skip)))
(defun shen/shen\.default_semantics
    (V4180)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V4180))
    nil)
   ((shen/and
     (shen/cons\? V4180)
     (shen/and
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V4180)))
      (shen/shen\.grammar_symbol\?
       (shen/hd V4180))))
    (shen/hd V4180))
   ((shen/and
     (shen/cons\? V4180)
     (shen/shen\.grammar_symbol\?
      (shen/hd V4180)))
    (list 'append
          (shen/hd V4180)
          (shen/shen\.default_semantics
           (nthcdr 1 V4180))))
   ((shen/cons\? V4180)
    (list 'cons
          (shen/hd V4180)
          (shen/shen\.default_semantics
           (nthcdr 1 V4180))))
   (shen/true
    (shen/shen\.f_error 'shen\.default_semantics))))
(defun shen/shen\.grammar_symbol\?
    (V4182)
  (shen/and
   (shen/symbol\? V4182)
   (shen/let Cs
             (shen/shen\.strip-pathname
              (shen/explode V4182))
             (shen/and
              (shen/=
               (shen/hd Cs)
               "<")
              (shen/=
               (shen/hd
                (shen/reverse Cs))
               ">")))))
(defun shen/shen\.yacc_cases
    (V4184)
  (shen/cond
   ((shen/and
     (shen/cons\? V4184)
     (shen/internal/predicate->shen
      (null
       (nthcdr 1 V4184))))
    (shen/hd V4184))
   ((shen/cons\? V4184)
    (shen/let P 'YaccParse
              (list 'let P
                    (shen/hd V4184)
                    (list 'if
                          (list '= P
                                (list 'fail))
                          (shen/shen\.yacc_cases
                           (nthcdr 1 V4184))
                          P))))
   (shen/true
    (shen/shen\.f_error 'shen\.yacc_cases))))
(defun shen/shen\.cc_body
    (V4186)
  (shen/cond
   ((shen/and
     (shen/cons\? V4186)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V4186))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V4186)))))
    (shen/shen\.syntax
     (shen/hd V4186)
     'Stream
     (shen/hd
      (nthcdr 1 V4186))))
   (shen/true
    (shen/shen\.f_error 'shen\.cc_body))))
(defun shen/shen\.syntax
    (V4190 V4191 V4192)
  (shen/cond
   ((shen/and
     (shen/internal/predicate->shen
      (null V4190))
     (shen/and
      (shen/cons\? V4192)
      (shen/and
       (shen/= 'where
               (shen/hd V4192))
       (shen/and
        (shen/cons\?
         (nthcdr 1 V4192))
        (shen/and
         (shen/cons\?
          (nthcdr 2 V4192))
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V4192))))))))
    (list 'if
          (shen/shen\.semantics
           (shen/hd
            (nthcdr 1 V4192)))
          (list 'shen\.pair
                (list 'hd V4191)
                (shen/shen\.semantics
                 (shen/hd
                  (nthcdr 2 V4192))))
          (list 'fail)))
   ((shen/internal/predicate->shen
     (null V4190))
    (list 'shen\.pair
          (list 'hd V4191)
          (shen/shen\.semantics V4192)))
   ((shen/cons\? V4190)
    (shen/if
     (shen/shen\.grammar_symbol\?
      (shen/hd V4190))
     (shen/shen\.recursive_descent V4190 V4191 V4192)
     (shen/if
      (shen/variable\?
       (shen/hd V4190))
      (shen/shen\.variable-match V4190 V4191 V4192)
      (shen/if
       (shen/shen\.jump_stream\?
        (shen/hd V4190))
       (shen/shen\.jump_stream V4190 V4191 V4192)
       (shen/if
        (shen/shen\.terminal\?
         (shen/hd V4190))
        (shen/shen\.check_stream V4190 V4191 V4192)
        (shen/if
         (shen/cons\?
          (shen/hd V4190))
         (shen/shen\.list-stream
          (shen/shen\.decons
           (shen/hd V4190))
          (nthcdr 1 V4190)
          V4191 V4192)
         (shen/simple-error
          (shen/shen\.app
           (shen/hd V4190)
           " is not legal syntax\n" 'shen\.a))))))))
   (shen/true
    (shen/shen\.f_error 'shen\.syntax))))
(defun shen/shen\.list-stream
    (V4197 V4198 V4199 V4200)
  (shen/let Test
            (list 'and
                  (list 'cons\?
                        (list 'hd V4199))
                  (list 'cons\?
                        (list 'hd
                              (list 'hd V4199))))
            (shen/let Placeholder
                      (shen/gensym 'shen\.place)
                      (shen/let RunOn
                                (shen/shen\.syntax V4198
                                                   (list 'shen\.pair
                                                         (list 'tl
                                                               (list 'hd V4199))
                                                         (list 'hd
                                                               (list 'tl V4199)))
                                                   V4200)
                                (shen/let Action
                                          (shen/shen\.insert-runon RunOn Placeholder
                                                                   (shen/shen\.syntax V4197
                                                                                      (list 'shen\.pair
                                                                                            (list 'hd
                                                                                                  (list 'hd V4199))
                                                                                            (list 'hd
                                                                                                  (list 'tl V4199)))
                                                                                      Placeholder))
                                          (list 'if Test Action
                                                (list 'fail)))))))
(defun shen/shen\.decons
    (V4202)
  (shen/cond
   ((shen/and
     (shen/cons\? V4202)
     (shen/and
      (shen/= 'cons
              (shen/hd V4202))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V4202))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V4202))
        (shen/and
         (shen/internal/predicate->shen
          (null
           (shen/hd
            (nthcdr 2 V4202))))
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V4202))))))))
    (list
     (shen/hd
      (nthcdr 1 V4202))))
   ((shen/and
     (shen/cons\? V4202)
     (shen/and
      (shen/= 'cons
              (shen/hd V4202))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V4202))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V4202))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V4202)))))))
    (append
     (list
      (shen/hd
       (nthcdr 1 V4202)))
     (shen/shen\.decons
      (shen/hd
       (nthcdr 2 V4202)))))
   (shen/true V4202)))
(defun shen/shen\.insert-runon
    (V4217 V4218 V4219)
  (shen/cond
   ((shen/and
     (shen/cons\? V4219)
     (shen/and
      (shen/= 'shen\.pair
              (shen/hd V4219))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V4219))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V4219))
        (shen/and
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V4219)))
         (shen/=
          (shen/hd
           (nthcdr 2 V4219))
          V4218))))))
    V4217)
   ((shen/cons\? V4219)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.insert-runon V4217 V4218 Z))
     V4219))
   (shen/true V4219)))
(defun shen/shen\.strip-pathname
    (V4225)
  (cl-flet
      ((tail-trampoline
        (V4225)
        (shen/cond
         ((shen/not
           (shen/element\? "." V4225))
          V4225)
         ((shen/cons\? V4225)
          (vector
           (list
            (nthcdr 1 V4225))))
         (shen/true
          (shen/shen\.f_error 'shen\.strip-pathname)))))
    (let
        ((result
          (funcall #'tail-trampoline V4225)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.recursive_descent
    (V4229 V4230 V4231)
  (shen/cond
   ((shen/cons\? V4229)
    (shen/let Test
              (list
               (shen/hd V4229)
               V4230)
              (shen/let Action
                        (shen/shen\.syntax
                         (nthcdr 1 V4229)
                         (shen/concat 'Parse_
                                      (shen/hd V4229))
                         V4231)
                        (shen/let Else
                                  (list 'fail)
                                  (list 'let
                                        (shen/concat 'Parse_
                                                     (shen/hd V4229))
                                        Test
                                        (list 'if
                                              (list 'not
                                                    (list '=
                                                          (list 'fail)
                                                          (shen/concat 'Parse_
                                                                       (shen/hd V4229))))
                                              Action Else))))))
   (shen/true
    (shen/shen\.f_error 'shen\.recursive_descent))))
(defun shen/shen\.variable-match
    (V4235 V4236 V4237)
  (shen/cond
   ((shen/cons\? V4235)
    (shen/let Test
              (list 'cons\?
                    (list 'hd V4236))
              (shen/let Action
                        (list 'let
                              (shen/concat 'Parse_
                                           (shen/hd V4235))
                              (list 'hd
                                    (list 'hd V4236))
                              (shen/shen\.syntax
                               (nthcdr 1 V4235)
                               (list 'shen\.pair
                                     (list 'tl
                                           (list 'hd V4236))
                                     (list 'shen\.hdtl V4236))
                               V4237))
                        (shen/let Else
                                  (list 'fail)
                                  (list 'if Test Action Else)))))
   (shen/true
    (shen/shen\.f_error 'shen\.variable-match))))
(defun shen/shen\.terminal\?
    (V4247)
  (shen/cond
   ((shen/cons\? V4247)
    'false)
   ((shen/variable\? V4247)
    'false)
   (shen/true 'true)))
(defun shen/shen\.jump_stream\?
    (V4253)
  (shen/cond
   ((shen/= V4253 '_)
    'true)
   (shen/true 'false)))
(defun shen/shen\.check_stream
    (V4257 V4258 V4259)
  (shen/cond
   ((shen/cons\? V4257)
    (shen/let Test
              (list 'and
                    (list 'cons\?
                          (list 'hd V4258))
                    (list '=
                          (shen/hd V4257)
                          (list 'hd
                                (list 'hd V4258))))
              (shen/let Action
                        (shen/shen\.syntax
                         (nthcdr 1 V4257)
                         (list 'shen\.pair
                               (list 'tl
                                     (list 'hd V4258))
                               (list 'shen\.hdtl V4258))
                         V4259)
                        (shen/let Else
                                  (list 'fail)
                                  (list 'if Test Action Else)))))
   (shen/true
    (shen/shen\.f_error 'shen\.check_stream))))
(defun shen/shen\.jump_stream
    (V4263 V4264 V4265)
  (shen/cond
   ((shen/cons\? V4263)
    (shen/let Test
              (list 'cons\?
                    (list 'hd V4264))
              (shen/let Action
                        (shen/shen\.syntax
                         (nthcdr 1 V4263)
                         (list 'shen\.pair
                               (list 'tl
                                     (list 'hd V4264))
                               (list 'shen\.hdtl V4264))
                         V4265)
                        (shen/let Else
                                  (list 'fail)
                                  (list 'if Test Action Else)))))
   (shen/true
    (shen/shen\.f_error 'shen\.jump_stream))))
(defun shen/shen\.semantics
    (V4267)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V4267))
    nil)
   ((shen/shen\.grammar_symbol\? V4267)
    (list 'shen\.hdtl
          (shen/concat 'Parse_ V4267)))
   ((shen/variable\? V4267)
    (shen/concat 'Parse_ V4267))
   ((shen/cons\? V4267)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.semantics Z))
     V4267))
   (shen/true V4267)))
(defun shen/shen\.snd-or-fail
    (V4275)
  (shen/cond
   ((shen/and
     (shen/cons\? V4275)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V4275))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V4275)))))
    (shen/hd
     (nthcdr 1 V4275)))
   (shen/true
    (shen/fail))))
(defun shen/fail nil 'shen\.fail!)
(defun shen/shen\.pair
    (V4278 V4279)
  (list V4278 V4279))
(defun shen/shen\.hdtl
    (V4281)
  (shen/hd
   (nthcdr 1 V4281)))
(defun shen/<!>
    (V4289)
  (shen/cond
   ((shen/and
     (shen/cons\? V4289)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V4289))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V4289)))))
    (list nil
          (shen/hd V4289)))
   (shen/true
    (shen/fail))))
(defun shen/<e>
    (V4295)
  (shen/cond
   ((shen/and
     (shen/cons\? V4295)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V4295))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V4295)))))
    (list
     (shen/hd V4295)
     nil))
   (shen/true
    (shen/shen\.f_error '<e>))))
(defun shen/read-char-code
    (V2163)
  (shen/read-byte V2163))
(defun shen/read-file-as-bytelist
    (V2165)
  (shen/shen\.read-file-as-Xlist V2165
                                 (shen/lambda S
                                              (shen/read-byte S))))
(defun shen/read-file-as-charlist
    (V2167)
  (shen/shen\.read-file-as-Xlist V2167
                                 (shen/lambda S
                                              (shen/read-char-code S))))
(defun shen/shen\.read-file-as-Xlist
    (V2170 V2171)
  (shen/let Stream
            (shen/open V2170 'in)
            (shen/let X
                      (shen/internal/apply-higher-order-function V2171
                                                                 (list Stream))
                      (shen/let Xs
                                (shen/shen\.read-file-as-Xlist-help Stream V2171 X nil)
                                (shen/let Close
                                          (shen/close Stream)
                                          (shen/reverse Xs))))))
(defun shen/shen\.read-file-as-Xlist-help
    (V2176 V2177 V2178 V2179)
  (cl-flet
      ((tail-trampoline
        (V2176 V2177 V2178 V2179)
        (shen/cond
         ((shen/= -1 V2178)
          V2179)
         (shen/true
          (vector
           (list V2176 V2177
                 (shen/internal/apply-higher-order-function V2177
                                                            (list V2176))
                 (append
                  (list V2178)
                  V2179)))))))
    (let
        ((result
          (funcall #'tail-trampoline V2176 V2177 V2178 V2179)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/read-file-as-string
    (V2181)
  (shen/let Stream
            (shen/open V2181 'in)
            (shen/shen\.rfas-h Stream
                               (shen/read-char-code Stream)
                               "")))
(defun shen/shen\.rfas-h
    (V2185 V2186 V2187)
  (cl-flet
      ((tail-trampoline
        (V2185 V2186 V2187)
        (shen/cond
         ((shen/= -1 V2186)
          (shen/do
           (shen/close V2185)
           V2187))
         (shen/true
          (vector
           (list V2185
                 (shen/read-char-code V2185)
                 (shen/cn V2187
                          (shen/n->string V2186))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2185 V2186 V2187)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/input
    (V2189)
  (shen/eval-kl
   (shen/read V2189)))
(defun shen/input+
    (V2192 V2193)
  (shen/let Mono\?
            (shen/shen\.monotype V2192)
            (shen/let Input
                      (shen/read V2193)
                      (shen/if
                       (shen/= 'false
                               (shen/shen\.typecheck Input
                                                     (shen/shen\.demodulate V2192)))
                       (shen/simple-error
                        (shen/cn "type error: "
                                 (shen/shen\.app Input
                                                 (shen/cn " is not of type "
                                                          (shen/shen\.app V2192 "\n" 'shen\.r))
                                                 'shen\.r)))
                       (shen/eval-kl Input)))))
(defun shen/shen\.monotype
    (V2195)
  (shen/cond
   ((shen/cons\? V2195)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.monotype Z))
     V2195))
   (shen/true
    (shen/if
     (shen/variable\? V2195)
     (shen/simple-error
      (shen/cn "input+ expects a monotype: not "
               (shen/shen\.app V2195 "\n" 'shen\.a)))
     V2195))))
(defun shen/read
    (V2197)
  (shen/hd
   (shen/shen\.read-loop V2197
                         (shen/read-char-code V2197)
                         nil)))
(defun shen/it nil
  (shen/value 'shen\.*it*))
(defun shen/shen\.read-loop
    (V2205 V2206 V2207)
  (cl-flet
      ((tail-trampoline
        (V2205 V2206 V2207)
        (shen/cond
         ((shen/= 94 V2206)
          (shen/simple-error "read aborted"))
         ((shen/= -1 V2206)
          (shen/if
           (shen/empty\? V2207)
           (shen/simple-error "error: empty stream")
           (shen/compile
            (shen/lambda X
                         (shen/shen\.<st_input> X))
            V2207
            (shen/lambda E E))))
         ((shen/shen\.terminator\? V2206)
          (shen/let AllChars
                    (shen/append V2207
                                 (list V2206))
                    (shen/let It
                              (shen/shen\.record-it AllChars)
                              (shen/let Read
                                        (shen/compile
                                         (shen/lambda X
                                                      (shen/shen\.<st_input> X))
                                         AllChars
                                         (shen/lambda E 'shen\.nextbyte))
                                        (shen/if
                                         (shen/or
                                          (shen/= Read 'shen\.nextbyte)
                                          (shen/empty\? Read))
                                         (vector
                                          (list V2205
                                                (shen/read-char-code V2205)
                                                AllChars))
                                         Read)))))
         (shen/true
          (vector
           (list V2205
                 (shen/read-char-code V2205)
                 (shen/append V2207
                              (list V2206))))))))
    (let
        ((result
          (funcall #'tail-trampoline V2205 V2206 V2207)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.terminator\?
    (V2209)
  (shen/element\? V2209
                  (list 9 10 13 32 34 41 93)))
(defun shen/lineread
    (V2211)
  (shen/shen\.lineread-loop
   (shen/read-char-code V2211)
   nil V2211))
(defun shen/shen\.lineread-loop
    (V2216 V2217 V2218)
  (cl-flet
      ((tail-trampoline
        (V2216 V2217 V2218)
        (shen/cond
         ((shen/= -1 V2216)
          (shen/if
           (shen/empty\? V2217)
           (shen/simple-error "empty stream")
           (shen/compile
            (shen/lambda X
                         (shen/shen\.<st_input> X))
            V2217
            (shen/lambda E E))))
         ((shen/= V2216
                  (shen/shen\.hat))
          (shen/simple-error "line read aborted"))
         ((shen/element\? V2216
                          (list
                           (shen/shen\.newline)
                           (shen/shen\.carriage-return)))
          (shen/let Line
                    (shen/compile
                     (shen/lambda X
                                  (shen/shen\.<st_input> X))
                     V2217
                     (shen/lambda E 'shen\.nextline))
                    (shen/let It
                              (shen/shen\.record-it V2217)
                              (shen/if
                               (shen/or
                                (shen/= Line 'shen\.nextline)
                                (shen/empty\? Line))
                               (vector
                                (list
                                 (shen/read-char-code V2218)
                                 (shen/append V2217
                                              (list V2216))
                                 V2218))
                               Line))))
         (shen/true
          (vector
           (list
            (shen/read-char-code V2218)
            (shen/append V2217
                         (list V2216))
            V2218))))))
    (let
        ((result
          (funcall #'tail-trampoline V2216 V2217 V2218)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.record-it
    (V2220)
  (shen/let TrimLeft
            (shen/shen\.trim-whitespace V2220)
            (shen/let TrimRight
                      (shen/shen\.trim-whitespace
                       (shen/reverse TrimLeft))
                      (shen/let Trimmed
                                (shen/reverse TrimRight)
                                (shen/shen\.record-it-h Trimmed)))))
(defun shen/shen\.trim-whitespace
    (V2222)
  (cl-flet
      ((tail-trampoline
        (V2222)
        (shen/cond
         ((shen/and
           (shen/cons\? V2222)
           (shen/element\?
            (shen/hd V2222)
            (list 9 10 13 32)))
          (vector
           (list
            (nthcdr 1 V2222))))
         (shen/true V2222))))
    (let
        ((result
          (funcall #'tail-trampoline V2222)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.record-it-h
    (V2224)
  (shen/do
   (shen/set 'shen\.*it*
             (shen/shen\.cn-all
              (shen/map
               (shen/lambda X
                            (shen/n->string X))
               V2224)))
   V2224))
(defun shen/shen\.cn-all
    (V2226)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2226))
    "")
   ((shen/cons\? V2226)
    (shen/cn
     (shen/hd V2226)
     (shen/shen\.cn-all
      (nthcdr 1 V2226))))
   (shen/true
    (shen/shen\.f_error 'shen\.cn-all))))
(defun shen/read-file
    (V2228)
  (shen/let Charlist
            (shen/read-file-as-charlist V2228)
            (shen/compile
             (shen/lambda X
                          (shen/shen\.<st_input> X))
             Charlist
             (shen/lambda X
                          (shen/shen\.read-error X)))))
(defun shen/read-from-string
    (V2230)
  (shen/let Ns
            (shen/map
             (shen/lambda X
                          (shen/string->n X))
             (shen/explode V2230))
            (shen/compile
             (shen/lambda X
                          (shen/shen\.<st_input> X))
             Ns
             (shen/lambda X
                          (shen/shen\.read-error X)))))
(defun shen/shen\.read-error
    (V2238)
  (shen/cond
   ((shen/and
     (shen/cons\? V2238)
     (shen/and
      (shen/cons\?
       (shen/hd V2238))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2238))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V2238))))))
    (shen/simple-error
     (shen/cn "read error here:\n\n "
              (shen/shen\.app
               (shen/shen\.compress-50 50
                                       (shen/hd V2238))
               "\n" 'shen\.a))))
   (shen/true
    (shen/simple-error "read error\n"))))
(defun shen/shen\.compress-50
    (V2245 V2246)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2246))
    "")
   ((shen/= 0 V2245)
    "")
   ((shen/cons\? V2246)
    (shen/cn
     (shen/n->string
      (shen/hd V2246))
     (shen/shen\.compress-50
      (shen/- V2245 1)
      (nthcdr 1 V2246))))
   (shen/true
    (shen/shen\.f_error 'shen\.compress-50))))
(defun shen/shen\.<st_input>
    (V2248)
  (shen/let YaccParse
            (shen/let Parse_shen\.<lsb>
                      (shen/shen\.<lsb> V2248)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<lsb>))
                       (shen/let Parse_shen\.<st_input1>
                                 (shen/shen\.<st_input1> Parse_shen\.<lsb>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<st_input1>))
                                  (shen/let Parse_shen\.<rsb>
                                            (shen/shen\.<rsb> Parse_shen\.<st_input1>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<rsb>))
                                             (shen/let Parse_shen\.<st_input2>
                                                       (shen/shen\.<st_input2> Parse_shen\.<rsb>)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<st_input2>))
                                                        (shen/shen\.pair
                                                         (shen/hd Parse_shen\.<st_input2>)
                                                         (append
                                                          (list
                                                           (shen/macroexpand
                                                            (shen/shen\.cons_form
                                                             (shen/shen\.hdtl Parse_shen\.<st_input1>))))
                                                          (shen/shen\.hdtl Parse_shen\.<st_input2>)))
                                                        (shen/fail)))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<lrb>
                                 (shen/shen\.<lrb> V2248)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<lrb>))
                                  (shen/let Parse_shen\.<st_input1>
                                            (shen/shen\.<st_input1> Parse_shen\.<lrb>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<st_input1>))
                                             (shen/let Parse_shen\.<rrb>
                                                       (shen/shen\.<rrb> Parse_shen\.<st_input1>)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<rrb>))
                                                        (shen/let Parse_shen\.<st_input2>
                                                                  (shen/shen\.<st_input2> Parse_shen\.<rrb>)
                                                                  (shen/if
                                                                   (shen/not
                                                                    (shen/=
                                                                     (shen/fail)
                                                                     Parse_shen\.<st_input2>))
                                                                   (shen/shen\.pair
                                                                    (shen/hd Parse_shen\.<st_input2>)
                                                                    (shen/shen\.package-macro
                                                                     (shen/macroexpand
                                                                      (shen/shen\.hdtl Parse_shen\.<st_input1>))
                                                                     (shen/shen\.hdtl Parse_shen\.<st_input2>)))
                                                                   (shen/fail)))
                                                        (shen/fail)))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let YaccParse
                                  (shen/let Parse_shen\.<lcurly>
                                            (shen/shen\.<lcurly> V2248)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<lcurly>))
                                             (shen/let Parse_shen\.<st_input>
                                                       (shen/shen\.<st_input> Parse_shen\.<lcurly>)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<st_input>))
                                                        (shen/shen\.pair
                                                         (shen/hd Parse_shen\.<st_input>)
                                                         (append
                                                          (list '{)
                                                          (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                        (shen/fail)))
                                             (shen/fail)))
                                  (shen/if
                                   (shen/= YaccParse
                                           (shen/fail))
                                   (shen/let YaccParse
                                             (shen/let Parse_shen\.<rcurly>
                                                       (shen/shen\.<rcurly> V2248)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<rcurly>))
                                                        (shen/let Parse_shen\.<st_input>
                                                                  (shen/shen\.<st_input> Parse_shen\.<rcurly>)
                                                                  (shen/if
                                                                   (shen/not
                                                                    (shen/=
                                                                     (shen/fail)
                                                                     Parse_shen\.<st_input>))
                                                                   (shen/shen\.pair
                                                                    (shen/hd Parse_shen\.<st_input>)
                                                                    (append
                                                                     (list '})
                                                                     (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                   (shen/fail)))
                                                        (shen/fail)))
                                             (shen/if
                                              (shen/= YaccParse
                                                      (shen/fail))
                                              (shen/let YaccParse
                                                        (shen/let Parse_shen\.<bar>
                                                                  (shen/shen\.<bar> V2248)
                                                                  (shen/if
                                                                   (shen/not
                                                                    (shen/=
                                                                     (shen/fail)
                                                                     Parse_shen\.<bar>))
                                                                   (shen/let Parse_shen\.<st_input>
                                                                             (shen/shen\.<st_input> Parse_shen\.<bar>)
                                                                             (shen/if
                                                                              (shen/not
                                                                               (shen/=
                                                                                (shen/fail)
                                                                                Parse_shen\.<st_input>))
                                                                              (shen/shen\.pair
                                                                               (shen/hd Parse_shen\.<st_input>)
                                                                               (append
                                                                                (list 'bar!)
                                                                                (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                              (shen/fail)))
                                                                   (shen/fail)))
                                                        (shen/if
                                                         (shen/= YaccParse
                                                                 (shen/fail))
                                                         (shen/let YaccParse
                                                                   (shen/let Parse_shen\.<semicolon>
                                                                             (shen/shen\.<semicolon> V2248)
                                                                             (shen/if
                                                                              (shen/not
                                                                               (shen/=
                                                                                (shen/fail)
                                                                                Parse_shen\.<semicolon>))
                                                                              (shen/let Parse_shen\.<st_input>
                                                                                        (shen/shen\.<st_input> Parse_shen\.<semicolon>)
                                                                                        (shen/if
                                                                                         (shen/not
                                                                                          (shen/=
                                                                                           (shen/fail)
                                                                                           Parse_shen\.<st_input>))
                                                                                         (shen/shen\.pair
                                                                                          (shen/hd Parse_shen\.<st_input>)
                                                                                          (append
                                                                                           (list '\;)
                                                                                           (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                                         (shen/fail)))
                                                                              (shen/fail)))
                                                                   (shen/if
                                                                    (shen/= YaccParse
                                                                            (shen/fail))
                                                                    (shen/let YaccParse
                                                                              (shen/let Parse_shen\.<colon>
                                                                                        (shen/shen\.<colon> V2248)
                                                                                        (shen/if
                                                                                         (shen/not
                                                                                          (shen/=
                                                                                           (shen/fail)
                                                                                           Parse_shen\.<colon>))
                                                                                         (shen/let Parse_shen\.<equal>
                                                                                                   (shen/shen\.<equal> Parse_shen\.<colon>)
                                                                                                   (shen/if
                                                                                                    (shen/not
                                                                                                     (shen/=
                                                                                                      (shen/fail)
                                                                                                      Parse_shen\.<equal>))
                                                                                                    (shen/let Parse_shen\.<st_input>
                                                                                                              (shen/shen\.<st_input> Parse_shen\.<equal>)
                                                                                                              (shen/if
                                                                                                               (shen/not
                                                                                                                (shen/=
                                                                                                                 (shen/fail)
                                                                                                                 Parse_shen\.<st_input>))
                                                                                                               (shen/shen\.pair
                                                                                                                (shen/hd Parse_shen\.<st_input>)
                                                                                                                (append
                                                                                                                 (list ':=)
                                                                                                                 (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                                                               (shen/fail)))
                                                                                                    (shen/fail)))
                                                                                         (shen/fail)))
                                                                              (shen/if
                                                                               (shen/= YaccParse
                                                                                       (shen/fail))
                                                                               (shen/let YaccParse
                                                                                         (shen/let Parse_shen\.<colon>
                                                                                                   (shen/shen\.<colon> V2248)
                                                                                                   (shen/if
                                                                                                    (shen/not
                                                                                                     (shen/=
                                                                                                      (shen/fail)
                                                                                                      Parse_shen\.<colon>))
                                                                                                    (shen/let Parse_shen\.<minus>
                                                                                                              (shen/shen\.<minus> Parse_shen\.<colon>)
                                                                                                              (shen/if
                                                                                                               (shen/not
                                                                                                                (shen/=
                                                                                                                 (shen/fail)
                                                                                                                 Parse_shen\.<minus>))
                                                                                                               (shen/let Parse_shen\.<st_input>
                                                                                                                         (shen/shen\.<st_input> Parse_shen\.<minus>)
                                                                                                                         (shen/if
                                                                                                                          (shen/not
                                                                                                                           (shen/=
                                                                                                                            (shen/fail)
                                                                                                                            Parse_shen\.<st_input>))
                                                                                                                          (shen/shen\.pair
                                                                                                                           (shen/hd Parse_shen\.<st_input>)
                                                                                                                           (append
                                                                                                                            (list ':-)
                                                                                                                            (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                                                                          (shen/fail)))
                                                                                                               (shen/fail)))
                                                                                                    (shen/fail)))
                                                                                         (shen/if
                                                                                          (shen/= YaccParse
                                                                                                  (shen/fail))
                                                                                          (shen/let YaccParse
                                                                                                    (shen/let Parse_shen\.<colon>
                                                                                                              (shen/shen\.<colon> V2248)
                                                                                                              (shen/if
                                                                                                               (shen/not
                                                                                                                (shen/=
                                                                                                                 (shen/fail)
                                                                                                                 Parse_shen\.<colon>))
                                                                                                               (shen/let Parse_shen\.<st_input>
                                                                                                                         (shen/shen\.<st_input> Parse_shen\.<colon>)
                                                                                                                         (shen/if
                                                                                                                          (shen/not
                                                                                                                           (shen/=
                                                                                                                            (shen/fail)
                                                                                                                            Parse_shen\.<st_input>))
                                                                                                                          (shen/shen\.pair
                                                                                                                           (shen/hd Parse_shen\.<st_input>)
                                                                                                                           (append
                                                                                                                            (list ':)
                                                                                                                            (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                                                                          (shen/fail)))
                                                                                                               (shen/fail)))
                                                                                                    (shen/if
                                                                                                     (shen/= YaccParse
                                                                                                             (shen/fail))
                                                                                                     (shen/let YaccParse
                                                                                                               (shen/let Parse_shen\.<comma>
                                                                                                                         (shen/shen\.<comma> V2248)
                                                                                                                         (shen/if
                                                                                                                          (shen/not
                                                                                                                           (shen/=
                                                                                                                            (shen/fail)
                                                                                                                            Parse_shen\.<comma>))
                                                                                                                          (shen/let Parse_shen\.<st_input>
                                                                                                                                    (shen/shen\.<st_input> Parse_shen\.<comma>)
                                                                                                                                    (shen/if
                                                                                                                                     (shen/not
                                                                                                                                      (shen/=
                                                                                                                                       (shen/fail)
                                                                                                                                       Parse_shen\.<st_input>))
                                                                                                                                     (shen/shen\.pair
                                                                                                                                      (shen/hd Parse_shen\.<st_input>)
                                                                                                                                      (append
                                                                                                                                       (list
                                                                                                                                        (shen/intern ","))
                                                                                                                                       (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                                                                                     (shen/fail)))
                                                                                                                          (shen/fail)))
                                                                                                               (shen/if
                                                                                                                (shen/= YaccParse
                                                                                                                        (shen/fail))
                                                                                                                (shen/let YaccParse
                                                                                                                          (shen/let Parse_shen\.<comment>
                                                                                                                                    (shen/shen\.<comment> V2248)
                                                                                                                                    (shen/if
                                                                                                                                     (shen/not
                                                                                                                                      (shen/=
                                                                                                                                       (shen/fail)
                                                                                                                                       Parse_shen\.<comment>))
                                                                                                                                     (shen/let Parse_shen\.<st_input>
                                                                                                                                               (shen/shen\.<st_input> Parse_shen\.<comment>)
                                                                                                                                               (shen/if
                                                                                                                                                (shen/not
                                                                                                                                                 (shen/=
                                                                                                                                                  (shen/fail)
                                                                                                                                                  Parse_shen\.<st_input>))
                                                                                                                                                (shen/shen\.pair
                                                                                                                                                 (shen/hd Parse_shen\.<st_input>)
                                                                                                                                                 (shen/shen\.hdtl Parse_shen\.<st_input>))
                                                                                                                                                (shen/fail)))
                                                                                                                                     (shen/fail)))
                                                                                                                          (shen/if
                                                                                                                           (shen/= YaccParse
                                                                                                                                   (shen/fail))
                                                                                                                           (shen/let YaccParse
                                                                                                                                     (shen/let Parse_shen\.<atom>
                                                                                                                                               (shen/shen\.<atom> V2248)
                                                                                                                                               (shen/if
                                                                                                                                                (shen/not
                                                                                                                                                 (shen/=
                                                                                                                                                  (shen/fail)
                                                                                                                                                  Parse_shen\.<atom>))
                                                                                                                                                (shen/let Parse_shen\.<st_input>
                                                                                                                                                          (shen/shen\.<st_input> Parse_shen\.<atom>)
                                                                                                                                                          (shen/if
                                                                                                                                                           (shen/not
                                                                                                                                                            (shen/=
                                                                                                                                                             (shen/fail)
                                                                                                                                                             Parse_shen\.<st_input>))
                                                                                                                                                           (shen/shen\.pair
                                                                                                                                                            (shen/hd Parse_shen\.<st_input>)
                                                                                                                                                            (append
                                                                                                                                                             (list
                                                                                                                                                              (shen/macroexpand
                                                                                                                                                               (shen/shen\.hdtl Parse_shen\.<atom>)))
                                                                                                                                                             (shen/shen\.hdtl Parse_shen\.<st_input>)))
                                                                                                                                                           (shen/fail)))
                                                                                                                                                (shen/fail)))
                                                                                                                                     (shen/if
                                                                                                                                      (shen/= YaccParse
                                                                                                                                              (shen/fail))
                                                                                                                                      (shen/let YaccParse
                                                                                                                                                (shen/let Parse_shen\.<whitespaces>
                                                                                                                                                          (shen/shen\.<whitespaces> V2248)
                                                                                                                                                          (shen/if
                                                                                                                                                           (shen/not
                                                                                                                                                            (shen/=
                                                                                                                                                             (shen/fail)
                                                                                                                                                             Parse_shen\.<whitespaces>))
                                                                                                                                                           (shen/let Parse_shen\.<st_input>
                                                                                                                                                                     (shen/shen\.<st_input> Parse_shen\.<whitespaces>)
                                                                                                                                                                     (shen/if
                                                                                                                                                                      (shen/not
                                                                                                                                                                       (shen/=
                                                                                                                                                                        (shen/fail)
                                                                                                                                                                        Parse_shen\.<st_input>))
                                                                                                                                                                      (shen/shen\.pair
                                                                                                                                                                       (shen/hd Parse_shen\.<st_input>)
                                                                                                                                                                       (shen/shen\.hdtl Parse_shen\.<st_input>))
                                                                                                                                                                      (shen/fail)))
                                                                                                                                                           (shen/fail)))
                                                                                                                                                (shen/if
                                                                                                                                                 (shen/= YaccParse
                                                                                                                                                         (shen/fail))
                                                                                                                                                 (shen/let Parse_<e>
                                                                                                                                                           (shen/<e> V2248)
                                                                                                                                                           (shen/if
                                                                                                                                                            (shen/not
                                                                                                                                                             (shen/=
                                                                                                                                                              (shen/fail)
                                                                                                                                                              Parse_<e>))
                                                                                                                                                            (shen/shen\.pair
                                                                                                                                                             (shen/hd Parse_<e>)
                                                                                                                                                             nil)
                                                                                                                                                            (shen/fail)))
                                                                                                                                                 YaccParse))
                                                                                                                                      YaccParse))
                                                                                                                           YaccParse))
                                                                                                                YaccParse))
                                                                                                     YaccParse))
                                                                                          YaccParse))
                                                                               YaccParse))
                                                                    YaccParse))
                                                         YaccParse))
                                              YaccParse))
                                   YaccParse))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.<lsb>
    (V2250)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2250))
    (shen/= 91
            (shen/hd
             (shen/hd V2250))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2250))
      (shen/shen\.hdtl V2250)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<rsb>
    (V2252)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2252))
    (shen/= 93
            (shen/hd
             (shen/hd V2252))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2252))
      (shen/shen\.hdtl V2252)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<lcurly>
    (V2254)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2254))
    (shen/= 123
            (shen/hd
             (shen/hd V2254))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2254))
      (shen/shen\.hdtl V2254)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<rcurly>
    (V2256)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2256))
    (shen/= 125
            (shen/hd
             (shen/hd V2256))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2256))
      (shen/shen\.hdtl V2256)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<bar>
    (V2258)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2258))
    (shen/= 124
            (shen/hd
             (shen/hd V2258))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2258))
      (shen/shen\.hdtl V2258)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<semicolon>
    (V2260)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2260))
    (shen/= 59
            (shen/hd
             (shen/hd V2260))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2260))
      (shen/shen\.hdtl V2260)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<colon>
    (V2262)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2262))
    (shen/= 58
            (shen/hd
             (shen/hd V2262))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2262))
      (shen/shen\.hdtl V2262)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<comma>
    (V2264)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2264))
    (shen/= 44
            (shen/hd
             (shen/hd V2264))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2264))
      (shen/shen\.hdtl V2264)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<equal>
    (V2266)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2266))
    (shen/= 61
            (shen/hd
             (shen/hd V2266))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2266))
      (shen/shen\.hdtl V2266)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<minus>
    (V2268)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2268))
    (shen/= 45
            (shen/hd
             (shen/hd V2268))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2268))
      (shen/shen\.hdtl V2268)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<lrb>
    (V2270)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2270))
    (shen/= 40
            (shen/hd
             (shen/hd V2270))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2270))
      (shen/shen\.hdtl V2270)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<rrb>
    (V2272)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2272))
    (shen/= 41
            (shen/hd
             (shen/hd V2272))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2272))
      (shen/shen\.hdtl V2272)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<atom>
    (V2274)
  (shen/let YaccParse
            (shen/let Parse_shen\.<str>
                      (shen/shen\.<str> V2274)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<str>))
                       (shen/shen\.pair
                        (shen/hd Parse_shen\.<str>)
                        (shen/shen\.control-chars
                         (shen/shen\.hdtl Parse_shen\.<str>)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<number>
                                 (shen/shen\.<number> V2274)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<number>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<number>)
                                   (shen/shen\.hdtl Parse_shen\.<number>))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let Parse_shen\.<sym>
                                  (shen/shen\.<sym> V2274)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<sym>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<sym>)
                                    (shen/if
                                     (shen/=
                                      (shen/shen\.hdtl Parse_shen\.<sym>)
                                      "<>")
                                     (list 'vector 0)
                                     (shen/intern
                                      (shen/shen\.hdtl Parse_shen\.<sym>))))
                                   (shen/fail)))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.control-chars
    (V2276)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2276))
    "")
   ((shen/and
     (shen/cons\? V2276)
     (shen/and
      (shen/= "c"
              (shen/hd V2276))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2276))
       (shen/= "#"
               (shen/hd
                (nthcdr 1 V2276))))))
    (shen/let CodePoint
              (shen/shen\.code-point
               (nthcdr 2 V2276))
              (shen/let AfterCodePoint
                        (shen/shen\.after-codepoint
                         (nthcdr 2 V2276))
                        (concat
                         (concat
                          (shen/n->string
                           (shen/shen\.decimalise CodePoint)))
                         (shen/shen\.control-chars AfterCodePoint)))))
   ((shen/cons\? V2276)
    (concat
     (concat
      (shen/hd V2276))
     (shen/shen\.control-chars
      (nthcdr 1 V2276))))
   (shen/true
    (shen/shen\.f_error 'shen\.control-chars))))
(defun shen/shen\.code-point
    (V2280)
  (shen/cond
   ((shen/and
     (shen/cons\? V2280)
     (shen/= ";"
             (shen/hd V2280)))
    "")
   ((shen/and
     (shen/cons\? V2280)
     (shen/element\?
      (shen/hd V2280)
      (list "0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "0")))
    (append
     (list
      (shen/hd V2280))
     (shen/shen\.code-point
      (nthcdr 1 V2280))))
   (shen/true
    (shen/simple-error
     (shen/cn "code point parse error "
              (shen/shen\.app V2280 "\n" 'shen\.a))))))
(defun shen/shen\.after-codepoint
    (V2286)
  (cl-flet
      ((tail-trampoline
        (V2286)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2286))
          nil)
         ((shen/and
           (shen/cons\? V2286)
           (shen/= ";"
                   (shen/hd V2286)))
          (nthcdr 1 V2286))
         ((shen/cons\? V2286)
          (vector
           (list
            (nthcdr 1 V2286))))
         (shen/true
          (shen/shen\.f_error 'shen\.after-codepoint)))))
    (let
        ((result
          (funcall #'tail-trampoline V2286)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.decimalise
    (V2288)
  (shen/shen\.pre
   (shen/reverse
    (shen/shen\.digits->integers V2288))
   0))
(defun shen/shen\.digits->integers
    (V2294)
  (shen/cond
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "0"
             (shen/hd V2294)))
    (append
     (list 0)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "1"
             (shen/hd V2294)))
    (append
     (list 1)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "2"
             (shen/hd V2294)))
    (append
     (list 2)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "3"
             (shen/hd V2294)))
    (append
     (list 3)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "4"
             (shen/hd V2294)))
    (append
     (list 4)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "5"
             (shen/hd V2294)))
    (append
     (list 5)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "6"
             (shen/hd V2294)))
    (append
     (list 6)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "7"
             (shen/hd V2294)))
    (append
     (list 7)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "8"
             (shen/hd V2294)))
    (append
     (list 8)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   ((shen/and
     (shen/cons\? V2294)
     (shen/= "9"
             (shen/hd V2294)))
    (append
     (list 9)
     (shen/shen\.digits->integers
      (nthcdr 1 V2294))))
   (shen/true nil)))
(defun shen/shen\.<sym>
    (V2296)
  (shen/let Parse_shen\.<alpha>
            (shen/shen\.<alpha> V2296)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<alpha>))
             (shen/let Parse_shen\.<alphanums>
                       (shen/shen\.<alphanums> Parse_shen\.<alpha>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<alphanums>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<alphanums>)
                         (concat
                          (concat
                           (shen/shen\.hdtl Parse_shen\.<alpha>))
                          (shen/shen\.hdtl Parse_shen\.<alphanums>)))
                        (shen/fail)))
             (shen/fail))))
(defun shen/shen\.<alphanums>
    (V2298)
  (shen/let YaccParse
            (shen/let Parse_shen\.<alphanum>
                      (shen/shen\.<alphanum> V2298)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<alphanum>))
                       (shen/let Parse_shen\.<alphanums>
                                 (shen/shen\.<alphanums> Parse_shen\.<alphanum>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<alphanums>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<alphanums>)
                                   (concat
                                    (concat
                                     (shen/shen\.hdtl Parse_shen\.<alphanum>))
                                    (shen/shen\.hdtl Parse_shen\.<alphanums>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2298)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         "")
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<alphanum>
    (V2300)
  (shen/let YaccParse
            (shen/let Parse_shen\.<alpha>
                      (shen/shen\.<alpha> V2300)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<alpha>))
                       (shen/shen\.pair
                        (shen/hd Parse_shen\.<alpha>)
                        (shen/shen\.hdtl Parse_shen\.<alpha>))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<num>
                       (shen/shen\.<num> V2300)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<num>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<num>)
                         (shen/shen\.hdtl Parse_shen\.<num>))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<num>
    (V2302)
  (shen/if
   (shen/cons\?
    (shen/hd V2302))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2302))
             (shen/if
              (shen/shen\.numbyte\? Parse_Char)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2302))
                 (shen/shen\.hdtl V2302)))
               (shen/n->string Parse_Char))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.numbyte\?
    (V2308)
  (shen/cond
   ((shen/= 48 V2308)
    'true)
   ((shen/= 49 V2308)
    'true)
   ((shen/= 50 V2308)
    'true)
   ((shen/= 51 V2308)
    'true)
   ((shen/= 52 V2308)
    'true)
   ((shen/= 53 V2308)
    'true)
   ((shen/= 54 V2308)
    'true)
   ((shen/= 55 V2308)
    'true)
   ((shen/= 56 V2308)
    'true)
   ((shen/= 57 V2308)
    'true)
   (shen/true 'false)))
(defun shen/shen\.<alpha>
    (V2310)
  (shen/if
   (shen/cons\?
    (shen/hd V2310))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2310))
             (shen/if
              (shen/shen\.symbol-code\? Parse_Char)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2310))
                 (shen/shen\.hdtl V2310)))
               (shen/n->string Parse_Char))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.symbol-code\?
    (V2312)
  (shen/or
   (shen/= V2312 126)
   (shen/or
    (shen/and
     (shen/> V2312 94)
     (shen/< V2312 123))
    (shen/or
     (shen/and
      (shen/> V2312 59)
      (shen/< V2312 91))
     (shen/or
      (shen/and
       (shen/> V2312 41)
       (shen/and
        (shen/< V2312 58)
        (shen/not
         (shen/= V2312 44))))
      (shen/or
       (shen/and
        (shen/> V2312 34)
        (shen/< V2312 40))
       (shen/= V2312 33)))))))
(defun shen/shen\.<str>
    (V2314)
  (shen/let Parse_shen\.<dbq>
            (shen/shen\.<dbq> V2314)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<dbq>))
             (shen/let Parse_shen\.<strcontents>
                       (shen/shen\.<strcontents> Parse_shen\.<dbq>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<strcontents>))
                        (shen/let Parse_shen\.<dbq>
                                  (shen/shen\.<dbq> Parse_shen\.<strcontents>)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<dbq>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<dbq>)
                                    (shen/shen\.hdtl Parse_shen\.<strcontents>))
                                   (shen/fail)))
                        (shen/fail)))
             (shen/fail))))
(defun shen/shen\.<dbq>
    (V2316)
  (shen/if
   (shen/cons\?
    (shen/hd V2316))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2316))
             (shen/if
              (shen/= Parse_Char 34)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2316))
                 (shen/shen\.hdtl V2316)))
               Parse_Char)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<strcontents>
    (V2318)
  (shen/let YaccParse
            (shen/let Parse_shen\.<strc>
                      (shen/shen\.<strc> V2318)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<strc>))
                       (shen/let Parse_shen\.<strcontents>
                                 (shen/shen\.<strcontents> Parse_shen\.<strc>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<strcontents>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<strcontents>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<strc>))
                                    (shen/shen\.hdtl Parse_shen\.<strcontents>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2318)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<byte>
    (V2320)
  (shen/if
   (shen/cons\?
    (shen/hd V2320))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2320))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V2320))
                (shen/shen\.hdtl V2320)))
              (shen/n->string Parse_Char)))
   (shen/fail)))
(defun shen/shen\.<strc>
    (V2322)
  (shen/if
   (shen/cons\?
    (shen/hd V2322))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2322))
             (shen/if
              (shen/not
               (shen/= Parse_Char 34))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2322))
                 (shen/shen\.hdtl V2322)))
               (shen/n->string Parse_Char))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<number>
    (V2324)
  (shen/let YaccParse
            (shen/let Parse_shen\.<minus>
                      (shen/shen\.<minus> V2324)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<minus>))
                       (shen/let Parse_shen\.<number>
                                 (shen/shen\.<number> Parse_shen\.<minus>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<number>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<number>)
                                   (shen/- 0
                                           (shen/shen\.hdtl Parse_shen\.<number>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<plus>
                                 (shen/shen\.<plus> V2324)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<plus>))
                                  (shen/let Parse_shen\.<number>
                                            (shen/shen\.<number> Parse_shen\.<plus>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<number>))
                                             (shen/shen\.pair
                                              (shen/hd Parse_shen\.<number>)
                                              (shen/shen\.hdtl Parse_shen\.<number>))
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/let YaccParse
                                  (shen/let Parse_shen\.<predigits>
                                            (shen/shen\.<predigits> V2324)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<predigits>))
                                             (shen/let Parse_shen\.<stop>
                                                       (shen/shen\.<stop> Parse_shen\.<predigits>)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<stop>))
                                                        (shen/let Parse_shen\.<postdigits>
                                                                  (shen/shen\.<postdigits> Parse_shen\.<stop>)
                                                                  (shen/if
                                                                   (shen/not
                                                                    (shen/=
                                                                     (shen/fail)
                                                                     Parse_shen\.<postdigits>))
                                                                   (shen/let Parse_shen\.<E>
                                                                             (shen/shen\.<E> Parse_shen\.<postdigits>)
                                                                             (shen/if
                                                                              (shen/not
                                                                               (shen/=
                                                                                (shen/fail)
                                                                                Parse_shen\.<E>))
                                                                              (shen/let Parse_shen\.<log10>
                                                                                        (shen/shen\.<log10> Parse_shen\.<E>)
                                                                                        (shen/if
                                                                                         (shen/not
                                                                                          (shen/=
                                                                                           (shen/fail)
                                                                                           Parse_shen\.<log10>))
                                                                                         (shen/shen\.pair
                                                                                          (shen/hd Parse_shen\.<log10>)
                                                                                          (shen/*
                                                                                           (shen/shen\.expt 10
                                                                                                            (shen/shen\.hdtl Parse_shen\.<log10>))
                                                                                           (shen/+
                                                                                            (shen/shen\.pre
                                                                                             (shen/reverse
                                                                                              (shen/shen\.hdtl Parse_shen\.<predigits>))
                                                                                             0)
                                                                                            (shen/shen\.post
                                                                                             (shen/shen\.hdtl Parse_shen\.<postdigits>)
                                                                                             1))))
                                                                                         (shen/fail)))
                                                                              (shen/fail)))
                                                                   (shen/fail)))
                                                        (shen/fail)))
                                             (shen/fail)))
                                  (shen/if
                                   (shen/= YaccParse
                                           (shen/fail))
                                   (shen/let YaccParse
                                             (shen/let Parse_shen\.<digits>
                                                       (shen/shen\.<digits> V2324)
                                                       (shen/if
                                                        (shen/not
                                                         (shen/=
                                                          (shen/fail)
                                                          Parse_shen\.<digits>))
                                                        (shen/let Parse_shen\.<E>
                                                                  (shen/shen\.<E> Parse_shen\.<digits>)
                                                                  (shen/if
                                                                   (shen/not
                                                                    (shen/=
                                                                     (shen/fail)
                                                                     Parse_shen\.<E>))
                                                                   (shen/let Parse_shen\.<log10>
                                                                             (shen/shen\.<log10> Parse_shen\.<E>)
                                                                             (shen/if
                                                                              (shen/not
                                                                               (shen/=
                                                                                (shen/fail)
                                                                                Parse_shen\.<log10>))
                                                                              (shen/shen\.pair
                                                                               (shen/hd Parse_shen\.<log10>)
                                                                               (shen/*
                                                                                (shen/shen\.expt 10
                                                                                                 (shen/shen\.hdtl Parse_shen\.<log10>))
                                                                                (shen/shen\.pre
                                                                                 (shen/reverse
                                                                                  (shen/shen\.hdtl Parse_shen\.<digits>))
                                                                                 0)))
                                                                              (shen/fail)))
                                                                   (shen/fail)))
                                                        (shen/fail)))
                                             (shen/if
                                              (shen/= YaccParse
                                                      (shen/fail))
                                              (shen/let YaccParse
                                                        (shen/let Parse_shen\.<predigits>
                                                                  (shen/shen\.<predigits> V2324)
                                                                  (shen/if
                                                                   (shen/not
                                                                    (shen/=
                                                                     (shen/fail)
                                                                     Parse_shen\.<predigits>))
                                                                   (shen/let Parse_shen\.<stop>
                                                                             (shen/shen\.<stop> Parse_shen\.<predigits>)
                                                                             (shen/if
                                                                              (shen/not
                                                                               (shen/=
                                                                                (shen/fail)
                                                                                Parse_shen\.<stop>))
                                                                              (shen/let Parse_shen\.<postdigits>
                                                                                        (shen/shen\.<postdigits> Parse_shen\.<stop>)
                                                                                        (shen/if
                                                                                         (shen/not
                                                                                          (shen/=
                                                                                           (shen/fail)
                                                                                           Parse_shen\.<postdigits>))
                                                                                         (shen/shen\.pair
                                                                                          (shen/hd Parse_shen\.<postdigits>)
                                                                                          (shen/+
                                                                                           (shen/shen\.pre
                                                                                            (shen/reverse
                                                                                             (shen/shen\.hdtl Parse_shen\.<predigits>))
                                                                                            0)
                                                                                           (shen/shen\.post
                                                                                            (shen/shen\.hdtl Parse_shen\.<postdigits>)
                                                                                            1)))
                                                                                         (shen/fail)))
                                                                              (shen/fail)))
                                                                   (shen/fail)))
                                                        (shen/if
                                                         (shen/= YaccParse
                                                                 (shen/fail))
                                                         (shen/let Parse_shen\.<digits>
                                                                   (shen/shen\.<digits> V2324)
                                                                   (shen/if
                                                                    (shen/not
                                                                     (shen/=
                                                                      (shen/fail)
                                                                      Parse_shen\.<digits>))
                                                                    (shen/shen\.pair
                                                                     (shen/hd Parse_shen\.<digits>)
                                                                     (shen/shen\.pre
                                                                      (shen/reverse
                                                                       (shen/shen\.hdtl Parse_shen\.<digits>))
                                                                      0))
                                                                    (shen/fail)))
                                                         YaccParse))
                                              YaccParse))
                                   YaccParse))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.<E>
    (V2326)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2326))
    (shen/= 101
            (shen/hd
             (shen/hd V2326))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2326))
      (shen/shen\.hdtl V2326)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<log10>
    (V2328)
  (shen/let YaccParse
            (shen/let Parse_shen\.<minus>
                      (shen/shen\.<minus> V2328)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<minus>))
                       (shen/let Parse_shen\.<digits>
                                 (shen/shen\.<digits> Parse_shen\.<minus>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<digits>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<digits>)
                                   (shen/- 0
                                           (shen/shen\.pre
                                            (shen/reverse
                                             (shen/shen\.hdtl Parse_shen\.<digits>))
                                            0)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<digits>
                       (shen/shen\.<digits> V2328)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<digits>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<digits>)
                         (shen/shen\.pre
                          (shen/reverse
                           (shen/shen\.hdtl Parse_shen\.<digits>))
                          0))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<plus>
    (V2330)
  (shen/if
   (shen/cons\?
    (shen/hd V2330))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2330))
             (shen/if
              (shen/= Parse_Char 43)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2330))
                 (shen/shen\.hdtl V2330)))
               Parse_Char)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<stop>
    (V2332)
  (shen/if
   (shen/cons\?
    (shen/hd V2332))
   (shen/let Parse_Char
             (shen/hd
              (shen/hd V2332))
             (shen/if
              (shen/= Parse_Char 46)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2332))
                 (shen/shen\.hdtl V2332)))
               Parse_Char)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<predigits>
    (V2334)
  (shen/let YaccParse
            (shen/let Parse_shen\.<digits>
                      (shen/shen\.<digits> V2334)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<digits>))
                       (shen/shen\.pair
                        (shen/hd Parse_shen\.<digits>)
                        (shen/shen\.hdtl Parse_shen\.<digits>))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2334)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<postdigits>
    (V2336)
  (shen/let Parse_shen\.<digits>
            (shen/shen\.<digits> V2336)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<digits>))
             (shen/shen\.pair
              (shen/hd Parse_shen\.<digits>)
              (shen/shen\.hdtl Parse_shen\.<digits>))
             (shen/fail))))
(defun shen/shen\.<digits>
    (V2338)
  (shen/let YaccParse
            (shen/let Parse_shen\.<digit>
                      (shen/shen\.<digit> V2338)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<digit>))
                       (shen/let Parse_shen\.<digits>
                                 (shen/shen\.<digits> Parse_shen\.<digit>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<digits>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<digits>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<digit>))
                                    (shen/shen\.hdtl Parse_shen\.<digits>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<digit>
                       (shen/shen\.<digit> V2338)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<digit>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<digit>)
                         (list
                          (shen/shen\.hdtl Parse_shen\.<digit>)))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<digit>
    (V2340)
  (shen/if
   (shen/cons\?
    (shen/hd V2340))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2340))
             (shen/if
              (shen/shen\.numbyte\? Parse_X)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2340))
                 (shen/shen\.hdtl V2340)))
               (shen/shen\.byte->digit Parse_X))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.byte->digit
    (V2342)
  (shen/cond
   ((shen/= 48 V2342)
    0)
   ((shen/= 49 V2342)
    1)
   ((shen/= 50 V2342)
    2)
   ((shen/= 51 V2342)
    3)
   ((shen/= 52 V2342)
    4)
   ((shen/= 53 V2342)
    5)
   ((shen/= 54 V2342)
    6)
   ((shen/= 55 V2342)
    7)
   ((shen/= 56 V2342)
    8)
   ((shen/= 57 V2342)
    9)
   (shen/true
    (shen/shen\.f_error 'shen\.byte->digit))))
(defun shen/shen\.pre
    (V2347 V2348)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2347))
    0)
   ((shen/cons\? V2347)
    (shen/+
     (shen/*
      (shen/shen\.expt 10 V2348)
      (shen/hd V2347))
     (shen/shen\.pre
      (nthcdr 1 V2347)
      (1+ V2348))))
   (shen/true
    (shen/shen\.f_error 'shen\.pre))))
(defun shen/shen\.post
    (V2353 V2354)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2353))
    0)
   ((shen/cons\? V2353)
    (shen/+
     (shen/*
      (shen/shen\.expt 10
                       (shen/- 0 V2354))
      (shen/hd V2353))
     (shen/shen\.post
      (nthcdr 1 V2353)
      (1+ V2354))))
   (shen/true
    (shen/shen\.f_error 'shen\.post))))
(defun shen/shen\.expt
    (V2359 V2360)
  (shen/cond
   ((shen/= 0 V2360)
    1)
   ((shen/> V2360 0)
    (shen/* V2359
            (shen/shen\.expt V2359
                             (shen/- V2360 1))))
   (shen/true
    (shen/* 1.0
            (shen//
             (shen/shen\.expt V2359
                              (1+ V2360))
             V2359)))))
(defun shen/shen\.<st_input1>
    (V2362)
  (shen/let Parse_shen\.<st_input>
            (shen/shen\.<st_input> V2362)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<st_input>))
             (shen/shen\.pair
              (shen/hd Parse_shen\.<st_input>)
              (shen/shen\.hdtl Parse_shen\.<st_input>))
             (shen/fail))))
(defun shen/shen\.<st_input2>
    (V2364)
  (shen/let Parse_shen\.<st_input>
            (shen/shen\.<st_input> V2364)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<st_input>))
             (shen/shen\.pair
              (shen/hd Parse_shen\.<st_input>)
              (shen/shen\.hdtl Parse_shen\.<st_input>))
             (shen/fail))))
(defun shen/shen\.<comment>
    (V2366)
  (shen/let YaccParse
            (shen/let Parse_shen\.<singleline>
                      (shen/shen\.<singleline> V2366)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<singleline>))
                       (shen/shen\.pair
                        (shen/hd Parse_shen\.<singleline>)
                        'shen\.skip)
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<multiline>
                       (shen/shen\.<multiline> V2366)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<multiline>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<multiline>)
                         'shen\.skip)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<singleline>
    (V2368)
  (shen/let Parse_shen\.<backslash>
            (shen/shen\.<backslash> V2368)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<backslash>))
             (shen/let Parse_shen\.<backslash>
                       (shen/shen\.<backslash> Parse_shen\.<backslash>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<backslash>))
                        (shen/let Parse_shen\.<anysingle>
                                  (shen/shen\.<anysingle> Parse_shen\.<backslash>)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<anysingle>))
                                   (shen/let Parse_shen\.<return>
                                             (shen/shen\.<return> Parse_shen\.<anysingle>)
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<return>))
                                              (shen/shen\.pair
                                               (shen/hd Parse_shen\.<return>)
                                               'shen\.skip)
                                              (shen/fail)))
                                   (shen/fail)))
                        (shen/fail)))
             (shen/fail))))
(defun shen/shen\.<backslash>
    (V2370)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2370))
    (shen/= 92
            (shen/hd
             (shen/hd V2370))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2370))
      (shen/shen\.hdtl V2370)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<anysingle>
    (V2372)
  (shen/let YaccParse
            (shen/let Parse_shen\.<non-return>
                      (shen/shen\.<non-return> V2372)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<non-return>))
                       (shen/let Parse_shen\.<anysingle>
                                 (shen/shen\.<anysingle> Parse_shen\.<non-return>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<anysingle>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<anysingle>)
                                   'shen\.skip)
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V2372)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         'shen\.skip)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<non-return>
    (V2374)
  (shen/if
   (shen/cons\?
    (shen/hd V2374))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2374))
             (shen/if
              (shen/not
               (shen/element\? Parse_X
                               (list 10 13)))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2374))
                 (shen/shen\.hdtl V2374)))
               'shen\.skip)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<return>
    (V2376)
  (shen/if
   (shen/cons\?
    (shen/hd V2376))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2376))
             (shen/if
              (shen/element\? Parse_X
                              (list 10 13))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2376))
                 (shen/shen\.hdtl V2376)))
               'shen\.skip)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.<multiline>
    (V2378)
  (shen/let Parse_shen\.<backslash>
            (shen/shen\.<backslash> V2378)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<backslash>))
             (shen/let Parse_shen\.<times>
                       (shen/shen\.<times> Parse_shen\.<backslash>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<times>))
                        (shen/let Parse_shen\.<anymulti>
                                  (shen/shen\.<anymulti> Parse_shen\.<times>)
                                  (shen/if
                                   (shen/not
                                    (shen/=
                                     (shen/fail)
                                     Parse_shen\.<anymulti>))
                                   (shen/shen\.pair
                                    (shen/hd Parse_shen\.<anymulti>)
                                    'shen\.skip)
                                   (shen/fail)))
                        (shen/fail)))
             (shen/fail))))
(defun shen/shen\.<times>
    (V2380)
  (shen/if
   (shen/and
    (shen/cons\?
     (shen/hd V2380))
    (shen/= 42
            (shen/hd
             (shen/hd V2380))))
   (shen/shen\.pair
    (shen/hd
     (shen/shen\.pair
      (nthcdr 1
              (shen/hd V2380))
      (shen/shen\.hdtl V2380)))
    'shen\.skip)
   (shen/fail)))
(defun shen/shen\.<anymulti>
    (V2382)
  (shen/let YaccParse
            (shen/let Parse_shen\.<comment>
                      (shen/shen\.<comment> V2382)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<comment>))
                       (shen/let Parse_shen\.<anymulti>
                                 (shen/shen\.<anymulti> Parse_shen\.<comment>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<anymulti>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<anymulti>)
                                   'shen\.skip)
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let YaccParse
                       (shen/let Parse_shen\.<times>
                                 (shen/shen\.<times> V2382)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<times>))
                                  (shen/let Parse_shen\.<backslash>
                                            (shen/shen\.<backslash> Parse_shen\.<times>)
                                            (shen/if
                                             (shen/not
                                              (shen/=
                                               (shen/fail)
                                               Parse_shen\.<backslash>))
                                             (shen/shen\.pair
                                              (shen/hd Parse_shen\.<backslash>)
                                              'shen\.skip)
                                             (shen/fail)))
                                  (shen/fail)))
                       (shen/if
                        (shen/= YaccParse
                                (shen/fail))
                        (shen/if
                         (shen/cons\?
                          (shen/hd V2382))
                         (shen/let Parse_X
                                   (shen/hd
                                    (shen/hd V2382))
                                   (shen/let Parse_shen\.<anymulti>
                                             (shen/shen\.<anymulti>
                                              (shen/shen\.pair
                                               (nthcdr 1
                                                       (shen/hd V2382))
                                               (shen/shen\.hdtl V2382)))
                                             (shen/if
                                              (shen/not
                                               (shen/=
                                                (shen/fail)
                                                Parse_shen\.<anymulti>))
                                              (shen/shen\.pair
                                               (shen/hd Parse_shen\.<anymulti>)
                                               'shen\.skip)
                                              (shen/fail))))
                         (shen/fail))
                        YaccParse))
             YaccParse)))
(defun shen/shen\.<whitespaces>
    (V2384)
  (shen/let YaccParse
            (shen/let Parse_shen\.<whitespace>
                      (shen/shen\.<whitespace> V2384)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<whitespace>))
                       (shen/let Parse_shen\.<whitespaces>
                                 (shen/shen\.<whitespaces> Parse_shen\.<whitespace>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<whitespaces>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<whitespaces>)
                                   'shen\.skip)
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<whitespace>
                       (shen/shen\.<whitespace> V2384)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<whitespace>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<whitespace>)
                         'shen\.skip)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<whitespace>
    (V2386)
  (shen/if
   (shen/cons\?
    (shen/hd V2386))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V2386))
             (shen/if
              (shen/let Parse_Case Parse_X
                        (shen/or
                         (shen/= Parse_Case 32)
                         (shen/or
                          (shen/= Parse_Case 13)
                          (shen/or
                           (shen/= Parse_Case 10)
                           (shen/= Parse_Case 9)))))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V2386))
                 (shen/shen\.hdtl V2386)))
               'shen\.skip)
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.cons_form
    (V2388)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V2388))
    nil)
   ((shen/and
     (shen/cons\? V2388)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2388))
      (shen/and
       (shen/cons\?
        (nthcdr 2 V2388))
       (shen/and
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V2388)))
        (shen/=
         (shen/hd
          (nthcdr 1 V2388))
         'bar!)))))
    (append
     (list 'cons
           (shen/hd V2388))
     (nthcdr 2 V2388)))
   ((shen/cons\? V2388)
    (list 'cons
          (shen/hd V2388)
          (shen/shen\.cons_form
           (nthcdr 1 V2388))))
   (shen/true
    (shen/shen\.f_error 'shen\.cons_form))))
(defun shen/shen\.package-macro
    (V2393 V2394)
  (shen/cond
   ((shen/and
     (shen/cons\? V2393)
     (shen/and
      (shen/= '$
              (shen/hd V2393))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2393))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V2393))))))
    (shen/append
     (shen/explode
      (shen/hd
       (nthcdr 1 V2393)))
     V2394))
   ((shen/and
     (shen/cons\? V2393)
     (shen/and
      (shen/= 'package
              (shen/hd V2393))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2393))
       (shen/and
        (shen/= 'null
                (shen/hd
                 (nthcdr 1 V2393)))
        (shen/cons\?
         (nthcdr 2 V2393))))))
    (shen/append
     (nthcdr 3 V2393)
     V2394))
   ((shen/and
     (shen/cons\? V2393)
     (shen/and
      (shen/= 'package
              (shen/hd V2393))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V2393))
       (shen/cons\?
        (nthcdr 2 V2393)))))
    (shen/let ListofExceptions
              (shen/shen\.eval-without-macros
               (shen/hd
                (nthcdr 2 V2393)))
              (shen/let External
                        (shen/shen\.record-exceptions ListofExceptions
                                                      (shen/hd
                                                       (nthcdr 1 V2393)))
                        (shen/let PackageNameDot
                                  (shen/intern
                                   (shen/cn
                                    (shen/str
                                     (shen/hd
                                      (nthcdr 1 V2393)))
                                    "."))
                                  (shen/let ExpPackageNameDot
                                            (shen/explode PackageNameDot)
                                            (shen/let Packaged
                                                      (shen/shen\.packageh PackageNameDot ListofExceptions
                                                                           (nthcdr 3 V2393)
                                                                           ExpPackageNameDot)
                                                      (shen/let Internal
                                                                (shen/shen\.record-internal
                                                                 (shen/hd
                                                                  (nthcdr 1 V2393))
                                                                 (shen/shen\.internal-symbols ExpPackageNameDot Packaged))
                                                                (shen/append Packaged V2394))))))))
   (shen/true
    (append
     (list V2393)
     V2394))))
(defun shen/shen\.record-exceptions
    (V2397 V2398)
  (shen/let CurrExceptions
            (shen/get/or V2398 'shen\.external-symbols
                         (shen/freeze nil)
                         (shen/value '*property-vector*))
            (shen/let AllExceptions
                      (shen/union V2397 CurrExceptions)
                      (shen/put V2398 'shen\.external-symbols AllExceptions
                                (shen/value '*property-vector*)))))
(defun shen/shen\.record-internal
    (V2401 V2402)
  (shen/put V2401 'shen\.internal-symbols
            (shen/union V2402
                        (shen/get/or V2401 'shen\.internal-symbols
                                     (shen/freeze nil)
                                     (shen/value '*property-vector*)))
            (shen/value '*property-vector*)))
(defun shen/shen\.internal-symbols
    (V2413 V2414)
  (shen/cond
   ((shen/and
     (shen/symbol\? V2414)
     (shen/shen\.prefix\? V2413
                          (shen/explode V2414)))
    (list V2414))
   ((shen/cons\? V2414)
    (shen/union
     (shen/shen\.internal-symbols V2413
                                  (shen/hd V2414))
     (shen/shen\.internal-symbols V2413
                                  (nthcdr 1 V2414))))
   (shen/true nil)))
(defun shen/shen\.packageh
    (V2431 V2432 V2433 V2434)
  (shen/cond
   ((shen/cons\? V2433)
    (append
     (list
      (shen/shen\.packageh V2431 V2432
                           (shen/hd V2433)
                           V2434))
     (shen/shen\.packageh V2431 V2432
                          (nthcdr 1 V2433)
                          V2434)))
   ((shen/or
     (shen/shen\.sysfunc\? V2433)
     (shen/or
      (shen/variable\? V2433)
      (shen/or
       (shen/element\? V2433 V2432)
       (shen/or
        (shen/shen\.doubleunderline\? V2433)
        (shen/shen\.singleunderline\? V2433)))))
    V2433)
   ((shen/and
     (shen/symbol\? V2433)
     (shen/let ExplodeX
               (shen/explode V2433)
               (shen/and
                (shen/not
                 (shen/shen\.prefix\?
                  (list "s" "h" "e" "n" ".")
                  ExplodeX))
                (shen/not
                 (shen/shen\.prefix\? V2434 ExplodeX)))))
    (shen/concat V2431 V2433))
   (shen/true V2433)))
(defun shen/shen\.<defprolog>
    (V1618)
  (shen/let Parse_shen\.<predicate*>
            (shen/shen\.<predicate*> V1618)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<predicate*>))
             (shen/let Parse_shen\.<clauses*>
                       (shen/shen\.<clauses*> Parse_shen\.<predicate*>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<clauses*>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<clauses*>)
                         (shen/hd
                          (shen/shen\.prolog->shen
                           (shen/map
                            (shen/lambda Parse_X
                                         (shen/shen\.insert-predicate
                                          (shen/shen\.hdtl Parse_shen\.<predicate*>)
                                          Parse_X))
                            (shen/shen\.hdtl Parse_shen\.<clauses*>)))))
                        (shen/fail)))
             (shen/fail))))
(defun shen/shen\.prolog-error
    (V1627 V1628)
  (shen/cond
   ((shen/and
     (shen/cons\? V1628)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1628))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1628)))))
    (shen/simple-error
     (shen/cn "prolog syntax error in "
              (shen/shen\.app V1627
                              (shen/cn " here:\n\n "
                                       (shen/shen\.app
                                        (shen/shen\.next-50 50
                                                            (shen/hd V1628))
                                        "\n" 'shen\.a))
                              'shen\.a))))
   (shen/true
    (shen/simple-error
     (shen/cn "prolog syntax error in "
              (shen/shen\.app V1627 "\n" 'shen\.a))))))
(defun shen/shen\.next-50
    (V1635 V1636)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1636))
    "")
   ((shen/= 0 V1635)
    "")
   ((shen/cons\? V1636)
    (shen/cn
     (shen/shen\.decons-string
      (shen/hd V1636))
     (shen/shen\.next-50
      (shen/- V1635 1)
      (nthcdr 1 V1636))))
   (shen/true
    (shen/shen\.f_error 'shen\.next-50))))
(defun shen/shen\.decons-string
    (V1638)
  (shen/cond
   ((shen/and
     (shen/cons\? V1638)
     (shen/and
      (shen/= 'cons
              (shen/hd V1638))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1638))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1638))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1638)))))))
    (shen/shen\.app
     (shen/shen\.eval-cons V1638)
     " " 'shen\.s))
   (shen/true
    (shen/shen\.app V1638 " " 'shen\.r))))
(defun shen/shen\.insert-predicate
    (V1641 V1642)
  (shen/cond
   ((shen/and
     (shen/cons\? V1642)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1642))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1642)))))
    (append
     (list
      (append
       (list V1641)
       (shen/hd V1642))
      ':-)
     (nthcdr 1 V1642)))
   (shen/true
    (shen/shen\.f_error 'shen\.insert-predicate))))
(defun shen/shen\.<predicate*>
    (V1644)
  (shen/if
   (shen/cons\?
    (shen/hd V1644))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V1644))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V1644))
                (shen/shen\.hdtl V1644)))
              Parse_X))
   (shen/fail)))
(defun shen/shen\.<clauses*>
    (V1646)
  (shen/let YaccParse
            (shen/let Parse_shen\.<clause*>
                      (shen/shen\.<clause*> V1646)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<clause*>))
                       (shen/let Parse_shen\.<clauses*>
                                 (shen/shen\.<clauses*> Parse_shen\.<clause*>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<clauses*>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<clauses*>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<clause*>))
                                    (shen/shen\.hdtl Parse_shen\.<clauses*>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V1646)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<clause*>
    (V1648)
  (shen/let Parse_shen\.<head*>
            (shen/shen\.<head*> V1648)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<head*>))
             (shen/if
              (shen/and
               (shen/cons\?
                (shen/hd Parse_shen\.<head*>))
               (shen/= '<--
                       (shen/hd
                        (shen/hd Parse_shen\.<head*>))))
              (shen/let Parse_shen\.<body*>
                        (shen/shen\.<body*>
                         (shen/shen\.pair
                          (nthcdr 1
                                  (shen/hd Parse_shen\.<head*>))
                          (shen/shen\.hdtl Parse_shen\.<head*>)))
                        (shen/if
                         (shen/not
                          (shen/=
                           (shen/fail)
                           Parse_shen\.<body*>))
                         (shen/let Parse_shen\.<end*>
                                   (shen/shen\.<end*> Parse_shen\.<body*>)
                                   (shen/if
                                    (shen/not
                                     (shen/=
                                      (shen/fail)
                                      Parse_shen\.<end*>))
                                    (shen/shen\.pair
                                     (shen/hd Parse_shen\.<end*>)
                                     (list
                                      (shen/shen\.hdtl Parse_shen\.<head*>)
                                      (shen/shen\.hdtl Parse_shen\.<body*>)))
                                    (shen/fail)))
                         (shen/fail)))
              (shen/fail))
             (shen/fail))))
(defun shen/shen\.<head*>
    (V1650)
  (shen/let YaccParse
            (shen/let Parse_shen\.<term*>
                      (shen/shen\.<term*> V1650)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<term*>))
                       (shen/let Parse_shen\.<head*>
                                 (shen/shen\.<head*> Parse_shen\.<term*>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<head*>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<head*>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<term*>))
                                    (shen/shen\.hdtl Parse_shen\.<head*>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V1650)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<term*>
    (V1652)
  (shen/if
   (shen/cons\?
    (shen/hd V1652))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V1652))
             (shen/if
              (shen/and
               (shen/not
                (shen/= '<-- Parse_X))
               (shen/shen\.legitimate-term\? Parse_X))
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V1652))
                 (shen/shen\.hdtl V1652)))
               (shen/shen\.eval-cons Parse_X))
              (shen/fail)))
   (shen/fail)))
(defun shen/shen\.legitimate-term\?
    (V1658)
  (cl-flet
      ((tail-trampoline
        (V1658)
        (shen/cond
         ((shen/and
           (shen/cons\? V1658)
           (shen/and
            (shen/= 'cons
                    (shen/hd V1658))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1658))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1658))
              (shen/internal/predicate->shen
               (null
                (nthcdr 3 V1658)))))))
          (shen/and
           (shen/shen\.legitimate-term\?
            (shen/hd
             (nthcdr 1 V1658)))
           (shen/shen\.legitimate-term\?
            (shen/hd
             (nthcdr 2 V1658)))))
         ((shen/and
           (shen/cons\? V1658)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1658))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1658))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1658))
              (shen/and
               (shen/= '+
                       (shen/hd
                        (nthcdr 2 V1658)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1658))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 1 V1658)))))
         ((shen/and
           (shen/cons\? V1658)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1658))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1658))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1658))
              (shen/and
               (shen/= '-
                       (shen/hd
                        (nthcdr 2 V1658)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1658))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 1 V1658)))))
         ((shen/cons\? V1658)
          'false)
         (shen/true 'true))))
    (let
        ((result
          (funcall #'tail-trampoline V1658)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.eval-cons
    (V1660)
  (shen/cond
   ((shen/and
     (shen/cons\? V1660)
     (shen/and
      (shen/= 'cons
              (shen/hd V1660))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1660))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1660))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1660)))))))
    (append
     (list
      (shen/shen\.eval-cons
       (shen/hd
        (nthcdr 1 V1660))))
     (shen/shen\.eval-cons
      (shen/hd
       (nthcdr 2 V1660)))))
   ((shen/and
     (shen/cons\? V1660)
     (shen/and
      (shen/= 'mode
              (shen/hd V1660))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1660))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1660))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1660)))))))
    (append
     (list 'mode
           (shen/shen\.eval-cons
            (shen/hd
             (nthcdr 1 V1660))))
     (nthcdr 2 V1660)))
   (shen/true V1660)))
(defun shen/shen\.<body*>
    (V1662)
  (shen/let YaccParse
            (shen/let Parse_shen\.<literal*>
                      (shen/shen\.<literal*> V1662)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<literal*>))
                       (shen/let Parse_shen\.<body*>
                                 (shen/shen\.<body*> Parse_shen\.<literal*>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<body*>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<body*>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<literal*>))
                                    (shen/shen\.hdtl Parse_shen\.<body*>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_<e>
                       (shen/<e> V1662)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<e>))
                        (shen/shen\.pair
                         (shen/hd Parse_<e>)
                         nil)
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.<literal*>
    (V1664)
  (shen/let YaccParse
            (shen/if
             (shen/and
              (shen/cons\?
               (shen/hd V1664))
              (shen/= '!
                      (shen/hd
                       (shen/hd V1664))))
             (shen/shen\.pair
              (shen/hd
               (shen/shen\.pair
                (nthcdr 1
                        (shen/hd V1664))
                (shen/shen\.hdtl V1664)))
              (list 'cut
                    (shen/intern "Throwcontrol")))
             (shen/fail))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/if
              (shen/cons\?
               (shen/hd V1664))
              (shen/let Parse_X
                        (shen/hd
                         (shen/hd V1664))
                        (shen/if
                         (shen/cons\? Parse_X)
                         (shen/shen\.pair
                          (shen/hd
                           (shen/shen\.pair
                            (nthcdr 1
                                    (shen/hd V1664))
                            (shen/shen\.hdtl V1664)))
                          Parse_X)
                         (shen/fail)))
              (shen/fail))
             YaccParse)))
(defun shen/shen\.<end*>
    (V1666)
  (shen/if
   (shen/cons\?
    (shen/hd V1666))
   (shen/let Parse_X
             (shen/hd
              (shen/hd V1666))
             (shen/if
              (shen/= Parse_X '\;)
              (shen/shen\.pair
               (shen/hd
                (shen/shen\.pair
                 (nthcdr 1
                         (shen/hd V1666))
                 (shen/shen\.hdtl V1666)))
               Parse_X)
              (shen/fail)))
   (shen/fail)))
(defun shen/cut
    (V1670 V1671 V1672)
  (shen/let Result
            (shen/thaw V1672)
            (shen/if
             (shen/= Result 'false)
             V1670 Result)))
(defun shen/shen\.insert_modes
    (V1674)
  (shen/cond
   ((shen/and
     (shen/cons\? V1674)
     (shen/and
      (shen/= 'mode
              (shen/hd V1674))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1674))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1674))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1674)))))))
    V1674)
   ((shen/internal/predicate->shen
     (null V1674))
    nil)
   ((shen/cons\? V1674)
    (list
     (list 'mode
           (shen/hd V1674)
           '+)
     'mode
     (shen/shen\.insert_modes
      (nthcdr 1 V1674))
     '-))
   (shen/true V1674)))
(defun shen/shen\.s-prolog
    (V1676)
  (shen/map
   (shen/lambda X
                (shen/eval X))
   (shen/shen\.prolog->shen V1676)))
(defun shen/shen\.prolog->shen
    (V1678)
  (shen/map
   (shen/lambda X
                (shen/shen\.compile_prolog_procedure X))
   (shen/shen\.group_clauses
    (shen/map
     (shen/lambda X
                  (shen/shen\.s-prolog_clause X))
     (shen/mapcan
      (shen/lambda X
                   (shen/shen\.head_abstraction X))
      V1678)))))
(defun shen/shen\.s-prolog_clause
    (V1680)
  (shen/cond
   ((shen/and
     (shen/cons\? V1680)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1680))
      (shen/and
       (shen/= ':-
               (shen/hd
                (nthcdr 1 V1680)))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1680))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1680)))))))
    (list
     (shen/hd V1680)
     ':-
     (shen/map
      (shen/lambda X
                   (shen/shen\.s-prolog_literal X))
      (shen/hd
       (nthcdr 2 V1680)))))
   (shen/true
    (shen/shen\.f_error 'shen\.s-prolog_clause))))
(defun shen/shen\.head_abstraction
    (V1682)
  (shen/cond
   ((shen/and
     (shen/cons\? V1682)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1682))
      (shen/and
       (shen/= ':-
               (shen/hd
                (nthcdr 1 V1682)))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1682))
        (shen/and
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V1682)))
         (shen/<
          (shen/shen\.complexity_head
           (shen/hd V1682))
          (shen/value 'shen\.*maxcomplexity*)))))))
    (list V1682))
   ((shen/and
     (shen/cons\? V1682)
     (shen/and
      (shen/cons\?
       (shen/hd V1682))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1682))
       (shen/and
        (shen/= ':-
                (shen/hd
                 (nthcdr 1 V1682)))
        (shen/and
         (shen/cons\?
          (nthcdr 2 V1682))
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V1682))))))))
    (shen/let Terms
              (shen/map
               (shen/lambda Y
                            (shen/gensym 'V))
               (nthcdr 1
                       (shen/hd V1682)))
              (shen/let XTerms
                        (shen/shen\.rcons_form
                         (shen/shen\.remove_modes
                          (nthcdr 1
                                  (shen/hd V1682))))
                        (shen/let Literal
                                  (list 'unify
                                        (shen/shen\.cons_form Terms)
                                        XTerms)
                                  (shen/let Clause
                                            (list
                                             (append
                                              (list
                                               (shen/hd
                                                (shen/hd V1682)))
                                              Terms)
                                             ':-
                                             (append
                                              (list Literal)
                                              (shen/hd
                                               (nthcdr 2 V1682))))
                                            (list Clause))))))
   (shen/true
    (shen/shen\.f_error 'shen\.head_abstraction))))
(defun shen/shen\.complexity_head
    (V1688)
  (shen/cond
   ((shen/cons\? V1688)
    (shen/shen\.product
     (shen/map
      (shen/lambda X
                   (shen/shen\.complexity X))
      (nthcdr 1 V1688))))
   (shen/true
    (shen/shen\.f_error 'shen\.complexity_head))))
(defun shen/shen\.complexity
    (V1697)
  (cl-flet
      ((tail-trampoline
        (V1697)
        (shen/cond
         ((shen/and
           (shen/cons\? V1697)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1697))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1697))
             (shen/and
              (shen/cons\?
               (shen/hd
                (nthcdr 1 V1697)))
              (shen/and
               (shen/= 'mode
                       (shen/hd
                        (shen/hd
                         (nthcdr 1 V1697))))
               (shen/and
                (shen/cons\?
                 (nthcdr 1
                         (shen/hd
                          (nthcdr 1 V1697))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 2
                          (shen/hd
                           (nthcdr 1 V1697))))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 3
                            (shen/hd
                             (nthcdr 1 V1697)))))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 2 V1697))
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 3 V1697))))))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 1 V1697)))))
         ((shen/and
           (shen/cons\? V1697)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1697))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1697))
             (shen/and
              (shen/cons\?
               (shen/hd
                (nthcdr 1 V1697)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1697))
               (shen/and
                (shen/= '+
                        (shen/hd
                         (nthcdr 2 V1697)))
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3 V1697)))))))))
          (shen/* 2
                  (shen/*
                   (shen/shen\.complexity
                    (append
                     (list 'mode
                           (shen/hd
                            (shen/hd
                             (nthcdr 1 V1697))))
                     (nthcdr 2 V1697)))
                   (shen/shen\.complexity
                    (append
                     (list 'mode
                           (nthcdr 1
                                   (shen/hd
                                    (nthcdr 1 V1697))))
                     (nthcdr 2 V1697))))))
         ((shen/and
           (shen/cons\? V1697)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1697))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1697))
             (shen/and
              (shen/cons\?
               (shen/hd
                (nthcdr 1 V1697)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1697))
               (shen/and
                (shen/= '-
                        (shen/hd
                         (nthcdr 2 V1697)))
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3 V1697)))))))))
          (shen/*
           (shen/shen\.complexity
            (append
             (list 'mode
                   (shen/hd
                    (shen/hd
                     (nthcdr 1 V1697))))
             (nthcdr 2 V1697)))
           (shen/shen\.complexity
            (append
             (list 'mode
                   (nthcdr 1
                           (shen/hd
                            (nthcdr 1 V1697))))
             (nthcdr 2 V1697)))))
         ((shen/and
           (shen/cons\? V1697)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1697))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1697))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1697))
              (shen/and
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1697)))
               (shen/variable\?
                (shen/hd
                 (nthcdr 1 V1697))))))))
          1)
         ((shen/and
           (shen/cons\? V1697)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1697))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1697))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1697))
              (shen/and
               (shen/= '+
                       (shen/hd
                        (nthcdr 2 V1697)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1697))))))))
          2)
         ((shen/and
           (shen/cons\? V1697)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1697))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1697))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1697))
              (shen/and
               (shen/= '-
                       (shen/hd
                        (nthcdr 2 V1697)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1697))))))))
          1)
         (shen/true
          (vector
           (list
            (list 'mode V1697 '+)))))))
    (let
        ((result
          (funcall #'tail-trampoline V1697)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.product
    (V1699)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1699))
    1)
   ((shen/cons\? V1699)
    (shen/*
     (shen/hd V1699)
     (shen/shen\.product
      (nthcdr 1 V1699))))
   (shen/true
    (shen/shen\.f_error 'shen\.product))))
(defun shen/shen\.s-prolog_literal
    (V1701)
  (shen/cond
   ((shen/and
     (shen/cons\? V1701)
     (shen/and
      (shen/= 'is
              (shen/hd V1701))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1701))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1701))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1701)))))))
    (list 'bind
          (shen/hd
           (nthcdr 1 V1701))
          (shen/shen\.insert_deref
           (shen/hd
            (nthcdr 2 V1701)))))
   ((shen/and
     (shen/cons\? V1701)
     (shen/and
      (shen/= 'when
              (shen/hd V1701))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1701))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1701))))))
    (list 'fwhen
          (shen/shen\.insert_deref
           (shen/hd
            (nthcdr 1 V1701)))))
   ((shen/and
     (shen/cons\? V1701)
     (shen/and
      (shen/= 'bind
              (shen/hd V1701))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1701))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1701))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1701)))))))
    (list 'bind
          (shen/hd
           (nthcdr 1 V1701))
          (shen/shen\.insert_lazyderef
           (shen/hd
            (nthcdr 2 V1701)))))
   ((shen/and
     (shen/cons\? V1701)
     (shen/and
      (shen/= 'fwhen
              (shen/hd V1701))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1701))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1701))))))
    (list 'fwhen
          (shen/shen\.insert_lazyderef
           (shen/hd
            (nthcdr 1 V1701)))))
   ((shen/cons\? V1701)
    V1701)
   (shen/true
    (shen/shen\.f_error 'shen\.s-prolog_literal))))
(defun shen/shen\.insert_deref
    (V1703)
  (shen/cond
   ((shen/variable\? V1703)
    (list 'shen\.deref V1703 'ProcessN))
   ((shen/cons\? V1703)
    (append
     (list
      (shen/shen\.insert_deref
       (shen/hd V1703)))
     (shen/shen\.insert_deref
      (nthcdr 1 V1703))))
   (shen/true V1703)))
(defun shen/shen\.insert_lazyderef
    (V1705)
  (shen/cond
   ((shen/variable\? V1705)
    (list 'shen\.lazyderef V1705 'ProcessN))
   ((shen/cons\? V1705)
    (append
     (list
      (shen/shen\.insert_lazyderef
       (shen/hd V1705)))
     (shen/shen\.insert_lazyderef
      (nthcdr 1 V1705))))
   (shen/true V1705)))
(defun shen/shen\.group_clauses
    (V1707)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1707))
    nil)
   ((shen/cons\? V1707)
    (shen/let Group
              (shen/shen\.collect
               (shen/lambda X
                            (shen/shen\.same_predicate\?
                             (shen/hd V1707)
                             X))
               V1707)
              (shen/let Rest
                        (shen/difference V1707 Group)
                        (append
                         (list Group)
                         (shen/shen\.group_clauses Rest)))))
   (shen/true
    (shen/shen\.f_error 'shen\.group_clauses))))
(defun shen/shen\.collect
    (V1712 V1713)
  (cl-flet
      ((tail-trampoline
        (V1712 V1713)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1713))
          nil)
         ((shen/cons\? V1713)
          (shen/if
           (shen/internal/apply-higher-order-function V1712
                                                      (list
                                                       (shen/hd V1713)))
           (append
            (list
             (shen/hd V1713))
            (shen/shen\.collect V1712
                                (nthcdr 1 V1713)))
           (vector
            (list V1712
                  (nthcdr 1 V1713)))))
         (shen/true
          (shen/shen\.f_error 'shen\.collect)))))
    (let
        ((result
          (funcall #'tail-trampoline V1712 V1713)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.same_predicate\?
    (V1732 V1733)
  (shen/cond
   ((shen/and
     (shen/cons\? V1732)
     (shen/and
      (shen/cons\?
       (shen/hd V1732))
      (shen/and
       (shen/cons\? V1733)
       (shen/cons\?
        (shen/hd V1733)))))
    (shen/=
     (shen/hd
      (shen/hd V1732))
     (shen/hd
      (shen/hd V1733))))
   (shen/true
    (shen/shen\.f_error 'shen\.same_predicate\?))))
(defun shen/shen\.compile_prolog_procedure
    (V1735)
  (shen/let F
            (shen/shen\.procedure_name V1735)
            (shen/let Shen
                      (shen/shen\.clauses-to-shen F V1735)
                      Shen)))
(defun shen/shen\.procedure_name
    (V1749)
  (shen/cond
   ((shen/and
     (shen/cons\? V1749)
     (shen/and
      (shen/cons\?
       (shen/hd V1749))
      (shen/cons\?
       (shen/hd
        (shen/hd V1749)))))
    (shen/hd
     (shen/hd
      (shen/hd V1749))))
   (shen/true
    (shen/shen\.f_error 'shen\.procedure_name))))
(defun shen/shen\.clauses-to-shen
    (V1752 V1753)
  (shen/let Linear
            (shen/map
             (shen/lambda X
                          (shen/shen\.linearise-clause X))
             V1753)
            (shen/let Arity
                      (shen/shen\.prolog-aritycheck V1752
                                                    (shen/map
                                                     (shen/lambda X
                                                                  (shen/head X))
                                                     V1753))
                      (shen/let Parameters
                                (shen/shen\.parameters Arity)
                                (shen/let AUM_instructions
                                          (shen/map
                                           (shen/lambda X
                                                        (shen/shen\.aum X Parameters))
                                           Linear)
                                          (shen/let Code
                                                    (shen/shen\.catch-cut
                                                     (shen/shen\.nest-disjunct
                                                      (shen/map
                                                       (shen/lambda X
                                                                    (shen/shen\.aum_to_shen X))
                                                       AUM_instructions)))
                                                    (shen/let ShenDef
                                                              (append
                                                               (list 'define V1752)
                                                               (shen/append Parameters
                                                                            (shen/append
                                                                             (list 'ProcessN 'Continuation)
                                                                             (list '-> Code))))
                                                              ShenDef)))))))
(defun shen/shen\.catch-cut
    (V1755)
  (shen/cond
   ((shen/not
     (shen/shen\.occurs\? 'cut V1755))
    V1755)
   (shen/true
    (list 'let 'Throwcontrol
          (list 'shen\.catchpoint)
          (list 'shen\.cutpoint 'Throwcontrol V1755)))))
(defun shen/shen\.catchpoint nil
  (shen/set 'shen\.*catch*
            (1+
             (shen/value 'shen\.*catch*))))
(defun shen/shen\.cutpoint
    (V1763 V1764)
  (shen/cond
   ((shen/= V1764 V1763)
    'false)
   (shen/true V1764)))
(defun shen/shen\.nest-disjunct
    (V1766)
  (shen/cond
   ((shen/and
     (shen/cons\? V1766)
     (shen/internal/predicate->shen
      (null
       (nthcdr 1 V1766))))
    (shen/hd V1766))
   ((shen/cons\? V1766)
    (shen/shen\.lisp-or
     (shen/hd V1766)
     (shen/shen\.nest-disjunct
      (nthcdr 1 V1766))))
   (shen/true
    (shen/shen\.f_error 'shen\.nest-disjunct))))
(defun shen/shen\.lisp-or
    (V1769 V1770)
  (list 'let 'Case V1769
        (list 'if
              (list '= 'Case 'false)
              V1770 'Case)))
(defun shen/shen\.prolog-aritycheck
    (V1775 V1776)
  (cl-flet
      ((tail-trampoline
        (V1775 V1776)
        (shen/cond
         ((shen/and
           (shen/cons\? V1776)
           (shen/internal/predicate->shen
            (null
             (nthcdr 1 V1776))))
          (shen/-
           (shen/length
            (shen/hd V1776))
           1))
         ((shen/and
           (shen/cons\? V1776)
           (shen/cons\?
            (nthcdr 1 V1776)))
          (shen/if
           (shen/=
            (shen/length
             (shen/hd V1776))
            (shen/length
             (shen/hd
              (nthcdr 1 V1776))))
           (vector
            (list V1775
                  (nthcdr 1 V1776)))
           (shen/simple-error
            (shen/cn "arity error in prolog procedure "
                     (shen/shen\.app
                      (list V1775)
                      "\n" 'shen\.a)))))
         (shen/true
          (shen/shen\.f_error 'shen\.prolog-aritycheck)))))
    (let
        ((result
          (funcall #'tail-trampoline V1775 V1776)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.linearise-clause
    (V1778)
  (shen/cond
   ((shen/and
     (shen/cons\? V1778)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1778))
      (shen/and
       (shen/= ':-
               (shen/hd
                (nthcdr 1 V1778)))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1778))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1778)))))))
    (shen/let Linear
              (shen/shen\.linearise
               (append
                (list
                 (shen/hd V1778))
                (nthcdr 2 V1778)))
              (shen/shen\.clause_form Linear)))
   (shen/true
    (shen/shen\.f_error 'shen\.linearise-clause))))
(defun shen/shen\.clause_form
    (V1780)
  (shen/cond
   ((shen/and
     (shen/cons\? V1780)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1780))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V1780)))))
    (list
     (shen/shen\.explicit_modes
      (shen/hd V1780))
     ':-
     (shen/shen\.cf_help
      (shen/hd
       (nthcdr 1 V1780)))))
   (shen/true
    (shen/shen\.f_error 'shen\.clause_form))))
(defun shen/shen\.explicit_modes
    (V1782)
  (shen/cond
   ((shen/cons\? V1782)
    (append
     (list
      (shen/hd V1782))
     (shen/map
      (shen/lambda X
                   (shen/shen\.em_help X))
      (nthcdr 1 V1782))))
   (shen/true
    (shen/shen\.f_error 'shen\.explicit_modes))))
(defun shen/shen\.em_help
    (V1784)
  (shen/cond
   ((shen/and
     (shen/cons\? V1784)
     (shen/and
      (shen/= 'mode
              (shen/hd V1784))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1784))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1784))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1784)))))))
    V1784)
   (shen/true
    (list 'mode V1784 '+))))
(defun shen/shen\.cf_help
    (V1786)
  (shen/cond
   ((shen/and
     (shen/cons\? V1786)
     (shen/and
      (shen/= 'where
              (shen/hd V1786))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1786))
       (shen/and
        (shen/cons\?
         (shen/hd
          (nthcdr 1 V1786)))
        (shen/and
         (shen/= '=
                 (shen/hd
                  (shen/hd
                   (nthcdr 1 V1786))))
         (shen/and
          (shen/cons\?
           (nthcdr 1
                   (shen/hd
                    (nthcdr 1 V1786))))
          (shen/and
           (shen/cons\?
            (nthcdr 2
                    (shen/hd
                     (nthcdr 1 V1786))))
           (shen/and
            (shen/internal/predicate->shen
             (null
              (nthcdr 3
                      (shen/hd
                       (nthcdr 1 V1786)))))
            (shen/and
             (shen/cons\?
              (nthcdr 2 V1786))
             (shen/internal/predicate->shen
              (null
               (nthcdr 3 V1786))))))))))))
    (append
     (list
      (append
       (list
        (shen/if
         (shen/value 'shen\.*occurs*)
         'unify! 'unify))
       (nthcdr 1
               (shen/hd
                (nthcdr 1 V1786)))))
     (shen/shen\.cf_help
      (shen/hd
       (nthcdr 2 V1786)))))
   (shen/true V1786)))
(defun shen/occurs-check
    (V1792)
  (shen/cond
   ((shen/= '+ V1792)
    (shen/set 'shen\.*occurs* 'true))
   ((shen/= '- V1792)
    (shen/set 'shen\.*occurs* 'false))
   (shen/true
    (shen/simple-error "occurs-check expects + or -\n"))))
(defun shen/shen\.aum
    (V1795 V1796)
  (shen/cond
   ((shen/and
     (shen/cons\? V1795)
     (shen/and
      (shen/cons\?
       (shen/hd V1795))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1795))
       (shen/and
        (shen/= ':-
                (shen/hd
                 (nthcdr 1 V1795)))
        (shen/and
         (shen/cons\?
          (nthcdr 2 V1795))
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V1795))))))))
    (shen/let MuApplication
              (shen/shen\.make_mu_application
               (list 'shen\.mu
                     (nthcdr 1
                             (shen/hd V1795))
                     (shen/shen\.continuation_call
                      (nthcdr 1
                              (shen/hd V1795))
                      (shen/hd
                       (nthcdr 2 V1795))))
               V1796)
              (shen/shen\.mu_reduction MuApplication '+)))
   (shen/true
    (shen/shen\.f_error 'shen\.aum))))
(defun shen/shen\.continuation_call
    (V1799 V1800)
  (shen/let VTerms
            (append
             (list 'ProcessN)
             (shen/shen\.extract_vars V1799))
            (shen/let VBody
                      (shen/shen\.extract_vars V1800)
                      (shen/let Free
                                (shen/remove 'Throwcontrol
                                             (shen/difference VBody VTerms))
                                (shen/shen\.cc_help Free V1800)))))
(defun shen/remove
    (V1803 V1804)
  (shen/shen\.remove-h V1803 V1804 nil))
(defun shen/shen\.remove-h
    (V1811 V1812 V1813)
  (cl-flet
      ((tail-trampoline
        (V1811 V1812 V1813)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1812))
          (shen/reverse V1813))
         ((shen/and
           (shen/cons\? V1812)
           (shen/=
            (shen/hd V1812)
            V1811))
          (vector
           (list
            (shen/hd V1812)
            (nthcdr 1 V1812)
            V1813)))
         ((shen/cons\? V1812)
          (vector
           (list V1811
                 (nthcdr 1 V1812)
                 (append
                  (list
                   (shen/hd V1812))
                  V1813))))
         (shen/true
          (shen/shen\.f_error 'shen\.remove-h)))))
    (let
        ((result
          (funcall #'tail-trampoline V1811 V1812 V1813)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.cc_help
    (V1816 V1817)
  (shen/cond
   ((shen/and
     (shen/internal/predicate->shen
      (null V1816))
     (shen/internal/predicate->shen
      (null V1817)))
    (list 'shen\.pop 'shen\.the 'shen\.stack))
   ((shen/internal/predicate->shen
     (null V1817))
    (list 'shen\.rename 'shen\.the 'shen\.variables 'in V1816 'and 'shen\.then
          (list 'shen\.pop 'shen\.the 'shen\.stack)))
   ((shen/internal/predicate->shen
     (null V1816))
    (list 'call 'shen\.the 'shen\.continuation V1817))
   (shen/true
    (list 'shen\.rename 'shen\.the 'shen\.variables 'in V1816 'and 'shen\.then
          (list 'call 'shen\.the 'shen\.continuation V1817)))))
(defun shen/shen\.make_mu_application
    (V1820 V1821)
  (shen/cond
   ((shen/and
     (shen/cons\? V1820)
     (shen/and
      (shen/= 'shen\.mu
              (shen/hd V1820))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1820))
       (shen/and
        (shen/internal/predicate->shen
         (null
          (shen/hd
           (nthcdr 1 V1820))))
        (shen/and
         (shen/cons\?
          (nthcdr 2 V1820))
         (shen/and
          (shen/internal/predicate->shen
           (null
            (nthcdr 3 V1820)))
          (shen/internal/predicate->shen
           (null V1821))))))))
    (shen/hd
     (nthcdr 2 V1820)))
   ((shen/and
     (shen/cons\? V1820)
     (shen/and
      (shen/= 'shen\.mu
              (shen/hd V1820))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1820))
       (shen/and
        (shen/cons\?
         (shen/hd
          (nthcdr 1 V1820)))
        (shen/and
         (shen/cons\?
          (nthcdr 2 V1820))
         (shen/and
          (shen/internal/predicate->shen
           (null
            (nthcdr 3 V1820)))
          (shen/cons\? V1821)))))))
    (list
     (list 'shen\.mu
           (shen/hd
            (shen/hd
             (nthcdr 1 V1820)))
           (shen/shen\.make_mu_application
            (append
             (list 'shen\.mu
                   (nthcdr 1
                           (shen/hd
                            (nthcdr 1 V1820))))
             (nthcdr 2 V1820))
            (nthcdr 1 V1821)))
     (shen/hd V1821)))
   (shen/true
    (shen/shen\.f_error 'shen\.make_mu_application))))
(defun shen/shen\.mu_reduction
    (V1830 V1831)
  (cl-flet
      ((tail-trampoline
        (V1830 V1831)
        (shen/cond
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1830))))
               (shen/and
                (shen/= 'mode
                        (shen/hd
                         (shen/hd
                          (nthcdr 1
                                  (shen/hd V1830)))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1830)))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 2
                           (shen/hd
                            (nthcdr 1
                                    (shen/hd V1830)))))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 3
                             (shen/hd
                              (nthcdr 1
                                      (shen/hd V1830))))))
                   (shen/and
                    (shen/cons\?
                     (nthcdr 2
                             (shen/hd V1830)))
                    (shen/and
                     (shen/internal/predicate->shen
                      (null
                       (nthcdr 3
                               (shen/hd V1830))))
                     (shen/and
                      (shen/cons\?
                       (nthcdr 1 V1830))
                      (shen/internal/predicate->shen
                       (null
                        (nthcdr 2 V1830)))))))))))))))
          (vector
           (list
            (append
             (list
              (append
               (list 'shen\.mu
                     (shen/hd
                      (nthcdr 1
                              (shen/hd
                               (nthcdr 1
                                       (shen/hd V1830))))))
               (nthcdr 2
                       (shen/hd V1830))))
             (nthcdr 1 V1830))
            (shen/hd
             (nthcdr 2
                     (shen/hd
                      (nthcdr 1
                              (shen/hd V1830))))))))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1830)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1830))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1830))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 2 V1830)))
                  (shen/= '_
                          (shen/hd
                           (nthcdr 1
                                   (shen/hd V1830))))))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 2
                     (shen/hd V1830)))
            V1831)))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1830)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1830))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1830))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 2 V1830)))
                  (shen/shen\.ephemeral_variable\?
                   (shen/hd
                    (nthcdr 1
                            (shen/hd V1830)))
                   (shen/hd
                    (nthcdr 1 V1830)))))))))))
          (shen/subst
           (shen/hd
            (nthcdr 1 V1830))
           (shen/hd
            (nthcdr 1
                    (shen/hd V1830)))
           (shen/shen\.mu_reduction
            (shen/hd
             (nthcdr 2
                     (shen/hd V1830)))
            V1831)))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1830)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1830))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1830))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 2 V1830)))
                  (shen/variable\?
                   (shen/hd
                    (nthcdr 1
                            (shen/hd V1830))))))))))))
          (list 'let
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1830)))
                'shen\.be
                (shen/hd
                 (nthcdr 1 V1830))
                'in
                (shen/shen\.mu_reduction
                 (shen/hd
                  (nthcdr 2
                          (shen/hd V1830)))
                 V1831)))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1830)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1830))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1830))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 2 V1830)))
                  (shen/and
                   (shen/= '- V1831)
                   (shen/shen\.prolog_constant\?
                    (shen/hd
                     (nthcdr 1
                             (shen/hd V1830)))))))))))))
          (shen/let Z
                    (shen/gensym 'V)
                    (list 'let Z 'shen\.be
                          (append
                           (list 'shen\.the 'shen\.result 'shen\.of 'shen\.dereferencing)
                           (nthcdr 1 V1830))
                          'in
                          (list 'if
                                (list Z 'is 'identical 'shen\.to
                                      (shen/hd
                                       (nthcdr 1
                                               (shen/hd V1830))))
                                'shen\.then
                                (shen/shen\.mu_reduction
                                 (shen/hd
                                  (nthcdr 2
                                          (shen/hd V1830)))
                                 '-)
                                'shen\.else 'shen\.failed!))))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (nthcdr 2
                        (shen/hd V1830)))
               (shen/and
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3
                          (shen/hd V1830))))
                (shen/and
                 (shen/cons\?
                  (nthcdr 1 V1830))
                 (shen/and
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 2 V1830)))
                  (shen/and
                   (shen/= '+ V1831)
                   (shen/shen\.prolog_constant\?
                    (shen/hd
                     (nthcdr 1
                             (shen/hd V1830)))))))))))))
          (shen/let Z
                    (shen/gensym 'V)
                    (list 'let Z 'shen\.be
                          (append
                           (list 'shen\.the 'shen\.result 'shen\.of 'shen\.dereferencing)
                           (nthcdr 1 V1830))
                          'in
                          (list 'if
                                (list Z 'is 'identical 'shen\.to
                                      (shen/hd
                                       (nthcdr 1
                                               (shen/hd V1830))))
                                'shen\.then
                                (shen/shen\.mu_reduction
                                 (shen/hd
                                  (nthcdr 2
                                          (shen/hd V1830)))
                                 '+)
                                'shen\.else
                                (list 'if
                                      (list Z 'is 'shen\.a 'shen\.variable)
                                      'shen\.then
                                      (list 'bind Z 'shen\.to
                                            (shen/hd
                                             (nthcdr 1
                                                     (shen/hd V1830)))
                                            'in
                                            (shen/shen\.mu_reduction
                                             (shen/hd
                                              (nthcdr 2
                                                      (shen/hd V1830)))
                                             '+))
                                      'shen\.else 'shen\.failed!)))))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1830))))
               (shen/and
                (shen/cons\?
                 (nthcdr 2
                         (shen/hd V1830)))
                (shen/and
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 3
                           (shen/hd V1830))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 1 V1830))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 2 V1830)))
                   (shen/= '- V1831))))))))))
          (shen/let Z
                    (shen/gensym 'V)
                    (list 'let Z 'shen\.be
                          (append
                           (list 'shen\.the 'shen\.result 'shen\.of 'shen\.dereferencing)
                           (nthcdr 1 V1830))
                          'in
                          (list 'if
                                (list Z 'is 'shen\.a 'shen\.non-empty 'list)
                                'shen\.then
                                (shen/shen\.mu_reduction
                                 (list
                                  (list 'shen\.mu
                                        (shen/hd
                                         (shen/hd
                                          (nthcdr 1
                                                  (shen/hd V1830))))
                                        (list
                                         (append
                                          (list 'shen\.mu
                                                (nthcdr 1
                                                        (shen/hd
                                                         (nthcdr 1
                                                                 (shen/hd V1830)))))
                                          (nthcdr 2
                                                  (shen/hd V1830)))
                                         (list 'shen\.the 'tail 'shen\.of Z)))
                                  (list 'shen\.the 'head 'shen\.of Z))
                                 '-)
                                'shen\.else 'shen\.failed!))))
         ((shen/and
           (shen/cons\? V1830)
           (shen/and
            (shen/cons\?
             (shen/hd V1830))
            (shen/and
             (shen/= 'shen\.mu
                     (shen/hd
                      (shen/hd V1830)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1830)))
              (shen/and
               (shen/cons\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd V1830))))
               (shen/and
                (shen/cons\?
                 (nthcdr 2
                         (shen/hd V1830)))
                (shen/and
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 3
                           (shen/hd V1830))))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 1 V1830))
                  (shen/and
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 2 V1830)))
                   (shen/= '+ V1831))))))))))
          (shen/let Z
                    (shen/gensym 'V)
                    (list 'let Z 'shen\.be
                          (append
                           (list 'shen\.the 'shen\.result 'shen\.of 'shen\.dereferencing)
                           (nthcdr 1 V1830))
                          'in
                          (list 'if
                                (list Z 'is 'shen\.a 'shen\.non-empty 'list)
                                'shen\.then
                                (shen/shen\.mu_reduction
                                 (list
                                  (list 'shen\.mu
                                        (shen/hd
                                         (shen/hd
                                          (nthcdr 1
                                                  (shen/hd V1830))))
                                        (list
                                         (append
                                          (list 'shen\.mu
                                                (nthcdr 1
                                                        (shen/hd
                                                         (nthcdr 1
                                                                 (shen/hd V1830)))))
                                          (nthcdr 2
                                                  (shen/hd V1830)))
                                         (list 'shen\.the 'tail 'shen\.of Z)))
                                  (list 'shen\.the 'head 'shen\.of Z))
                                 '+)
                                'shen\.else
                                (list 'if
                                      (list Z 'is 'shen\.a 'shen\.variable)
                                      'shen\.then
                                      (list 'shen\.rename 'shen\.the 'shen\.variables 'in
                                            (shen/shen\.extract_vars
                                             (shen/hd
                                              (nthcdr 1
                                                      (shen/hd V1830))))
                                            'and 'shen\.then
                                            (list 'bind Z 'shen\.to
                                                  (shen/shen\.rcons_form
                                                   (shen/shen\.remove_modes
                                                    (shen/hd
                                                     (nthcdr 1
                                                             (shen/hd V1830)))))
                                                  'in
                                                  (shen/shen\.mu_reduction
                                                   (shen/hd
                                                    (nthcdr 2
                                                            (shen/hd V1830)))
                                                   '+)))
                                      'shen\.else 'shen\.failed!)))))
         (shen/true V1830))))
    (let
        ((result
          (funcall #'tail-trampoline V1830 V1831)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.rcons_form
    (V1833)
  (shen/cond
   ((shen/cons\? V1833)
    (list 'cons
          (shen/shen\.rcons_form
           (shen/hd V1833))
          (shen/shen\.rcons_form
           (nthcdr 1 V1833))))
   (shen/true V1833)))
(defun shen/shen\.remove_modes
    (V1835)
  (cl-flet
      ((tail-trampoline
        (V1835)
        (shen/cond
         ((shen/and
           (shen/cons\? V1835)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1835))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1835))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1835))
              (shen/and
               (shen/= '+
                       (shen/hd
                        (nthcdr 2 V1835)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1835))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 1 V1835)))))
         ((shen/and
           (shen/cons\? V1835)
           (shen/and
            (shen/= 'mode
                    (shen/hd V1835))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1835))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1835))
              (shen/and
               (shen/= '-
                       (shen/hd
                        (nthcdr 2 V1835)))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1835))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 1 V1835)))))
         ((shen/cons\? V1835)
          (append
           (list
            (shen/shen\.remove_modes
             (shen/hd V1835)))
           (shen/shen\.remove_modes
            (nthcdr 1 V1835))))
         (shen/true V1835))))
    (let
        ((result
          (funcall #'tail-trampoline V1835)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.ephemeral_variable\?
    (V1838 V1839)
  (shen/and
   (shen/variable\? V1838)
   (shen/variable\? V1839)))
(defun shen/shen\.prolog_constant\?
    (V1849)
  (shen/cond
   ((shen/cons\? V1849)
    'false)
   (shen/true 'true)))
(defun shen/shen\.aum_to_shen
    (V1851)
  (cl-flet
      ((tail-trampoline
        (V1851)
        (shen/cond
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'let
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1851))
              (shen/and
               (shen/= 'shen\.be
                       (shen/hd
                        (nthcdr 2 V1851)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V1851))
                (shen/and
                 (shen/cons\?
                  (nthcdr 4 V1851))
                 (shen/and
                  (shen/= 'in
                          (shen/hd
                           (nthcdr 4 V1851)))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 5 V1851))
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 6 V1851))))))))))))
          (list 'let
                (shen/hd
                 (nthcdr 1 V1851))
                (shen/shen\.aum_to_shen
                 (shen/hd
                  (nthcdr 3 V1851)))
                (shen/shen\.aum_to_shen
                 (shen/hd
                  (nthcdr 5 V1851)))))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'shen\.the
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'shen\.result
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.of
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 3 V1851))
                 (shen/and
                  (shen/= 'shen\.dereferencing
                          (shen/hd
                           (nthcdr 3 V1851)))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 4 V1851))
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 5 V1851))))))))))))
          (list 'shen\.lazyderef
                (shen/shen\.aum_to_shen
                 (shen/hd
                  (nthcdr 4 V1851)))
                'ProcessN))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'if
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1851))
              (shen/and
               (shen/= 'shen\.then
                       (shen/hd
                        (nthcdr 2 V1851)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V1851))
                (shen/and
                 (shen/cons\?
                  (nthcdr 4 V1851))
                 (shen/and
                  (shen/= 'shen\.else
                          (shen/hd
                           (nthcdr 4 V1851)))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 5 V1851))
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 6 V1851))))))))))))
          (list 'if
                (shen/shen\.aum_to_shen
                 (shen/hd
                  (nthcdr 1 V1851)))
                (shen/shen\.aum_to_shen
                 (shen/hd
                  (nthcdr 3 V1851)))
                (shen/shen\.aum_to_shen
                 (shen/hd
                  (nthcdr 5 V1851)))))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1851))
            (shen/and
             (shen/= 'is
                     (shen/hd
                      (nthcdr 1 V1851)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1851))
              (shen/and
               (shen/= 'shen\.a
                       (shen/hd
                        (nthcdr 2 V1851)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V1851))
                (shen/and
                 (shen/= 'shen\.variable
                         (shen/hd
                          (nthcdr 3 V1851)))
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 4 V1851))))))))))
          (list 'shen\.pvar\?
                (shen/hd V1851)))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1851))
            (shen/and
             (shen/= 'is
                     (shen/hd
                      (nthcdr 1 V1851)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1851))
              (shen/and
               (shen/= 'shen\.a
                       (shen/hd
                        (nthcdr 2 V1851)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V1851))
                (shen/and
                 (shen/= 'shen\.non-empty
                         (shen/hd
                          (nthcdr 3 V1851)))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 4 V1851))
                  (shen/and
                   (shen/= 'list
                           (shen/hd
                            (nthcdr 4 V1851)))
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 5 V1851))))))))))))
          (list 'cons\?
                (shen/hd V1851)))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'shen\.rename
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'shen\.the
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.variables
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 3 V1851))
                 (shen/and
                  (shen/= 'in
                          (shen/hd
                           (nthcdr 3 V1851)))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 4 V1851))
                   (shen/and
                    (shen/internal/predicate->shen
                     (null
                      (shen/hd
                       (nthcdr 4 V1851))))
                    (shen/and
                     (shen/cons\?
                      (nthcdr 5 V1851))
                     (shen/and
                      (shen/= 'and
                              (shen/hd
                               (nthcdr 5 V1851)))
                      (shen/and
                       (shen/cons\?
                        (nthcdr 6 V1851))
                       (shen/and
                        (shen/= 'shen\.then
                                (shen/hd
                                 (nthcdr 6 V1851)))
                        (shen/and
                         (shen/cons\?
                          (nthcdr 7 V1851))
                         (shen/internal/predicate->shen
                          (null
                           (nthcdr 8 V1851))))))))))))))))))
          (vector
           (list
            (shen/hd
             (nthcdr 7 V1851)))))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'shen\.rename
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'shen\.the
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.variables
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 3 V1851))
                 (shen/and
                  (shen/= 'in
                          (shen/hd
                           (nthcdr 3 V1851)))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 4 V1851))
                   (shen/and
                    (shen/cons\?
                     (shen/hd
                      (nthcdr 4 V1851)))
                    (shen/and
                     (shen/cons\?
                      (nthcdr 5 V1851))
                     (shen/and
                      (shen/= 'and
                              (shen/hd
                               (nthcdr 5 V1851)))
                      (shen/and
                       (shen/cons\?
                        (nthcdr 6 V1851))
                       (shen/and
                        (shen/= 'shen\.then
                                (shen/hd
                                 (nthcdr 6 V1851)))
                        (shen/and
                         (shen/cons\?
                          (nthcdr 7 V1851))
                         (shen/internal/predicate->shen
                          (null
                           (nthcdr 8 V1851))))))))))))))))))
          (list 'let
                (shen/hd
                 (shen/hd
                  (nthcdr 4 V1851)))
                (list 'shen\.newpv 'ProcessN)
                (shen/shen\.aum_to_shen
                 (append
                  (list 'shen\.rename 'shen\.the 'shen\.variables 'in
                        (nthcdr 1
                                (shen/hd
                                 (nthcdr 4 V1851))))
                  (nthcdr 5 V1851)))))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'bind
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1851))
              (shen/and
               (shen/= 'shen\.to
                       (shen/hd
                        (nthcdr 2 V1851)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V1851))
                (shen/and
                 (shen/cons\?
                  (nthcdr 4 V1851))
                 (shen/and
                  (shen/= 'in
                          (shen/hd
                           (nthcdr 4 V1851)))
                  (shen/and
                   (shen/cons\?
                    (nthcdr 5 V1851))
                   (shen/internal/predicate->shen
                    (null
                     (nthcdr 6 V1851))))))))))))
          (list 'do
                (list 'shen\.bindv
                      (shen/hd
                       (nthcdr 1 V1851))
                      (shen/shen\.chwild
                       (shen/hd
                        (nthcdr 3 V1851)))
                      'ProcessN)
                (list 'let 'Result
                      (shen/shen\.aum_to_shen
                       (shen/hd
                        (nthcdr 5 V1851)))
                      (list 'do
                            (list 'shen\.unbindv
                                  (shen/hd
                                   (nthcdr 1 V1851))
                                  'ProcessN)
                            'Result))))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V1851))
            (shen/and
             (shen/= 'is
                     (shen/hd
                      (nthcdr 1 V1851)))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1851))
              (shen/and
               (shen/= 'identical
                       (shen/hd
                        (nthcdr 2 V1851)))
               (shen/and
                (shen/cons\?
                 (nthcdr 3 V1851))
                (shen/and
                 (shen/= 'shen\.to
                         (shen/hd
                          (nthcdr 3 V1851)))
                 (shen/and
                  (shen/cons\?
                   (nthcdr 4 V1851))
                  (shen/internal/predicate->shen
                   (null
                    (nthcdr 5 V1851)))))))))))
          (list '=
                (shen/hd
                 (nthcdr 4 V1851))
                (shen/hd V1851)))
         ((shen/= 'shen\.failed! V1851)
          'false)
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'shen\.the
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'head
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.of
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 3 V1851))
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 4 V1851))))))))))
          (append
           (list 'hd)
           (nthcdr 3 V1851)))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'shen\.the
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'tail
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.of
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 3 V1851))
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 4 V1851))))))))))
          (append
           (list 'tl)
           (nthcdr 3 V1851)))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'shen\.pop
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'shen\.the
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.stack
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/internal/predicate->shen
                 (null
                  (nthcdr 3 V1851)))))))))
          (list 'do
                (list 'shen\.incinfs)
                (list 'thaw 'Continuation)))
         ((shen/and
           (shen/cons\? V1851)
           (shen/and
            (shen/= 'call
                    (shen/hd V1851))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1851))
             (shen/and
              (shen/= 'shen\.the
                      (shen/hd
                       (nthcdr 1 V1851)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V1851))
               (shen/and
                (shen/= 'shen\.continuation
                        (shen/hd
                         (nthcdr 2 V1851)))
                (shen/and
                 (shen/cons\?
                  (nthcdr 3 V1851))
                 (shen/internal/predicate->shen
                  (null
                   (nthcdr 4 V1851))))))))))
          (list 'do
                (list 'shen\.incinfs)
                (shen/shen\.call_the_continuation
                 (shen/shen\.chwild
                  (shen/hd
                   (nthcdr 3 V1851)))
                 'ProcessN 'Continuation)))
         (shen/true V1851))))
    (let
        ((result
          (funcall #'tail-trampoline V1851)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.chwild
    (V1853)
  (shen/cond
   ((shen/= V1853 '_)
    (list 'shen\.newpv 'ProcessN))
   ((shen/cons\? V1853)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.chwild Z))
     V1853))
   (shen/true V1853)))
(defun shen/shen\.newpv
    (V1855)
  (shen/let Count+1
            (1+
             (shen/<-address
              (shen/value 'shen\.*varcounter*)
              V1855))
            (shen/let IncVar
                      (shen/address->
                       (shen/value 'shen\.*varcounter*)
                       V1855 Count+1)
                      (shen/let Vector
                                (shen/<-address
                                 (shen/value 'shen\.*prologvectors*)
                                 V1855)
                                (shen/let ResizeVectorIfNeeded
                                          (shen/if
                                           (shen/= Count+1
                                                   (shen/limit Vector))
                                           (shen/shen\.resizeprocessvector V1855 Count+1)
                                           'shen\.skip)
                                          (shen/shen\.mk-pvar Count+1))))))
(defun shen/shen\.resizeprocessvector
    (V1858 V1859)
  (shen/let Vector
            (shen/<-address
             (shen/value 'shen\.*prologvectors*)
             V1858)
            (shen/let BigVector
                      (shen/shen\.resize-vector Vector
                                                (shen/+ V1859 V1859)
                                                'shen\.-null-)
                      (shen/address->
                       (shen/value 'shen\.*prologvectors*)
                       V1858 BigVector))))
(defun shen/shen\.resize-vector
    (V1863 V1864 V1865)
  (shen/let BigVector
            (shen/address->
             (shen/absvector
              (1+ V1864))
             0 V1864)
            (shen/shen\.copy-vector V1863 BigVector
                                    (shen/limit V1863)
                                    V1864 V1865)))
(defun shen/shen\.copy-vector
    (V1871 V1872 V1873 V1874 V1875)
  (shen/shen\.copy-vector-stage-2
   (1+ V1873)
   (1+ V1874)
   V1875
   (shen/shen\.copy-vector-stage-1 1 V1871 V1872
                                   (1+ V1873))))
(defun shen/shen\.copy-vector-stage-1
    (V1883 V1884 V1885 V1886)
  (cl-flet
      ((tail-trampoline
        (V1883 V1884 V1885 V1886)
        (shen/cond
         ((shen/= V1886 V1883)
          V1885)
         (shen/true
          (vector
           (list
            (1+ V1883)
            V1884
            (shen/address-> V1885 V1883
                            (shen/<-address V1884 V1883))
            V1886))))))
    (let
        ((result
          (funcall #'tail-trampoline V1883 V1884 V1885 V1886)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.copy-vector-stage-2
    (V1894 V1895 V1896 V1897)
  (cl-flet
      ((tail-trampoline
        (V1894 V1895 V1896 V1897)
        (shen/cond
         ((shen/= V1895 V1894)
          V1897)
         (shen/true
          (vector
           (list
            (1+ V1894)
            V1895 V1896
            (shen/address-> V1897 V1894 V1896)))))))
    (let
        ((result
          (funcall #'tail-trampoline V1894 V1895 V1896 V1897)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.mk-pvar
    (V1899)
  (shen/address->
   (shen/address->
    (shen/absvector 2)
    0 'shen\.pvar)
   1 V1899))
(defun shen/shen\.pvar\?
    (V1901)
  (shen/and
   (shen/absvector\? V1901)
   (shen/=
    (shen/<-address/or V1901 0
                       (shen/freeze 'shen\.not-pvar))
    'shen\.pvar)))
(defun shen/shen\.bindv
    (V1905 V1906 V1907)
  (shen/let Vector
            (shen/<-address
             (shen/value 'shen\.*prologvectors*)
             V1907)
            (shen/address-> Vector
                            (shen/<-address V1905 1)
                            V1906)))
(defun shen/shen\.unbindv
    (V1910 V1911)
  (shen/let Vector
            (shen/<-address
             (shen/value 'shen\.*prologvectors*)
             V1911)
            (shen/address-> Vector
                            (shen/<-address V1910 1)
                            'shen\.-null-)))
(defun shen/shen\.incinfs nil
  (shen/set 'shen\.*infs*
            (1+
             (shen/value 'shen\.*infs*))))
(defun shen/shen\.call_the_continuation
    (V1915 V1916 V1917)
  (shen/cond
   ((shen/and
     (shen/cons\? V1915)
     (shen/and
      (shen/cons\?
       (shen/hd V1915))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1915)))))
    (append
     (list
      (shen/hd
       (shen/hd V1915)))
     (shen/append
      (nthcdr 1
              (shen/hd V1915))
      (list V1916 V1917))))
   ((shen/and
     (shen/cons\? V1915)
     (shen/cons\?
      (shen/hd V1915)))
    (shen/let NewContinuation
              (shen/shen\.newcontinuation
               (nthcdr 1 V1915)
               V1916 V1917)
              (append
               (list
                (shen/hd
                 (shen/hd V1915)))
               (shen/append
                (nthcdr 1
                        (shen/hd V1915))
                (list V1916 NewContinuation)))))
   (shen/true
    (shen/shen\.f_error 'shen\.call_the_continuation))))
(defun shen/shen\.newcontinuation
    (V1921 V1922 V1923)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1921))
    V1923)
   ((shen/and
     (shen/cons\? V1921)
     (shen/cons\?
      (shen/hd V1921)))
    (list 'freeze
          (append
           (list
            (shen/hd
             (shen/hd V1921)))
           (shen/append
            (nthcdr 1
                    (shen/hd V1921))
            (list V1922
                  (shen/shen\.newcontinuation
                   (nthcdr 1 V1921)
                   V1922 V1923))))))
   (shen/true
    (shen/shen\.f_error 'shen\.newcontinuation))))
(defun shen/return
    (V1931 V1932 V1933)
  (shen/shen\.deref V1931 V1932))
(defun shen/shen\.measure&return
    (V1941 V1942 V1943)
  (shen/do
   (shen/shen\.prhush
    (shen/shen\.app
     (shen/value 'shen\.*infs*)
     " inferences\n" 'shen\.a)
    (shen/stoutput))
   (shen/shen\.deref V1941 V1942)))
(defun shen/unify
    (V1948 V1949 V1950 V1951)
  (shen/shen\.lzy=
   (shen/shen\.lazyderef V1948 V1950)
   (shen/shen\.lazyderef V1949 V1950)
   V1950 V1951))
(defun shen/shen\.lzy=
    (V1973 V1974 V1975 V1976)
  (cl-flet
      ((tail-trampoline
        (V1973 V1974 V1975 V1976)
        (shen/cond
         ((shen/= V1974 V1973)
          (shen/thaw V1976))
         ((shen/shen\.pvar\? V1973)
          (shen/bind V1973 V1974 V1975 V1976))
         ((shen/shen\.pvar\? V1974)
          (shen/bind V1974 V1973 V1975 V1976))
         ((shen/and
           (shen/cons\? V1973)
           (shen/cons\? V1974))
          (vector
           (list
            (shen/shen\.lazyderef
             (shen/hd V1973)
             V1975)
            (shen/shen\.lazyderef
             (shen/hd V1974)
             V1975)
            V1975
            (shen/freeze
             (shen/shen\.lzy=
              (shen/shen\.lazyderef
               (nthcdr 1 V1973)
               V1975)
              (shen/shen\.lazyderef
               (nthcdr 1 V1974)
               V1975)
              V1975 V1976)))))
         (shen/true 'false))))
    (let
        ((result
          (funcall #'tail-trampoline V1973 V1974 V1975 V1976)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.deref
    (V1979 V1980)
  (cl-flet
      ((tail-trampoline
        (V1979 V1980)
        (shen/cond
         ((shen/cons\? V1979)
          (append
           (list
            (shen/shen\.deref
             (shen/hd V1979)
             V1980))
           (shen/shen\.deref
            (nthcdr 1 V1979)
            V1980)))
         (shen/true
          (shen/if
           (shen/shen\.pvar\? V1979)
           (shen/let Value
                     (shen/shen\.valvector V1979 V1980)
                     (shen/if
                      (shen/= Value 'shen\.-null-)
                      V1979
                      (vector
                       (list Value V1980))))
           V1979)))))
    (let
        ((result
          (funcall #'tail-trampoline V1979 V1980)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.lazyderef
    (X ProcessN)
  (let
      ((Current X)
       (KeepLooking t))
    (while KeepLooking
      (shen/if
       (shen/shen\.pvar\? Current)
       (shen/let Value
                 (shen/shen\.valvector Current ProcessN)
                 (shen/if
                  (shen/= Value 'shen\.-null-)
                  (setq KeepLooking nil)
                  (setq Current Value)))
       (setq KeepLooking nil)))
    Current))
(defun shen/shen\.valvector
    (V1987 V1988)
  (shen/<-address
   (shen/<-address
    (shen/value 'shen\.*prologvectors*)
    V1988)
   (shen/<-address V1987 1)))
(defun shen/unify!
    (V1993 V1994 V1995 V1996)
  (shen/shen\.lzy=!
   (shen/shen\.lazyderef V1993 V1995)
   (shen/shen\.lazyderef V1994 V1995)
   V1995 V1996))
(defun shen/shen\.lzy=!
    (V2018 V2019 V2020 V2021)
  (cl-flet
      ((tail-trampoline
        (V2018 V2019 V2020 V2021)
        (shen/cond
         ((shen/= V2019 V2018)
          (shen/thaw V2021))
         ((shen/and
           (shen/shen\.pvar\? V2018)
           (shen/not
            (shen/shen\.occurs\? V2018
                                 (shen/shen\.deref V2019 V2020))))
          (shen/bind V2018 V2019 V2020 V2021))
         ((shen/and
           (shen/shen\.pvar\? V2019)
           (shen/not
            (shen/shen\.occurs\? V2019
                                 (shen/shen\.deref V2018 V2020))))
          (shen/bind V2019 V2018 V2020 V2021))
         ((shen/and
           (shen/cons\? V2018)
           (shen/cons\? V2019))
          (vector
           (list
            (shen/shen\.lazyderef
             (shen/hd V2018)
             V2020)
            (shen/shen\.lazyderef
             (shen/hd V2019)
             V2020)
            V2020
            (shen/freeze
             (shen/shen\.lzy=!
              (shen/shen\.lazyderef
               (nthcdr 1 V2018)
               V2020)
              (shen/shen\.lazyderef
               (nthcdr 1 V2019)
               V2020)
              V2020 V2021)))))
         (shen/true 'false))))
    (let
        ((result
          (funcall #'tail-trampoline V2018 V2019 V2020 V2021)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.occurs\?
    (V2033 V2034)
  (shen/cond
   ((shen/= V2034 V2033)
    'true)
   ((shen/cons\? V2034)
    (shen/or
     (shen/shen\.occurs\? V2033
                          (shen/hd V2034))
     (shen/shen\.occurs\? V2033
                          (nthcdr 1 V2034))))
   (shen/true 'false)))
(defun shen/identical
    (V2039 V2040 V2041 V2042)
  (shen/shen\.lzy==
   (shen/shen\.lazyderef V2039 V2041)
   (shen/shen\.lazyderef V2040 V2041)
   V2041 V2042))
(defun shen/shen\.lzy==
    (V2064 V2065 V2066 V2067)
  (cl-flet
      ((tail-trampoline
        (V2064 V2065 V2066 V2067)
        (shen/cond
         ((shen/= V2065 V2064)
          (shen/thaw V2067))
         ((shen/and
           (shen/cons\? V2064)
           (shen/cons\? V2065))
          (vector
           (list
            (shen/shen\.lazyderef
             (shen/hd V2064)
             V2066)
            (shen/shen\.lazyderef
             (shen/hd V2065)
             V2066)
            V2066
            (shen/freeze
             (shen/shen\.lzy==
              (nthcdr 1 V2064)
              (nthcdr 1 V2065)
              V2066 V2067)))))
         (shen/true 'false))))
    (let
        ((result
          (funcall #'tail-trampoline V2064 V2065 V2066 V2067)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.pvar
    (V2069)
  (shen/cn "Var"
           (shen/shen\.app
            (shen/<-address V2069 1)
            "" 'shen\.a)))
(defun shen/bind
    (V2074 V2075 V2076 V2077)
  (shen/do
   (shen/shen\.bindv V2074 V2075 V2076)
   (shen/let Result
             (shen/thaw V2077)
             (shen/do
              (shen/shen\.unbindv V2074 V2076)
              Result))))
(defun shen/fwhen
    (V2095 V2096 V2097)
  (shen/cond
   ((shen/= 'true V2095)
    (shen/thaw V2097))
   ((shen/= 'false V2095)
    'false)
   (shen/true
    (shen/simple-error
     (shen/cn "fwhen expects a boolean: not "
              (shen/shen\.app V2095 "%" 'shen\.s))))))
(defun shen/call
    (V2113 V2114 V2115)
  (shen/cond
   ((shen/cons\? V2113)
    (shen/shen\.call-help
     (shen/function
      (shen/shen\.lazyderef
       (shen/hd V2113)
       V2114))
     (nthcdr 1 V2113)
     V2114 V2115))
   (shen/true 'false)))
(defun shen/shen\.call-help
    (V2120 V2121 V2122 V2123)
  (cl-flet
      ((tail-trampoline
        (V2120 V2121 V2122 V2123)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2121))
          (shen/internal/apply-higher-order-function V2120
                                                     (list V2122 V2123)))
         ((shen/cons\? V2121)
          (vector
           (list
            (shen/internal/apply-higher-order-function V2120
                                                       (list
                                                        (shen/hd V2121)))
            (nthcdr 1 V2121)
            V2122 V2123)))
         (shen/true
          (shen/shen\.f_error 'shen\.call-help)))))
    (let
        ((result
          (funcall #'tail-trampoline V2120 V2121 V2122 V2123)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.intprolog
    (V2125)
  (shen/cond
   ((shen/and
     (shen/cons\? V2125)
     (shen/cons\?
      (shen/hd V2125)))
    (shen/let ProcessN
              (shen/shen\.start-new-prolog-process)
              (shen/shen\.intprolog-help
               (shen/hd
                (shen/hd V2125))
               (shen/shen\.insert-prolog-variables
                (list
                 (nthcdr 1
                         (shen/hd V2125))
                 (nthcdr 1 V2125))
                ProcessN)
               ProcessN)))
   (shen/true
    (shen/shen\.f_error 'shen\.intprolog))))
(defun shen/shen\.intprolog-help
    (V2129 V2130 V2131)
  (shen/cond
   ((shen/and
     (shen/cons\? V2130)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V2130))
      (shen/internal/predicate->shen
       (null
        (nthcdr 2 V2130)))))
    (shen/shen\.intprolog-help-help V2129
                                    (shen/hd V2130)
                                    (shen/hd
                                     (nthcdr 1 V2130))
                                    V2131))
   (shen/true
    (shen/shen\.f_error 'shen\.intprolog-help))))
(defun shen/shen\.intprolog-help-help
    (V2136 V2137 V2138 V2139)
  (cl-flet
      ((tail-trampoline
        (V2136 V2137 V2138 V2139)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2137))
          (shen/internal/apply-higher-order-function V2136
                                                     (list V2139
                                                           (shen/freeze
                                                            (shen/shen\.call-rest V2138 V2139)))))
         ((shen/cons\? V2137)
          (vector
           (list
            (shen/internal/apply-higher-order-function V2136
                                                       (list
                                                        (shen/hd V2137)))
            (nthcdr 1 V2137)
            V2138 V2139)))
         (shen/true
          (shen/shen\.f_error 'shen\.intprolog-help-help)))))
    (let
        ((result
          (funcall #'tail-trampoline V2136 V2137 V2138 V2139)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.call-rest
    (V2144 V2145)
  (cl-flet
      ((tail-trampoline
        (V2144 V2145)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2144))
          'true)
         ((shen/and
           (shen/cons\? V2144)
           (shen/and
            (shen/cons\?
             (shen/hd V2144))
            (shen/cons\?
             (nthcdr 1
                     (shen/hd V2144)))))
          (vector
           (list
            (append
             (list
              (append
               (list
                (shen/internal/apply-function-expression
                 (shen/hd
                  (shen/hd V2144))
                 (list
                  (shen/hd
                   (nthcdr 1
                           (shen/hd V2144))))))
               (nthcdr 2
                       (shen/hd V2144))))
             (nthcdr 1 V2144))
            V2145)))
         ((shen/and
           (shen/cons\? V2144)
           (shen/and
            (shen/cons\?
             (shen/hd V2144))
            (shen/internal/predicate->shen
             (null
              (nthcdr 1
                      (shen/hd V2144))))))
          (shen/internal/apply-function-expression
           (shen/hd
            (shen/hd V2144))
           (list V2145
                 (shen/freeze
                  (shen/shen\.call-rest
                   (nthcdr 1 V2144)
                   V2145)))))
         (shen/true
          (shen/shen\.f_error 'shen\.call-rest)))))
    (let
        ((result
          (funcall #'tail-trampoline V2144 V2145)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.start-new-prolog-process nil
  (shen/let IncrementProcessCounter
            (shen/set 'shen\.*process-counter*
                      (1+
                       (shen/value 'shen\.*process-counter*)))
            (shen/shen\.initialise-prolog IncrementProcessCounter)))
(defun shen/shen\.insert-prolog-variables
    (V2148 V2149)
  (shen/shen\.insert-prolog-variables-help V2148
                                           (shen/shen\.flatten V2148)
                                           V2149))
(defun shen/shen\.insert-prolog-variables-help
    (V2157 V2158 V2159)
  (cl-flet
      ((tail-trampoline
        (V2157 V2158 V2159)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V2158))
          V2157)
         ((shen/and
           (shen/cons\? V2158)
           (shen/variable\?
            (shen/hd V2158)))
          (shen/let V
                    (shen/shen\.newpv V2159)
                    (shen/let XV/Y
                              (shen/subst V
                                          (shen/hd V2158)
                                          V2157)
                              (shen/let Z-Y
                                        (shen/remove
                                         (shen/hd V2158)
                                         (nthcdr 1 V2158))
                                        (vector
                                         (list XV/Y Z-Y V2159))))))
         ((shen/cons\? V2158)
          (vector
           (list V2157
                 (nthcdr 1 V2158)
                 V2159)))
         (shen/true
          (shen/shen\.f_error 'shen\.insert-prolog-variables-help)))))
    (let
        ((result
          (funcall #'tail-trampoline V2157 V2158 V2159)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.initialise-prolog
    (V2161)
  (shen/let Vector
            (shen/address->
             (shen/value 'shen\.*prologvectors*)
             V2161
             (shen/shen\.fillvector
              (shen/vector 10)
              1 10 'shen\.-null-))
            (shen/let Counter
                      (shen/address->
                       (shen/value 'shen\.*varcounter*)
                       V2161 1)
                      V2161)))
(defun shen/shen\.f_error
    (V3913)
  (shen/do
   (shen/shen\.prhush
    (shen/cn "partial function "
             (shen/shen\.app V3913 ";\n" 'shen\.a))
    (shen/stoutput))
   (shen/do
    (shen/if
     (shen/and
      (shen/not
       (shen/shen\.tracked\? V3913))
      (shen/y-or-n\?
       (shen/cn "track "
                (shen/shen\.app V3913 "? " 'shen\.a))))
     (shen/shen\.track-function
      (shen/ps V3913))
     'shen\.ok)
    (shen/simple-error "aborted"))))
(defun shen/shen\.tracked\?
    (V3915)
  (shen/element\? V3915
                  (shen/value 'shen\.*tracking*)))
(defun shen/track
    (V3917)
  (shen/let Source
            (shen/ps V3917)
            (shen/shen\.track-function Source)))
(defun shen/shen\.track-function
    (V3919)
  (shen/cond
   ((shen/and
     (shen/cons\? V3919)
     (shen/and
      (shen/= 'defun
              (shen/hd V3919))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V3919))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V3919))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V3919))
         (shen/internal/predicate->shen
          (null
           (nthcdr 4 V3919))))))))
    (shen/let KL
              (list 'defun
                    (shen/hd
                     (nthcdr 1 V3919))
                    (shen/hd
                     (nthcdr 2 V3919))
                    (shen/shen\.insert-tracking-code
                     (shen/hd
                      (nthcdr 1 V3919))
                     (shen/hd
                      (nthcdr 2 V3919))
                     (shen/hd
                      (nthcdr 3 V3919))))
              (shen/let Ob
                        (shen/eval-kl KL)
                        (shen/let Tr
                                  (shen/set 'shen\.*tracking*
                                            (append
                                             (list Ob)
                                             (shen/value 'shen\.*tracking*)))
                                  Ob))))
   (shen/true
    (shen/shen\.f_error 'shen\.track-function))))
(defun shen/shen\.insert-tracking-code
    (V3923 V3924 V3925)
  (list 'do
        (list 'set 'shen\.*call*
              (list '+
                    (list 'value 'shen\.*call*)
                    1))
        (list 'do
              (list 'shen\.input-track
                    (list 'value 'shen\.*call*)
                    V3923
                    (shen/shen\.cons_form V3924))
              (list 'do
                    (list 'shen\.terpri-or-read-char)
                    (list 'let 'Result V3925
                          (list 'do
                                (list 'shen\.output-track
                                      (list 'value 'shen\.*call*)
                                      V3923 'Result)
                                (list 'do
                                      (list 'set 'shen\.*call*
                                            (list '-
                                                  (list 'value 'shen\.*call*)
                                                  1))
                                      (list 'do
                                            (list 'shen\.terpri-or-read-char)
                                            'Result))))))))
(shen/set 'shen\.*step* 'false)
(defun shen/step
    (V3931)
  (shen/cond
   ((shen/= '+ V3931)
    (shen/set 'shen\.*step* 'true))
   ((shen/= '- V3931)
    (shen/set 'shen\.*step* 'false))
   (shen/true
    (shen/simple-error "step expects a + or a -.\n"))))
(defun shen/spy
    (V3937)
  (shen/cond
   ((shen/= '+ V3937)
    (shen/set 'shen\.*spy* 'true))
   ((shen/= '- V3937)
    (shen/set 'shen\.*spy* 'false))
   (shen/true
    (shen/simple-error "spy expects a + or a -.\n"))))
(defun shen/shen\.terpri-or-read-char nil
  (shen/if
   (shen/value 'shen\.*step*)
   (shen/shen\.check-byte
    (shen/read-byte
     (shen/value '*stinput*)))
   (shen/nl 1)))
(defun shen/shen\.check-byte
    (V3943)
  (shen/cond
   ((shen/= V3943
            (shen/shen\.hat))
    (shen/simple-error "aborted"))
   (shen/true 'true)))
(defun shen/shen\.input-track
    (V3947 V3948 V3949)
  (shen/do
   (shen/shen\.prhush
    (shen/cn "\n"
             (shen/shen\.app
              (shen/shen\.spaces V3947)
              (shen/cn "<"
                       (shen/shen\.app V3947
                                       (shen/cn "> Inputs to "
                                                (shen/shen\.app V3948
                                                                (shen/cn " \n"
                                                                         (shen/shen\.app
                                                                          (shen/shen\.spaces V3947)
                                                                          "" 'shen\.a))
                                                                'shen\.a))
                                       'shen\.a))
              'shen\.a))
    (shen/stoutput))
   (shen/shen\.recursively-print V3949)))
(defun shen/shen\.recursively-print
    (V3951)
  (cl-flet
      ((tail-trampoline
        (V3951)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3951))
          (shen/shen\.prhush " ==>"
                             (shen/stoutput)))
         ((shen/cons\? V3951)
          (shen/do
           (shen/print
            (shen/hd V3951))
           (shen/do
            (shen/shen\.prhush ", "
                               (shen/stoutput))
            (vector
             (list
              (nthcdr 1 V3951))))))
         (shen/true
          (shen/shen\.f_error 'shen\.recursively-print)))))
    (let
        ((result
          (funcall #'tail-trampoline V3951)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.spaces
    (V3953)
  (shen/cond
   ((shen/= 0 V3953)
    "")
   (shen/true
    (shen/cn " "
             (shen/shen\.spaces
              (shen/- V3953 1))))))
(defun shen/shen\.output-track
    (V3957 V3958 V3959)
  (shen/shen\.prhush
   (shen/cn "\n"
            (shen/shen\.app
             (shen/shen\.spaces V3957)
             (shen/cn "<"
                      (shen/shen\.app V3957
                                      (shen/cn "> Output from "
                                               (shen/shen\.app V3958
                                                               (shen/cn " \n"
                                                                        (shen/shen\.app
                                                                         (shen/shen\.spaces V3957)
                                                                         (shen/cn "==> "
                                                                                  (shen/shen\.app V3959 "" 'shen\.s))
                                                                         'shen\.a))
                                                               'shen\.a))
                                      'shen\.a))
             'shen\.a))
   (shen/stoutput)))
(defun shen/untrack
    (F)
  (progn
    (shen/set shen\.*tracking*
              (shen/internal/delete-first-eq F
                                             (shen/value shen\.*tracking*)))
    (shen/eval
     (shen/ps F))))
(defun shen/profile
    (V3963)
  (shen/shen\.profile-help
   (shen/ps V3963)))
(defun shen/shen\.profile-help
    (V3969)
  (shen/cond
   ((shen/and
     (shen/cons\? V3969)
     (shen/and
      (shen/= 'defun
              (shen/hd V3969))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V3969))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V3969))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V3969))
         (shen/internal/predicate->shen
          (null
           (nthcdr 4 V3969))))))))
    (shen/let G
              (shen/gensym 'shen\.f)
              (shen/let Profile
                        (list 'defun
                              (shen/hd
                               (nthcdr 1 V3969))
                              (shen/hd
                               (nthcdr 2 V3969))
                              (shen/shen\.profile-func
                               (shen/hd
                                (nthcdr 1 V3969))
                               (shen/hd
                                (nthcdr 2 V3969))
                               (append
                                (list G)
                                (shen/hd
                                 (nthcdr 2 V3969)))))
                        (shen/let Def
                                  (list 'defun G
                                        (shen/hd
                                         (nthcdr 2 V3969))
                                        (shen/subst G
                                                    (shen/hd
                                                     (nthcdr 1 V3969))
                                                    (shen/hd
                                                     (nthcdr 3 V3969))))
                                  (shen/let CompileProfile
                                            (shen/shen\.eval-without-macros Profile)
                                            (shen/let CompileG
                                                      (shen/shen\.eval-without-macros Def)
                                                      (shen/hd
                                                       (nthcdr 1 V3969))))))))
   (shen/true
    (shen/simple-error "Cannot profile.\n"))))
(defun shen/unprofile
    (V3971)
  (shen/untrack V3971))
(defun shen/shen\.profile-func
    (V3975 V3976 V3977)
  (list 'let 'Start
        (list 'get-time 'run)
        (list 'let 'Result V3977
              (list 'let 'Finish
                    (list '-
                          (list 'get-time 'run)
                          'Start)
                    (list 'let 'Record
                          (list 'shen\.put-profile V3975
                                (list '+
                                      (list 'shen\.get-profile V3975)
                                      'Finish))
                          'Result)))))
(defun shen/profile-results
    (V3979)
  (shen/let Results
            (shen/shen\.get-profile V3979)
            (shen/let Initialise
                      (shen/shen\.put-profile V3979 0)
                      (shen/@p V3979 Results))))
(defun shen/shen\.get-profile
    (V3981)
  (shen/get/or V3981 'profile
               (shen/freeze 0)
               (shen/value '*property-vector*)))
(defun shen/shen\.put-profile
    (V3984 V3985)
  (shen/put V3984 'profile V3985
            (shen/value '*property-vector*)))
(defun shen/load
    (V1482)
  (shen/let Load
            (shen/let Start
                      (shen/get-time 'run)
                      (shen/let Result
                                (shen/shen\.load-help
                                 (shen/value 'shen\.*tc*)
                                 (shen/read-file V1482))
                                (shen/let Finish
                                          (shen/get-time 'run)
                                          (shen/let Time
                                                    (shen/- Finish Start)
                                                    (shen/let Message
                                                              (shen/shen\.prhush
                                                               (shen/cn "\nrun time: "
                                                                        (shen/cn
                                                                         (shen/str Time)
                                                                         " secs\n"))
                                                               (shen/stoutput))
                                                              Result)))))
            (shen/let Infs
                      (shen/if
                       (shen/value 'shen\.*tc*)
                       (shen/shen\.prhush
                        (shen/cn "\ntypechecked in "
                                 (shen/shen\.app
                                  (shen/inferences)
                                  " inferences\n" 'shen\.a))
                        (shen/stoutput))
                       'shen\.skip)
                      'loaded)))
(defun shen/shen\.load-help
    (V1489 V1490)
  (shen/cond
   ((shen/= 'false V1489)
    (shen/for-each
     (shen/lambda X
                  (shen/shen\.prhush
                   (shen/shen\.app
                    (shen/shen\.eval-without-macros X)
                    "\n" 'shen\.s)
                   (shen/stoutput)))
     V1490))
   (shen/true
    (shen/let RemoveSynonyms
              (shen/mapcan
               (shen/lambda X
                            (shen/shen\.remove-synonyms X))
               V1490)
              (shen/let Table
                        (shen/mapcan
                         (shen/lambda X
                                      (shen/shen\.typetable X))
                         RemoveSynonyms)
                        (shen/let Assume
                                  (shen/for-each
                                   (shen/lambda X
                                                (shen/shen\.assumetype X))
                                   Table)
                                  (shen/trap-error
                                   (shen/for-each
                                    (shen/lambda X
                                                 (shen/shen\.typecheck-and-load X))
                                    RemoveSynonyms)
                                   (shen/lambda E
                                                (shen/shen\.unwind-types E Table)))))))))
(defun shen/shen\.remove-synonyms
    (V1492)
  (shen/cond
   ((shen/and
     (shen/cons\? V1492)
     (shen/= 'shen\.synonyms-help
             (shen/hd V1492)))
    (shen/do
     (shen/eval V1492)
     nil))
   (shen/true
    (list V1492))))
(defun shen/shen\.typecheck-and-load
    (V1494)
  (shen/do
   (shen/nl 1)
   (shen/shen\.typecheck-and-evaluate V1494
                                      (shen/gensym 'A))))
(defun shen/shen\.typetable
    (V1500)
  (shen/cond
   ((shen/and
     (shen/cons\? V1500)
     (shen/and
      (shen/= 'define
              (shen/hd V1500))
      (shen/cons\?
       (nthcdr 1 V1500))))
    (shen/let Sig
              (shen/compile
               (shen/lambda Y
                            (shen/shen\.<sig+rest> Y))
               (nthcdr 2 V1500)
               (shen/lambda E
                            (shen/simple-error
                             (shen/shen\.app
                              (shen/hd
                               (nthcdr 1 V1500))
                              " lacks a proper signature.\n" 'shen\.a))))
              (list
               (append
                (list
                 (shen/hd
                  (nthcdr 1 V1500)))
                Sig))))
   (shen/true nil)))
(defun shen/shen\.assumetype
    (V1502)
  (shen/cond
   ((shen/cons\? V1502)
    (shen/declare
     (shen/hd V1502)
     (nthcdr 1 V1502)))
   (shen/true
    (shen/shen\.f_error 'shen\.assumetype))))
(defun shen/shen\.unwind-types
    (V1509 V1510)
  (cl-flet
      ((tail-trampoline
        (V1509 V1510)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1510))
          (shen/simple-error
           (shen/error-to-string V1509)))
         ((shen/and
           (shen/cons\? V1510)
           (shen/cons\?
            (shen/hd V1510)))
          (shen/do
           (shen/shen\.remtype
            (shen/hd
             (shen/hd V1510)))
           (vector
            (list V1509
                  (nthcdr 1 V1510)))))
         (shen/true
          (shen/shen\.f_error 'shen\.unwind-types)))))
    (let
        ((result
          (funcall #'tail-trampoline V1509 V1510)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.remtype
    (V1512)
  (shen/set 'shen\.*signedfuncs*
            (shen/shen\.removetype V1512
                                   (shen/value 'shen\.*signedfuncs*))))
(defun shen/shen\.removetype
    (V1520 V1521)
  (cl-flet
      ((tail-trampoline
        (V1520 V1521)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1521))
          nil)
         ((shen/and
           (shen/cons\? V1521)
           (shen/and
            (shen/cons\?
             (shen/hd V1521))
            (shen/=
             (shen/hd
              (shen/hd V1521))
             V1520)))
          (vector
           (list
            (shen/hd
             (shen/hd V1521))
            (nthcdr 1 V1521))))
         ((shen/cons\? V1521)
          (append
           (list
            (shen/hd V1521))
           (shen/shen\.removetype V1520
                                  (nthcdr 1 V1521))))
         (shen/true
          (shen/shen\.f_error 'shen\.removetype)))))
    (let
        ((result
          (funcall #'tail-trampoline V1520 V1521)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.<sig+rest>
    (V1523)
  (shen/let Parse_shen\.<signature>
            (shen/shen\.<signature> V1523)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<signature>))
             (shen/let Parse_<!>
                       (shen/<!> Parse_shen\.<signature>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_<!>))
                        (shen/shen\.pair
                         (shen/hd Parse_<!>)
                         (shen/shen\.hdtl Parse_shen\.<signature>))
                        (shen/fail)))
             (shen/fail))))
(defun shen/write-to-file
    (V1526 V1527)
  (shen/let Stream
            (shen/open V1526 'out)
            (shen/let String
                      (shen/if
                       (shen/string\? V1527)
                       (shen/shen\.app V1527 "\n\n" 'shen\.a)
                       (shen/shen\.app V1527 "\n\n" 'shen\.s))
                      (shen/let Write
                                (shen/pr String Stream)
                                (shen/let Close
                                          (shen/close Stream)
                                          V1527)))))
(defun shen/pr
    (V4012 V4013)
  (shen/trap-error
   (shen/shen\.prh V4012 V4013 0)
   (shen/lambda E V4012)))
(defun shen/shen\.prh
    (V4017 V4018 V4019)
  (cl-flet
      ((tail-trampoline
        (V4017 V4018 V4019)
        (vector
         (list V4017 V4018
               (shen/shen\.write-char-and-inc V4017 V4018 V4019)))))
    (let
        ((result
          (funcall #'tail-trampoline V4017 V4018 V4019)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.write-char-and-inc
    (V4023 V4024 V4025)
  (shen/do
   (shen/write-byte
    (shen/string->n
     (shen/pos V4023 V4025))
    V4024)
   (1+ V4025)))
(defun shen/print
    (V4027)
  (shen/let String
            (shen/shen\.insert V4027 "~S")
            (shen/let Print
                      (shen/shen\.prhush String
                                         (shen/stoutput))
                      V4027)))
(defun shen/shen\.prhush
    (V4030 V4031)
  (shen/if
   (shen/value '*hush*)
   V4030
   (shen/pr V4030 V4031)))
(defun shen/shen\.mkstr
    (V4034 V4035)
  (shen/cond
   ((shen/string\? V4034)
    (shen/shen\.mkstr-l
     (shen/shen\.proc-nl V4034)
     V4035))
   (shen/true
    (shen/shen\.mkstr-r
     (list 'shen\.proc-nl V4034)
     V4035))))
(defun shen/shen\.mkstr-l
    (V4038 V4039)
  (cl-flet
      ((tail-trampoline
        (V4038 V4039)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V4039))
          V4038)
         ((shen/cons\? V4039)
          (vector
           (list
            (shen/shen\.insert-l
             (shen/hd V4039)
             V4038)
            (nthcdr 1 V4039))))
         (shen/true
          (shen/shen\.f_error 'shen\.mkstr-l)))))
    (let
        ((result
          (funcall #'tail-trampoline V4038 V4039)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.insert-l
    (V4044 V4045)
  (shen/cond
   ((shen/= "" V4045)
    "")
   ((shen/and
     (shen/shen\.+string\? V4045)
     (shen/and
      (shen/= "~"
              (shen/pos V4045 0))
      (shen/and
       (shen/shen\.+string\?
        (shen/tlstr V4045))
       (shen/= "A"
               (shen/pos
                (shen/tlstr V4045)
                0)))))
    (list 'shen\.app V4044
          (shen/tlstr
           (shen/tlstr V4045))
          'shen\.a))
   ((shen/and
     (shen/shen\.+string\? V4045)
     (shen/and
      (shen/= "~"
              (shen/pos V4045 0))
      (shen/and
       (shen/shen\.+string\?
        (shen/tlstr V4045))
       (shen/= "R"
               (shen/pos
                (shen/tlstr V4045)
                0)))))
    (list 'shen\.app V4044
          (shen/tlstr
           (shen/tlstr V4045))
          'shen\.r))
   ((shen/and
     (shen/shen\.+string\? V4045)
     (shen/and
      (shen/= "~"
              (shen/pos V4045 0))
      (shen/and
       (shen/shen\.+string\?
        (shen/tlstr V4045))
       (shen/= "S"
               (shen/pos
                (shen/tlstr V4045)
                0)))))
    (list 'shen\.app V4044
          (shen/tlstr
           (shen/tlstr V4045))
          'shen\.s))
   ((shen/shen\.+string\? V4045)
    (shen/shen\.factor-cn
     (list 'cn
           (shen/pos V4045 0)
           (shen/shen\.insert-l V4044
                                (shen/tlstr V4045)))))
   ((shen/and
     (shen/cons\? V4045)
     (shen/and
      (shen/= 'cn
              (shen/hd V4045))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V4045))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V4045))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V4045)))))))
    (list 'cn
          (shen/hd
           (nthcdr 1 V4045))
          (shen/shen\.insert-l V4044
                               (shen/hd
                                (nthcdr 2 V4045)))))
   ((shen/and
     (shen/cons\? V4045)
     (shen/and
      (shen/= 'shen\.app
              (shen/hd V4045))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V4045))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V4045))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V4045))
         (shen/internal/predicate->shen
          (null
           (nthcdr 4 V4045))))))))
    (append
     (list 'shen\.app
           (shen/hd
            (nthcdr 1 V4045))
           (shen/shen\.insert-l V4044
                                (shen/hd
                                 (nthcdr 2 V4045))))
     (nthcdr 3 V4045)))
   (shen/true
    (shen/shen\.f_error 'shen\.insert-l))))
(defun shen/shen\.factor-cn
    (V4047)
  (shen/cond
   ((shen/and
     (shen/cons\? V4047)
     (shen/and
      (shen/= 'cn
              (shen/hd V4047))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V4047))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V4047))
        (shen/and
         (shen/cons\?
          (shen/hd
           (nthcdr 2 V4047)))
         (shen/and
          (shen/= 'cn
                  (shen/hd
                   (shen/hd
                    (nthcdr 2 V4047))))
          (shen/and
           (shen/cons\?
            (nthcdr 1
                    (shen/hd
                     (nthcdr 2 V4047))))
           (shen/and
            (shen/cons\?
             (nthcdr 2
                     (shen/hd
                      (nthcdr 2 V4047))))
            (shen/and
             (shen/internal/predicate->shen
              (null
               (nthcdr 3
                       (shen/hd
                        (nthcdr 2 V4047)))))
             (shen/and
              (shen/internal/predicate->shen
               (null
                (nthcdr 3 V4047)))
              (shen/and
               (shen/string\?
                (shen/hd
                 (nthcdr 1 V4047)))
               (shen/string\?
                (shen/hd
                 (nthcdr 1
                         (shen/hd
                          (nthcdr 2 V4047))))))))))))))))
    (append
     (list 'cn
           (shen/cn
            (shen/hd
             (nthcdr 1 V4047))
            (shen/hd
             (nthcdr 1
                     (shen/hd
                      (nthcdr 2 V4047))))))
     (nthcdr 2
             (shen/hd
              (nthcdr 2 V4047)))))
   (shen/true V4047)))
(defun shen/shen\.proc-nl
    (V4049)
  (shen/cond
   ((shen/= "" V4049)
    "")
   ((shen/and
     (shen/shen\.+string\? V4049)
     (shen/and
      (shen/= "~"
              (shen/pos V4049 0))
      (shen/and
       (shen/shen\.+string\?
        (shen/tlstr V4049))
       (shen/= "%"
               (shen/pos
                (shen/tlstr V4049)
                0)))))
    (shen/cn
     (shen/n->string 10)
     (shen/shen\.proc-nl
      (shen/tlstr
       (shen/tlstr V4049)))))
   ((shen/shen\.+string\? V4049)
    (shen/cn
     (shen/pos V4049 0)
     (shen/shen\.proc-nl
      (shen/tlstr V4049))))
   (shen/true
    (shen/shen\.f_error 'shen\.proc-nl))))
(defun shen/shen\.mkstr-r
    (V4052 V4053)
  (cl-flet
      ((tail-trampoline
        (V4052 V4053)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V4053))
          V4052)
         ((shen/cons\? V4053)
          (vector
           (list
            (list 'shen\.insert
                  (shen/hd V4053)
                  V4052)
            (nthcdr 1 V4053))))
         (shen/true
          (shen/shen\.f_error 'shen\.mkstr-r)))))
    (let
        ((result
          (funcall #'tail-trampoline V4052 V4053)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.insert
    (V4056 V4057)
  (shen/shen\.insert-h V4056 V4057 ""))
(defun shen/shen\.insert-h
    (V4063 V4064 V4065)
  (cl-flet
      ((tail-trampoline
        (V4063 V4064 V4065)
        (shen/cond
         ((shen/= "" V4064)
          V4065)
         ((shen/and
           (shen/shen\.+string\? V4064)
           (shen/and
            (shen/= "~"
                    (shen/pos V4064 0))
            (shen/and
             (shen/shen\.+string\?
              (shen/tlstr V4064))
             (shen/= "A"
                     (shen/pos
                      (shen/tlstr V4064)
                      0)))))
          (shen/cn V4065
                   (shen/shen\.app V4063
                                   (shen/tlstr
                                    (shen/tlstr V4064))
                                   'shen\.a)))
         ((shen/and
           (shen/shen\.+string\? V4064)
           (shen/and
            (shen/= "~"
                    (shen/pos V4064 0))
            (shen/and
             (shen/shen\.+string\?
              (shen/tlstr V4064))
             (shen/= "R"
                     (shen/pos
                      (shen/tlstr V4064)
                      0)))))
          (shen/cn V4065
                   (shen/shen\.app V4063
                                   (shen/tlstr
                                    (shen/tlstr V4064))
                                   'shen\.r)))
         ((shen/and
           (shen/shen\.+string\? V4064)
           (shen/and
            (shen/= "~"
                    (shen/pos V4064 0))
            (shen/and
             (shen/shen\.+string\?
              (shen/tlstr V4064))
             (shen/= "S"
                     (shen/pos
                      (shen/tlstr V4064)
                      0)))))
          (shen/cn V4065
                   (shen/shen\.app V4063
                                   (shen/tlstr
                                    (shen/tlstr V4064))
                                   'shen\.s)))
         ((shen/shen\.+string\? V4064)
          (vector
           (list V4063
                 (shen/tlstr V4064)
                 (shen/cn V4065
                          (shen/pos V4064 0)))))
         (shen/true
          (shen/shen\.f_error 'shen\.insert-h)))))
    (let
        ((result
          (funcall #'tail-trampoline V4063 V4064 V4065)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.app
    (V4069 V4070 V4071)
  (shen/cn
   (shen/shen\.arg->str V4069 V4071)
   V4070))
(defun shen/shen\.arg->str
    (V4079 V4080)
  (shen/cond
   ((shen/= V4079
            (shen/fail))
    "...")
   ((shen/shen\.list\? V4079)
    (shen/shen\.list->str V4079 V4080))
   ((shen/string\? V4079)
    (shen/shen\.str->str V4079 V4080))
   ((shen/absvector\? V4079)
    (shen/shen\.vector->str V4079 V4080))
   (shen/true
    (shen/shen\.atom->str V4079))))
(defun shen/shen\.list->str
    (V4083 V4084)
  (shen/cond
   ((shen/= 'shen\.r V4084)
    (concat
     (concat "("
             (shen/shen\.iter-list V4083 'shen\.r
                                   (shen/shen\.maxseq)))
     ")"))
   (shen/true
    (concat
     (concat "["
             (shen/shen\.iter-list V4083 V4084
                                   (shen/shen\.maxseq)))
     "]"))))
(defun shen/shen\.maxseq nil
  (shen/value '*maximum-print-sequence-size*))
(defun shen/shen\.iter-list
    (V4098 V4099 V4100)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V4098))
    "")
   ((shen/= 0 V4100)
    "... etc")
   ((shen/and
     (shen/cons\? V4098)
     (shen/internal/predicate->shen
      (null
       (nthcdr 1 V4098))))
    (shen/shen\.arg->str
     (shen/hd V4098)
     V4099))
   ((shen/cons\? V4098)
    (concat
     (concat
      (shen/shen\.arg->str
       (shen/hd V4098)
       V4099)
      " ")
     (shen/shen\.iter-list
      (nthcdr 1 V4098)
      V4099
      (shen/- V4100 1))))
   (shen/true
    (concat
     (concat "|" " ")
     (shen/shen\.arg->str V4098 V4099)))))
(defun shen/shen\.str->str
    (V4107 V4108)
  (shen/cond
   ((shen/= 'shen\.a V4108)
    V4107)
   (shen/true
    (concat
     (concat
      (shen/n->string 34)
      V4107)
     (shen/n->string 34)))))
(defun shen/shen\.vector->str
    (V4111 V4112)
  (shen/if
   (shen/shen\.print-vector\? V4111)
   (shen/internal/apply-function-expression
    (shen/function
     (shen/<-address V4111 0))
    (list V4111))
   (shen/if
    (shen/vector\? V4111)
    (concat
     (concat "<"
             (shen/shen\.iter-vector V4111 1 V4112
                                     (shen/shen\.maxseq)))
     ">")
    (concat
     (concat "<" "<"
             (shen/shen\.iter-vector V4111 0 V4112
                                     (shen/shen\.maxseq)))
     ">>"))))
(defun shen/shen\.print-vector\?
    (V4114)
  (shen/let Zero
            (shen/<-address V4114 0)
            (shen/if
             (shen/= Zero 'shen\.tuple)
             'true
             (shen/if
              (shen/= Zero 'shen\.pvar)
              'true
              (shen/if
               (shen/= Zero 'shen\.dictionary)
               'true
               (shen/if
                (shen/not
                 (shen/number\? Zero))
                (shen/shen\.fbound\? Zero)
                'false))))))
(defun shen/shen\.fbound\?
    (V4116)
  (shen/trap-error
   (shen/do
    (shen/shen\.lookup-func V4116)
    'true)
   (shen/lambda E 'false)))
(defun shen/shen\.tuple
    (V4118)
  (shen/cn "(@p "
           (shen/shen\.app
            (shen/<-address V4118 1)
            (shen/cn " "
                     (shen/shen\.app
                      (shen/<-address V4118 2)
                      ")" 'shen\.s))
            'shen\.s)))
(defun shen/shen\.dictionary
    (V4120)
  "(dict ...)")
(defun shen/shen\.iter-vector
    (V4131 V4132 V4133 V4134)
  (shen/cond
   ((shen/= 0 V4134)
    "... etc")
   (shen/true
    (shen/let Item
              (shen/<-address/or V4131 V4132
                                 (shen/freeze 'shen\.out-of-bounds))
              (shen/let Next
                        (shen/<-address/or V4131
                                           (1+ V4132)
                                           (shen/freeze 'shen\.out-of-bounds))
                        (shen/if
                         (shen/= Item 'shen\.out-of-bounds)
                         ""
                         (shen/if
                          (shen/= Next 'shen\.out-of-bounds)
                          (shen/shen\.arg->str Item V4133)
                          (concat
                           (concat
                            (shen/shen\.arg->str Item V4133)
                            " ")
                           (shen/shen\.iter-vector V4131
                                                   (1+ V4132)
                                                   V4133
                                                   (shen/- V4134 1))))))))))
(defun shen/shen\.atom->str
    (V4136)
  (shen/trap-error
   (shen/str V4136)
   (shen/lambda E
                (shen/shen\.funexstring))))
(defun shen/shen\.funexstring nil
  (concat
   (concat "" "f" "u" "n" "e"
           (shen/shen\.arg->str
            (shen/gensym
             (shen/intern "x"))
            'shen\.a))
   ""))
(defun shen/shen\.list\?
    (V4138)
  (shen/or
   (shen/empty\? V4138)
   (shen/cons\? V4138)))
(defun shen/macroexpand
    (V1529)
  (shen/let Y
            (shen/shen\.compose
             (shen/value '*macros*)
             V1529)
            (shen/if
             (shen/= V1529 Y)
             V1529
             (shen/shen\.walk
              (shen/lambda Z
                           (shen/macroexpand Z))
              Y))))
(defun shen/shen\.error-macro
    (V1531)
  (shen/cond
   ((shen/and
     (shen/cons\? V1531)
     (shen/and
      (shen/= 'error
              (shen/hd V1531))
      (shen/cons\?
       (nthcdr 1 V1531))))
    (list 'simple-error
          (shen/shen\.mkstr
           (shen/hd
            (nthcdr 1 V1531))
           (nthcdr 2 V1531))))
   (shen/true V1531)))
(defun shen/shen\.output-macro
    (V1533)
  (shen/cond
   ((shen/and
     (shen/cons\? V1533)
     (shen/and
      (shen/= 'output
              (shen/hd V1533))
      (shen/cons\?
       (nthcdr 1 V1533))))
    (list 'shen\.prhush
          (shen/shen\.mkstr
           (shen/hd
            (nthcdr 1 V1533))
           (nthcdr 2 V1533))
          (list 'stoutput)))
   ((shen/and
     (shen/cons\? V1533)
     (shen/and
      (shen/= 'pr
              (shen/hd V1533))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1533))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1533))))))
    (list 'pr
          (shen/hd
           (nthcdr 1 V1533))
          (list 'stoutput)))
   (shen/true V1533)))
(defun shen/shen\.make-string-macro
    (V1535)
  (shen/cond
   ((shen/and
     (shen/cons\? V1535)
     (shen/and
      (shen/= 'make-string
              (shen/hd V1535))
      (shen/cons\?
       (nthcdr 1 V1535))))
    (shen/shen\.mkstr
     (shen/hd
      (nthcdr 1 V1535))
     (nthcdr 2 V1535)))
   (shen/true V1535)))
(defun shen/shen\.input-macro
    (V1537)
  (shen/cond
   ((shen/and
     (shen/cons\? V1537)
     (shen/and
      (shen/= 'lineread
              (shen/hd V1537))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1537)))))
    (list 'lineread
          (list 'stinput)))
   ((shen/and
     (shen/cons\? V1537)
     (shen/and
      (shen/= 'input
              (shen/hd V1537))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1537)))))
    (list 'input
          (list 'stinput)))
   ((shen/and
     (shen/cons\? V1537)
     (shen/and
      (shen/= 'read
              (shen/hd V1537))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1537)))))
    (list 'read
          (list 'stinput)))
   ((shen/and
     (shen/cons\? V1537)
     (shen/and
      (shen/= 'input+
              (shen/hd V1537))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1537))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1537))))))
    (list 'input+
          (shen/hd
           (nthcdr 1 V1537))
          (list 'stinput)))
   ((shen/and
     (shen/cons\? V1537)
     (shen/and
      (shen/= 'read-byte
              (shen/hd V1537))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1537)))))
    (list 'read-byte
          (list 'stinput)))
   ((shen/and
     (shen/cons\? V1537)
     (shen/and
      (shen/= 'read-char-code
              (shen/hd V1537))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1537)))))
    (list 'read-char-code
          (list 'stinput)))
   (shen/true V1537)))
(defun shen/shen\.compose
    (Fs X)
  (let
      ((Result X))
    (dolist
        (F Fs Result)
      (setq Result
            (funcall F Result)))))
(defun shen/shen\.compile-macro
    (V1543)
  (shen/cond
   ((shen/and
     (shen/cons\? V1543)
     (shen/and
      (shen/= 'compile
              (shen/hd V1543))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1543))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1543))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1543)))))))
    (list 'compile
          (shen/hd
           (nthcdr 1 V1543))
          (shen/hd
           (nthcdr 2 V1543))
          (list 'lambda 'E
                (list 'if
                      (list 'cons\? 'E)
                      (list 'error "parse error here: ~S~%" 'E)
                      (list 'error "parse error~%")))))
   (shen/true V1543)))
(defun shen/shen\.prolog-macro
    (V1545)
  (shen/cond
   ((shen/and
     (shen/cons\? V1545)
     (shen/= 'prolog\?
             (shen/hd V1545)))
    (shen/let F
              (shen/gensym 'shen\.f)
              (shen/let Receive
                        (shen/shen\.receive-terms
                         (nthcdr 1 V1545))
                        (shen/let PrologDef
                                  (shen/eval
                                   (shen/append
                                    (list 'defprolog F)
                                    (shen/append Receive
                                                 (shen/append
                                                  (list '<--)
                                                  (shen/append
                                                   (shen/shen\.pass-literals
                                                    (nthcdr 1 V1545))
                                                   (list '\;))))))
                                  (shen/let Query
                                            (append
                                             (list F)
                                             (shen/append Receive
                                                          (list
                                                           (list 'shen\.start-new-prolog-process)
                                                           (list 'freeze 'true))))
                                            Query)))))
   (shen/true V1545)))
(defun shen/shen\.receive-terms
    (V1551)
  (cl-flet
      ((tail-trampoline
        (V1551)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1551))
          nil)
         ((shen/and
           (shen/cons\? V1551)
           (shen/and
            (shen/cons\?
             (shen/hd V1551))
            (shen/and
             (shen/= 'receive
                     (shen/hd
                      (shen/hd V1551)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1551)))
              (shen/internal/predicate->shen
               (null
                (nthcdr 2
                        (shen/hd V1551))))))))
          (append
           (list
            (shen/hd
             (nthcdr 1
                     (shen/hd V1551))))
           (shen/shen\.receive-terms
            (nthcdr 1 V1551))))
         ((shen/cons\? V1551)
          (vector
           (list
            (nthcdr 1 V1551))))
         (shen/true
          (shen/shen\.f_error 'shen\.receive-terms)))))
    (let
        ((result
          (funcall #'tail-trampoline V1551)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.pass-literals
    (V1555)
  (cl-flet
      ((tail-trampoline
        (V1555)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1555))
          nil)
         ((shen/and
           (shen/cons\? V1555)
           (shen/and
            (shen/cons\?
             (shen/hd V1555))
            (shen/and
             (shen/= 'receive
                     (shen/hd
                      (shen/hd V1555)))
             (shen/and
              (shen/cons\?
               (nthcdr 1
                       (shen/hd V1555)))
              (shen/internal/predicate->shen
               (null
                (nthcdr 2
                        (shen/hd V1555))))))))
          (vector
           (list
            (nthcdr 1 V1555))))
         ((shen/cons\? V1555)
          (append
           (list
            (shen/hd V1555))
           (shen/shen\.pass-literals
            (nthcdr 1 V1555))))
         (shen/true
          (shen/shen\.f_error 'shen\.pass-literals)))))
    (let
        ((result
          (funcall #'tail-trampoline V1555)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.defprolog-macro
    (V1557)
  (shen/cond
   ((shen/and
     (shen/cons\? V1557)
     (shen/and
      (shen/= 'defprolog
              (shen/hd V1557))
      (shen/cons\?
       (nthcdr 1 V1557))))
    (shen/compile
     (shen/lambda Y
                  (shen/shen\.<defprolog> Y))
     (nthcdr 1 V1557)
     (shen/lambda Y
                  (shen/shen\.prolog-error
                   (shen/hd
                    (nthcdr 1 V1557))
                   Y))))
   (shen/true V1557)))
(defun shen/shen\.datatype-macro
    (V1559)
  (shen/cond
   ((shen/and
     (shen/cons\? V1559)
     (shen/and
      (shen/= 'datatype
              (shen/hd V1559))
      (shen/cons\?
       (nthcdr 1 V1559))))
    (list 'shen\.process-datatype
          (shen/shen\.intern-type
           (shen/hd
            (nthcdr 1 V1559)))
          (list 'compile
                (list 'lambda 'X
                      (list 'shen\.<datatype-rules> 'X))
                (shen/shen\.rcons_form
                 (nthcdr 2 V1559))
                (list 'function 'shen\.datatype-error))))
   (shen/true V1559)))
(defun shen/shen\.intern-type
    (V1561)
  (shen/intern
   (shen/cn "type#"
            (shen/str V1561))))
(defun shen/shen\.@s-macro
    (V1563)
  (cl-flet
      ((tail-trampoline
        (V1563)
        (shen/cond
         ((shen/and
           (shen/cons\? V1563)
           (shen/and
            (shen/= '@s
                    (shen/hd V1563))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1563))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1563))
              (shen/cons\?
               (nthcdr 3 V1563))))))
          (list '@s
                (shen/hd
                 (nthcdr 1 V1563))
                (shen/shen\.@s-macro
                 (append
                  (list '@s)
                  (nthcdr 2 V1563)))))
         ((shen/and
           (shen/cons\? V1563)
           (shen/and
            (shen/= '@s
                    (shen/hd V1563))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V1563))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V1563))
              (shen/and
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V1563)))
               (shen/string\?
                (shen/hd
                 (nthcdr 1 V1563))))))))
          (shen/let E
                    (shen/explode
                     (shen/hd
                      (nthcdr 1 V1563)))
                    (shen/if
                     (shen/>
                      (shen/length E)
                      1)
                     (vector
                      (list
                       (append
                        (list '@s)
                        (shen/append E
                                     (nthcdr 2 V1563)))))
                     V1563)))
         (shen/true V1563))))
    (let
        ((result
          (funcall #'tail-trampoline V1563)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.synonyms-macro
    (V1565)
  (shen/cond
   ((shen/and
     (shen/cons\? V1565)
     (shen/= 'synonyms
             (shen/hd V1565)))
    (list 'shen\.synonyms-help
          (shen/shen\.rcons_form
           (shen/shen\.curry-synonyms
            (nthcdr 1 V1565)))))
   (shen/true V1565)))
(defun shen/shen\.curry-synonyms
    (V1567)
  (shen/map
   (shen/lambda X
                (shen/shen\.curry-type X))
   V1567))
(defun shen/shen\.nl-macro
    (V1569)
  (shen/cond
   ((shen/and
     (shen/cons\? V1569)
     (shen/and
      (shen/= 'nl
              (shen/hd V1569))
      (shen/internal/predicate->shen
       (null
        (nthcdr 1 V1569)))))
    (list 'nl 1))
   (shen/true V1569)))
(defun shen/shen\.assoc-macro
    (V1571)
  (shen/cond
   ((shen/and
     (shen/cons\? V1571)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V1571))
      (shen/and
       (shen/cons\?
        (nthcdr 2 V1571))
       (shen/and
        (shen/cons\?
         (nthcdr 3 V1571))
        (shen/element\?
         (shen/hd V1571)
         (list '@p '@v 'append 'and 'or '+ '* 'do))))))
    (list
     (shen/hd V1571)
     (shen/hd
      (nthcdr 1 V1571))
     (shen/shen\.assoc-macro
      (append
       (list
        (shen/hd V1571))
       (nthcdr 2 V1571)))))
   (shen/true V1571)))
(defun shen/shen\.let-macro
    (V1573)
  (shen/cond
   ((shen/and
     (shen/cons\? V1573)
     (shen/and
      (shen/= 'let
              (shen/hd V1573))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1573))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1573))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V1573))
         (shen/cons\?
          (nthcdr 4 V1573)))))))
    (list 'let
          (shen/hd
           (nthcdr 1 V1573))
          (shen/hd
           (nthcdr 2 V1573))
          (shen/shen\.let-macro
           (append
            (list 'let)
            (nthcdr 3 V1573)))))
   (shen/true V1573)))
(defun shen/shen\.abs-macro
    (V1575)
  (shen/cond
   ((shen/and
     (shen/cons\? V1575)
     (shen/and
      (shen/= '/\.
              (shen/hd V1575))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1575))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1575))
        (shen/cons\?
         (nthcdr 3 V1575))))))
    (list 'lambda
          (shen/hd
           (nthcdr 1 V1575))
          (shen/shen\.abs-macro
           (append
            (list '/\.)
            (nthcdr 2 V1575)))))
   ((shen/and
     (shen/cons\? V1575)
     (shen/and
      (shen/= '/\.
              (shen/hd V1575))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1575))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1575))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1575)))))))
    (append
     (list 'lambda)
     (nthcdr 1 V1575)))
   (shen/true V1575)))
(defun shen/shen\.cases-macro
    (V1579)
  (shen/cond
   ((shen/and
     (shen/cons\? V1579)
     (shen/and
      (shen/= 'cases
              (shen/hd V1579))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1579))
       (shen/and
        (shen/= 'true
                (shen/hd
                 (nthcdr 1 V1579)))
        (shen/cons\?
         (nthcdr 2 V1579))))))
    (shen/hd
     (nthcdr 2 V1579)))
   ((shen/and
     (shen/cons\? V1579)
     (shen/and
      (shen/= 'cases
              (shen/hd V1579))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1579))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1579))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1579)))))))
    (list 'if
          (shen/hd
           (nthcdr 1 V1579))
          (shen/hd
           (nthcdr 2 V1579))
          (list 'simple-error "error: cases exhausted")))
   ((shen/and
     (shen/cons\? V1579)
     (shen/and
      (shen/= 'cases
              (shen/hd V1579))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1579))
       (shen/cons\?
        (nthcdr 2 V1579)))))
    (list 'if
          (shen/hd
           (nthcdr 1 V1579))
          (shen/hd
           (nthcdr 2 V1579))
          (shen/shen\.cases-macro
           (append
            (list 'cases)
            (nthcdr 3 V1579)))))
   ((shen/and
     (shen/cons\? V1579)
     (shen/and
      (shen/= 'cases
              (shen/hd V1579))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1579))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1579))))))
    (shen/simple-error "error: odd number of case elements\n"))
   (shen/true V1579)))
(defun shen/shen\.timer-macro
    (V1581)
  (shen/cond
   ((shen/and
     (shen/cons\? V1581)
     (shen/and
      (shen/= 'time
              (shen/hd V1581))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1581))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1581))))))
    (shen/shen\.let-macro
     (list 'let 'Start
           (list 'get-time 'run)
           'Result
           (shen/hd
            (nthcdr 1 V1581))
           'Finish
           (list 'get-time 'run)
           'Time
           (list '- 'Finish 'Start)
           'Message
           (list 'shen\.prhush
                 (list 'cn "\nrun time: "
                       (list 'cn
                             (list 'str 'Time)
                             " secs\n"))
                 (list 'stoutput))
           'Result)))
   (shen/true V1581)))
(defun shen/shen\.tuple-up
    (V1583)
  (shen/cond
   ((shen/cons\? V1583)
    (list '@p
          (shen/hd V1583)
          (shen/shen\.tuple-up
           (nthcdr 1 V1583))))
   (shen/true V1583)))
(defun shen/shen\.put/get-macro
    (V1585)
  (shen/cond
   ((shen/and
     (shen/cons\? V1585)
     (shen/and
      (shen/= 'put
              (shen/hd V1585))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1585))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1585))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V1585))
         (shen/internal/predicate->shen
          (null
           (nthcdr 4 V1585))))))))
    (list 'put
          (shen/hd
           (nthcdr 1 V1585))
          (shen/hd
           (nthcdr 2 V1585))
          (shen/hd
           (nthcdr 3 V1585))
          (list 'value '*property-vector*)))
   ((shen/and
     (shen/cons\? V1585)
     (shen/and
      (shen/= 'get
              (shen/hd V1585))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1585))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1585))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1585)))))))
    (list 'get
          (shen/hd
           (nthcdr 1 V1585))
          (shen/hd
           (nthcdr 2 V1585))
          (list 'value '*property-vector*)))
   ((shen/and
     (shen/cons\? V1585)
     (shen/and
      (shen/= 'get/or
              (shen/hd V1585))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1585))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1585))
        (shen/and
         (shen/cons\?
          (nthcdr 3 V1585))
         (shen/internal/predicate->shen
          (null
           (nthcdr 4 V1585))))))))
    (list 'get/or
          (shen/hd
           (nthcdr 1 V1585))
          (shen/hd
           (nthcdr 2 V1585))
          (shen/hd
           (nthcdr 3 V1585))
          (list 'value '*property-vector*)))
   ((shen/and
     (shen/cons\? V1585)
     (shen/and
      (shen/= 'unput
              (shen/hd V1585))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1585))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V1585))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V1585)))))))
    (list 'unput
          (shen/hd
           (nthcdr 1 V1585))
          (shen/hd
           (nthcdr 2 V1585))
          (list 'value '*property-vector*)))
   (shen/true V1585)))
(defun shen/shen\.function-macro
    (V1587)
  (shen/cond
   ((shen/and
     (shen/cons\? V1587)
     (shen/and
      (shen/= 'function
              (shen/hd V1587))
      (shen/and
       (shen/cons\?
        (nthcdr 1 V1587))
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V1587))))))
    (shen/shen\.function-abstraction
     (shen/hd
      (nthcdr 1 V1587))
     (shen/arity
      (shen/hd
       (nthcdr 1 V1587)))))
   (shen/true V1587)))
(defun shen/shen\.function-abstraction
    (V1590 V1591)
  (shen/cond
   ((shen/= 0 V1591)
    (shen/simple-error
     (shen/shen\.app V1590 " has no lambda form\n" 'shen\.a)))
   ((shen/= -1 V1591)
    (list 'function V1590))
   (shen/true
    (shen/shen\.function-abstraction-help V1590 V1591 nil))))
(defun shen/shen\.function-abstraction-help
    (V1595 V1596 V1597)
  (shen/cond
   ((shen/= 0 V1596)
    (append
     (list V1595)
     V1597))
   (shen/true
    (shen/let X
              (shen/gensym 'V)
              (list '/\. X
                    (shen/shen\.function-abstraction-help V1595
                                                          (shen/- V1596 1)
                                                          (shen/append V1597
                                                                       (list X))))))))
(defun shen/undefmacro
    (V1599)
  (shen/let MacroReg
            (shen/value 'shen\.*macroreg*)
            (shen/let Pos
                      (shen/shen\.findpos V1599 MacroReg)
                      (shen/let Remove1
                                (shen/set 'shen\.*macroreg*
                                          (shen/remove V1599 MacroReg))
                                (shen/let Remove2
                                          (shen/set '*macros*
                                                    (shen/shen\.remove-nth Pos
                                                                           (shen/value '*macros*)))
                                          V1599)))))
(defun shen/shen\.findpos
    (V1609 V1610)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V1610))
    (shen/simple-error
     (shen/shen\.app V1609 " is not a macro\n" 'shen\.a)))
   ((shen/and
     (shen/cons\? V1610)
     (shen/=
      (shen/hd V1610)
      V1609))
    1)
   ((shen/cons\? V1610)
    (1+
     (shen/shen\.findpos V1609
                         (nthcdr 1 V1610))))
   (shen/true
    (shen/shen\.f_error 'shen\.findpos))))
(defun shen/shen\.remove-nth
    (V1615 V1616)
  (shen/cond
   ((shen/and
     (shen/= 1 V1615)
     (shen/cons\? V1616))
    (nthcdr 1 V1616))
   ((shen/cons\? V1616)
    (append
     (list
      (shen/hd V1616))
     (shen/shen\.remove-nth
      (shen/- V1615 1)
      (nthcdr 1 V1616))))
   (shen/true
    (shen/shen\.f_error 'shen\.remove-nth))))
(shen/set 'shen\.*installing-kl* 'false)
(shen/set 'shen\.*history* nil)
(shen/set 'shen\.*tc* 'false)
(shen/set '*property-vector*
          (shen/dict 20000))
(shen/set 'shen\.*process-counter* 0)
(shen/set 'shen\.*varcounter*
          (shen/vector 1000))
(shen/set 'shen\.*prologvectors*
          (shen/vector 1000))
(shen/set 'shen\.*demodulation-function*
          (shen/lambda X X))
(shen/set 'shen\.*macroreg*
          (list 'shen\.timer-macro 'shen\.cases-macro 'shen\.abs-macro 'shen\.put/get-macro 'shen\.compile-macro 'shen\.datatype-macro 'shen\.let-macro 'shen\.assoc-macro 'shen\.make-string-macro 'shen\.output-macro 'shen\.input-macro 'shen\.error-macro 'shen\.prolog-macro 'shen\.synonyms-macro 'shen\.nl-macro 'shen\.@s-macro 'shen\.defprolog-macro 'shen\.function-macro))
(shen/set '*macros*
          (list
           (shen/lambda X
                        (shen/shen\.timer-macro X))
           (shen/lambda X
                        (shen/shen\.cases-macro X))
           (shen/lambda X
                        (shen/shen\.abs-macro X))
           (shen/lambda X
                        (shen/shen\.put/get-macro X))
           (shen/lambda X
                        (shen/shen\.compile-macro X))
           (shen/lambda X
                        (shen/shen\.datatype-macro X))
           (shen/lambda X
                        (shen/shen\.let-macro X))
           (shen/lambda X
                        (shen/shen\.assoc-macro X))
           (shen/lambda X
                        (shen/shen\.make-string-macro X))
           (shen/lambda X
                        (shen/shen\.output-macro X))
           (shen/lambda X
                        (shen/shen\.input-macro X))
           (shen/lambda X
                        (shen/shen\.error-macro X))
           (shen/lambda X
                        (shen/shen\.prolog-macro X))
           (shen/lambda X
                        (shen/shen\.synonyms-macro X))
           (shen/lambda X
                        (shen/shen\.nl-macro X))
           (shen/lambda X
                        (shen/shen\.@s-macro X))
           (shen/lambda X
                        (shen/shen\.defprolog-macro X))
           (shen/lambda X
                        (shen/shen\.function-macro X))))
(shen/set 'shen\.*gensym* 0)
(shen/set 'shen\.*tracking* nil)
(shen/set 'shen\.*alphabet*
          (list 'A 'B 'C 'D 'E 'F 'G 'H 'I 'J 'K 'L 'M 'N 'O 'P 'Q 'R 'S 'T 'U 'V 'W 'X 'Y 'Z))
(shen/set 'shen\.*special*
          (list '@p '@s '@v 'cons 'lambda 'let 'where 'set 'open))
(shen/set 'shen\.*extraspecial*
          (list 'define 'shen\.process-datatype 'input+ 'defcc 'shen\.read+ 'defmacro))
(shen/set 'shen\.*spy* 'false)
(shen/set 'shen\.*datatypes* nil)
(shen/set 'shen\.*alldatatypes* nil)
(shen/set 'shen\.*shen-type-theory-enabled\?* 'true)
(shen/set 'shen\.*synonyms* nil)
(shen/set 'shen\.*system* nil)
(shen/set 'shen\.*signedfuncs* nil)
(shen/set 'shen\.*maxcomplexity* 128)
(shen/set 'shen\.*occurs* 'true)
(shen/set 'shen\.*maxinferences* 1000000)
(shen/set '*maximum-print-sequence-size* 20)
(shen/set 'shen\.*catch* 0)
(shen/set 'shen\.*call* 0)
(shen/set 'shen\.*infs* 0)
(shen/set '*hush* 'false)
(shen/set 'shen\.*optimise* 'false)
(shen/set '*version* "Shen 20.0")
(shen/if
 (shen/not
  (shen/bound\? '*home-directory*))
 (shen/set '*home-directory* "")
 'shen\.skip)
(shen/if
 (shen/not
  (shen/bound\? 'shen\.*sterror*))
 (shen/set 'shen\.*sterror*
           (shen/value '*stoutput*))
 'shen\.skip)
(defun shen/shen\.initialise_arity_table
    (V1456)
  (cl-flet
      ((tail-trampoline
        (V1456)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V1456))
          nil)
         ((shen/and
           (shen/cons\? V1456)
           (shen/cons\?
            (nthcdr 1 V1456)))
          (shen/let DecArity
                    (shen/put
                     (shen/hd V1456)
                     'arity
                     (shen/hd
                      (nthcdr 1 V1456))
                     (shen/value '*property-vector*))
                    (vector
                     (list
                      (nthcdr 2 V1456)))))
         (shen/true
          (shen/shen\.f_error 'shen\.initialise_arity_table)))))
    (let
        ((result
          (funcall #'tail-trampoline V1456)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/arity
    (V1458)
  (shen/get/or V1458 'arity
               (shen/freeze -1)
               (shen/value '*property-vector*)))
(shen/shen\.initialise_arity_table
 (list 'abort 0 'absvector\? 1 'absvector 1 'adjoin 2 'and 2 'append 2 'arity 1 'assoc 2 'boolean\? 1 'bound\? 1 'cd 1 'close 1 'compile 3 'concat 2 'cons 2 'cons\? 1 'cn 2 'declare 2 'destroy 1 'difference 2 'do 2 'element\? 2 'empty\? 1 'enable-type-theory 1 'error-to-string 1 'shen\.interror 2 'eval 1 'eval-kl 1 'exit 1 'explode 1 'external 1 'fail-if 2 'fail 0 'fix 2 'fold-left 3 'fold-right 3 'filter 2 'for-each 2 'findall 5 'freeze 1 'fst 1 'gensym 1 'get 3 'get/or 4 'get-time 1 'address-> 3 '<-address 2 '<-address/or 3 '<-vector 2 '<-vector/or 3 '> 2 '>= 2 '= 2 'hash 2 'hd 1 'hdv 1 'hdstr 1 'head 1 'if 3 'integer\? 1 'intern 1 'identical 4 'inferences 0 'input 1 'input+ 2 'implementation 0 'intersection 2 'internal 1 'it 0 'kill 0 'language 0 'length 1 'limit 1 'lineread 1 'load 1 '< 2 '<= 2 'vector 1 'macroexpand 1 'map 2 'mapcan 2 'maxinferences 1 'nl 1 'not 1 'nth 2 'n->string 1 'number\? 1 'occurs-check 1 'occurrences 2 'occurs-check 1 'open 2 'optimise 1 'or 2 'os 0 'package 3 'package\? 1 'port 0 'porters 0 'pos 2 'print 1 'profile 1 'profile-results 1 'pr 2 'ps 1 'preclude 1 'preclude-all-but 1 'protect 1 'address-> 3 'put 4 'shen\.reassemble 2 'read-file-as-string 1 'read-file 1 'read-file-as-charlist 1 'read-file-as-bytelist 1 'read 1 'read-byte 1 'read-from-string 1 'read-char-code 1 'receive 1 'release 0 'remove 2 'shen\.require 3 'reverse 1 'set 2 'simple-error 1 'snd 1 'specialise 1 'spy 1 'step 1 'stinput 0 'stoutput 0 'sterror 0 'string->n 1 'string->symbol 1 'string\? 1 'str 1 'subst 3 'sum 1 'symbol\? 1 'systemf 1 'tail 1 'tl 1 'tc 1 'tc\? 0 'thaw 1 'tlstr 1 'track 1 'trap-error 2 'tuple\? 1 'type 2 'return 3 'undefmacro 1 'unput 3 'unprofile 1 'unify 4 'unify! 4 'union 2 'untrack 1 'unspecialise 1 'undefmacro 1 'vector 1 'vector\? 1 'vector-> 3 'value 1 'value/or 2 'variable\? 1 'version 0 'write-byte 2 'write-to-file 2 'y-or-n\? 1 '+ 2 '* 2 '/ 2 '- 2 '== 2 '<e> 1 '<!> 1 '@p 2 '@v 2 '@s 2 'preclude 1 'include 1 'preclude-all-but 1 'include-all-but 1 'dict 1 'dict\? 1 'dict-count 1 'dict-> 3 '<-dict/or 3 '<-dict 2 'dict-rm 2 'dict-fold 3 'dict-keys 1 'dict-values 1))
(defun shen/systemf
    (V1460)
  (shen/let Shen
            (shen/intern "shen")
            (shen/let External
                      (shen/get Shen 'shen\.external-symbols
                                (shen/value '*property-vector*))
                      (shen/let Place
                                (shen/put Shen 'shen\.external-symbols
                                          (shen/adjoin V1460 External)
                                          (shen/value '*property-vector*))
                                V1460))))
(defun shen/adjoin
    (V1463 V1464)
  (shen/if
   (shen/element\? V1463 V1464)
   V1464
   (append
    (list V1463)
    V1464)))
(shen/put
 (shen/intern "shen")
 'shen\.external-symbols
 (list '! '} '{ '--> '<-- '&& ': '\; ':- ':= '_ '*language* '*implementation* '*stinput* '*stoutput* 'shen\.*sterror* '*home-directory* '*version* '*maximum-print-sequence-size* '*macros* '*os* '*release* '*property-vector* '*port* '*porters* '*hush* '@v '@p '@s '<- '-> '<e> '<!> '== '= '>= '> '/\. '=! '$ '- '/ '* '+ '<= '< '>>
       (shen/vector 0)
       'y-or-n\? 'write-to-file 'write-byte 'where 'when 'warn 'version 'verified 'variable\? 'value 'value/or 'vector-> '<-vector '<-vector/or 'vector 'vector\? 'unspecialise 'untrack 'unit 'shen\.unix 'union 'unify 'unify! 'unput 'unprofile 'undefmacro 'return 'type 'tuple\? 'true 'trap-error 'track 'time 'thaw 'tc\? 'tc 'tl 'tlstr 'tlv 'tail 'systemf 'synonyms 'symbol 'symbol\? 'string->symbol 'sum 'subst 'string\? 'string->n 'stream 'string 'stinput 'sterror 'stoutput 'step 'spy 'specialise 'snd 'simple-error 'set 'save 'str 'run 'reverse 'remove 'release 'read 'receive 'read-file 'read-file-as-bytelist 'read-file-as-string 'read-byte 'read-file-as-charlist 'read-char-code 'read-from-string 'package\? 'put 'preclude 'preclude-all-but 'ps 'prolog\? 'protect 'profile-results 'profile 'print 'pr 'pos 'porters 'port 'package 'output 'out 'os 'or 'optimise 'open 'occurrences 'occurs-check 'n->string 'number\? 'number 'null 'nth 'not 'nl 'mode 'macroexpand 'maxinferences 'mapcan 'map 'make-string 'load 'loaded 'list 'lineread 'limit 'length 'let 'lazy 'lambda 'language 'kill 'is 'intersection 'inferences 'intern 'integer\? 'input 'input+ 'include 'include-all-but 'it 'in 'internal 'implementation 'if 'identical 'head 'hd 'hdv 'hdstr 'hash 'get 'get/or 'get-time 'gensym 'function 'fst 'freeze 'fix 'file 'fail 'fail-if 'fwhen 'findall 'for-each 'fold-right 'fold-left 'filter 'false 'enable-type-theory 'explode 'external 'exception 'eval-kl 'eval 'error-to-string 'error 'empty\? 'exit 'element\? 'do 'difference 'destroy 'defun 'define 'defmacro 'defcc 'defprolog 'declare 'datatype 'cut 'cn 'cons\? 'cons 'cond 'concat 'compile 'cd 'cases 'call 'close 'bind 'bound\? 'boolean\? 'boolean 'bar! 'assoc 'arity 'abort 'append 'and 'adjoin '<-address '<-address/or 'address-> 'absvector\? 'absvector 'dict 'dict\? 'dict-count 'dict-> '<-dict/or '<-dict 'dict-rm 'dict-fold 'dict-keys 'dict-values)
 (shen/value '*property-vector*))
(defun shen/shen\.lambda-form-entry
    (V1466)
  (shen/cond
   ((shen/= 'package V1466)
    nil)
   ((shen/= 'receive V1466)
    nil)
   (shen/true
    (shen/let ArityF
              (shen/arity V1466)
              (shen/if
               (shen/= ArityF -1)
               nil
               (shen/if
                (shen/= ArityF 0)
                nil
                (list
                 (append
                  (list V1466)
                  (shen/eval-kl
                   (shen/shen\.lambda-form V1466 ArityF))))))))))
(defun shen/shen\.lambda-form
    (V1469 V1470)
  (shen/cond
   ((shen/= 0 V1470)
    V1469)
   (shen/true
    (shen/let X
              (shen/gensym 'V)
              (list 'lambda X
                    (shen/shen\.lambda-form
                     (shen/shen\.add-end V1469 X)
                     (shen/- V1470 1)))))))
(defun shen/shen\.add-end
    (V1473 V1474)
  (shen/cond
   ((shen/cons\? V1473)
    (shen/append V1473
                 (list V1474)))
   (shen/true
    (list V1473 V1474))))
(defun shen/shen\.set-lambda-form-entry
    (V1476)
  (shen/cond
   ((shen/cons\? V1476)
    (shen/put
     (shen/hd V1476)
     'shen\.lambda-form
     (nthcdr 1 V1476)
     (shen/value '*property-vector*)))
   (shen/true
    (shen/shen\.f_error 'shen\.set-lambda-form-entry))))
(shen/for-each
 (shen/lambda Entry
              (shen/shen\.set-lambda-form-entry Entry))
 (append
  (list
   (append
    (list 'shen\.datatype-error)
    (shen/lambda X
                 (shen/shen\.datatype-error X)))
   (append
    (list 'shen\.tuple)
    (shen/lambda X
                 (shen/shen\.tuple X)))
   (append
    (list 'shen\.pvar)
    (shen/lambda X
                 (shen/shen\.pvar X)))
   (append
    (list 'shen\.dictionary)
    (shen/lambda X
                 (shen/shen\.dictionary X))))
  (shen/mapcan
   (shen/lambda X
                (shen/shen\.lambda-form-entry X))
   (shen/external
    (shen/intern "shen")))))
(defun shen/specialise
    (V1478)
  (shen/do
   (shen/set 'shen\.*special*
             (append
              (list V1478)
              (shen/value 'shen\.*special*)))
   V1478))
(defun shen/unspecialise
    (V1480)
  (shen/do
   (shen/set 'shen\.*special*
             (shen/remove V1480
                          (shen/value 'shen\.*special*)))
   V1480))
(defun shen/declare
    (V3988 V3989)
  (shen/let Record
            (shen/set 'shen\.*signedfuncs*
                      (append
                       (list
                        (append
                         (list V3988)
                         V3989))
                       (shen/value 'shen\.*signedfuncs*)))
            (shen/let Variancy
                      (shen/trap-error
                       (shen/shen\.variancy-test V3988 V3989)
                       (shen/lambda E 'shen\.skip))
                      (shen/let Type
                                (shen/shen\.rcons_form
                                 (shen/shen\.demodulate V3989))
                                (shen/let F*
                                          (shen/concat 'shen\.type-signature-of- V3988)
                                          (shen/let Parameters
                                                    (shen/shen\.parameters 1)
                                                    (shen/let Clause
                                                              (list
                                                               (list F* 'X)
                                                               ':-
                                                               (list
                                                                (list 'unify! 'X Type)))
                                                              (shen/let AUM_instruction
                                                                        (shen/shen\.aum Clause Parameters)
                                                                        (shen/let Code
                                                                                  (shen/shen\.aum_to_shen AUM_instruction)
                                                                                  (shen/let ShenDef
                                                                                            (append
                                                                                             (list 'define F*)
                                                                                             (shen/append Parameters
                                                                                                          (shen/append
                                                                                                           (list 'ProcessN 'Continuation)
                                                                                                           (list '-> Code))))
                                                                                            (shen/let Eval
                                                                                                      (shen/shen\.eval-without-macros ShenDef)
                                                                                                      V3988)))))))))))
(defun shen/shen\.demodulate
    (V3991)
  (cl-flet
      ((tail-trampoline
        (V3991)
        (shen/let Demod
                  (shen/shen\.walk
                   (shen/value 'shen\.*demodulation-function*)
                   V3991)
                  (shen/if
                   (shen/= Demod V3991)
                   V3991
                   (vector
                    (list Demod))))))
    (let
        ((result
          (funcall #'tail-trampoline V3991)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.variancy-test
    (V3994 V3995)
  (shen/let TypeF
            (shen/shen\.typecheck V3994 'B)
            (shen/let Check
                      (shen/if
                       (shen/= 'symbol TypeF)
                       'shen\.skip
                       (shen/if
                        (shen/shen\.variant\? TypeF V3995)
                        'shen\.skip
                        (shen/shen\.prhush
                         (shen/cn "warning: changing the type of "
                                  (shen/shen\.app V3994 " may create errors\n" 'shen\.a))
                         (shen/stoutput))))
                      'shen\.skip)))
(defun shen/shen\.variant\?
    (V4008 V4009)
  (cl-flet
      ((tail-trampoline
        (V4008 V4009)
        (shen/cond
         ((shen/= V4009 V4008)
          'true)
         ((shen/and
           (shen/cons\? V4008)
           (shen/and
            (shen/cons\? V4009)
            (shen/=
             (shen/hd V4009)
             (shen/hd V4008))))
          (vector
           (list
            (nthcdr 1 V4008)
            (nthcdr 1 V4009))))
         ((shen/and
           (shen/cons\? V4008)
           (shen/and
            (shen/cons\? V4009)
            (shen/and
             (shen/shen\.pvar\?
              (shen/hd V4008))
             (shen/variable\?
              (shen/hd V4009)))))
          (vector
           (list
            (shen/subst 'shen\.a
                        (shen/hd V4008)
                        (nthcdr 1 V4008))
            (shen/subst 'shen\.a
                        (shen/hd V4009)
                        (nthcdr 1 V4009)))))
         ((shen/and
           (shen/cons\? V4008)
           (shen/and
            (shen/cons\?
             (shen/hd V4008))
            (shen/and
             (shen/cons\? V4009)
             (shen/cons\?
              (shen/hd V4009)))))
          (vector
           (list
            (shen/append
             (shen/hd V4008)
             (nthcdr 1 V4008))
            (shen/append
             (shen/hd V4009)
             (nthcdr 1 V4009)))))
         (shen/true 'false))))
    (let
        ((result
          (funcall #'tail-trampoline V4008 V4009)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(shen/declare 'absvector\?
              (list 'A '--> 'boolean))
(shen/declare 'adjoin
              (list 'A '-->
                    (list
                     (list 'list 'A)
                     '-->
                     (list 'list 'A))))
(shen/declare 'and
              (list 'boolean '-->
                    (list 'boolean '--> 'boolean)))
(shen/declare 'shen\.app
              (list 'A '-->
                    (list 'string '-->
                          (list 'symbol '--> 'string))))
(shen/declare 'append
              (list
               (list 'list 'A)
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'A))))
(shen/declare 'arity
              (list 'A '--> 'number))
(shen/declare 'assoc
              (list 'A '-->
                    (list
                     (list 'list
                           (list 'list 'A))
                     '-->
                     (list 'list 'A))))
(shen/declare 'boolean\?
              (list 'A '--> 'boolean))
(shen/declare 'bound\?
              (list 'symbol '--> 'boolean))
(shen/declare 'cd
              (list 'string '--> 'string))
(shen/declare 'close
              (list
               (list 'stream 'A)
               '-->
               (list 'list 'B)))
(shen/declare 'cn
              (list 'string '-->
                    (list 'string '--> 'string)))
(shen/declare 'compile
              (list
               (list 'A 'shen\.==> 'B)
               '-->
               (list 'A '-->
                     (list
                      (list 'A '--> 'B)
                      '--> 'B))))
(shen/declare 'cons\?
              (list 'A '--> 'boolean))
(shen/declare 'destroy
              (list
               (list 'A '--> 'B)
               '--> 'symbol))
(shen/declare 'difference
              (list
               (list 'list 'A)
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'A))))
(shen/declare 'do
              (list 'A '-->
                    (list 'B '--> 'B)))
(shen/declare '<e>
              (list
               (list 'list 'A)
               'shen\.==>
               (list 'list 'B)))
(shen/declare '<!>
              (list
               (list 'list 'A)
               'shen\.==>
               (list 'list 'A)))
(shen/declare 'element\?
              (list 'A '-->
                    (list
                     (list 'list 'A)
                     '--> 'boolean)))
(shen/declare 'empty\?
              (list 'A '--> 'boolean))
(shen/declare 'enable-type-theory
              (list 'symbol '--> 'boolean))
(shen/declare 'external
              (list 'symbol '-->
                    (list 'list 'symbol)))
(shen/declare 'error-to-string
              (list 'exception '--> 'string))
(shen/declare 'explode
              (list 'A '-->
                    (list 'list 'string)))
(shen/declare 'fail
              (list '--> 'symbol))
(shen/declare 'fail-if
              (list
               (list 'symbol '--> 'boolean)
               '-->
               (list 'symbol '--> 'symbol)))
(shen/declare 'fix
              (list
               (list 'A '--> 'A)
               '-->
               (list 'A '--> 'A)))
(shen/declare 'freeze
              (list 'A '-->
                    (list 'lazy 'A)))
(shen/declare 'fst
              (list
               (list 'A '* 'B)
               '--> 'A))
(shen/declare 'function
              (list
               (list 'A '--> 'B)
               '-->
               (list 'A '--> 'B)))
(shen/declare 'gensym
              (list 'symbol '--> 'symbol))
(shen/declare '<-vector
              (list
               (list 'vector 'A)
               '-->
               (list 'number '--> 'A)))
(shen/declare '<-vector/or
              (list
               (list 'vector 'A)
               '-->
               (list 'number '-->
                     (list
                      (list 'lazy 'A)
                      '--> 'A))))
(shen/declare 'vector->
              (list
               (list 'vector 'A)
               '-->
               (list 'number '-->
                     (list 'A '-->
                           (list 'vector 'A)))))
(shen/declare 'vector
              (list 'number '-->
                    (list 'vector 'A)))
(shen/declare 'dict
              (list 'number '-->
                    (list 'dict 'K 'V)))
(shen/declare 'dict\?
              (list 'A '--> 'boolean))
(shen/declare 'dict-count
              (list
               (list 'dict 'K 'V)
               '--> 'number))
(shen/declare '<-dict
              (list
               (list 'dict 'K 'V)
               '-->
               (list 'K '--> 'V)))
(shen/declare '<-dict/or
              (list
               (list 'dict 'K 'V)
               '-->
               (list 'K '-->
                     (list
                      (list 'lazy 'V)
                      '--> 'V))))
(shen/declare 'dict->
              (list
               (list 'dict 'K 'V)
               '-->
               (list 'K '-->
                     (list 'V '--> 'V))))
(shen/declare 'dict-rm
              (list
               (list 'dict 'K 'V)
               '-->
               (list 'K '--> 'K)))
(shen/declare 'dict-fold
              (list
               (list 'K '-->
                     (list 'V '-->
                           (list 'A '--> 'A)))
               '-->
               (list
                (list 'dict 'K 'V)
                '-->
                (list 'A '--> 'A))))
(shen/declare 'dict-keys
              (list
               (list 'dict 'K 'V)
               '-->
               (list 'list 'K)))
(shen/declare 'dict-values
              (list
               (list 'dict 'K 'V)
               '-->
               (list 'list 'V)))
(shen/declare 'exit
              (list 'number '--> 'unit))
(shen/declare 'get-time
              (list 'symbol '--> 'number))
(shen/declare 'hash
              (list 'A '-->
                    (list 'number '--> 'number)))
(shen/declare 'head
              (list
               (list 'list 'A)
               '--> 'A))
(shen/declare 'hdv
              (list
               (list 'vector 'A)
               '--> 'A))
(shen/declare 'hdstr
              (list 'string '--> 'string))
(shen/declare 'if
              (list 'boolean '-->
                    (list 'A '-->
                          (list 'A '--> 'A))))
(shen/declare 'it
              (list '--> 'string))
(shen/declare 'implementation
              (list '--> 'string))
(shen/declare 'include
              (list
               (list 'list 'symbol)
               '-->
               (list 'list 'symbol)))
(shen/declare 'include-all-but
              (list
               (list 'list 'symbol)
               '-->
               (list 'list 'symbol)))
(shen/declare 'inferences
              (list '--> 'number))
(shen/declare 'shen\.insert
              (list 'A '-->
                    (list 'string '--> 'string)))
(shen/declare 'integer\?
              (list 'A '--> 'boolean))
(shen/declare 'internal
              (list 'symbol '-->
                    (list 'list 'symbol)))
(shen/declare 'intersection
              (list
               (list 'list 'A)
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'A))))
(shen/declare 'kill
              (list '--> 'A))
(shen/declare 'language
              (list '--> 'string))
(shen/declare 'length
              (list
               (list 'list 'A)
               '--> 'number))
(shen/declare 'limit
              (list
               (list 'vector 'A)
               '--> 'number))
(shen/declare 'load
              (list 'string '--> 'symbol))
(shen/declare 'fold-left
              (list
               (list 'B '-->
                     (list 'A '--> 'B))
               '-->
               (list 'B '-->
                     (list
                      (list 'list 'A)
                      '--> 'B))))
(shen/declare 'fold-right
              (list
               (list 'A '-->
                     (list 'B '--> 'B))
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'B '--> 'B))))
(shen/declare 'for-each
              (list
               (list 'A '--> 'B)
               '-->
               (list
                (list 'list 'A)
                '--> 'unit)))
(shen/declare 'map
              (list
               (list 'A '--> 'B)
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'B))))
(shen/declare 'mapcan
              (list
               (list 'A '-->
                     (list 'list 'B))
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'B))))
(shen/declare 'filter
              (list
               (list 'A '--> 'boolean)
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'A))))
(shen/declare 'maxinferences
              (list 'number '--> 'number))
(shen/declare 'n->string
              (list 'number '--> 'string))
(shen/declare 'nl
              (list 'number '--> 'number))
(shen/declare 'not
              (list 'boolean '--> 'boolean))
(shen/declare 'nth
              (list 'number '-->
                    (list
                     (list 'list 'A)
                     '--> 'A)))
(shen/declare 'number\?
              (list 'A '--> 'boolean))
(shen/declare 'occurrences
              (list 'A '-->
                    (list 'B '--> 'number)))
(shen/declare 'occurs-check
              (list 'symbol '--> 'boolean))
(shen/declare 'optimise
              (list 'symbol '--> 'boolean))
(shen/declare 'or
              (list 'boolean '-->
                    (list 'boolean '--> 'boolean)))
(shen/declare 'os
              (list '--> 'string))
(shen/declare 'package\?
              (list 'symbol '--> 'boolean))
(shen/declare 'port
              (list '--> 'string))
(shen/declare 'porters
              (list '--> 'string))
(shen/declare 'pos
              (list 'string '-->
                    (list 'number '--> 'string)))
(shen/declare 'pr
              (list 'string '-->
                    (list
                     (list 'stream 'out)
                     '--> 'string)))
(shen/declare 'print
              (list 'A '--> 'A))
(shen/declare 'profile
              (list
               (list 'A '--> 'B)
               '-->
               (list 'A '--> 'B)))
(shen/declare 'preclude
              (list
               (list 'list 'symbol)
               '-->
               (list 'list 'symbol)))
(shen/declare 'shen\.proc-nl
              (list 'string '--> 'string))
(shen/declare 'profile-results
              (list
               (list 'A '--> 'B)
               '-->
               (list
                (list 'A '--> 'B)
                '* 'number)))
(shen/declare 'protect
              (list 'symbol '--> 'symbol))
(shen/declare 'preclude-all-but
              (list
               (list 'list 'symbol)
               '-->
               (list 'list 'symbol)))
(shen/declare 'shen\.prhush
              (list 'string '-->
                    (list
                     (list 'stream 'out)
                     '--> 'string)))
(shen/declare 'ps
              (list 'symbol '-->
                    (list 'list 'unit)))
(shen/declare 'read
              (list
               (list 'stream 'in)
               '--> 'unit))
(shen/declare 'read-byte
              (list
               (list 'stream 'in)
               '--> 'number))
(shen/declare 'read-char-code
              (list
               (list 'stream 'in)
               '--> 'number))
(shen/declare 'read-file-as-bytelist
              (list 'string '-->
                    (list 'list 'number)))
(shen/declare 'read-file-as-charlist
              (list 'string '-->
                    (list 'list 'number)))
(shen/declare 'read-file-as-string
              (list 'string '--> 'string))
(shen/declare 'read-file
              (list 'string '-->
                    (list 'list 'unit)))
(shen/declare 'read-from-string
              (list 'string '-->
                    (list 'list 'unit)))
(shen/declare 'release
              (list '--> 'string))
(shen/declare 'remove
              (list 'A '-->
                    (list
                     (list 'list 'A)
                     '-->
                     (list 'list 'A))))
(shen/declare 'reverse
              (list
               (list 'list 'A)
               '-->
               (list 'list 'A)))
(shen/declare 'simple-error
              (list 'string '--> 'A))
(shen/declare 'snd
              (list
               (list 'A '* 'B)
               '--> 'B))
(shen/declare 'specialise
              (list 'symbol '--> 'symbol))
(shen/declare 'spy
              (list 'symbol '--> 'boolean))
(shen/declare 'step
              (list 'symbol '--> 'boolean))
(shen/declare 'stinput
              (list '-->
                    (list 'stream 'in)))
(shen/declare 'sterror
              (list '-->
                    (list 'stream 'out)))
(shen/declare 'stoutput
              (list '-->
                    (list 'stream 'out)))
(shen/declare 'string\?
              (list 'A '--> 'boolean))
(shen/declare 'str
              (list 'A '--> 'string))
(shen/declare 'string->n
              (list 'string '--> 'number))
(shen/declare 'string->symbol
              (list 'string '--> 'symbol))
(shen/declare 'sum
              (list
               (list 'list 'number)
               '--> 'number))
(shen/declare 'symbol\?
              (list 'A '--> 'boolean))
(shen/declare 'systemf
              (list 'symbol '--> 'symbol))
(shen/declare 'tail
              (list
               (list 'list 'A)
               '-->
               (list 'list 'A)))
(shen/declare 'tlstr
              (list 'string '--> 'string))
(shen/declare 'tlv
              (list
               (list 'vector 'A)
               '-->
               (list 'vector 'A)))
(shen/declare 'tc
              (list 'symbol '--> 'boolean))
(shen/declare 'tc\?
              (list '--> 'boolean))
(shen/declare 'thaw
              (list
               (list 'lazy 'A)
               '--> 'A))
(shen/declare 'track
              (list 'symbol '--> 'symbol))
(shen/declare 'trap-error
              (list 'A '-->
                    (list
                     (list 'exception '--> 'A)
                     '--> 'A)))
(shen/declare 'tuple\?
              (list 'A '--> 'boolean))
(shen/declare 'undefmacro
              (list 'symbol '--> 'symbol))
(shen/declare 'union
              (list
               (list 'list 'A)
               '-->
               (list
                (list 'list 'A)
                '-->
                (list 'list 'A))))
(shen/declare 'unprofile
              (list
               (list 'A '--> 'B)
               '-->
               (list 'A '--> 'B)))
(shen/declare 'untrack
              (list 'symbol '--> 'symbol))
(shen/declare 'unspecialise
              (list 'symbol '--> 'symbol))
(shen/declare 'variable\?
              (list 'A '--> 'boolean))
(shen/declare 'vector\?
              (list 'A '--> 'boolean))
(shen/declare 'version
              (list '--> 'string))
(shen/declare 'write-to-file
              (list 'string '-->
                    (list 'A '--> 'A)))
(shen/declare 'write-byte
              (list 'number '-->
                    (list
                     (list 'stream 'out)
                     '--> 'number)))
(shen/declare 'y-or-n\?
              (list 'string '--> 'boolean))
(shen/declare '>
              (list 'number '-->
                    (list 'number '--> 'boolean)))
(shen/declare '<
              (list 'number '-->
                    (list 'number '--> 'boolean)))
(shen/declare '>=
              (list 'number '-->
                    (list 'number '--> 'boolean)))
(shen/declare '<=
              (list 'number '-->
                    (list 'number '--> 'boolean)))
(shen/declare '=
              (list 'A '-->
                    (list 'A '--> 'boolean)))
(shen/declare '+
              (list 'number '-->
                    (list 'number '--> 'number)))
(shen/declare '/
              (list 'number '-->
                    (list 'number '--> 'number)))
(shen/declare '-
              (list 'number '-->
                    (list 'number '--> 'number)))
(shen/declare '*
              (list 'number '-->
                    (list 'number '--> 'number)))
(shen/declare '==
              (list 'A '-->
                    (list 'B '--> 'boolean)))
(defun shen/shen\.typecheck
    (V3509 V3510)
  (shen/let Curry
            (shen/shen\.curry V3509)
            (shen/let ProcessN
                      (shen/shen\.start-new-prolog-process)
                      (shen/let Type
                                (shen/shen\.insert-prolog-variables
                                 (shen/shen\.demodulate
                                  (shen/shen\.curry-type V3510))
                                 ProcessN)
                                (shen/let Continuation
                                          (shen/freeze
                                           (shen/return Type ProcessN 'shen\.void))
                                          (shen/shen\.t*
                                           (list Curry ': Type)
                                           nil ProcessN Continuation))))))
(defun shen/shen\.curry
    (V3512)
  (cl-flet
      ((tail-trampoline
        (V3512)
        (shen/cond
         ((shen/and
           (shen/cons\? V3512)
           (shen/shen\.special\?
            (shen/hd V3512)))
          (append
           (list
            (shen/hd V3512))
           (shen/map
            (shen/lambda Y
                         (shen/shen\.curry Y))
            (nthcdr 1 V3512))))
         ((shen/and
           (shen/cons\? V3512)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V3512))
            (shen/shen\.extraspecial\?
             (shen/hd V3512))))
          V3512)
         ((shen/and
           (shen/cons\? V3512)
           (shen/and
            (shen/= 'type
                    (shen/hd V3512))
            (shen/and
             (shen/cons\?
              (nthcdr 1 V3512))
             (shen/and
              (shen/cons\?
               (nthcdr 2 V3512))
              (shen/internal/predicate->shen
               (null
                (nthcdr 3 V3512)))))))
          (append
           (list 'type
                 (shen/shen\.curry
                  (shen/hd
                   (nthcdr 1 V3512))))
           (nthcdr 2 V3512)))
         ((shen/and
           (shen/cons\? V3512)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V3512))
            (shen/cons\?
             (nthcdr 2 V3512))))
          (vector
           (list
            (append
             (list
              (list
               (shen/hd V3512)
               (shen/hd
                (nthcdr 1 V3512))))
             (nthcdr 2 V3512)))))
         ((shen/and
           (shen/cons\? V3512)
           (shen/and
            (shen/cons\?
             (nthcdr 1 V3512))
            (shen/internal/predicate->shen
             (null
              (nthcdr 2 V3512)))))
          (list
           (shen/shen\.curry
            (shen/hd V3512))
           (shen/shen\.curry
            (shen/hd
             (nthcdr 1 V3512)))))
         (shen/true V3512))))
    (let
        ((result
          (funcall #'tail-trampoline V3512)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.special\?
    (V3514)
  (shen/element\? V3514
                  (shen/value 'shen\.*special*)))
(defun shen/shen\.extraspecial\?
    (V3516)
  (shen/element\? V3516
                  (shen/value 'shen\.*extraspecial*)))
(defun shen/shen\.t*
    (V3521 V3522 V3523 V3524)
  (shen/let Throwcontrol
            (shen/shen\.catchpoint)
            (shen/shen\.cutpoint Throwcontrol
                                 (shen/let Case
                                           (shen/let Error
                                                     (shen/shen\.newpv V3523)
                                                     (shen/do
                                                      (shen/shen\.incinfs)
                                                      (shen/fwhen
                                                       (shen/shen\.maxinfexceeded\?)
                                                       V3523
                                                       (shen/freeze
                                                        (shen/bind Error
                                                                   (shen/shen\.errormaxinfs)
                                                                   V3523 V3524)))))
                                           (shen/if
                                            (shen/= Case 'false)
                                            (shen/let Case
                                                      (shen/let V3501
                                                                (shen/shen\.lazyderef V3521 V3523)
                                                                (shen/if
                                                                 (shen/= 'fail V3501)
                                                                 (shen/do
                                                                  (shen/shen\.incinfs)
                                                                  (shen/cut Throwcontrol V3523
                                                                            (shen/freeze
                                                                             (shen/shen\.prolog-failure V3523 V3524))))
                                                                 'false))
                                                      (shen/if
                                                       (shen/= Case 'false)
                                                       (shen/let Case
                                                                 (shen/let V3502
                                                                           (shen/shen\.lazyderef V3521 V3523)
                                                                           (shen/if
                                                                            (shen/cons\? V3502)
                                                                            (shen/let X
                                                                                      (shen/hd V3502)
                                                                                      (shen/let V3503
                                                                                                (shen/shen\.lazyderef
                                                                                                 (nthcdr 1 V3502)
                                                                                                 V3523)
                                                                                                (shen/if
                                                                                                 (shen/cons\? V3503)
                                                                                                 (shen/let V3504
                                                                                                           (shen/shen\.lazyderef
                                                                                                            (shen/hd V3503)
                                                                                                            V3523)
                                                                                                           (shen/if
                                                                                                            (shen/= ': V3504)
                                                                                                            (shen/let V3505
                                                                                                                      (shen/shen\.lazyderef
                                                                                                                       (nthcdr 1 V3503)
                                                                                                                       V3523)
                                                                                                                      (shen/if
                                                                                                                       (shen/cons\? V3505)
                                                                                                                       (shen/let A
                                                                                                                                 (shen/hd V3505)
                                                                                                                                 (shen/let V3506
                                                                                                                                           (shen/shen\.lazyderef
                                                                                                                                            (nthcdr 1 V3505)
                                                                                                                                            V3523)
                                                                                                                                           (shen/if
                                                                                                                                            (shen/internal/predicate->shen
                                                                                                                                             (null V3506))
                                                                                                                                            (shen/do
                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                             (shen/fwhen
                                                                                                                                              (shen/shen\.type-theory-enabled\?)
                                                                                                                                              V3523
                                                                                                                                              (shen/freeze
                                                                                                                                               (shen/cut Throwcontrol V3523
                                                                                                                                                         (shen/freeze
                                                                                                                                                          (shen/shen\.th* X A V3522 V3523 V3524))))))
                                                                                                                                            'false)))
                                                                                                                       'false))
                                                                                                            'false))
                                                                                                 'false)))
                                                                            'false))
                                                                 (shen/if
                                                                  (shen/= Case 'false)
                                                                  (shen/let Datatypes
                                                                            (shen/shen\.newpv V3523)
                                                                            (shen/do
                                                                             (shen/shen\.incinfs)
                                                                             (shen/shen\.show V3521 V3522 V3523
                                                                                              (shen/freeze
                                                                                               (shen/bind Datatypes
                                                                                                          (shen/value 'shen\.*datatypes*)
                                                                                                          V3523
                                                                                                          (shen/freeze
                                                                                                           (shen/shen\.udefs* V3521 V3522 Datatypes V3523 V3524)))))))
                                                                  Case))
                                                       Case))
                                            Case)))))
(defun shen/shen\.type-theory-enabled\? nil
  (shen/value 'shen\.*shen-type-theory-enabled\?*))
(defun shen/enable-type-theory
    (V3530)
  (shen/cond
   ((shen/= '+ V3530)
    (shen/set 'shen\.*shen-type-theory-enabled\?* 'true))
   ((shen/= '- V3530)
    (shen/set 'shen\.*shen-type-theory-enabled\?* 'false))
   (shen/true
    (shen/simple-error "enable-type-theory expects a + or a -\n"))))
(defun shen/shen\.prolog-failure
    (V3541 V3542)
  'false)
(defun shen/shen\.maxinfexceeded\? nil
  (shen/>
   (shen/inferences)
   (shen/value 'shen\.*maxinferences*)))
(defun shen/shen\.errormaxinfs nil
  (shen/simple-error "maximum inferences exceeded~%"))
(defun shen/shen\.udefs*
    (V3548 V3549 V3550 V3551 V3552)
  (cl-flet
      ((tail-trampoline
        (V3548 V3549 V3550 V3551 V3552)
        (shen/let Case
                  (shen/let V3497
                            (shen/shen\.lazyderef V3550 V3551)
                            (shen/if
                             (shen/cons\? V3497)
                             (shen/let D
                                       (shen/hd V3497)
                                       (shen/do
                                        (shen/shen\.incinfs)
                                        (shen/call
                                         (list D V3548 V3549)
                                         V3551 V3552)))
                             'false))
                  (shen/if
                   (shen/= Case 'false)
                   (shen/let V3498
                             (shen/shen\.lazyderef V3550 V3551)
                             (shen/if
                              (shen/cons\? V3498)
                              (shen/let Ds
                                        (nthcdr 1 V3498)
                                        (shen/do
                                         (shen/shen\.incinfs)
                                         (vector
                                          (list V3548 V3549 Ds V3551 V3552))))
                              'false))
                   Case))))
    (let
        ((result
          (funcall #'tail-trampoline V3548 V3549 V3550 V3551 V3552)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.th*
    (V3558 V3559 V3560 V3561 V3562)
  (shen/let Throwcontrol
            (shen/shen\.catchpoint)
            (shen/shen\.cutpoint Throwcontrol
                                 (shen/let Case
                                           (shen/do
                                            (shen/shen\.incinfs)
                                            (shen/shen\.show
                                             (list V3558 ': V3559)
                                             V3560 V3561
                                             (shen/freeze
                                              (shen/fwhen 'false V3561 V3562))))
                                           (shen/if
                                            (shen/= Case 'false)
                                            (shen/let Case
                                                      (shen/let F
                                                                (shen/shen\.newpv V3561)
                                                                (shen/do
                                                                 (shen/shen\.incinfs)
                                                                 (shen/fwhen
                                                                  (shen/shen\.typedf\?
                                                                   (shen/shen\.lazyderef V3558 V3561))
                                                                  V3561
                                                                  (shen/freeze
                                                                   (shen/bind F
                                                                              (shen/shen\.sigf
                                                                               (shen/shen\.lazyderef V3558 V3561))
                                                                              V3561
                                                                              (shen/freeze
                                                                               (shen/call
                                                                                (list F V3559)
                                                                                V3561 V3562)))))))
                                                      (shen/if
                                                       (shen/= Case 'false)
                                                       (shen/let Case
                                                                 (shen/do
                                                                  (shen/shen\.incinfs)
                                                                  (shen/shen\.base V3558 V3559 V3561 V3562))
                                                                 (shen/if
                                                                  (shen/= Case 'false)
                                                                  (shen/let Case
                                                                            (shen/do
                                                                             (shen/shen\.incinfs)
                                                                             (shen/shen\.by_hypothesis V3558 V3559 V3560 V3561 V3562))
                                                                            (shen/if
                                                                             (shen/= Case 'false)
                                                                             (shen/let Case
                                                                                       (shen/let V3393
                                                                                                 (shen/shen\.lazyderef V3558 V3561)
                                                                                                 (shen/if
                                                                                                  (shen/cons\? V3393)
                                                                                                  (shen/let F
                                                                                                            (shen/hd V3393)
                                                                                                            (shen/let V3394
                                                                                                                      (shen/shen\.lazyderef
                                                                                                                       (nthcdr 1 V3393)
                                                                                                                       V3561)
                                                                                                                      (shen/if
                                                                                                                       (shen/internal/predicate->shen
                                                                                                                        (null V3394))
                                                                                                                       (shen/do
                                                                                                                        (shen/shen\.incinfs)
                                                                                                                        (shen/shen\.th* F
                                                                                                                                        (list '--> V3559)
                                                                                                                                        V3560 V3561 V3562))
                                                                                                                       'false)))
                                                                                                  'false))
                                                                                       (shen/if
                                                                                        (shen/= Case 'false)
                                                                                        (shen/let Case
                                                                                                  (shen/let V3395
                                                                                                            (shen/shen\.lazyderef V3558 V3561)
                                                                                                            (shen/if
                                                                                                             (shen/cons\? V3395)
                                                                                                             (shen/let F
                                                                                                                       (shen/hd V3395)
                                                                                                                       (shen/let V3396
                                                                                                                                 (shen/shen\.lazyderef
                                                                                                                                  (nthcdr 1 V3395)
                                                                                                                                  V3561)
                                                                                                                                 (shen/if
                                                                                                                                  (shen/cons\? V3396)
                                                                                                                                  (shen/let X
                                                                                                                                            (shen/hd V3396)
                                                                                                                                            (shen/let V3397
                                                                                                                                                      (shen/shen\.lazyderef
                                                                                                                                                       (nthcdr 1 V3396)
                                                                                                                                                       V3561)
                                                                                                                                                      (shen/if
                                                                                                                                                       (shen/internal/predicate->shen
                                                                                                                                                        (null V3397))
                                                                                                                                                       (shen/let B
                                                                                                                                                                 (shen/shen\.newpv V3561)
                                                                                                                                                                 (shen/do
                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                  (shen/shen\.th* F
                                                                                                                                                                                  (list B '--> V3559)
                                                                                                                                                                                  V3560 V3561
                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                   (shen/shen\.th* X B V3560 V3561 V3562)))))
                                                                                                                                                       'false)))
                                                                                                                                  'false)))
                                                                                                             'false))
                                                                                                  (shen/if
                                                                                                   (shen/= Case 'false)
                                                                                                   (shen/let Case
                                                                                                             (shen/let V3398
                                                                                                                       (shen/shen\.lazyderef V3558 V3561)
                                                                                                                       (shen/if
                                                                                                                        (shen/cons\? V3398)
                                                                                                                        (shen/let V3399
                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                   (shen/hd V3398)
                                                                                                                                   V3561)
                                                                                                                                  (shen/if
                                                                                                                                   (shen/= 'cons V3399)
                                                                                                                                   (shen/let V3400
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (nthcdr 1 V3398)
                                                                                                                                              V3561)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/cons\? V3400)
                                                                                                                                              (shen/let X
                                                                                                                                                        (shen/hd V3400)
                                                                                                                                                        (shen/let V3401
                                                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                                                   (nthcdr 1 V3400)
                                                                                                                                                                   V3561)
                                                                                                                                                                  (shen/if
                                                                                                                                                                   (shen/cons\? V3401)
                                                                                                                                                                   (shen/let Y
                                                                                                                                                                             (shen/hd V3401)
                                                                                                                                                                             (shen/let V3402
                                                                                                                                                                                       (shen/shen\.lazyderef
                                                                                                                                                                                        (nthcdr 1 V3401)
                                                                                                                                                                                        V3561)
                                                                                                                                                                                       (shen/if
                                                                                                                                                                                        (shen/internal/predicate->shen
                                                                                                                                                                                         (null V3402))
                                                                                                                                                                                        (shen/let V3403
                                                                                                                                                                                                  (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                  (shen/if
                                                                                                                                                                                                   (shen/cons\? V3403)
                                                                                                                                                                                                   (shen/let V3404
                                                                                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                                                                                              (shen/hd V3403)
                                                                                                                                                                                                              V3561)
                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                              (shen/= 'list V3404)
                                                                                                                                                                                                              (shen/let V3405
                                                                                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                                                                                         (nthcdr 1 V3403)
                                                                                                                                                                                                                         V3561)
                                                                                                                                                                                                                        (shen/if
                                                                                                                                                                                                                         (shen/cons\? V3405)
                                                                                                                                                                                                                         (shen/let A
                                                                                                                                                                                                                                   (shen/hd V3405)
                                                                                                                                                                                                                                   (shen/let V3406
                                                                                                                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                                                                                                                              (nthcdr 1 V3405)
                                                                                                                                                                                                                                              V3561)
                                                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                                                              (shen/internal/predicate->shen
                                                                                                                                                                                                                                               (null V3406))
                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                               (shen/shen\.incinfs)
                                                                                                                                                                                                                                               (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                               (shen/freeze
                                                                                                                                                                                                                                                                (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                (list 'list A)
                                                                                                                                                                                                                                                                                V3560 V3561 V3562))))
                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                               (shen/shen\.pvar\? V3406)
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.bindv V3406 nil V3561)
                                                                                                                                                                                                                                                (shen/let Result
                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                                                           (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                            (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                            (list 'list A)
                                                                                                                                                                                                                                                                                            V3560 V3561 V3562))))
                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                           (shen/shen\.unbindv V3406 V3561)
                                                                                                                                                                                                                                                           Result)))
                                                                                                                                                                                                                                               'false))))
                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                          (shen/shen\.pvar\? V3405)
                                                                                                                                                                                                                          (shen/let A
                                                                                                                                                                                                                                    (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                     (shen/shen\.bindv V3405
                                                                                                                                                                                                                                                       (list A)
                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                     (shen/let Result
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                                                (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                (shen/freeze
                                                                                                                                                                                                                                                                 (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                 (list 'list A)
                                                                                                                                                                                                                                                                                 V3560 V3561 V3562))))
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.unbindv V3405 V3561)
                                                                                                                                                                                                                                                Result))))
                                                                                                                                                                                                                          'false)))
                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                               (shen/shen\.pvar\? V3404)
                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                (shen/shen\.bindv V3404 'list V3561)
                                                                                                                                                                                                                (shen/let Result
                                                                                                                                                                                                                          (shen/let V3407
                                                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                                                     (nthcdr 1 V3403)
                                                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                     (shen/cons\? V3407)
                                                                                                                                                                                                                                     (shen/let A
                                                                                                                                                                                                                                               (shen/hd V3407)
                                                                                                                                                                                                                                               (shen/let V3408
                                                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                                                          (nthcdr 1 V3407)
                                                                                                                                                                                                                                                          V3561)
                                                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                                                          (shen/internal/predicate->shen
                                                                                                                                                                                                                                                           (null V3408))
                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                                                           (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                            (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                            (list 'list A)
                                                                                                                                                                                                                                                                                            V3560 V3561 V3562))))
                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                           (shen/shen\.pvar\? V3408)
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.bindv V3408 nil V3561)
                                                                                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                       (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                        (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                        (list 'list A)
                                                                                                                                                                                                                                                                                                        V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                       (shen/shen\.unbindv V3408 V3561)
                                                                                                                                                                                                                                                                       Result)))
                                                                                                                                                                                                                                                           'false))))
                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                      (shen/shen\.pvar\? V3407)
                                                                                                                                                                                                                                      (shen/let A
                                                                                                                                                                                                                                                (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.bindv V3407
                                                                                                                                                                                                                                                                   (list A)
                                                                                                                                                                                                                                                                   V3561)
                                                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                            (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                            (shen/freeze
                                                                                                                                                                                                                                                                             (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                             (list 'list A)
                                                                                                                                                                                                                                                                                             V3560 V3561 V3562))))
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.unbindv V3407 V3561)
                                                                                                                                                                                                                                                            Result))))
                                                                                                                                                                                                                                      'false)))
                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                           (shen/shen\.unbindv V3404 V3561)
                                                                                                                                                                                                                           Result)))
                                                                                                                                                                                                               'false)))
                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                    (shen/shen\.pvar\? V3403)
                                                                                                                                                                                                    (shen/let A
                                                                                                                                                                                                              (shen/shen\.newpv V3561)
                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                               (shen/shen\.bindv V3403
                                                                                                                                                                                                                                 (list 'list A)
                                                                                                                                                                                                                                 V3561)
                                                                                                                                                                                                               (shen/let Result
                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                                                                                          (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                           (shen/shen\.th* Y
                                                                                                                                                                                                                                                           (list 'list A)
                                                                                                                                                                                                                                                           V3560 V3561 V3562))))
                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                          (shen/shen\.unbindv V3403 V3561)
                                                                                                                                                                                                                          Result))))
                                                                                                                                                                                                    'false)))
                                                                                                                                                                                        'false)))
                                                                                                                                                                   'false)))
                                                                                                                                              'false))
                                                                                                                                   'false))
                                                                                                                        'false))
                                                                                                             (shen/if
                                                                                                              (shen/= Case 'false)
                                                                                                              (shen/let Case
                                                                                                                        (shen/let V3409
                                                                                                                                  (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                  (shen/if
                                                                                                                                   (shen/cons\? V3409)
                                                                                                                                   (shen/let V3410
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (shen/hd V3409)
                                                                                                                                              V3561)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/= '@p V3410)
                                                                                                                                              (shen/let V3411
                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                         (nthcdr 1 V3409)
                                                                                                                                                         V3561)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/cons\? V3411)
                                                                                                                                                         (shen/let X
                                                                                                                                                                   (shen/hd V3411)
                                                                                                                                                                   (shen/let V3412
                                                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                                                              (nthcdr 1 V3411)
                                                                                                                                                                              V3561)
                                                                                                                                                                             (shen/if
                                                                                                                                                                              (shen/cons\? V3412)
                                                                                                                                                                              (shen/let Y
                                                                                                                                                                                        (shen/hd V3412)
                                                                                                                                                                                        (shen/let V3413
                                                                                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                                                                                   (nthcdr 1 V3412)
                                                                                                                                                                                                   V3561)
                                                                                                                                                                                                  (shen/if
                                                                                                                                                                                                   (shen/internal/predicate->shen
                                                                                                                                                                                                    (null V3413))
                                                                                                                                                                                                   (shen/let V3414
                                                                                                                                                                                                             (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                              (shen/cons\? V3414)
                                                                                                                                                                                                              (shen/let A
                                                                                                                                                                                                                        (shen/hd V3414)
                                                                                                                                                                                                                        (shen/let V3415
                                                                                                                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                                                                                                                   (nthcdr 1 V3414)
                                                                                                                                                                                                                                   V3561)
                                                                                                                                                                                                                                  (shen/if
                                                                                                                                                                                                                                   (shen/cons\? V3415)
                                                                                                                                                                                                                                   (shen/let V3416
                                                                                                                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                                                                                                                              (shen/hd V3415)
                                                                                                                                                                                                                                              V3561)
                                                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                                                              (shen/= '* V3416)
                                                                                                                                                                                                                                              (shen/let V3417
                                                                                                                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                                                                                                                         (nthcdr 1 V3415)
                                                                                                                                                                                                                                                         V3561)
                                                                                                                                                                                                                                                        (shen/if
                                                                                                                                                                                                                                                         (shen/cons\? V3417)
                                                                                                                                                                                                                                                         (shen/let B
                                                                                                                                                                                                                                                                   (shen/hd V3417)
                                                                                                                                                                                                                                                                   (shen/let V3418
                                                                                                                                                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                              (nthcdr 1 V3417)
                                                                                                                                                                                                                                                                              V3561)
                                                                                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                                                                                              (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                               (null V3418))
                                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                                               (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                               (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                               (shen/freeze
                                                                                                                                                                                                                                                                                                (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                                                               (shen/shen\.pvar\? V3418)
                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                (shen/shen\.bindv V3418 nil V3561)
                                                                                                                                                                                                                                                                                (shen/let Result
                                                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                           (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                            (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                                                           (shen/shen\.unbindv V3418 V3561)
                                                                                                                                                                                                                                                                                           Result)))
                                                                                                                                                                                                                                                                               'false))))
                                                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                                                          (shen/shen\.pvar\? V3417)
                                                                                                                                                                                                                                                          (shen/let B
                                                                                                                                                                                                                                                                    (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                                                     (shen/shen\.bindv V3417
                                                                                                                                                                                                                                                                                       (list B)
                                                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                                                     (shen/let Result
                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                (shen/freeze
                                                                                                                                                                                                                                                                                                 (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                (shen/shen\.unbindv V3417 V3561)
                                                                                                                                                                                                                                                                                Result))))
                                                                                                                                                                                                                                                          'false)))
                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                               (shen/shen\.pvar\? V3416)
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.bindv V3416 '* V3561)
                                                                                                                                                                                                                                                (shen/let Result
                                                                                                                                                                                                                                                          (shen/let V3419
                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                     (nthcdr 1 V3415)
                                                                                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                                                     (shen/cons\? V3419)
                                                                                                                                                                                                                                                                     (shen/let B
                                                                                                                                                                                                                                                                               (shen/hd V3419)
                                                                                                                                                                                                                                                                               (shen/let V3420
                                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                          (nthcdr 1 V3419)
                                                                                                                                                                                                                                                                                          V3561)
                                                                                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                                                                                          (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                           (null V3420))
                                                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                           (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                            (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                                                           (shen/shen\.pvar\? V3420)
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.bindv V3420 nil V3561)
                                                                                                                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                       (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                                                        (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                                                       (shen/shen\.unbindv V3420 V3561)
                                                                                                                                                                                                                                                                                                       Result)))
                                                                                                                                                                                                                                                                                           'false))))
                                                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                                                      (shen/shen\.pvar\? V3419)
                                                                                                                                                                                                                                                                      (shen/let B
                                                                                                                                                                                                                                                                                (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                 (shen/shen\.bindv V3419
                                                                                                                                                                                                                                                                                                   (list B)
                                                                                                                                                                                                                                                                                                   V3561)
                                                                                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                            (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                            (shen/freeze
                                                                                                                                                                                                                                                                                                             (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.unbindv V3419 V3561)
                                                                                                                                                                                                                                                                                            Result))))
                                                                                                                                                                                                                                                                      'false)))
                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                           (shen/shen\.unbindv V3416 V3561)
                                                                                                                                                                                                                                                           Result)))
                                                                                                                                                                                                                                               'false)))
                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                    (shen/shen\.pvar\? V3415)
                                                                                                                                                                                                                                    (shen/let B
                                                                                                                                                                                                                                              (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                               (shen/shen\.bindv V3415
                                                                                                                                                                                                                                                                 (list '* B)
                                                                                                                                                                                                                                                                 V3561)
                                                                                                                                                                                                                                               (shen/let Result
                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                                                                                                                          (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                           (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                          (shen/shen\.unbindv V3415 V3561)
                                                                                                                                                                                                                                                          Result))))
                                                                                                                                                                                                                                    'false))))
                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                               (shen/shen\.pvar\? V3414)
                                                                                                                                                                                                               (shen/let A
                                                                                                                                                                                                                         (shen/shen\.newpv V3561)
                                                                                                                                                                                                                         (shen/let B
                                                                                                                                                                                                                                   (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                    (shen/shen\.bindv V3414
                                                                                                                                                                                                                                                      (list A '* B)
                                                                                                                                                                                                                                                      V3561)
                                                                                                                                                                                                                                    (shen/let Result
                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                               (shen/shen\.incinfs)
                                                                                                                                                                                                                                               (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                               (shen/freeze
                                                                                                                                                                                                                                                                (shen/shen\.th* Y B V3560 V3561 V3562))))
                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                               (shen/shen\.unbindv V3414 V3561)
                                                                                                                                                                                                                                               Result)))))
                                                                                                                                                                                                               'false)))
                                                                                                                                                                                                   'false)))
                                                                                                                                                                              'false)))
                                                                                                                                                         'false))
                                                                                                                                              'false))
                                                                                                                                   'false))
                                                                                                                        (shen/if
                                                                                                                         (shen/= Case 'false)
                                                                                                                         (shen/let Case
                                                                                                                                   (shen/let V3421
                                                                                                                                             (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/cons\? V3421)
                                                                                                                                              (shen/let V3422
                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                         (shen/hd V3421)
                                                                                                                                                         V3561)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/= '@v V3422)
                                                                                                                                                         (shen/let V3423
                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                    (nthcdr 1 V3421)
                                                                                                                                                                    V3561)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/cons\? V3423)
                                                                                                                                                                    (shen/let X
                                                                                                                                                                              (shen/hd V3423)
                                                                                                                                                                              (shen/let V3424
                                                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                                                         (nthcdr 1 V3423)
                                                                                                                                                                                         V3561)
                                                                                                                                                                                        (shen/if
                                                                                                                                                                                         (shen/cons\? V3424)
                                                                                                                                                                                         (shen/let Y
                                                                                                                                                                                                   (shen/hd V3424)
                                                                                                                                                                                                   (shen/let V3425
                                                                                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                                                                                              (nthcdr 1 V3424)
                                                                                                                                                                                                              V3561)
                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                              (shen/internal/predicate->shen
                                                                                                                                                                                                               (null V3425))
                                                                                                                                                                                                              (shen/let V3426
                                                                                                                                                                                                                        (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                        (shen/if
                                                                                                                                                                                                                         (shen/cons\? V3426)
                                                                                                                                                                                                                         (shen/let V3427
                                                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                                                    (shen/hd V3426)
                                                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                    (shen/= 'vector V3427)
                                                                                                                                                                                                                                    (shen/let V3428
                                                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                                                               (nthcdr 1 V3426)
                                                                                                                                                                                                                                               V3561)
                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                               (shen/cons\? V3428)
                                                                                                                                                                                                                                               (shen/let A
                                                                                                                                                                                                                                                         (shen/hd V3428)
                                                                                                                                                                                                                                                         (shen/let V3429
                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                    (nthcdr 1 V3428)
                                                                                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                                                    (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                     (null V3429))
                                                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                     (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                     (shen/freeze
                                                                                                                                                                                                                                                                                      (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                      (list 'vector A)
                                                                                                                                                                                                                                                                                                      V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                                                     (shen/shen\.pvar\? V3429)
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.bindv V3429 nil V3561)
                                                                                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                 (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                 (shen/freeze
                                                                                                                                                                                                                                                                                                  (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                                  (list 'vector A)
                                                                                                                                                                                                                                                                                                                  V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                 (shen/shen\.unbindv V3429 V3561)
                                                                                                                                                                                                                                                                                 Result)))
                                                                                                                                                                                                                                                                     'false))))
                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                (shen/shen\.pvar\? V3428)
                                                                                                                                                                                                                                                (shen/let A
                                                                                                                                                                                                                                                          (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                           (shen/shen\.bindv V3428
                                                                                                                                                                                                                                                                             (list A)
                                                                                                                                                                                                                                                                             V3561)
                                                                                                                                                                                                                                                           (shen/let Result
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                      (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                       (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                       (list 'vector A)
                                                                                                                                                                                                                                                                                                       V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.unbindv V3428 V3561)
                                                                                                                                                                                                                                                                      Result))))
                                                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                     (shen/shen\.pvar\? V3427)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.bindv V3427 'vector V3561)
                                                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                                                (shen/let V3430
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                           (nthcdr 1 V3426)
                                                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                           (shen/cons\? V3430)
                                                                                                                                                                                                                                                           (shen/let A
                                                                                                                                                                                                                                                                     (shen/hd V3430)
                                                                                                                                                                                                                                                                     (shen/let V3431
                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                (nthcdr 1 V3430)
                                                                                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                 (null V3431))
                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                 (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                 (shen/freeze
                                                                                                                                                                                                                                                                                                  (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                                  (list 'vector A)
                                                                                                                                                                                                                                                                                                                  V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                                                 (shen/shen\.pvar\? V3431)
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.bindv V3431 nil V3561)
                                                                                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                             (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                             (shen/freeze
                                                                                                                                                                                                                                                                                                              (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                                              (list 'vector A)
                                                                                                                                                                                                                                                                                                                              V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                             (shen/shen\.unbindv V3431 V3561)
                                                                                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                                                                                 'false))))
                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                            (shen/shen\.pvar\? V3430)
                                                                                                                                                                                                                                                            (shen/let A
                                                                                                                                                                                                                                                                      (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                       (shen/shen\.bindv V3430
                                                                                                                                                                                                                                                                                         (list A)
                                                                                                                                                                                                                                                                                         V3561)
                                                                                                                                                                                                                                                                       (shen/let Result
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                  (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                                                                                                                                   (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                                                   (list 'vector A)
                                                                                                                                                                                                                                                                                                                   V3560 V3561 V3562))))
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.unbindv V3430 V3561)
                                                                                                                                                                                                                                                                                  Result))))
                                                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.unbindv V3427 V3561)
                                                                                                                                                                                                                                                 Result)))
                                                                                                                                                                                                                                     'false)))
                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                          (shen/shen\.pvar\? V3426)
                                                                                                                                                                                                                          (shen/let A
                                                                                                                                                                                                                                    (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                     (shen/shen\.bindv V3426
                                                                                                                                                                                                                                                       (list 'vector A)
                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                     (shen/let Result
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                                                (shen/shen\.th* X A V3560 V3561
                                                                                                                                                                                                                                                                (shen/freeze
                                                                                                                                                                                                                                                                 (shen/shen\.th* Y
                                                                                                                                                                                                                                                                                 (list 'vector A)
                                                                                                                                                                                                                                                                                 V3560 V3561 V3562))))
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.unbindv V3426 V3561)
                                                                                                                                                                                                                                                Result))))
                                                                                                                                                                                                                          'false)))
                                                                                                                                                                                                              'false)))
                                                                                                                                                                                         'false)))
                                                                                                                                                                    'false))
                                                                                                                                                         'false))
                                                                                                                                              'false))
                                                                                                                                   (shen/if
                                                                                                                                    (shen/= Case 'false)
                                                                                                                                    (shen/let Case
                                                                                                                                              (shen/let V3432
                                                                                                                                                        (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/cons\? V3432)
                                                                                                                                                         (shen/let V3433
                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                    (shen/hd V3432)
                                                                                                                                                                    V3561)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/= '@s V3433)
                                                                                                                                                                    (shen/let V3434
                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                               (nthcdr 1 V3432)
                                                                                                                                                                               V3561)
                                                                                                                                                                              (shen/if
                                                                                                                                                                               (shen/cons\? V3434)
                                                                                                                                                                               (shen/let X
                                                                                                                                                                                         (shen/hd V3434)
                                                                                                                                                                                         (shen/let V3435
                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                    (nthcdr 1 V3434)
                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                    (shen/cons\? V3435)
                                                                                                                                                                                                    (shen/let Y
                                                                                                                                                                                                              (shen/hd V3435)
                                                                                                                                                                                                              (shen/let V3436
                                                                                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                                                                                         (nthcdr 1 V3435)
                                                                                                                                                                                                                         V3561)
                                                                                                                                                                                                                        (shen/if
                                                                                                                                                                                                                         (shen/internal/predicate->shen
                                                                                                                                                                                                                          (null V3436))
                                                                                                                                                                                                                         (shen/let V3437
                                                                                                                                                                                                                                   (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                    (shen/= 'string V3437)
                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                                                                                     (shen/shen\.th* X 'string V3560 V3561
                                                                                                                                                                                                                                                     (shen/freeze
                                                                                                                                                                                                                                                      (shen/shen\.th* Y 'string V3560 V3561 V3562))))
                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                     (shen/shen\.pvar\? V3437)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.bindv V3437 'string V3561)
                                                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                 (shen/shen\.th* X 'string V3560 V3561
                                                                                                                                                                                                                                                                 (shen/freeze
                                                                                                                                                                                                                                                                  (shen/shen\.th* Y 'string V3560 V3561 V3562))))
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.unbindv V3437 V3561)
                                                                                                                                                                                                                                                 Result)))
                                                                                                                                                                                                                                     'false)))
                                                                                                                                                                                                                         'false)))
                                                                                                                                                                                                    'false)))
                                                                                                                                                                               'false))
                                                                                                                                                                    'false))
                                                                                                                                                         'false))
                                                                                                                                              (shen/if
                                                                                                                                               (shen/= Case 'false)
                                                                                                                                               (shen/let Case
                                                                                                                                                         (shen/let V3438
                                                                                                                                                                   (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/cons\? V3438)
                                                                                                                                                                    (shen/let V3439
                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                               (shen/hd V3438)
                                                                                                                                                                               V3561)
                                                                                                                                                                              (shen/if
                                                                                                                                                                               (shen/= 'lambda V3439)
                                                                                                                                                                               (shen/let V3440
                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                          (nthcdr 1 V3438)
                                                                                                                                                                                          V3561)
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/cons\? V3440)
                                                                                                                                                                                          (shen/let X
                                                                                                                                                                                                    (shen/hd V3440)
                                                                                                                                                                                                    (shen/let V3441
                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                               (nthcdr 1 V3440)
                                                                                                                                                                                                               V3561)
                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                               (shen/cons\? V3441)
                                                                                                                                                                                                               (shen/let Y
                                                                                                                                                                                                                         (shen/hd V3441)
                                                                                                                                                                                                                         (shen/let V3442
                                                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                                                    (nthcdr 1 V3441)
                                                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                    (shen/internal/predicate->shen
                                                                                                                                                                                                                                     (null V3442))
                                                                                                                                                                                                                                    (shen/let V3443
                                                                                                                                                                                                                                              (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                               (shen/cons\? V3443)
                                                                                                                                                                                                                                               (shen/let A
                                                                                                                                                                                                                                                         (shen/hd V3443)
                                                                                                                                                                                                                                                         (shen/let V3444
                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                    (nthcdr 1 V3443)
                                                                                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                                                    (shen/cons\? V3444)
                                                                                                                                                                                                                                                                    (shen/let V3445
                                                                                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                               (shen/hd V3444)
                                                                                                                                                                                                                                                                               V3561)
                                                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                                                               (shen/= '--> V3445)
                                                                                                                                                                                                                                                                               (shen/let V3446
                                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                          (nthcdr 1 V3444)
                                                                                                                                                                                                                                                                                          V3561)
                                                                                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                                                                                          (shen/cons\? V3446)
                                                                                                                                                                                                                                                                                          (shen/let B
                                                                                                                                                                                                                                                                                                    (shen/hd V3446)
                                                                                                                                                                                                                                                                                                    (shen/let V3447
                                                                                                                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                                               (nthcdr 1 V3446)
                                                                                                                                                                                                                                                                                                               V3561)
                                                                                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                                                                                               (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                                                (null V3447))
                                                                                                                                                                                                                                                                                                               (shen/let Z
                                                                                                                                                                                                                                                                                                                         (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                         (shen/let X&&
                                                                                                                                                                                                                                                                                                                                   (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                                                                                                    (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                    (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                              (shen/freeze
                                                                                                                                                                                                                                                                                                                                               (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                                          (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                                          V3561
                                                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                                                           (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                                      (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                                      V3561
                                                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                       (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                                                                                                                                         (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                                        V3560)
                                                                                                                                                                                                                                                                                                                                                                                       V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                                                                                (shen/shen\.pvar\? V3447)
                                                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                                                 (shen/shen\.bindv V3447 nil V3561)
                                                                                                                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                                                                                                                           (shen/let Z
                                                                                                                                                                                                                                                                                                                                     (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                     (shen/let X&&
                                                                                                                                                                                                                                                                                                                                               (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                                (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                                                           (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                                                      (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                                                      V3561
                                                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                       (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                                                  V3561
                                                                                                                                                                                                                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                                                   (append
                                                                                                                                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                                                                                                                                     (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                                                    V3560)
                                                                                                                                                                                                                                                                                                                                                                                                   V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                                                            (shen/shen\.unbindv V3447 V3561)
                                                                                                                                                                                                                                                                                                                            Result)))
                                                                                                                                                                                                                                                                                                                'false))))
                                                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                                                           (shen/shen\.pvar\? V3446)
                                                                                                                                                                                                                                                                                           (shen/let B
                                                                                                                                                                                                                                                                                                     (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                                                      (shen/shen\.bindv V3446
                                                                                                                                                                                                                                                                                                                        (list B)
                                                                                                                                                                                                                                                                                                                        V3561)
                                                                                                                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                                                                                                                (shen/let Z
                                                                                                                                                                                                                                                                                                                          (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                          (shen/let X&&
                                                                                                                                                                                                                                                                                                                                    (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                     (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                               (shen/freeze
                                                                                                                                                                                                                                                                                                                                                (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                                           (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                                           V3561
                                                                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                                                                            (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                                       (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                                       V3561
                                                                                                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                                        (append
                                                                                                                                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                                                                                                                                          (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                                         V3560)
                                                                                                                                                                                                                                                                                                                                                                                        V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                                                 (shen/shen\.unbindv V3446 V3561)
                                                                                                                                                                                                                                                                                                                 Result))))
                                                                                                                                                                                                                                                                                           'false)))
                                                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                                                (shen/shen\.pvar\? V3445)
                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                 (shen/shen\.bindv V3445 '--> V3561)
                                                                                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                                                                                           (shen/let V3448
                                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                                      (nthcdr 1 V3444)
                                                                                                                                                                                                                                                                                                      V3561)
                                                                                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                                                                                      (shen/cons\? V3448)
                                                                                                                                                                                                                                                                                                      (shen/let B
                                                                                                                                                                                                                                                                                                                (shen/hd V3448)
                                                                                                                                                                                                                                                                                                                (shen/let V3449
                                                                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                                                           (nthcdr 1 V3448)
                                                                                                                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                                                                                           (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                                                            (null V3449))
                                                                                                                                                                                                                                                                                                                           (shen/let Z
                                                                                                                                                                                                                                                                                                                                     (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                     (shen/let X&&
                                                                                                                                                                                                                                                                                                                                               (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                                (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                                                           (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                                                      (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                                                      V3561
                                                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                       (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                                                  V3561
                                                                                                                                                                                                                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                                                   (append
                                                                                                                                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                                                                                                                                     (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                                                    V3560)
                                                                                                                                                                                                                                                                                                                                                                                                   V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                                                                                            (shen/shen\.pvar\? V3449)
                                                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                                                             (shen/shen\.bindv V3449 nil V3561)
                                                                                                                                                                                                                                                                                                                             (shen/let Result
                                                                                                                                                                                                                                                                                                                                       (shen/let Z
                                                                                                                                                                                                                                                                                                                                                 (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                                 (shen/let X&&
                                                                                                                                                                                                                                                                                                                                                           (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                                            (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                       (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                                                                  V3561
                                                                                                                                                                                                                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                                   (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                                                              (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                                                              V3561
                                                                                                                                                                                                                                                                                                                                                                                              (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                                               (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                                                               (append
                                                                                                                                                                                                                                                                                                                                                                                                                (list
                                                                                                                                                                                                                                                                                                                                                                                                                 (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                                                                V3560)
                                                                                                                                                                                                                                                                                                                                                                                                               V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                                                                                                                        (shen/shen\.unbindv V3449 V3561)
                                                                                                                                                                                                                                                                                                                                        Result)))
                                                                                                                                                                                                                                                                                                                            'false))))
                                                                                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                                                                                       (shen/shen\.pvar\? V3448)
                                                                                                                                                                                                                                                                                                       (shen/let B
                                                                                                                                                                                                                                                                                                                 (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                                                  (shen/shen\.bindv V3448
                                                                                                                                                                                                                                                                                                                                    (list B)
                                                                                                                                                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                                                                                                                            (shen/let Z
                                                                                                                                                                                                                                                                                                                                      (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                      (shen/let X&&
                                                                                                                                                                                                                                                                                                                                                (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                                 (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                                                                            (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                                                       (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                                                       V3561
                                                                                                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                        (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                                                   (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                                                   V3561
                                                                                                                                                                                                                                                                                                                                                                                   (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                                    (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                                                    (append
                                                                                                                                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                                                                                                                                      (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                                                     V3560)
                                                                                                                                                                                                                                                                                                                                                                                                    V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                                                             (shen/shen\.unbindv V3448 V3561)
                                                                                                                                                                                                                                                                                                                             Result))))
                                                                                                                                                                                                                                                                                                       'false)))
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.unbindv V3445 V3561)
                                                                                                                                                                                                                                                                                            Result)))
                                                                                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                                                     (shen/shen\.pvar\? V3444)
                                                                                                                                                                                                                                                                     (shen/let B
                                                                                                                                                                                                                                                                               (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                (shen/shen\.bindv V3444
                                                                                                                                                                                                                                                                                                  (list '--> B)
                                                                                                                                                                                                                                                                                                  V3561)
                                                                                                                                                                                                                                                                                (shen/let Result
                                                                                                                                                                                                                                                                                          (shen/let Z
                                                                                                                                                                                                                                                                                                    (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                    (shen/let X&&
                                                                                                                                                                                                                                                                                                              (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                                                                               (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                               (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                         (shen/freeze
                                                                                                                                                                                                                                                                                                                          (shen/bind X&&
                                                                                                                                                                                                                                                                                                                                     (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                                     V3561
                                                                                                                                                                                                                                                                                                                                     (shen/freeze
                                                                                                                                                                                                                                                                                                                                      (shen/bind Z
                                                                                                                                                                                                                                                                                                                                                 (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                                 V3561
                                                                                                                                                                                                                                                                                                                                                 (shen/freeze
                                                                                                                                                                                                                                                                                                                                                  (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                                  (append
                                                                                                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                                                                                                    (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                                   V3560)
                                                                                                                                                                                                                                                                                                                                                                  V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                                                           (shen/shen\.unbindv V3444 V3561)
                                                                                                                                                                                                                                                                                           Result))))
                                                                                                                                                                                                                                                                     'false))))
                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                (shen/shen\.pvar\? V3443)
                                                                                                                                                                                                                                                (shen/let A
                                                                                                                                                                                                                                                          (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                          (shen/let B
                                                                                                                                                                                                                                                                    (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                                                     (shen/shen\.bindv V3443
                                                                                                                                                                                                                                                                                       (list A '--> B)
                                                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                                                     (shen/let Result
                                                                                                                                                                                                                                                                               (shen/let Z
                                                                                                                                                                                                                                                                                         (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                         (shen/let X&&
                                                                                                                                                                                                                                                                                                   (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                                                                    (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                    (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                              (shen/freeze
                                                                                                                                                                                                                                                                                                               (shen/bind X&&
                                                                                                                                                                                                                                                                                                                          (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                          V3561
                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                           (shen/bind Z
                                                                                                                                                                                                                                                                                                                                      (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef Y V3561))
                                                                                                                                                                                                                                                                                                                                      V3561
                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                       (shen/shen\.th* Z B
                                                                                                                                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                                                                                                         (list X&& ': A))
                                                                                                                                                                                                                                                                                                                                                        V3560)
                                                                                                                                                                                                                                                                                                                                                       V3561 V3562))))))))))
                                                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                                                (shen/shen\.unbindv V3443 V3561)
                                                                                                                                                                                                                                                                                Result)))))
                                                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                                                    'false)))
                                                                                                                                                                                                               'false)))
                                                                                                                                                                                          'false))
                                                                                                                                                                               'false))
                                                                                                                                                                    'false))
                                                                                                                                                         (shen/if
                                                                                                                                                          (shen/= Case 'false)
                                                                                                                                                          (shen/let Case
                                                                                                                                                                    (shen/let V3450
                                                                                                                                                                              (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                              (shen/if
                                                                                                                                                                               (shen/cons\? V3450)
                                                                                                                                                                               (shen/let V3451
                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                          (shen/hd V3450)
                                                                                                                                                                                          V3561)
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/= 'let V3451)
                                                                                                                                                                                          (shen/let V3452
                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                     (nthcdr 1 V3450)
                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/cons\? V3452)
                                                                                                                                                                                                     (shen/let X
                                                                                                                                                                                                               (shen/hd V3452)
                                                                                                                                                                                                               (shen/let V3453
                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                          (nthcdr 1 V3452)
                                                                                                                                                                                                                          V3561)
                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                          (shen/cons\? V3453)
                                                                                                                                                                                                                          (shen/let Y
                                                                                                                                                                                                                                    (shen/hd V3453)
                                                                                                                                                                                                                                    (shen/let V3454
                                                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                                                               (nthcdr 1 V3453)
                                                                                                                                                                                                                                               V3561)
                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                               (shen/cons\? V3454)
                                                                                                                                                                                                                                               (shen/let Z
                                                                                                                                                                                                                                                         (shen/hd V3454)
                                                                                                                                                                                                                                                         (shen/let V3455
                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                    (nthcdr 1 V3454)
                                                                                                                                                                                                                                                                    V3561)
                                                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                                                    (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                     (null V3455))
                                                                                                                                                                                                                                                                    (shen/let W
                                                                                                                                                                                                                                                                              (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                              (shen/let X&&
                                                                                                                                                                                                                                                                                        (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                        (shen/let B
                                                                                                                                                                                                                                                                                                  (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                                                                   (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                   (shen/shen\.th* Y B V3560 V3561
                                                                                                                                                                                                                                                                                                                   (shen/freeze
                                                                                                                                                                                                                                                                                                                    (shen/bind X&&
                                                                                                                                                                                                                                                                                                                               (shen/shen\.placeholder)
                                                                                                                                                                                                                                                                                                                               V3561
                                                                                                                                                                                                                                                                                                                               (shen/freeze
                                                                                                                                                                                                                                                                                                                                (shen/bind W
                                                                                                                                                                                                                                                                                                                                           (shen/shen\.ebr
                                                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef X&& V3561)
                                                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef X V3561)
                                                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef Z V3561))
                                                                                                                                                                                                                                                                                                                                           V3561
                                                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                                                            (shen/shen\.th* W V3559
                                                                                                                                                                                                                                                                                                                                                            (append
                                                                                                                                                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                                                                                                                                                              (list X&& ': B))
                                                                                                                                                                                                                                                                                                                                                             V3560)
                                                                                                                                                                                                                                                                                                                                                            V3561 V3562)))))))))))
                                                                                                                                                                                                                                                                    'false)))
                                                                                                                                                                                                                                               'false)))
                                                                                                                                                                                                                          'false)))
                                                                                                                                                                                                     'false))
                                                                                                                                                                                          'false))
                                                                                                                                                                               'false))
                                                                                                                                                                    (shen/if
                                                                                                                                                                     (shen/= Case 'false)
                                                                                                                                                                     (shen/let Case
                                                                                                                                                                               (shen/let V3456
                                                                                                                                                                                         (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/cons\? V3456)
                                                                                                                                                                                          (shen/let V3457
                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                     (shen/hd V3456)
                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/= 'open V3457)
                                                                                                                                                                                                     (shen/let V3458
                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                (nthcdr 1 V3456)
                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/cons\? V3458)
                                                                                                                                                                                                                (shen/let FileName
                                                                                                                                                                                                                          (shen/hd V3458)
                                                                                                                                                                                                                          (shen/let V3459
                                                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                                                     (nthcdr 1 V3458)
                                                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                     (shen/cons\? V3459)
                                                                                                                                                                                                                                     (shen/let Direction3389
                                                                                                                                                                                                                                               (shen/hd V3459)
                                                                                                                                                                                                                                               (shen/let V3460
                                                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                                                          (nthcdr 1 V3459)
                                                                                                                                                                                                                                                          V3561)
                                                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                                                          (shen/internal/predicate->shen
                                                                                                                                                                                                                                                           (null V3460))
                                                                                                                                                                                                                                                          (shen/let V3461
                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                                                     (shen/cons\? V3461)
                                                                                                                                                                                                                                                                     (shen/let V3462
                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                (shen/hd V3461)
                                                                                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                                                (shen/= 'stream V3462)
                                                                                                                                                                                                                                                                                (shen/let V3463
                                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                           (nthcdr 1 V3461)
                                                                                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                                                           (shen/cons\? V3463)
                                                                                                                                                                                                                                                                                           (shen/let Direction
                                                                                                                                                                                                                                                                                                     (shen/hd V3463)
                                                                                                                                                                                                                                                                                                     (shen/let V3464
                                                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                                                (nthcdr 1 V3463)
                                                                                                                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                                                 (null V3464))
                                                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                 (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                                              (shen/freeze
                                                                                                                                                                                                                                                                                                                               (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                         (shen/freeze
                                                                                                                                                                                                                                                                                                                                          (shen/fwhen
                                                                                                                                                                                                                                                                                                                                           (shen/element\?
                                                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                                            (list 'in 'out))
                                                                                                                                                                                                                                                                                                                                           V3561
                                                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                                                            (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                                                                                 (shen/shen\.pvar\? V3464)
                                                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                                                  (shen/shen\.bindv V3464 nil V3561)
                                                                                                                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                             (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                                           (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                     (shen/freeze
                                                                                                                                                                                                                                                                                                                                                      (shen/fwhen
                                                                                                                                                                                                                                                                                                                                                       (shen/element\?
                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                                                        (list 'in 'out))
                                                                                                                                                                                                                                                                                                                                                       V3561
                                                                                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                                                             (shen/shen\.unbindv V3464 V3561)
                                                                                                                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                                                                                                                 'false))))
                                                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                                                            (shen/shen\.pvar\? V3463)
                                                                                                                                                                                                                                                                                            (shen/let Direction
                                                                                                                                                                                                                                                                                                      (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                                                       (shen/shen\.bindv V3463
                                                                                                                                                                                                                                                                                                                         (list Direction)
                                                                                                                                                                                                                                                                                                                         V3561)
                                                                                                                                                                                                                                                                                                       (shen/let Result
                                                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                  (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                                               (shen/freeze
                                                                                                                                                                                                                                                                                                                                (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                                           (shen/fwhen
                                                                                                                                                                                                                                                                                                                                            (shen/element\?
                                                                                                                                                                                                                                                                                                                                             (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                                             (list 'in 'out))
                                                                                                                                                                                                                                                                                                                                            V3561
                                                                                                                                                                                                                                                                                                                                            (shen/freeze
                                                                                                                                                                                                                                                                                                                                             (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                                                  (shen/shen\.unbindv V3463 V3561)
                                                                                                                                                                                                                                                                                                                  Result))))
                                                                                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                                                 (shen/shen\.pvar\? V3462)
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.bindv V3462 'stream V3561)
                                                                                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                                                                                            (shen/let V3465
                                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                                       (nthcdr 1 V3461)
                                                                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                                                                                       (shen/cons\? V3465)
                                                                                                                                                                                                                                                                                                       (shen/let Direction
                                                                                                                                                                                                                                                                                                                 (shen/hd V3465)
                                                                                                                                                                                                                                                                                                                 (shen/let V3466
                                                                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                                                            (nthcdr 1 V3465)
                                                                                                                                                                                                                                                                                                                            V3561)
                                                                                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                                                                                            (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                                                             (null V3466))
                                                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                             (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                                                                                                           (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                     (shen/freeze
                                                                                                                                                                                                                                                                                                                                                      (shen/fwhen
                                                                                                                                                                                                                                                                                                                                                       (shen/element\?
                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                                                        (list 'in 'out))
                                                                                                                                                                                                                                                                                                                                                       V3561
                                                                                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                                                                                        (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                                                                                                                             (shen/shen\.pvar\? V3466)
                                                                                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                                                                                              (shen/shen\.bindv V3466 nil V3561)
                                                                                                                                                                                                                                                                                                                              (shen/let Result
                                                                                                                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                                                                                                                         (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                                         (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                                       (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                                 (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                  (shen/fwhen
                                                                                                                                                                                                                                                                                                                                                                   (shen/element\?
                                                                                                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                                                                    (list 'in 'out))
                                                                                                                                                                                                                                                                                                                                                                   V3561
                                                                                                                                                                                                                                                                                                                                                                   (shen/freeze
                                                                                                                                                                                                                                                                                                                                                                    (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                                                                                                                         (shen/shen\.unbindv V3466 V3561)
                                                                                                                                                                                                                                                                                                                                         Result)))
                                                                                                                                                                                                                                                                                                                             'false))))
                                                                                                                                                                                                                                                                                                       (shen/if
                                                                                                                                                                                                                                                                                                        (shen/shen\.pvar\? V3465)
                                                                                                                                                                                                                                                                                                        (shen/let Direction
                                                                                                                                                                                                                                                                                                                  (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                                                                                   (shen/shen\.bindv V3465
                                                                                                                                                                                                                                                                                                                                     (list Direction)
                                                                                                                                                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                                                                                                                                   (shen/let Result
                                                                                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                                                                                              (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                              (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                                                           (shen/freeze
                                                                                                                                                                                                                                                                                                                                            (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                                                       (shen/fwhen
                                                                                                                                                                                                                                                                                                                                                        (shen/element\?
                                                                                                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                                                         (list 'in 'out))
                                                                                                                                                                                                                                                                                                                                                        V3561
                                                                                                                                                                                                                                                                                                                                                        (shen/freeze
                                                                                                                                                                                                                                                                                                                                                         (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                                                                                              (shen/shen\.unbindv V3465 V3561)
                                                                                                                                                                                                                                                                                                                              Result))))
                                                                                                                                                                                                                                                                                                        'false)))
                                                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                                                             (shen/shen\.unbindv V3462 V3561)
                                                                                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                                                                                 'false)))
                                                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                                                      (shen/shen\.pvar\? V3461)
                                                                                                                                                                                                                                                                      (shen/let Direction
                                                                                                                                                                                                                                                                                (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                                                 (shen/shen\.bindv V3461
                                                                                                                                                                                                                                                                                                   (list 'stream Direction)
                                                                                                                                                                                                                                                                                                   V3561)
                                                                                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                            (shen/unify! Direction Direction3389 V3561
                                                                                                                                                                                                                                                                                                         (shen/freeze
                                                                                                                                                                                                                                                                                                          (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                    (shen/freeze
                                                                                                                                                                                                                                                                                                                     (shen/fwhen
                                                                                                                                                                                                                                                                                                                      (shen/element\?
                                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef Direction V3561)
                                                                                                                                                                                                                                                                                                                       (list 'in 'out))
                                                                                                                                                                                                                                                                                                                      V3561
                                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                                       (shen/shen\.th* FileName 'string V3560 V3561 V3562))))))))
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.unbindv V3461 V3561)
                                                                                                                                                                                                                                                                                            Result))))
                                                                                                                                                                                                                                                                      'false)))
                                                                                                                                                                                                                                                          'false)))
                                                                                                                                                                                                                                     'false)))
                                                                                                                                                                                                                'false))
                                                                                                                                                                                                     'false))
                                                                                                                                                                                          'false))
                                                                                                                                                                               (shen/if
                                                                                                                                                                                (shen/= Case 'false)
                                                                                                                                                                                (shen/let Case
                                                                                                                                                                                          (shen/let V3467
                                                                                                                                                                                                    (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/cons\? V3467)
                                                                                                                                                                                                     (shen/let V3468
                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                (shen/hd V3467)
                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/= 'type V3468)
                                                                                                                                                                                                                (shen/let V3469
                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                           (nthcdr 1 V3467)
                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/cons\? V3469)
                                                                                                                                                                                                                           (shen/let X
                                                                                                                                                                                                                                     (shen/hd V3469)
                                                                                                                                                                                                                                     (shen/let V3470
                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                (nthcdr 1 V3469)
                                                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                (shen/cons\? V3470)
                                                                                                                                                                                                                                                (shen/let A
                                                                                                                                                                                                                                                          (shen/hd V3470)
                                                                                                                                                                                                                                                          (shen/let V3471
                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                     (nthcdr 1 V3470)
                                                                                                                                                                                                                                                                     V3561)
                                                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                                                     (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                      (null V3471))
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                      (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                (shen/freeze
                                                                                                                                                                                                                                                                                 (shen/unify A V3559 V3561
                                                                                                                                                                                                                                                                                             (shen/freeze
                                                                                                                                                                                                                                                                                              (shen/shen\.th* X A V3560 V3561 V3562))))))
                                                                                                                                                                                                                                                                     'false)))
                                                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                                           'false))
                                                                                                                                                                                                                'false))
                                                                                                                                                                                                     'false))
                                                                                                                                                                                          (shen/if
                                                                                                                                                                                           (shen/= Case 'false)
                                                                                                                                                                                           (shen/let Case
                                                                                                                                                                                                     (shen/let V3472
                                                                                                                                                                                                               (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/cons\? V3472)
                                                                                                                                                                                                                (shen/let V3473
                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                           (shen/hd V3472)
                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/= 'input+ V3473)
                                                                                                                                                                                                                           (shen/let V3474
                                                                                                                                                                                                                                     (shen/shen\.lazyderef
                                                                                                                                                                                                                                      (nthcdr 1 V3472)
                                                                                                                                                                                                                                      V3561)
                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                      (shen/cons\? V3474)
                                                                                                                                                                                                                                      (shen/let A
                                                                                                                                                                                                                                                (shen/hd V3474)
                                                                                                                                                                                                                                                (shen/let V3475
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                           (nthcdr 1 V3474)
                                                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                           (shen/cons\? V3475)
                                                                                                                                                                                                                                                           (shen/let Stream
                                                                                                                                                                                                                                                                     (shen/hd V3475)
                                                                                                                                                                                                                                                                     (shen/let V3476
                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                (nthcdr 1 V3475)
                                                                                                                                                                                                                                                                                V3561)
                                                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                 (null V3476))
                                                                                                                                                                                                                                                                                (shen/let C
                                                                                                                                                                                                                                                                                          (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                           (shen/bind C
                                                                                                                                                                                                                                                                                                      (shen/shen\.demodulate
                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef A V3561))
                                                                                                                                                                                                                                                                                                      V3561
                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                       (shen/unify V3559 C V3561
                                                                                                                                                                                                                                                                                                                   (shen/freeze
                                                                                                                                                                                                                                                                                                                    (shen/shen\.th* Stream
                                                                                                                                                                                                                                                                                                                                    (list 'stream 'in)
                                                                                                                                                                                                                                                                                                                                    V3560 V3561 V3562)))))))
                                                                                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                                                                           'false)))
                                                                                                                                                                                                                                      'false))
                                                                                                                                                                                                                           'false))
                                                                                                                                                                                                                'false))
                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                      (shen/= Case 'false)
                                                                                                                                                                                                      (shen/let Case
                                                                                                                                                                                                                (shen/let V3477
                                                                                                                                                                                                                          (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/cons\? V3477)
                                                                                                                                                                                                                           (shen/let V3478
                                                                                                                                                                                                                                     (shen/shen\.lazyderef
                                                                                                                                                                                                                                      (shen/hd V3477)
                                                                                                                                                                                                                                      V3561)
                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                      (shen/= 'set V3478)
                                                                                                                                                                                                                                      (shen/let V3479
                                                                                                                                                                                                                                                (shen/shen\.lazyderef
                                                                                                                                                                                                                                                 (nthcdr 1 V3477)
                                                                                                                                                                                                                                                 V3561)
                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                 (shen/cons\? V3479)
                                                                                                                                                                                                                                                 (shen/let Var
                                                                                                                                                                                                                                                           (shen/hd V3479)
                                                                                                                                                                                                                                                           (shen/let V3480
                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                      (nthcdr 1 V3479)
                                                                                                                                                                                                                                                                      V3561)
                                                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                                                      (shen/cons\? V3480)
                                                                                                                                                                                                                                                                      (shen/let Val
                                                                                                                                                                                                                                                                                (shen/hd V3480)
                                                                                                                                                                                                                                                                                (shen/let V3481
                                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                           (nthcdr 1 V3480)
                                                                                                                                                                                                                                                                                           V3561)
                                                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                                                           (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                                            (null V3481))
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                            (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                       (shen/shen\.th* Var 'symbol V3560 V3561
                                                                                                                                                                                                                                                                                                                       (shen/freeze
                                                                                                                                                                                                                                                                                                                        (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                                                                                                                                                                   (shen/shen\.th*
                                                                                                                                                                                                                                                                                                                                    (list 'value Var)
                                                                                                                                                                                                                                                                                                                                    V3559 V3560 V3561
                                                                                                                                                                                                                                                                                                                                    (shen/freeze
                                                                                                                                                                                                                                                                                                                                     (shen/shen\.th* Val V3559 V3560 V3561 V3562))))))))))
                                                                                                                                                                                                                                                                                           'false)))
                                                                                                                                                                                                                                                                      'false)))
                                                                                                                                                                                                                                                 'false))
                                                                                                                                                                                                                                      'false))
                                                                                                                                                                                                                           'false))
                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                 (shen/= Case 'false)
                                                                                                                                                                                                                 (shen/let Case
                                                                                                                                                                                                                           (shen/let NewHyp
                                                                                                                                                                                                                                     (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                      (shen/shen\.t*-hyps V3560 NewHyp V3561
                                                                                                                                                                                                                                                          (shen/freeze
                                                                                                                                                                                                                                                           (shen/shen\.th* V3558 V3559 NewHyp V3561 V3562)))))
                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                            (shen/= Case 'false)
                                                                                                                                                                                                                            (shen/let Case
                                                                                                                                                                                                                                      (shen/let V3482
                                                                                                                                                                                                                                                (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                 (shen/cons\? V3482)
                                                                                                                                                                                                                                                 (shen/let V3483
                                                                                                                                                                                                                                                           (shen/shen\.lazyderef
                                                                                                                                                                                                                                                            (shen/hd V3482)
                                                                                                                                                                                                                                                            V3561)
                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                            (shen/= 'define V3483)
                                                                                                                                                                                                                                                            (shen/let V3484
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                       (nthcdr 1 V3482)
                                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                                                       (shen/cons\? V3484)
                                                                                                                                                                                                                                                                       (shen/let F
                                                                                                                                                                                                                                                                                 (shen/hd V3484)
                                                                                                                                                                                                                                                                                 (shen/let X
                                                                                                                                                                                                                                                                                           (nthcdr 1 V3484)
                                                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                            (shen/cut Throwcontrol V3561
                                                                                                                                                                                                                                                                                                      (shen/freeze
                                                                                                                                                                                                                                                                                                       (shen/shen\.t*-def
                                                                                                                                                                                                                                                                                                        (append
                                                                                                                                                                                                                                                                                                         (list 'define F)
                                                                                                                                                                                                                                                                                                         X)
                                                                                                                                                                                                                                                                                                        V3559 V3560 V3561 V3562))))))
                                                                                                                                                                                                                                                                       'false))
                                                                                                                                                                                                                                                            'false))
                                                                                                                                                                                                                                                 'false))
                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                       (shen/= Case 'false)
                                                                                                                                                                                                                                       (shen/let Case
                                                                                                                                                                                                                                                 (shen/let V3485
                                                                                                                                                                                                                                                           (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                            (shen/cons\? V3485)
                                                                                                                                                                                                                                                            (shen/let V3486
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                       (shen/hd V3485)
                                                                                                                                                                                                                                                                       V3561)
                                                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                                                       (shen/= 'defmacro V3486)
                                                                                                                                                                                                                                                                       (shen/let V3487
                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                                                  (shen/= 'unit V3487)
                                                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                                                   (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                   (shen/cut Throwcontrol V3561 V3562))
                                                                                                                                                                                                                                                                                  (shen/if
                                                                                                                                                                                                                                                                                   (shen/shen\.pvar\? V3487)
                                                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                                                    (shen/shen\.bindv V3487 'unit V3561)
                                                                                                                                                                                                                                                                                    (shen/let Result
                                                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                                                               (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                               (shen/cut Throwcontrol V3561 V3562))
                                                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                                                               (shen/shen\.unbindv V3487 V3561)
                                                                                                                                                                                                                                                                                               Result)))
                                                                                                                                                                                                                                                                                   'false)))
                                                                                                                                                                                                                                                                       'false))
                                                                                                                                                                                                                                                            'false))
                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                  (shen/= Case 'false)
                                                                                                                                                                                                                                                  (shen/let Case
                                                                                                                                                                                                                                                            (shen/let V3488
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                                                       (shen/cons\? V3488)
                                                                                                                                                                                                                                                                       (shen/let V3489
                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                  (shen/hd V3488)
                                                                                                                                                                                                                                                                                  V3561)
                                                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                                                  (shen/= 'shen\.process-datatype V3489)
                                                                                                                                                                                                                                                                                  (shen/let V3490
                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                                                                                             (shen/= 'symbol V3490)
                                                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                                                              (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                              (shen/thaw V3562))
                                                                                                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                                                                                                              (shen/shen\.pvar\? V3490)
                                                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                                                               (shen/shen\.bindv V3490 'symbol V3561)
                                                                                                                                                                                                                                                                                               (shen/let Result
                                                                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                          (shen/thaw V3562))
                                                                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                                                                          (shen/shen\.unbindv V3490 V3561)
                                                                                                                                                                                                                                                                                                          Result)))
                                                                                                                                                                                                                                                                                              'false)))
                                                                                                                                                                                                                                                                                  'false))
                                                                                                                                                                                                                                                                       'false))
                                                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                                                             (shen/= Case 'false)
                                                                                                                                                                                                                                                             (shen/let Case
                                                                                                                                                                                                                                                                       (shen/let V3491
                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef V3558 V3561)
                                                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                                                  (shen/cons\? V3491)
                                                                                                                                                                                                                                                                                  (shen/let V3492
                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                                             (shen/hd V3491)
                                                                                                                                                                                                                                                                                             V3561)
                                                                                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                                                                                             (shen/= 'shen\.synonyms-help V3492)
                                                                                                                                                                                                                                                                                             (shen/let V3493
                                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef V3559 V3561)
                                                                                                                                                                                                                                                                                                       (shen/if
                                                                                                                                                                                                                                                                                                        (shen/= 'symbol V3493)
                                                                                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                                                                                         (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                         (shen/thaw V3562))
                                                                                                                                                                                                                                                                                                        (shen/if
                                                                                                                                                                                                                                                                                                         (shen/shen\.pvar\? V3493)
                                                                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                                                                          (shen/shen\.bindv V3493 'symbol V3561)
                                                                                                                                                                                                                                                                                                          (shen/let Result
                                                                                                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                                                     (shen/thaw V3562))
                                                                                                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                                                                                                     (shen/shen\.unbindv V3493 V3561)
                                                                                                                                                                                                                                                                                                                     Result)))
                                                                                                                                                                                                                                                                                                         'false)))
                                                                                                                                                                                                                                                                                             'false))
                                                                                                                                                                                                                                                                                  'false))
                                                                                                                                                                                                                                                                       (shen/if
                                                                                                                                                                                                                                                                        (shen/= Case 'false)
                                                                                                                                                                                                                                                                        (shen/let Datatypes
                                                                                                                                                                                                                                                                                  (shen/shen\.newpv V3561)
                                                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                                                   (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                   (shen/bind Datatypes
                                                                                                                                                                                                                                                                                              (shen/value 'shen\.*datatypes*)
                                                                                                                                                                                                                                                                                              V3561
                                                                                                                                                                                                                                                                                              (shen/freeze
                                                                                                                                                                                                                                                                                               (shen/shen\.udefs*
                                                                                                                                                                                                                                                                                                (list V3558 ': V3559)
                                                                                                                                                                                                                                                                                                V3560 Datatypes V3561 V3562)))))
                                                                                                                                                                                                                                                                        Case))
                                                                                                                                                                                                                                                             Case))
                                                                                                                                                                                                                                                  Case))
                                                                                                                                                                                                                                       Case))
                                                                                                                                                                                                                            Case))
                                                                                                                                                                                                                 Case))
                                                                                                                                                                                                      Case))
                                                                                                                                                                                           Case))
                                                                                                                                                                                Case))
                                                                                                                                                                     Case))
                                                                                                                                                          Case))
                                                                                                                                               Case))
                                                                                                                                    Case))
                                                                                                                         Case))
                                                                                                              Case))
                                                                                                   Case))
                                                                                        Case))
                                                                             Case))
                                                                  Case))
                                                       Case))
                                            Case)))))
(defun shen/shen\.t*-hyps
    (V3567 V3568 V3569 V3570)
  (shen/let Case
            (shen/let V3304
                      (shen/shen\.lazyderef V3567 V3569)
                      (shen/if
                       (shen/cons\? V3304)
                       (shen/let V3305
                                 (shen/shen\.lazyderef
                                  (shen/hd V3304)
                                  V3569)
                                 (shen/if
                                  (shen/cons\? V3305)
                                  (shen/let V3306
                                            (shen/shen\.lazyderef
                                             (shen/hd V3305)
                                             V3569)
                                            (shen/if
                                             (shen/cons\? V3306)
                                             (shen/let V3307
                                                       (shen/shen\.lazyderef
                                                        (shen/hd V3306)
                                                        V3569)
                                                       (shen/if
                                                        (shen/= 'cons V3307)
                                                        (shen/let V3308
                                                                  (shen/shen\.lazyderef
                                                                   (nthcdr 1 V3306)
                                                                   V3569)
                                                                  (shen/if
                                                                   (shen/cons\? V3308)
                                                                   (shen/let X
                                                                             (shen/hd V3308)
                                                                             (shen/let V3309
                                                                                       (shen/shen\.lazyderef
                                                                                        (nthcdr 1 V3308)
                                                                                        V3569)
                                                                                       (shen/if
                                                                                        (shen/cons\? V3309)
                                                                                        (shen/let Y
                                                                                                  (shen/hd V3309)
                                                                                                  (shen/let V3310
                                                                                                            (shen/shen\.lazyderef
                                                                                                             (nthcdr 1 V3309)
                                                                                                             V3569)
                                                                                                            (shen/if
                                                                                                             (shen/internal/predicate->shen
                                                                                                              (null V3310))
                                                                                                             (shen/let V3311
                                                                                                                       (shen/shen\.lazyderef
                                                                                                                        (nthcdr 1 V3305)
                                                                                                                        V3569)
                                                                                                                       (shen/if
                                                                                                                        (shen/cons\? V3311)
                                                                                                                        (shen/let V3312
                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                   (shen/hd V3311)
                                                                                                                                   V3569)
                                                                                                                                  (shen/if
                                                                                                                                   (shen/= ': V3312)
                                                                                                                                   (shen/let V3313
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (nthcdr 1 V3311)
                                                                                                                                              V3569)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/cons\? V3313)
                                                                                                                                              (shen/let V3314
                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                         (shen/hd V3313)
                                                                                                                                                         V3569)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/cons\? V3314)
                                                                                                                                                         (shen/let V3315
                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                    (shen/hd V3314)
                                                                                                                                                                    V3569)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/= 'list V3315)
                                                                                                                                                                    (shen/let V3316
                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                               (nthcdr 1 V3314)
                                                                                                                                                                               V3569)
                                                                                                                                                                              (shen/if
                                                                                                                                                                               (shen/cons\? V3316)
                                                                                                                                                                               (shen/let A
                                                                                                                                                                                         (shen/hd V3316)
                                                                                                                                                                                         (shen/let V3317
                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                    (nthcdr 1 V3316)
                                                                                                                                                                                                    V3569)
                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                    (shen/internal/predicate->shen
                                                                                                                                                                                                     (null V3317))
                                                                                                                                                                                                    (shen/let V3318
                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                               (nthcdr 1 V3313)
                                                                                                                                                                                                               V3569)
                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                               (shen/internal/predicate->shen
                                                                                                                                                                                                                (null V3318))
                                                                                                                                                                                                               (shen/let Hyp
                                                                                                                                                                                                                         (nthcdr 1 V3304)
                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                                                                                          (shen/bind V3568
                                                                                                                                                                                                                                     (append
                                                                                                                                                                                                                                      (list
                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                        (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                        ':
                                                                                                                                                                                                                                        (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                        (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                        ':
                                                                                                                                                                                                                                        (list 'list
                                                                                                                                                                                                                                              (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                      (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                     V3569 V3570)))
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/shen\.pvar\? V3318)
                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                 (shen/shen\.bindv V3318 nil V3569)
                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                     (nthcdr 1 V3304)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (list 'list
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.unbindv V3318 V3569)
                                                                                                                                                                                                                            Result)))
                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/shen\.pvar\? V3317)
                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                      (shen/shen\.bindv V3317 nil V3569)
                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                (shen/let V3319
                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                           (nthcdr 1 V3313)
                                                                                                                                                                                                                           V3569)
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/internal/predicate->shen
                                                                                                                                                                                                                            (null V3319))
                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                     (nthcdr 1 V3304)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (list 'list
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                            (shen/shen\.pvar\? V3319)
                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                             (shen/shen\.bindv V3319 nil V3569)
                                                                                                                                                                                                                             (shen/let Result
                                                                                                                                                                                                                                       (shen/let Hyp
                                                                                                                                                                                                                                                 (nthcdr 1 V3304)
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                  (shen/bind V3568
                                                                                                                                                                                                                                                             (append
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                (list 'list
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                             V3569 V3570)))
                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                        (shen/shen\.unbindv V3319 V3569)
                                                                                                                                                                                                                                        Result)))
                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                 (shen/shen\.unbindv V3317 V3569)
                                                                                                                                                                                                                 Result)))
                                                                                                                                                                                                     'false))))
                                                                                                                                                                               (shen/if
                                                                                                                                                                                (shen/shen\.pvar\? V3316)
                                                                                                                                                                                (shen/let A
                                                                                                                                                                                          (shen/shen\.newpv V3569)
                                                                                                                                                                                          (shen/do
                                                                                                                                                                                           (shen/shen\.bindv V3316
                                                                                                                                                                                                             (list A)
                                                                                                                                                                                                             V3569)
                                                                                                                                                                                           (shen/let Result
                                                                                                                                                                                                     (shen/let V3320
                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                (nthcdr 1 V3313)
                                                                                                                                                                                                                V3569)
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                 (null V3320))
                                                                                                                                                                                                                (shen/let Hyp
                                                                                                                                                                                                                          (nthcdr 1 V3304)
                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                           (shen/bind V3568
                                                                                                                                                                                                                                      (append
                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                         (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                         ':
                                                                                                                                                                                                                                         (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                         (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                         ':
                                                                                                                                                                                                                                         (list 'list
                                                                                                                                                                                                                                               (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                       (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                      V3569 V3570)))
                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                 (shen/shen\.pvar\? V3320)
                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                  (shen/shen\.bindv V3320 nil V3569)
                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                            (shen/let Hyp
                                                                                                                                                                                                                                      (nthcdr 1 V3304)
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                       (shen/bind V3568
                                                                                                                                                                                                                                                  (append
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                     (list 'list
                                                                                                                                                                                                                                                           (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                  V3569 V3570)))
                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                             (shen/shen\.unbindv V3320 V3569)
                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                 'false)))
                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                      (shen/shen\.unbindv V3316 V3569)
                                                                                                                                                                                                      Result))))
                                                                                                                                                                                'false)))
                                                                                                                                                                    (shen/if
                                                                                                                                                                     (shen/shen\.pvar\? V3315)
                                                                                                                                                                     (shen/do
                                                                                                                                                                      (shen/shen\.bindv V3315 'list V3569)
                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                (shen/let V3321
                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                           (nthcdr 1 V3314)
                                                                                                                                                                                           V3569)
                                                                                                                                                                                          (shen/if
                                                                                                                                                                                           (shen/cons\? V3321)
                                                                                                                                                                                           (shen/let A
                                                                                                                                                                                                     (shen/hd V3321)
                                                                                                                                                                                                     (shen/let V3322
                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                (nthcdr 1 V3321)
                                                                                                                                                                                                                V3569)
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                 (null V3322))
                                                                                                                                                                                                                (shen/let V3323
                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                           (nthcdr 1 V3313)
                                                                                                                                                                                                                           V3569)
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/internal/predicate->shen
                                                                                                                                                                                                                            (null V3323))
                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                     (nthcdr 1 V3304)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (list 'list
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                            (shen/shen\.pvar\? V3323)
                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                             (shen/shen\.bindv V3323 nil V3569)
                                                                                                                                                                                                                             (shen/let Result
                                                                                                                                                                                                                                       (shen/let Hyp
                                                                                                                                                                                                                                                 (nthcdr 1 V3304)
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                  (shen/bind V3568
                                                                                                                                                                                                                                                             (append
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                (list 'list
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                             V3569 V3570)))
                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                        (shen/shen\.unbindv V3323 V3569)
                                                                                                                                                                                                                                        Result)))
                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                 (shen/shen\.pvar\? V3322)
                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                  (shen/shen\.bindv V3322 nil V3569)
                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                            (shen/let V3324
                                                                                                                                                                                                                                      (shen/shen\.lazyderef
                                                                                                                                                                                                                                       (nthcdr 1 V3313)
                                                                                                                                                                                                                                       V3569)
                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                       (shen/internal/predicate->shen
                                                                                                                                                                                                                                        (null V3324))
                                                                                                                                                                                                                                       (shen/let Hyp
                                                                                                                                                                                                                                                 (nthcdr 1 V3304)
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                  (shen/bind V3568
                                                                                                                                                                                                                                                             (append
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                (list 'list
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                             V3569 V3570)))
                                                                                                                                                                                                                                       (shen/if
                                                                                                                                                                                                                                        (shen/shen\.pvar\? V3324)
                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                         (shen/shen\.bindv V3324 nil V3569)
                                                                                                                                                                                                                                         (shen/let Result
                                                                                                                                                                                                                                                   (shen/let Hyp
                                                                                                                                                                                                                                                             (nthcdr 1 V3304)
                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                              (shen/shen\.incinfs)
                                                                                                                                                                                                                                                              (shen/bind V3568
                                                                                                                                                                                                                                                                         (append
                                                                                                                                                                                                                                                                          (list
                                                                                                                                                                                                                                                                           (list
                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                            ':
                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                           (list
                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                            ':
                                                                                                                                                                                                                                                                            (list 'list
                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                         V3569 V3570)))
                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                    (shen/shen\.unbindv V3324 V3569)
                                                                                                                                                                                                                                                    Result)))
                                                                                                                                                                                                                                        'false)))
                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                             (shen/shen\.unbindv V3322 V3569)
                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                 'false))))
                                                                                                                                                                                           (shen/if
                                                                                                                                                                                            (shen/shen\.pvar\? V3321)
                                                                                                                                                                                            (shen/let A
                                                                                                                                                                                                      (shen/shen\.newpv V3569)
                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                       (shen/shen\.bindv V3321
                                                                                                                                                                                                                         (list A)
                                                                                                                                                                                                                         V3569)
                                                                                                                                                                                                       (shen/let Result
                                                                                                                                                                                                                 (shen/let V3325
                                                                                                                                                                                                                           (shen/shen\.lazyderef
                                                                                                                                                                                                                            (nthcdr 1 V3313)
                                                                                                                                                                                                                            V3569)
                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                            (shen/internal/predicate->shen
                                                                                                                                                                                                                             (null V3325))
                                                                                                                                                                                                                            (shen/let Hyp
                                                                                                                                                                                                                                      (nthcdr 1 V3304)
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                       (shen/bind V3568
                                                                                                                                                                                                                                                  (append
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                     (list 'list
                                                                                                                                                                                                                                                           (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                  V3569 V3570)))
                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                             (shen/shen\.pvar\? V3325)
                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                              (shen/shen\.bindv V3325 nil V3569)
                                                                                                                                                                                                                              (shen/let Result
                                                                                                                                                                                                                                        (shen/let Hyp
                                                                                                                                                                                                                                                  (nthcdr 1 V3304)
                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                   (shen/shen\.incinfs)
                                                                                                                                                                                                                                                   (shen/bind V3568
                                                                                                                                                                                                                                                              (append
                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                (list
                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                 ':
                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                (list
                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                 ':
                                                                                                                                                                                                                                                                 (list 'list
                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                              V3569 V3570)))
                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                         (shen/shen\.unbindv V3325 V3569)
                                                                                                                                                                                                                                         Result)))
                                                                                                                                                                                                                             'false)))
                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                  (shen/shen\.unbindv V3321 V3569)
                                                                                                                                                                                                                  Result))))
                                                                                                                                                                                            'false)))
                                                                                                                                                                                (shen/do
                                                                                                                                                                                 (shen/shen\.unbindv V3315 V3569)
                                                                                                                                                                                 Result)))
                                                                                                                                                                     'false)))
                                                                                                                                                         (shen/if
                                                                                                                                                          (shen/shen\.pvar\? V3314)
                                                                                                                                                          (shen/let A
                                                                                                                                                                    (shen/shen\.newpv V3569)
                                                                                                                                                                    (shen/do
                                                                                                                                                                     (shen/shen\.bindv V3314
                                                                                                                                                                                       (list 'list A)
                                                                                                                                                                                       V3569)
                                                                                                                                                                     (shen/let Result
                                                                                                                                                                               (shen/let V3326
                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                          (nthcdr 1 V3313)
                                                                                                                                                                                          V3569)
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/internal/predicate->shen
                                                                                                                                                                                           (null V3326))
                                                                                                                                                                                          (shen/let Hyp
                                                                                                                                                                                                    (nthcdr 1 V3304)
                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                                                     (shen/bind V3568
                                                                                                                                                                                                                (append
                                                                                                                                                                                                                 (list
                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                   (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                   ':
                                                                                                                                                                                                                   (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                   (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                   ':
                                                                                                                                                                                                                   (list 'list
                                                                                                                                                                                                                         (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                 (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                V3569 V3570)))
                                                                                                                                                                                          (shen/if
                                                                                                                                                                                           (shen/shen\.pvar\? V3326)
                                                                                                                                                                                           (shen/do
                                                                                                                                                                                            (shen/shen\.bindv V3326 nil V3569)
                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                      (shen/let Hyp
                                                                                                                                                                                                                (nthcdr 1 V3304)
                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                 (shen/bind V3568
                                                                                                                                                                                                                            (append
                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                               (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                               ':
                                                                                                                                                                                                                               (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                               (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                               ':
                                                                                                                                                                                                                               (list 'list
                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                             (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                            V3569 V3570)))
                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                       (shen/shen\.unbindv V3326 V3569)
                                                                                                                                                                                                       Result)))
                                                                                                                                                                                           'false)))
                                                                                                                                                                               (shen/do
                                                                                                                                                                                (shen/shen\.unbindv V3314 V3569)
                                                                                                                                                                                Result))))
                                                                                                                                                          'false)))
                                                                                                                                              'false))
                                                                                                                                   'false))
                                                                                                                        'false))
                                                                                                             'false)))
                                                                                        'false)))
                                                                   'false))
                                                        'false))
                                             'false))
                                  'false))
                       'false))
            (shen/if
             (shen/= Case 'false)
             (shen/let Case
                       (shen/let V3327
                                 (shen/shen\.lazyderef V3567 V3569)
                                 (shen/if
                                  (shen/cons\? V3327)
                                  (shen/let V3328
                                            (shen/shen\.lazyderef
                                             (shen/hd V3327)
                                             V3569)
                                            (shen/if
                                             (shen/cons\? V3328)
                                             (shen/let V3329
                                                       (shen/shen\.lazyderef
                                                        (shen/hd V3328)
                                                        V3569)
                                                       (shen/if
                                                        (shen/cons\? V3329)
                                                        (shen/let V3330
                                                                  (shen/shen\.lazyderef
                                                                   (shen/hd V3329)
                                                                   V3569)
                                                                  (shen/if
                                                                   (shen/= '@p V3330)
                                                                   (shen/let V3331
                                                                             (shen/shen\.lazyderef
                                                                              (nthcdr 1 V3329)
                                                                              V3569)
                                                                             (shen/if
                                                                              (shen/cons\? V3331)
                                                                              (shen/let X
                                                                                        (shen/hd V3331)
                                                                                        (shen/let V3332
                                                                                                  (shen/shen\.lazyderef
                                                                                                   (nthcdr 1 V3331)
                                                                                                   V3569)
                                                                                                  (shen/if
                                                                                                   (shen/cons\? V3332)
                                                                                                   (shen/let Y
                                                                                                             (shen/hd V3332)
                                                                                                             (shen/let V3333
                                                                                                                       (shen/shen\.lazyderef
                                                                                                                        (nthcdr 1 V3332)
                                                                                                                        V3569)
                                                                                                                       (shen/if
                                                                                                                        (shen/internal/predicate->shen
                                                                                                                         (null V3333))
                                                                                                                        (shen/let V3334
                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                   (nthcdr 1 V3328)
                                                                                                                                   V3569)
                                                                                                                                  (shen/if
                                                                                                                                   (shen/cons\? V3334)
                                                                                                                                   (shen/let V3335
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (shen/hd V3334)
                                                                                                                                              V3569)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/= ': V3335)
                                                                                                                                              (shen/let V3336
                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                         (nthcdr 1 V3334)
                                                                                                                                                         V3569)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/cons\? V3336)
                                                                                                                                                         (shen/let V3337
                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                    (shen/hd V3336)
                                                                                                                                                                    V3569)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/cons\? V3337)
                                                                                                                                                                    (shen/let A
                                                                                                                                                                              (shen/hd V3337)
                                                                                                                                                                              (shen/let V3338
                                                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                                                         (nthcdr 1 V3337)
                                                                                                                                                                                         V3569)
                                                                                                                                                                                        (shen/if
                                                                                                                                                                                         (shen/cons\? V3338)
                                                                                                                                                                                         (shen/let V3339
                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                    (shen/hd V3338)
                                                                                                                                                                                                    V3569)
                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                    (shen/= '* V3339)
                                                                                                                                                                                                    (shen/let V3340
                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                               (nthcdr 1 V3338)
                                                                                                                                                                                                               V3569)
                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                               (shen/cons\? V3340)
                                                                                                                                                                                                               (shen/let B
                                                                                                                                                                                                                         (shen/hd V3340)
                                                                                                                                                                                                                         (shen/let V3341
                                                                                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                                                                                    (nthcdr 1 V3340)
                                                                                                                                                                                                                                    V3569)
                                                                                                                                                                                                                                   (shen/if
                                                                                                                                                                                                                                    (shen/internal/predicate->shen
                                                                                                                                                                                                                                     (null V3341))
                                                                                                                                                                                                                                    (shen/let V3342
                                                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                                                               (nthcdr 1 V3336)
                                                                                                                                                                                                                                               V3569)
                                                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                                                               (shen/internal/predicate->shen
                                                                                                                                                                                                                                                (null V3342))
                                                                                                                                                                                                                                               (shen/let Hyp
                                                                                                                                                                                                                                                         (nthcdr 1 V3327)
                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                                                                                                                          (shen/bind V3568
                                                                                                                                                                                                                                                                     (append
                                                                                                                                                                                                                                                                      (list
                                                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                        ':
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                        ':
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                     V3569 V3570)))
                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                (shen/shen\.pvar\? V3342)
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.bindv V3342 nil V3569)
                                                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                                                     (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.unbindv V3342 V3569)
                                                                                                                                                                                                                                                            Result)))
                                                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                     (shen/shen\.pvar\? V3341)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.bindv V3341 nil V3569)
                                                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                                                (shen/let V3343
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                           (nthcdr 1 V3336)
                                                                                                                                                                                                                                                           V3569)
                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                           (shen/internal/predicate->shen
                                                                                                                                                                                                                                                            (null V3343))
                                                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                                                     (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                            (shen/shen\.pvar\? V3343)
                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                             (shen/shen\.bindv V3343 nil V3569)
                                                                                                                                                                                                                                                             (shen/let Result
                                                                                                                                                                                                                                                                       (shen/let Hyp
                                                                                                                                                                                                                                                                                 (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                  (shen/bind V3568
                                                                                                                                                                                                                                                                                             (append
                                                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                             V3569 V3570)))
                                                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                                                        (shen/shen\.unbindv V3343 V3569)
                                                                                                                                                                                                                                                                        Result)))
                                                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.unbindv V3341 V3569)
                                                                                                                                                                                                                                                 Result)))
                                                                                                                                                                                                                                     'false))))
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/shen\.pvar\? V3340)
                                                                                                                                                                                                                (shen/let B
                                                                                                                                                                                                                          (shen/shen\.newpv V3569)
                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                           (shen/shen\.bindv V3340
                                                                                                                                                                                                                                             (list B)
                                                                                                                                                                                                                                             V3569)
                                                                                                                                                                                                                           (shen/let Result
                                                                                                                                                                                                                                     (shen/let V3344
                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                (nthcdr 1 V3336)
                                                                                                                                                                                                                                                V3569)
                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                                                 (null V3344))
                                                                                                                                                                                                                                                (shen/let Hyp
                                                                                                                                                                                                                                                          (nthcdr 1 V3327)
                                                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                                                           (shen/bind V3568
                                                                                                                                                                                                                                                                      (append
                                                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                         ':
                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                         ':
                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                      V3569 V3570)))
                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                 (shen/shen\.pvar\? V3344)
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.bindv V3344 nil V3569)
                                                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                                                            (shen/let Hyp
                                                                                                                                                                                                                                                                      (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                       (shen/bind V3568
                                                                                                                                                                                                                                                                                  (append
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                  V3569 V3570)))
                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                             (shen/shen\.unbindv V3344 V3569)
                                                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                                                 'false)))
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.unbindv V3340 V3569)
                                                                                                                                                                                                                                      Result))))
                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/shen\.pvar\? V3339)
                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                      (shen/shen\.bindv V3339 '* V3569)
                                                                                                                                                                                                      (shen/let Result
                                                                                                                                                                                                                (shen/let V3345
                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                           (nthcdr 1 V3338)
                                                                                                                                                                                                                           V3569)
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/cons\? V3345)
                                                                                                                                                                                                                           (shen/let B
                                                                                                                                                                                                                                     (shen/hd V3345)
                                                                                                                                                                                                                                     (shen/let V3346
                                                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                                                (nthcdr 1 V3345)
                                                                                                                                                                                                                                                V3569)
                                                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                                                 (null V3346))
                                                                                                                                                                                                                                                (shen/let V3347
                                                                                                                                                                                                                                                          (shen/shen\.lazyderef
                                                                                                                                                                                                                                                           (nthcdr 1 V3336)
                                                                                                                                                                                                                                                           V3569)
                                                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                                                           (shen/internal/predicate->shen
                                                                                                                                                                                                                                                            (null V3347))
                                                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                                                     (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                            (shen/shen\.pvar\? V3347)
                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                             (shen/shen\.bindv V3347 nil V3569)
                                                                                                                                                                                                                                                             (shen/let Result
                                                                                                                                                                                                                                                                       (shen/let Hyp
                                                                                                                                                                                                                                                                                 (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                  (shen/bind V3568
                                                                                                                                                                                                                                                                                             (append
                                                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                             V3569 V3570)))
                                                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                                                        (shen/shen\.unbindv V3347 V3569)
                                                                                                                                                                                                                                                                        Result)))
                                                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                 (shen/shen\.pvar\? V3346)
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.bindv V3346 nil V3569)
                                                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                                                            (shen/let V3348
                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef
                                                                                                                                                                                                                                                                       (nthcdr 1 V3336)
                                                                                                                                                                                                                                                                       V3569)
                                                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                                                       (shen/internal/predicate->shen
                                                                                                                                                                                                                                                                        (null V3348))
                                                                                                                                                                                                                                                                       (shen/let Hyp
                                                                                                                                                                                                                                                                                 (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                                                  (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                  (shen/bind V3568
                                                                                                                                                                                                                                                                                             (append
                                                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                                ':
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                             V3569 V3570)))
                                                                                                                                                                                                                                                                       (shen/if
                                                                                                                                                                                                                                                                        (shen/shen\.pvar\? V3348)
                                                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                                                         (shen/shen\.bindv V3348 nil V3569)
                                                                                                                                                                                                                                                                         (shen/let Result
                                                                                                                                                                                                                                                                                   (shen/let Hyp
                                                                                                                                                                                                                                                                                             (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                                                              (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                              (shen/bind V3568
                                                                                                                                                                                                                                                                                                         (append
                                                                                                                                                                                                                                                                                                          (list
                                                                                                                                                                                                                                                                                                           (list
                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                                            ':
                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                                           (list
                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                                            ':
                                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                                         V3569 V3570)))
                                                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                                                    (shen/shen\.unbindv V3348 V3569)
                                                                                                                                                                                                                                                                                    Result)))
                                                                                                                                                                                                                                                                        'false)))
                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                             (shen/shen\.unbindv V3346 V3569)
                                                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                                                 'false))))
                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                            (shen/shen\.pvar\? V3345)
                                                                                                                                                                                                                            (shen/let B
                                                                                                                                                                                                                                      (shen/shen\.newpv V3569)
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.bindv V3345
                                                                                                                                                                                                                                                         (list B)
                                                                                                                                                                                                                                                         V3569)
                                                                                                                                                                                                                                       (shen/let Result
                                                                                                                                                                                                                                                 (shen/let V3349
                                                                                                                                                                                                                                                           (shen/shen\.lazyderef
                                                                                                                                                                                                                                                            (nthcdr 1 V3336)
                                                                                                                                                                                                                                                            V3569)
                                                                                                                                                                                                                                                           (shen/if
                                                                                                                                                                                                                                                            (shen/internal/predicate->shen
                                                                                                                                                                                                                                                             (null V3349))
                                                                                                                                                                                                                                                            (shen/let Hyp
                                                                                                                                                                                                                                                                      (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                       (shen/bind V3568
                                                                                                                                                                                                                                                                                  (append
                                                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                  V3569 V3570)))
                                                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                                                             (shen/shen\.pvar\? V3349)
                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                              (shen/shen\.bindv V3349 nil V3569)
                                                                                                                                                                                                                                                              (shen/let Result
                                                                                                                                                                                                                                                                        (shen/let Hyp
                                                                                                                                                                                                                                                                                  (nthcdr 1 V3327)
                                                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                                                   (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                   (shen/bind V3568
                                                                                                                                                                                                                                                                                              (append
                                                                                                                                                                                                                                                                                               (list
                                                                                                                                                                                                                                                                                                (list
                                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                                 ':
                                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                                (list
                                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                                 ':
                                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                                                               (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                              V3569 V3570)))
                                                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                                                         (shen/shen\.unbindv V3349 V3569)
                                                                                                                                                                                                                                                                         Result)))
                                                                                                                                                                                                                                                             'false)))
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.unbindv V3345 V3569)
                                                                                                                                                                                                                                                  Result))))
                                                                                                                                                                                                                            'false)))
                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                 (shen/shen\.unbindv V3339 V3569)
                                                                                                                                                                                                                 Result)))
                                                                                                                                                                                                     'false)))
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/shen\.pvar\? V3338)
                                                                                                                                                                                          (shen/let B
                                                                                                                                                                                                    (shen/shen\.newpv V3569)
                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                     (shen/shen\.bindv V3338
                                                                                                                                                                                                                       (list '* B)
                                                                                                                                                                                                                       V3569)
                                                                                                                                                                                                     (shen/let Result
                                                                                                                                                                                                               (shen/let V3350
                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                          (nthcdr 1 V3336)
                                                                                                                                                                                                                          V3569)
                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                          (shen/internal/predicate->shen
                                                                                                                                                                                                                           (null V3350))
                                                                                                                                                                                                                          (shen/let Hyp
                                                                                                                                                                                                                                    (nthcdr 1 V3327)
                                                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                                                                                     (shen/bind V3568
                                                                                                                                                                                                                                                (append
                                                                                                                                                                                                                                                 (list
                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                   ':
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                   ':
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                 (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                V3569 V3570)))
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/shen\.pvar\? V3350)
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.bindv V3350 nil V3569)
                                                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                                                      (shen/let Hyp
                                                                                                                                                                                                                                                (nthcdr 1 V3327)
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                 (shen/bind V3568
                                                                                                                                                                                                                                                            (append
                                                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                               ':
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                               ':
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                             (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                            V3569 V3570)))
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.unbindv V3350 V3569)
                                                                                                                                                                                                                                       Result)))
                                                                                                                                                                                                                           'false)))
                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                (shen/shen\.unbindv V3338 V3569)
                                                                                                                                                                                                                Result))))
                                                                                                                                                                                          'false))))
                                                                                                                                                                    (shen/if
                                                                                                                                                                     (shen/shen\.pvar\? V3337)
                                                                                                                                                                     (shen/let A
                                                                                                                                                                               (shen/shen\.newpv V3569)
                                                                                                                                                                               (shen/let B
                                                                                                                                                                                         (shen/shen\.newpv V3569)
                                                                                                                                                                                         (shen/do
                                                                                                                                                                                          (shen/shen\.bindv V3337
                                                                                                                                                                                                            (list A '* B)
                                                                                                                                                                                                            V3569)
                                                                                                                                                                                          (shen/let Result
                                                                                                                                                                                                    (shen/let V3351
                                                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                                                               (nthcdr 1 V3336)
                                                                                                                                                                                                               V3569)
                                                                                                                                                                                                              (shen/if
                                                                                                                                                                                                               (shen/internal/predicate->shen
                                                                                                                                                                                                                (null V3351))
                                                                                                                                                                                                               (shen/let Hyp
                                                                                                                                                                                                                         (nthcdr 1 V3327)
                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                                                                                          (shen/bind V3568
                                                                                                                                                                                                                                     (append
                                                                                                                                                                                                                                      (list
                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                        (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                        ':
                                                                                                                                                                                                                                        (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                        (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                        ':
                                                                                                                                                                                                                                        (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                      (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                     V3569 V3570)))
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/shen\.pvar\? V3351)
                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                 (shen/shen\.bindv V3351 nil V3569)
                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                           (shen/let Hyp
                                                                                                                                                                                                                                     (nthcdr 1 V3327)
                                                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                                                      (shen/shen\.incinfs)
                                                                                                                                                                                                                                      (shen/bind V3568
                                                                                                                                                                                                                                                 (append
                                                                                                                                                                                                                                                  (list
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                    ':
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef B V3569)))
                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                 V3569 V3570)))
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.unbindv V3351 V3569)
                                                                                                                                                                                                                            Result)))
                                                                                                                                                                                                                'false)))
                                                                                                                                                                                                    (shen/do
                                                                                                                                                                                                     (shen/shen\.unbindv V3337 V3569)
                                                                                                                                                                                                     Result)))))
                                                                                                                                                                     'false)))
                                                                                                                                                         'false))
                                                                                                                                              'false))
                                                                                                                                   'false))
                                                                                                                        'false)))
                                                                                                   'false)))
                                                                              'false))
                                                                   'false))
                                                        'false))
                                             'false))
                                  'false))
                       (shen/if
                        (shen/= Case 'false)
                        (shen/let Case
                                  (shen/let V3352
                                            (shen/shen\.lazyderef V3567 V3569)
                                            (shen/if
                                             (shen/cons\? V3352)
                                             (shen/let V3353
                                                       (shen/shen\.lazyderef
                                                        (shen/hd V3352)
                                                        V3569)
                                                       (shen/if
                                                        (shen/cons\? V3353)
                                                        (shen/let V3354
                                                                  (shen/shen\.lazyderef
                                                                   (shen/hd V3353)
                                                                   V3569)
                                                                  (shen/if
                                                                   (shen/cons\? V3354)
                                                                   (shen/let V3355
                                                                             (shen/shen\.lazyderef
                                                                              (shen/hd V3354)
                                                                              V3569)
                                                                             (shen/if
                                                                              (shen/= '@v V3355)
                                                                              (shen/let V3356
                                                                                        (shen/shen\.lazyderef
                                                                                         (nthcdr 1 V3354)
                                                                                         V3569)
                                                                                        (shen/if
                                                                                         (shen/cons\? V3356)
                                                                                         (shen/let X
                                                                                                   (shen/hd V3356)
                                                                                                   (shen/let V3357
                                                                                                             (shen/shen\.lazyderef
                                                                                                              (nthcdr 1 V3356)
                                                                                                              V3569)
                                                                                                             (shen/if
                                                                                                              (shen/cons\? V3357)
                                                                                                              (shen/let Y
                                                                                                                        (shen/hd V3357)
                                                                                                                        (shen/let V3358
                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                   (nthcdr 1 V3357)
                                                                                                                                   V3569)
                                                                                                                                  (shen/if
                                                                                                                                   (shen/internal/predicate->shen
                                                                                                                                    (null V3358))
                                                                                                                                   (shen/let V3359
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (nthcdr 1 V3353)
                                                                                                                                              V3569)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/cons\? V3359)
                                                                                                                                              (shen/let V3360
                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                         (shen/hd V3359)
                                                                                                                                                         V3569)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/= ': V3360)
                                                                                                                                                         (shen/let V3361
                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                    (nthcdr 1 V3359)
                                                                                                                                                                    V3569)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/cons\? V3361)
                                                                                                                                                                    (shen/let V3362
                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                               (shen/hd V3361)
                                                                                                                                                                               V3569)
                                                                                                                                                                              (shen/if
                                                                                                                                                                               (shen/cons\? V3362)
                                                                                                                                                                               (shen/let V3363
                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                          (shen/hd V3362)
                                                                                                                                                                                          V3569)
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/= 'vector V3363)
                                                                                                                                                                                          (shen/let V3364
                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                     (nthcdr 1 V3362)
                                                                                                                                                                                                     V3569)
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/cons\? V3364)
                                                                                                                                                                                                     (shen/let A
                                                                                                                                                                                                               (shen/hd V3364)
                                                                                                                                                                                                               (shen/let V3365
                                                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                                                          (nthcdr 1 V3364)
                                                                                                                                                                                                                          V3569)
                                                                                                                                                                                                                         (shen/if
                                                                                                                                                                                                                          (shen/internal/predicate->shen
                                                                                                                                                                                                                           (null V3365))
                                                                                                                                                                                                                          (shen/let V3366
                                                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                                                     (nthcdr 1 V3361)
                                                                                                                                                                                                                                     V3569)
                                                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                                                     (shen/internal/predicate->shen
                                                                                                                                                                                                                                      (null V3366))
                                                                                                                                                                                                                                     (shen/let Hyp
                                                                                                                                                                                                                                               (nthcdr 1 V3352)
                                                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                                                (shen/bind V3568
                                                                                                                                                                                                                                                           (append
                                                                                                                                                                                                                                                            (list
                                                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                                                              (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                              ':
                                                                                                                                                                                                                                                              (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                                                              (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                              ':
                                                                                                                                                                                                                                                              (list 'vector
                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                            (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                           V3569 V3570)))
                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                      (shen/shen\.pvar\? V3366)
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.bindv V3366 nil V3569)
                                                                                                                                                                                                                                       (shen/let Result
                                                                                                                                                                                                                                                 (shen/let Hyp
                                                                                                                                                                                                                                                           (nthcdr 1 V3352)
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                            (shen/bind V3568
                                                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                          ':
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                          ':
                                                                                                                                                                                                                                                                          (list 'vector
                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                       V3569 V3570)))
                                                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                                                  (shen/shen\.unbindv V3366 V3569)
                                                                                                                                                                                                                                                  Result)))
                                                                                                                                                                                                                                      'false)))
                                                                                                                                                                                                                          (shen/if
                                                                                                                                                                                                                           (shen/shen\.pvar\? V3365)
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.bindv V3365 nil V3569)
                                                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                                                      (shen/let V3367
                                                                                                                                                                                                                                                (shen/shen\.lazyderef
                                                                                                                                                                                                                                                 (nthcdr 1 V3361)
                                                                                                                                                                                                                                                 V3569)
                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                 (shen/internal/predicate->shen
                                                                                                                                                                                                                                                  (null V3367))
                                                                                                                                                                                                                                                 (shen/let Hyp
                                                                                                                                                                                                                                                           (nthcdr 1 V3352)
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                            (shen/bind V3568
                                                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                          ':
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                          ':
                                                                                                                                                                                                                                                                          (list 'vector
                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                       V3569 V3570)))
                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                  (shen/shen\.pvar\? V3367)
                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                   (shen/shen\.bindv V3367 nil V3569)
                                                                                                                                                                                                                                                   (shen/let Result
                                                                                                                                                                                                                                                             (shen/let Hyp
                                                                                                                                                                                                                                                                       (nthcdr 1 V3352)
                                                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                                                        (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                        (shen/bind V3568
                                                                                                                                                                                                                                                                                   (append
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                      ':
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                      ':
                                                                                                                                                                                                                                                                                      (list 'vector
                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                   V3569 V3570)))
                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                              (shen/shen\.unbindv V3367 V3569)
                                                                                                                                                                                                                                                              Result)))
                                                                                                                                                                                                                                                  'false)))
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.unbindv V3365 V3569)
                                                                                                                                                                                                                                       Result)))
                                                                                                                                                                                                                           'false))))
                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                      (shen/shen\.pvar\? V3364)
                                                                                                                                                                                                      (shen/let A
                                                                                                                                                                                                                (shen/shen\.newpv V3569)
                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                 (shen/shen\.bindv V3364
                                                                                                                                                                                                                                   (list A)
                                                                                                                                                                                                                                   V3569)
                                                                                                                                                                                                                 (shen/let Result
                                                                                                                                                                                                                           (shen/let V3368
                                                                                                                                                                                                                                     (shen/shen\.lazyderef
                                                                                                                                                                                                                                      (nthcdr 1 V3361)
                                                                                                                                                                                                                                      V3569)
                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                      (shen/internal/predicate->shen
                                                                                                                                                                                                                                       (null V3368))
                                                                                                                                                                                                                                      (shen/let Hyp
                                                                                                                                                                                                                                                (nthcdr 1 V3352)
                                                                                                                                                                                                                                                (shen/do
                                                                                                                                                                                                                                                 (shen/shen\.incinfs)
                                                                                                                                                                                                                                                 (shen/bind V3568
                                                                                                                                                                                                                                                            (append
                                                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                               ':
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                              (list
                                                                                                                                                                                                                                                               (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                               ':
                                                                                                                                                                                                                                                               (list 'vector
                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                             (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                            V3569 V3570)))
                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                       (shen/shen\.pvar\? V3368)
                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                        (shen/shen\.bindv V3368 nil V3569)
                                                                                                                                                                                                                                        (shen/let Result
                                                                                                                                                                                                                                                  (shen/let Hyp
                                                                                                                                                                                                                                                            (nthcdr 1 V3352)
                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                                                                                                                                             (shen/bind V3568
                                                                                                                                                                                                                                                                        (append
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (list
                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                           ':
                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                          (list
                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                           ':
                                                                                                                                                                                                                                                                           (list 'vector
                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                        V3569 V3570)))
                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                   (shen/shen\.unbindv V3368 V3569)
                                                                                                                                                                                                                                                   Result)))
                                                                                                                                                                                                                                       'false)))
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.unbindv V3364 V3569)
                                                                                                                                                                                                                            Result))))
                                                                                                                                                                                                      'false)))
                                                                                                                                                                                          (shen/if
                                                                                                                                                                                           (shen/shen\.pvar\? V3363)
                                                                                                                                                                                           (shen/do
                                                                                                                                                                                            (shen/shen\.bindv V3363 'vector V3569)
                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                      (shen/let V3369
                                                                                                                                                                                                                (shen/shen\.lazyderef
                                                                                                                                                                                                                 (nthcdr 1 V3362)
                                                                                                                                                                                                                 V3569)
                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                 (shen/cons\? V3369)
                                                                                                                                                                                                                 (shen/let A
                                                                                                                                                                                                                           (shen/hd V3369)
                                                                                                                                                                                                                           (shen/let V3370
                                                                                                                                                                                                                                     (shen/shen\.lazyderef
                                                                                                                                                                                                                                      (nthcdr 1 V3369)
                                                                                                                                                                                                                                      V3569)
                                                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                                                      (shen/internal/predicate->shen
                                                                                                                                                                                                                                       (null V3370))
                                                                                                                                                                                                                                      (shen/let V3371
                                                                                                                                                                                                                                                (shen/shen\.lazyderef
                                                                                                                                                                                                                                                 (nthcdr 1 V3361)
                                                                                                                                                                                                                                                 V3569)
                                                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                                                 (shen/internal/predicate->shen
                                                                                                                                                                                                                                                  (null V3371))
                                                                                                                                                                                                                                                 (shen/let Hyp
                                                                                                                                                                                                                                                           (nthcdr 1 V3352)
                                                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                                                            (shen/bind V3568
                                                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                          ':
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                          ':
                                                                                                                                                                                                                                                                          (list 'vector
                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                       V3569 V3570)))
                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                  (shen/shen\.pvar\? V3371)
                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                   (shen/shen\.bindv V3371 nil V3569)
                                                                                                                                                                                                                                                   (shen/let Result
                                                                                                                                                                                                                                                             (shen/let Hyp
                                                                                                                                                                                                                                                                       (nthcdr 1 V3352)
                                                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                                                        (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                        (shen/bind V3568
                                                                                                                                                                                                                                                                                   (append
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                      ':
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                      ':
                                                                                                                                                                                                                                                                                      (list 'vector
                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                   V3569 V3570)))
                                                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                                                              (shen/shen\.unbindv V3371 V3569)
                                                                                                                                                                                                                                                              Result)))
                                                                                                                                                                                                                                                  'false)))
                                                                                                                                                                                                                                      (shen/if
                                                                                                                                                                                                                                       (shen/shen\.pvar\? V3370)
                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                        (shen/shen\.bindv V3370 nil V3569)
                                                                                                                                                                                                                                        (shen/let Result
                                                                                                                                                                                                                                                  (shen/let V3372
                                                                                                                                                                                                                                                            (shen/shen\.lazyderef
                                                                                                                                                                                                                                                             (nthcdr 1 V3361)
                                                                                                                                                                                                                                                             V3569)
                                                                                                                                                                                                                                                            (shen/if
                                                                                                                                                                                                                                                             (shen/internal/predicate->shen
                                                                                                                                                                                                                                                              (null V3372))
                                                                                                                                                                                                                                                             (shen/let Hyp
                                                                                                                                                                                                                                                                       (nthcdr 1 V3352)
                                                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                                                        (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                        (shen/bind V3568
                                                                                                                                                                                                                                                                                   (append
                                                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                      ':
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                      ':
                                                                                                                                                                                                                                                                                      (list 'vector
                                                                                                                                                                                                                                                                                            (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                   V3569 V3570)))
                                                                                                                                                                                                                                                             (shen/if
                                                                                                                                                                                                                                                              (shen/shen\.pvar\? V3372)
                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                               (shen/shen\.bindv V3372 nil V3569)
                                                                                                                                                                                                                                                               (shen/let Result
                                                                                                                                                                                                                                                                         (shen/let Hyp
                                                                                                                                                                                                                                                                                   (nthcdr 1 V3352)
                                                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                                                    (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                                    (shen/bind V3568
                                                                                                                                                                                                                                                                                               (append
                                                                                                                                                                                                                                                                                                (list
                                                                                                                                                                                                                                                                                                 (list
                                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                                  ':
                                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                                 (list
                                                                                                                                                                                                                                                                                                  (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                                  ':
                                                                                                                                                                                                                                                                                                  (list 'vector
                                                                                                                                                                                                                                                                                                        (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                                                (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                               V3569 V3570)))
                                                                                                                                                                                                                                                                         (shen/do
                                                                                                                                                                                                                                                                          (shen/shen\.unbindv V3372 V3569)
                                                                                                                                                                                                                                                                          Result)))
                                                                                                                                                                                                                                                              'false)))
                                                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                                                   (shen/shen\.unbindv V3370 V3569)
                                                                                                                                                                                                                                                   Result)))
                                                                                                                                                                                                                                       'false))))
                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                  (shen/shen\.pvar\? V3369)
                                                                                                                                                                                                                  (shen/let A
                                                                                                                                                                                                                            (shen/shen\.newpv V3569)
                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                             (shen/shen\.bindv V3369
                                                                                                                                                                                                                                               (list A)
                                                                                                                                                                                                                                               V3569)
                                                                                                                                                                                                                             (shen/let Result
                                                                                                                                                                                                                                       (shen/let V3373
                                                                                                                                                                                                                                                 (shen/shen\.lazyderef
                                                                                                                                                                                                                                                  (nthcdr 1 V3361)
                                                                                                                                                                                                                                                  V3569)
                                                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                                                  (shen/internal/predicate->shen
                                                                                                                                                                                                                                                   (null V3373))
                                                                                                                                                                                                                                                  (shen/let Hyp
                                                                                                                                                                                                                                                            (nthcdr 1 V3352)
                                                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                                                                                                                                             (shen/bind V3568
                                                                                                                                                                                                                                                                        (append
                                                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                                                          (list
                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                           ':
                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                          (list
                                                                                                                                                                                                                                                                           (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                           ':
                                                                                                                                                                                                                                                                           (list 'vector
                                                                                                                                                                                                                                                                                 (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                         (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                        V3569 V3570)))
                                                                                                                                                                                                                                                  (shen/if
                                                                                                                                                                                                                                                   (shen/shen\.pvar\? V3373)
                                                                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                                                                    (shen/shen\.bindv V3373 nil V3569)
                                                                                                                                                                                                                                                    (shen/let Result
                                                                                                                                                                                                                                                              (shen/let Hyp
                                                                                                                                                                                                                                                                        (nthcdr 1 V3352)
                                                                                                                                                                                                                                                                        (shen/do
                                                                                                                                                                                                                                                                         (shen/shen\.incinfs)
                                                                                                                                                                                                                                                                         (shen/bind V3568
                                                                                                                                                                                                                                                                                    (append
                                                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                                                      (list
                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                                                       ':
                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                                                      (list
                                                                                                                                                                                                                                                                                       (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                                                       ':
                                                                                                                                                                                                                                                                                       (list 'vector
                                                                                                                                                                                                                                                                                             (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                                                     (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                                                    V3569 V3570)))
                                                                                                                                                                                                                                                              (shen/do
                                                                                                                                                                                                                                                               (shen/shen\.unbindv V3373 V3569)
                                                                                                                                                                                                                                                               Result)))
                                                                                                                                                                                                                                                   'false)))
                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                        (shen/shen\.unbindv V3369 V3569)
                                                                                                                                                                                                                                        Result))))
                                                                                                                                                                                                                  'false)))
                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                       (shen/shen\.unbindv V3363 V3569)
                                                                                                                                                                                                       Result)))
                                                                                                                                                                                           'false)))
                                                                                                                                                                               (shen/if
                                                                                                                                                                                (shen/shen\.pvar\? V3362)
                                                                                                                                                                                (shen/let A
                                                                                                                                                                                          (shen/shen\.newpv V3569)
                                                                                                                                                                                          (shen/do
                                                                                                                                                                                           (shen/shen\.bindv V3362
                                                                                                                                                                                                             (list 'vector A)
                                                                                                                                                                                                             V3569)
                                                                                                                                                                                           (shen/let Result
                                                                                                                                                                                                     (shen/let V3374
                                                                                                                                                                                                               (shen/shen\.lazyderef
                                                                                                                                                                                                                (nthcdr 1 V3361)
                                                                                                                                                                                                                V3569)
                                                                                                                                                                                                               (shen/if
                                                                                                                                                                                                                (shen/internal/predicate->shen
                                                                                                                                                                                                                 (null V3374))
                                                                                                                                                                                                                (shen/let Hyp
                                                                                                                                                                                                                          (nthcdr 1 V3352)
                                                                                                                                                                                                                          (shen/do
                                                                                                                                                                                                                           (shen/shen\.incinfs)
                                                                                                                                                                                                                           (shen/bind V3568
                                                                                                                                                                                                                                      (append
                                                                                                                                                                                                                                       (list
                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                         (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                         ':
                                                                                                                                                                                                                                         (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                         (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                         ':
                                                                                                                                                                                                                                         (list 'vector
                                                                                                                                                                                                                                               (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                       (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                      V3569 V3570)))
                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                 (shen/shen\.pvar\? V3374)
                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                  (shen/shen\.bindv V3374 nil V3569)
                                                                                                                                                                                                                  (shen/let Result
                                                                                                                                                                                                                            (shen/let Hyp
                                                                                                                                                                                                                                      (nthcdr 1 V3352)
                                                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                                                       (shen/shen\.incinfs)
                                                                                                                                                                                                                                       (shen/bind V3568
                                                                                                                                                                                                                                                  (append
                                                                                                                                                                                                                                                   (list
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef A V3569))
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                     ':
                                                                                                                                                                                                                                                     (list 'vector
                                                                                                                                                                                                                                                           (shen/shen\.lazyderef A V3569))))
                                                                                                                                                                                                                                                   (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                  V3569 V3570)))
                                                                                                                                                                                                                            (shen/do
                                                                                                                                                                                                                             (shen/shen\.unbindv V3374 V3569)
                                                                                                                                                                                                                             Result)))
                                                                                                                                                                                                                 'false)))
                                                                                                                                                                                                     (shen/do
                                                                                                                                                                                                      (shen/shen\.unbindv V3362 V3569)
                                                                                                                                                                                                      Result))))
                                                                                                                                                                                'false)))
                                                                                                                                                                    'false))
                                                                                                                                                         'false))
                                                                                                                                              'false))
                                                                                                                                   'false)))
                                                                                                              'false)))
                                                                                         'false))
                                                                              'false))
                                                                   'false))
                                                        'false))
                                             'false))
                                  (shen/if
                                   (shen/= Case 'false)
                                   (shen/let Case
                                             (shen/let V3375
                                                       (shen/shen\.lazyderef V3567 V3569)
                                                       (shen/if
                                                        (shen/cons\? V3375)
                                                        (shen/let V3376
                                                                  (shen/shen\.lazyderef
                                                                   (shen/hd V3375)
                                                                   V3569)
                                                                  (shen/if
                                                                   (shen/cons\? V3376)
                                                                   (shen/let V3377
                                                                             (shen/shen\.lazyderef
                                                                              (shen/hd V3376)
                                                                              V3569)
                                                                             (shen/if
                                                                              (shen/cons\? V3377)
                                                                              (shen/let V3378
                                                                                        (shen/shen\.lazyderef
                                                                                         (shen/hd V3377)
                                                                                         V3569)
                                                                                        (shen/if
                                                                                         (shen/= '@s V3378)
                                                                                         (shen/let V3379
                                                                                                   (shen/shen\.lazyderef
                                                                                                    (nthcdr 1 V3377)
                                                                                                    V3569)
                                                                                                   (shen/if
                                                                                                    (shen/cons\? V3379)
                                                                                                    (shen/let X
                                                                                                              (shen/hd V3379)
                                                                                                              (shen/let V3380
                                                                                                                        (shen/shen\.lazyderef
                                                                                                                         (nthcdr 1 V3379)
                                                                                                                         V3569)
                                                                                                                        (shen/if
                                                                                                                         (shen/cons\? V3380)
                                                                                                                         (shen/let Y
                                                                                                                                   (shen/hd V3380)
                                                                                                                                   (shen/let V3381
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (nthcdr 1 V3380)
                                                                                                                                              V3569)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/internal/predicate->shen
                                                                                                                                               (null V3381))
                                                                                                                                              (shen/let V3382
                                                                                                                                                        (shen/shen\.lazyderef
                                                                                                                                                         (nthcdr 1 V3376)
                                                                                                                                                         V3569)
                                                                                                                                                        (shen/if
                                                                                                                                                         (shen/cons\? V3382)
                                                                                                                                                         (shen/let V3383
                                                                                                                                                                   (shen/shen\.lazyderef
                                                                                                                                                                    (shen/hd V3382)
                                                                                                                                                                    V3569)
                                                                                                                                                                   (shen/if
                                                                                                                                                                    (shen/= ': V3383)
                                                                                                                                                                    (shen/let V3384
                                                                                                                                                                              (shen/shen\.lazyderef
                                                                                                                                                                               (nthcdr 1 V3382)
                                                                                                                                                                               V3569)
                                                                                                                                                                              (shen/if
                                                                                                                                                                               (shen/cons\? V3384)
                                                                                                                                                                               (shen/let V3385
                                                                                                                                                                                         (shen/shen\.lazyderef
                                                                                                                                                                                          (shen/hd V3384)
                                                                                                                                                                                          V3569)
                                                                                                                                                                                         (shen/if
                                                                                                                                                                                          (shen/= 'string V3385)
                                                                                                                                                                                          (shen/let V3386
                                                                                                                                                                                                    (shen/shen\.lazyderef
                                                                                                                                                                                                     (nthcdr 1 V3384)
                                                                                                                                                                                                     V3569)
                                                                                                                                                                                                    (shen/if
                                                                                                                                                                                                     (shen/internal/predicate->shen
                                                                                                                                                                                                      (null V3386))
                                                                                                                                                                                                     (shen/let Hyp
                                                                                                                                                                                                               (nthcdr 1 V3375)
                                                                                                                                                                                                               (shen/do
                                                                                                                                                                                                                (shen/shen\.incinfs)
                                                                                                                                                                                                                (shen/bind V3568
                                                                                                                                                                                                                           (append
                                                                                                                                                                                                                            (list
                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                              (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                              ': 'string)
                                                                                                                                                                                                                             (list
                                                                                                                                                                                                                              (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                              ': 'string))
                                                                                                                                                                                                                            (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                           V3569 V3570)))
                                                                                                                                                                                                     (shen/if
                                                                                                                                                                                                      (shen/shen\.pvar\? V3386)
                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                       (shen/shen\.bindv V3386 nil V3569)
                                                                                                                                                                                                       (shen/let Result
                                                                                                                                                                                                                 (shen/let Hyp
                                                                                                                                                                                                                           (nthcdr 1 V3375)
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                            (shen/bind V3568
                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                          (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                          ': 'string)
                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                          (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                          ': 'string))
                                                                                                                                                                                                                                        (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                       V3569 V3570)))
                                                                                                                                                                                                                 (shen/do
                                                                                                                                                                                                                  (shen/shen\.unbindv V3386 V3569)
                                                                                                                                                                                                                  Result)))
                                                                                                                                                                                                      'false)))
                                                                                                                                                                                          (shen/if
                                                                                                                                                                                           (shen/shen\.pvar\? V3385)
                                                                                                                                                                                           (shen/do
                                                                                                                                                                                            (shen/shen\.bindv V3385 'string V3569)
                                                                                                                                                                                            (shen/let Result
                                                                                                                                                                                                      (shen/let V3387
                                                                                                                                                                                                                (shen/shen\.lazyderef
                                                                                                                                                                                                                 (nthcdr 1 V3384)
                                                                                                                                                                                                                 V3569)
                                                                                                                                                                                                                (shen/if
                                                                                                                                                                                                                 (shen/internal/predicate->shen
                                                                                                                                                                                                                  (null V3387))
                                                                                                                                                                                                                 (shen/let Hyp
                                                                                                                                                                                                                           (nthcdr 1 V3375)
                                                                                                                                                                                                                           (shen/do
                                                                                                                                                                                                                            (shen/shen\.incinfs)
                                                                                                                                                                                                                            (shen/bind V3568
                                                                                                                                                                                                                                       (append
                                                                                                                                                                                                                                        (list
                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                          (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                          ': 'string)
                                                                                                                                                                                                                                         (list
                                                                                                                                                                                                                                          (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                          ': 'string))
                                                                                                                                                                                                                                        (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                       V3569 V3570)))
                                                                                                                                                                                                                 (shen/if
                                                                                                                                                                                                                  (shen/shen\.pvar\? V3387)
                                                                                                                                                                                                                  (shen/do
                                                                                                                                                                                                                   (shen/shen\.bindv V3387 nil V3569)
                                                                                                                                                                                                                   (shen/let Result
                                                                                                                                                                                                                             (shen/let Hyp
                                                                                                                                                                                                                                       (nthcdr 1 V3375)
                                                                                                                                                                                                                                       (shen/do
                                                                                                                                                                                                                                        (shen/shen\.incinfs)
                                                                                                                                                                                                                                        (shen/bind V3568
                                                                                                                                                                                                                                                   (append
                                                                                                                                                                                                                                                    (list
                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                      (shen/shen\.lazyderef X V3569)
                                                                                                                                                                                                                                                      ': 'string)
                                                                                                                                                                                                                                                     (list
                                                                                                                                                                                                                                                      (shen/shen\.lazyderef Y V3569)
                                                                                                                                                                                                                                                      ': 'string))
                                                                                                                                                                                                                                                    (shen/shen\.lazyderef Hyp V3569))
                                                                                                                                                                                                                                                   V3569 V3570)))
                                                                                                                                                                                                                             (shen/do
                                                                                                                                                                                                                              (shen/shen\.unbindv V3387 V3569)
                                                                                                                                                                                                                              Result)))
                                                                                                                                                                                                                  'false)))
                                                                                                                                                                                                      (shen/do
                                                                                                                                                                                                       (shen/shen\.unbindv V3385 V3569)
                                                                                                                                                                                                       Result)))
                                                                                                                                                                                           'false)))
                                                                                                                                                                               'false))
                                                                                                                                                                    'false))
                                                                                                                                                         'false))
                                                                                                                                              'false)))
                                                                                                                         'false)))
                                                                                                    'false))
                                                                                         'false))
                                                                              'false))
                                                                   'false))
                                                        'false))
                                             (shen/if
                                              (shen/= Case 'false)
                                              (shen/let V3388
                                                        (shen/shen\.lazyderef V3567 V3569)
                                                        (shen/if
                                                         (shen/cons\? V3388)
                                                         (shen/let X
                                                                   (shen/hd V3388)
                                                                   (shen/let Hyp
                                                                             (nthcdr 1 V3388)
                                                                             (shen/let NewHyps
                                                                                       (shen/shen\.newpv V3569)
                                                                                       (shen/do
                                                                                        (shen/shen\.incinfs)
                                                                                        (shen/bind V3568
                                                                                                   (append
                                                                                                    (list
                                                                                                     (shen/shen\.lazyderef X V3569))
                                                                                                    (shen/shen\.lazyderef NewHyps V3569))
                                                                                                   V3569
                                                                                                   (shen/freeze
                                                                                                    (shen/shen\.t*-hyps Hyp NewHyps V3569 V3570)))))))
                                                         'false))
                                              Case))
                                   Case))
                        Case))
             Case)))
(defun shen/shen\.show
    (V3587 V3588 V3589 V3590)
  (shen/cond
   ((shen/value 'shen\.*spy*)
    (shen/do
     (shen/shen\.line)
     (shen/do
      (shen/shen\.show-p
       (shen/shen\.deref V3587 V3589))
      (shen/do
       (shen/nl 1)
       (shen/do
        (shen/nl 1)
        (shen/do
         (shen/shen\.show-assumptions
          (shen/shen\.deref V3588 V3589)
          1)
         (shen/do
          (shen/shen\.prhush "\n> "
                             (shen/stoutput))
          (shen/do
           (shen/shen\.pause-for-user)
           (shen/thaw V3590)))))))))
   (shen/true
    (shen/thaw V3590))))
(defun shen/shen\.line nil
  (shen/let Infs
            (shen/inferences)
            (shen/shen\.prhush
             (shen/cn "____________________________________________________________ "
                      (shen/shen\.app Infs
                                      (shen/cn " inference"
                                               (shen/shen\.app
                                                (shen/if
                                                 (shen/= 1 Infs)
                                                 "" "s")
                                                " \n?- " 'shen\.a))
                                      'shen\.a))
             (shen/stoutput))))
(defun shen/shen\.show-p
    (V3592)
  (shen/cond
   ((shen/and
     (shen/cons\? V3592)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V3592))
      (shen/and
       (shen/= ':
               (shen/hd
                (nthcdr 1 V3592)))
       (shen/and
        (shen/cons\?
         (nthcdr 2 V3592))
        (shen/internal/predicate->shen
         (null
          (nthcdr 3 V3592)))))))
    (shen/shen\.prhush
     (shen/shen\.app
      (shen/hd V3592)
      (shen/cn " : "
               (shen/shen\.app
                (shen/hd
                 (nthcdr 2 V3592))
                "" 'shen\.r))
      'shen\.r)
     (shen/stoutput)))
   (shen/true
    (shen/shen\.prhush
     (shen/shen\.app V3592 "" 'shen\.r)
     (shen/stoutput)))))
(defun shen/shen\.show-assumptions
    (V3597 V3598)
  (cl-flet
      ((tail-trampoline
        (V3597 V3598)
        (shen/cond
         ((shen/internal/predicate->shen
           (null V3597))
          'shen\.skip)
         ((shen/cons\? V3597)
          (shen/do
           (shen/shen\.prhush
            (shen/shen\.app V3598 ". " 'shen\.a)
            (shen/stoutput))
           (shen/do
            (shen/shen\.show-p
             (shen/hd V3597))
            (shen/do
             (shen/nl 1)
             (vector
              (list
               (nthcdr 1 V3597)
               (1+ V3598)))))))
         (shen/true
          (shen/shen\.f_error 'shen\.show-assumptions)))))
    (let
        ((result
          (funcall #'tail-trampoline V3597 V3598)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.pause-for-user nil
  (shen/let Byte
            (shen/read-byte
             (shen/stinput))
            (shen/if
             (shen/= Byte 94)
             (shen/simple-error "input aborted\n")
             (shen/nl 1))))
(defun shen/shen\.typedf\?
    (V3600)
  (shen/cons\?
   (shen/assoc V3600
               (shen/value 'shen\.*signedfuncs*))))
(defun shen/shen\.sigf
    (V3602)
  (shen/concat 'shen\.type-signature-of- V3602))
(defun shen/shen\.placeholder nil
  (shen/gensym '&&))
(defun shen/shen\.base
    (V3607 V3608 V3609 V3610)
  (shen/let Case
            (shen/let V3291
                      (shen/shen\.lazyderef V3608 V3609)
                      (shen/if
                       (shen/= 'number V3291)
                       (shen/do
                        (shen/shen\.incinfs)
                        (shen/fwhen
                         (shen/number\?
                          (shen/shen\.lazyderef V3607 V3609))
                         V3609 V3610))
                       (shen/if
                        (shen/shen\.pvar\? V3291)
                        (shen/do
                         (shen/shen\.bindv V3291 'number V3609)
                         (shen/let Result
                                   (shen/do
                                    (shen/shen\.incinfs)
                                    (shen/fwhen
                                     (shen/number\?
                                      (shen/shen\.lazyderef V3607 V3609))
                                     V3609 V3610))
                                   (shen/do
                                    (shen/shen\.unbindv V3291 V3609)
                                    Result)))
                        'false)))
            (shen/if
             (shen/= Case 'false)
             (shen/let Case
                       (shen/let V3292
                                 (shen/shen\.lazyderef V3608 V3609)
                                 (shen/if
                                  (shen/= 'boolean V3292)
                                  (shen/do
                                   (shen/shen\.incinfs)
                                   (shen/fwhen
                                    (shen/boolean\?
                                     (shen/shen\.lazyderef V3607 V3609))
                                    V3609 V3610))
                                  (shen/if
                                   (shen/shen\.pvar\? V3292)
                                   (shen/do
                                    (shen/shen\.bindv V3292 'boolean V3609)
                                    (shen/let Result
                                              (shen/do
                                               (shen/shen\.incinfs)
                                               (shen/fwhen
                                                (shen/boolean\?
                                                 (shen/shen\.lazyderef V3607 V3609))
                                                V3609 V3610))
                                              (shen/do
                                               (shen/shen\.unbindv V3292 V3609)
                                               Result)))
                                   'false)))
                       (shen/if
                        (shen/= Case 'false)
                        (shen/let Case
                                  (shen/let V3293
                                            (shen/shen\.lazyderef V3608 V3609)
                                            (shen/if
                                             (shen/= 'string V3293)
                                             (shen/do
                                              (shen/shen\.incinfs)
                                              (shen/fwhen
                                               (shen/string\?
                                                (shen/shen\.lazyderef V3607 V3609))
                                               V3609 V3610))
                                             (shen/if
                                              (shen/shen\.pvar\? V3293)
                                              (shen/do
                                               (shen/shen\.bindv V3293 'string V3609)
                                               (shen/let Result
                                                         (shen/do
                                                          (shen/shen\.incinfs)
                                                          (shen/fwhen
                                                           (shen/string\?
                                                            (shen/shen\.lazyderef V3607 V3609))
                                                           V3609 V3610))
                                                         (shen/do
                                                          (shen/shen\.unbindv V3293 V3609)
                                                          Result)))
                                              'false)))
                                  (shen/if
                                   (shen/= Case 'false)
                                   (shen/let Case
                                             (shen/let V3294
                                                       (shen/shen\.lazyderef V3608 V3609)
                                                       (shen/if
                                                        (shen/= 'symbol V3294)
                                                        (shen/do
                                                         (shen/shen\.incinfs)
                                                         (shen/fwhen
                                                          (shen/symbol\?
                                                           (shen/shen\.lazyderef V3607 V3609))
                                                          V3609
                                                          (shen/freeze
                                                           (shen/fwhen
                                                            (shen/not
                                                             (shen/shen\.ue\?
                                                              (shen/shen\.lazyderef V3607 V3609)))
                                                            V3609 V3610))))
                                                        (shen/if
                                                         (shen/shen\.pvar\? V3294)
                                                         (shen/do
                                                          (shen/shen\.bindv V3294 'symbol V3609)
                                                          (shen/let Result
                                                                    (shen/do
                                                                     (shen/shen\.incinfs)
                                                                     (shen/fwhen
                                                                      (shen/symbol\?
                                                                       (shen/shen\.lazyderef V3607 V3609))
                                                                      V3609
                                                                      (shen/freeze
                                                                       (shen/fwhen
                                                                        (shen/not
                                                                         (shen/shen\.ue\?
                                                                          (shen/shen\.lazyderef V3607 V3609)))
                                                                        V3609 V3610))))
                                                                    (shen/do
                                                                     (shen/shen\.unbindv V3294 V3609)
                                                                     Result)))
                                                         'false)))
                                             (shen/if
                                              (shen/= Case 'false)
                                              (shen/let V3295
                                                        (shen/shen\.lazyderef V3607 V3609)
                                                        (shen/if
                                                         (shen/internal/predicate->shen
                                                          (null V3295))
                                                         (shen/let V3296
                                                                   (shen/shen\.lazyderef V3608 V3609)
                                                                   (shen/if
                                                                    (shen/cons\? V3296)
                                                                    (shen/let V3297
                                                                              (shen/shen\.lazyderef
                                                                               (shen/hd V3296)
                                                                               V3609)
                                                                              (shen/if
                                                                               (shen/= 'list V3297)
                                                                               (shen/let V3298
                                                                                         (shen/shen\.lazyderef
                                                                                          (nthcdr 1 V3296)
                                                                                          V3609)
                                                                                         (shen/if
                                                                                          (shen/cons\? V3298)
                                                                                          (shen/let A
                                                                                                    (shen/hd V3298)
                                                                                                    (shen/let V3299
                                                                                                              (shen/shen\.lazyderef
                                                                                                               (nthcdr 1 V3298)
                                                                                                               V3609)
                                                                                                              (shen/if
                                                                                                               (shen/internal/predicate->shen
                                                                                                                (null V3299))
                                                                                                               (shen/do
                                                                                                                (shen/shen\.incinfs)
                                                                                                                (shen/thaw V3610))
                                                                                                               (shen/if
                                                                                                                (shen/shen\.pvar\? V3299)
                                                                                                                (shen/do
                                                                                                                 (shen/shen\.bindv V3299 nil V3609)
                                                                                                                 (shen/let Result
                                                                                                                           (shen/do
                                                                                                                            (shen/shen\.incinfs)
                                                                                                                            (shen/thaw V3610))
                                                                                                                           (shen/do
                                                                                                                            (shen/shen\.unbindv V3299 V3609)
                                                                                                                            Result)))
                                                                                                                'false))))
                                                                                          (shen/if
                                                                                           (shen/shen\.pvar\? V3298)
                                                                                           (shen/let A
                                                                                                     (shen/shen\.newpv V3609)
                                                                                                     (shen/do
                                                                                                      (shen/shen\.bindv V3298
                                                                                                                        (list A)
                                                                                                                        V3609)
                                                                                                      (shen/let Result
                                                                                                                (shen/do
                                                                                                                 (shen/shen\.incinfs)
                                                                                                                 (shen/thaw V3610))
                                                                                                                (shen/do
                                                                                                                 (shen/shen\.unbindv V3298 V3609)
                                                                                                                 Result))))
                                                                                           'false)))
                                                                               (shen/if
                                                                                (shen/shen\.pvar\? V3297)
                                                                                (shen/do
                                                                                 (shen/shen\.bindv V3297 'list V3609)
                                                                                 (shen/let Result
                                                                                           (shen/let V3300
                                                                                                     (shen/shen\.lazyderef
                                                                                                      (nthcdr 1 V3296)
                                                                                                      V3609)
                                                                                                     (shen/if
                                                                                                      (shen/cons\? V3300)
                                                                                                      (shen/let A
                                                                                                                (shen/hd V3300)
                                                                                                                (shen/let V3301
                                                                                                                          (shen/shen\.lazyderef
                                                                                                                           (nthcdr 1 V3300)
                                                                                                                           V3609)
                                                                                                                          (shen/if
                                                                                                                           (shen/internal/predicate->shen
                                                                                                                            (null V3301))
                                                                                                                           (shen/do
                                                                                                                            (shen/shen\.incinfs)
                                                                                                                            (shen/thaw V3610))
                                                                                                                           (shen/if
                                                                                                                            (shen/shen\.pvar\? V3301)
                                                                                                                            (shen/do
                                                                                                                             (shen/shen\.bindv V3301 nil V3609)
                                                                                                                             (shen/let Result
                                                                                                                                       (shen/do
                                                                                                                                        (shen/shen\.incinfs)
                                                                                                                                        (shen/thaw V3610))
                                                                                                                                       (shen/do
                                                                                                                                        (shen/shen\.unbindv V3301 V3609)
                                                                                                                                        Result)))
                                                                                                                            'false))))
                                                                                                      (shen/if
                                                                                                       (shen/shen\.pvar\? V3300)
                                                                                                       (shen/let A
                                                                                                                 (shen/shen\.newpv V3609)
                                                                                                                 (shen/do
                                                                                                                  (shen/shen\.bindv V3300
                                                                                                                                    (list A)
                                                                                                                                    V3609)
                                                                                                                  (shen/let Result
                                                                                                                            (shen/do
                                                                                                                             (shen/shen\.incinfs)
                                                                                                                             (shen/thaw V3610))
                                                                                                                            (shen/do
                                                                                                                             (shen/shen\.unbindv V3300 V3609)
                                                                                                                             Result))))
                                                                                                       'false)))
                                                                                           (shen/do
                                                                                            (shen/shen\.unbindv V3297 V3609)
                                                                                            Result)))
                                                                                'false)))
                                                                    (shen/if
                                                                     (shen/shen\.pvar\? V3296)
                                                                     (shen/let A
                                                                               (shen/shen\.newpv V3609)
                                                                               (shen/do
                                                                                (shen/shen\.bindv V3296
                                                                                                  (list 'list A)
                                                                                                  V3609)
                                                                                (shen/let Result
                                                                                          (shen/do
                                                                                           (shen/shen\.incinfs)
                                                                                           (shen/thaw V3610))
                                                                                          (shen/do
                                                                                           (shen/shen\.unbindv V3296 V3609)
                                                                                           Result))))
                                                                     'false)))
                                                         'false))
                                              Case))
                                   Case))
                        Case))
             Case)))
(defun shen/shen\.by_hypothesis
    (V3616 V3617 V3618 V3619 V3620)
  (cl-flet
      ((tail-trampoline
        (V3616 V3617 V3618 V3619 V3620)
        (shen/let Case
                  (shen/let V3282
                            (shen/shen\.lazyderef V3618 V3619)
                            (shen/if
                             (shen/cons\? V3282)
                             (shen/let V3283
                                       (shen/shen\.lazyderef
                                        (shen/hd V3282)
                                        V3619)
                                       (shen/if
                                        (shen/cons\? V3283)
                                        (shen/let Y
                                                  (shen/hd V3283)
                                                  (shen/let V3284
                                                            (shen/shen\.lazyderef
                                                             (nthcdr 1 V3283)
                                                             V3619)
                                                            (shen/if
                                                             (shen/cons\? V3284)
                                                             (shen/let V3285
                                                                       (shen/shen\.lazyderef
                                                                        (shen/hd V3284)
                                                                        V3619)
                                                                       (shen/if
                                                                        (shen/= ': V3285)
                                                                        (shen/let V3286
                                                                                  (shen/shen\.lazyderef
                                                                                   (nthcdr 1 V3284)
                                                                                   V3619)
                                                                                  (shen/if
                                                                                   (shen/cons\? V3286)
                                                                                   (shen/let B
                                                                                             (shen/hd V3286)
                                                                                             (shen/let V3287
                                                                                                       (shen/shen\.lazyderef
                                                                                                        (nthcdr 1 V3286)
                                                                                                        V3619)
                                                                                                       (shen/if
                                                                                                        (shen/internal/predicate->shen
                                                                                                         (null V3287))
                                                                                                        (shen/do
                                                                                                         (shen/shen\.incinfs)
                                                                                                         (shen/identical V3616 Y V3619
                                                                                                                         (shen/freeze
                                                                                                                          (shen/unify! V3617 B V3619 V3620))))
                                                                                                        'false)))
                                                                                   'false))
                                                                        'false))
                                                             'false)))
                                        'false))
                             'false))
                  (shen/if
                   (shen/= Case 'false)
                   (shen/let V3288
                             (shen/shen\.lazyderef V3618 V3619)
                             (shen/if
                              (shen/cons\? V3288)
                              (shen/let Hyp
                                        (nthcdr 1 V3288)
                                        (shen/do
                                         (shen/shen\.incinfs)
                                         (vector
                                          (list V3616 V3617 Hyp V3619 V3620))))
                              'false))
                   Case))))
    (let
        ((result
          (funcall #'tail-trampoline V3616 V3617 V3618 V3619 V3620)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.t*-def
    (V3626 V3627 V3628 V3629 V3630)
  (shen/let V3276
            (shen/shen\.lazyderef V3626 V3629)
            (shen/if
             (shen/cons\? V3276)
             (shen/let V3277
                       (shen/shen\.lazyderef
                        (shen/hd V3276)
                        V3629)
                       (shen/if
                        (shen/= 'define V3277)
                        (shen/let V3278
                                  (shen/shen\.lazyderef
                                   (nthcdr 1 V3276)
                                   V3629)
                                  (shen/if
                                   (shen/cons\? V3278)
                                   (shen/let F
                                             (shen/hd V3278)
                                             (shen/let X
                                                       (nthcdr 1 V3278)
                                                       (shen/let Y
                                                                 (shen/shen\.newpv V3629)
                                                                 (shen/let E
                                                                           (shen/shen\.newpv V3629)
                                                                           (shen/do
                                                                            (shen/shen\.incinfs)
                                                                            (shen/shen\.t*-defh
                                                                             (shen/compile
                                                                              (shen/lambda Y
                                                                                           (shen/shen\.<sig+rules> Y))
                                                                              X
                                                                              (shen/lambda E
                                                                                           (shen/if
                                                                                            (shen/cons\? E)
                                                                                            (shen/simple-error
                                                                                             (shen/cn "parse error here: "
                                                                                                      (shen/shen\.app E "\n" 'shen\.s)))
                                                                                            (shen/simple-error "parse error\n"))))
                                                                             F V3627 V3628 V3629 V3630))))))
                                   'false))
                        'false))
             'false)))
(defun shen/shen\.t*-defh
    (V3637 V3638 V3639 V3640 V3641 V3642)
  (shen/let V3272
            (shen/shen\.lazyderef V3637 V3641)
            (shen/if
             (shen/cons\? V3272)
             (shen/let Sig
                       (shen/hd V3272)
                       (shen/let Rules
                                 (nthcdr 1 V3272)
                                 (shen/do
                                  (shen/shen\.incinfs)
                                  (shen/shen\.t*-defhh Sig
                                                       (shen/shen\.ue-sig Sig)
                                                       V3638 V3639 V3640 Rules V3641 V3642))))
             'false)))
(defun shen/shen\.t*-defhh
    (V3651 V3652 V3653 V3654 V3655 V3656 V3657 V3658)
  (shen/do
   (shen/shen\.incinfs)
   (shen/shen\.t*-rules V3656 V3652 1 V3653
                        (append
                         (list
                          (list V3653 ': V3652))
                         V3655)
                        V3657
                        (shen/freeze
                         (shen/shen\.memo V3653 V3651 V3654 V3657 V3658)))))
(defun shen/shen\.memo
    (V3664 V3665 V3666 V3667 V3668)
  (shen/let Jnk
            (shen/shen\.newpv V3667)
            (shen/do
             (shen/shen\.incinfs)
             (shen/unify! V3666 V3665 V3667
                          (shen/freeze
                           (shen/bind Jnk
                                      (shen/declare
                                       (shen/shen\.lazyderef V3664 V3667)
                                       (shen/shen\.lazyderef V3666 V3667))
                                      V3667 V3668))))))
(defun shen/shen\.<sig+rules>
    (V3670)
  (shen/let Parse_shen\.<signature>
            (shen/shen\.<signature> V3670)
            (shen/if
             (shen/not
              (shen/=
               (shen/fail)
               Parse_shen\.<signature>))
             (shen/let Parse_shen\.<non-ll-rules>
                       (shen/shen\.<non-ll-rules> Parse_shen\.<signature>)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<non-ll-rules>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<non-ll-rules>)
                         (append
                          (list
                           (shen/shen\.hdtl Parse_shen\.<signature>))
                          (shen/shen\.hdtl Parse_shen\.<non-ll-rules>)))
                        (shen/fail)))
             (shen/fail))))
(defun shen/shen\.<non-ll-rules>
    (V3672)
  (shen/let YaccParse
            (shen/let Parse_shen\.<rule>
                      (shen/shen\.<rule> V3672)
                      (shen/if
                       (shen/not
                        (shen/=
                         (shen/fail)
                         Parse_shen\.<rule>))
                       (shen/let Parse_shen\.<non-ll-rules>
                                 (shen/shen\.<non-ll-rules> Parse_shen\.<rule>)
                                 (shen/if
                                  (shen/not
                                   (shen/=
                                    (shen/fail)
                                    Parse_shen\.<non-ll-rules>))
                                  (shen/shen\.pair
                                   (shen/hd Parse_shen\.<non-ll-rules>)
                                   (append
                                    (list
                                     (shen/shen\.hdtl Parse_shen\.<rule>))
                                    (shen/shen\.hdtl Parse_shen\.<non-ll-rules>)))
                                  (shen/fail)))
                       (shen/fail)))
            (shen/if
             (shen/= YaccParse
                     (shen/fail))
             (shen/let Parse_shen\.<rule>
                       (shen/shen\.<rule> V3672)
                       (shen/if
                        (shen/not
                         (shen/=
                          (shen/fail)
                          Parse_shen\.<rule>))
                        (shen/shen\.pair
                         (shen/hd Parse_shen\.<rule>)
                         (list
                          (shen/shen\.hdtl Parse_shen\.<rule>)))
                        (shen/fail)))
             YaccParse)))
(defun shen/shen\.ue
    (V3674)
  (shen/cond
   ((shen/and
     (shen/cons\? V3674)
     (shen/and
      (shen/cons\?
       (nthcdr 1 V3674))
      (shen/and
       (shen/internal/predicate->shen
        (null
         (nthcdr 2 V3674)))
       (shen/=
        (shen/hd V3674)
        'protect))))
    V3674)
   ((shen/cons\? V3674)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.ue Z))
     V3674))
   ((shen/variable\? V3674)
    (shen/concat '&& V3674))
   (shen/true V3674)))
(defun shen/shen\.ue-sig
    (V3676)
  (shen/cond
   ((shen/cons\? V3676)
    (shen/map
     (shen/lambda Z
                  (shen/shen\.ue-sig Z))
     V3676))
   ((shen/variable\? V3676)
    (shen/concat '&&& V3676))
   (shen/true V3676)))
(defun shen/shen\.ues
    (V3682)
  (shen/cond
   ((shen/shen\.ue\? V3682)
    (list V3682))
   ((shen/cons\? V3682)
    (shen/union
     (shen/shen\.ues
      (shen/hd V3682))
     (shen/shen\.ues
      (nthcdr 1 V3682))))
   (shen/true nil)))
(defun shen/shen\.ue\?
    (V3684)
  (shen/and
   (shen/symbol\? V3684)
   (shen/shen\.ue-h\?
    (shen/str V3684))))
(defun shen/shen\.ue-h\?
    (V3692)
  (shen/cond
   ((shen/and
     (shen/shen\.+string\? V3692)
     (shen/and
      (shen/= "&"
              (shen/pos V3692 0))
      (shen/and
       (shen/shen\.+string\?
        (shen/tlstr V3692))
       (shen/= "&"
               (shen/pos
                (shen/tlstr V3692)
                0)))))
    'true)
   (shen/true 'false)))
(defun shen/shen\.t*-rules
    (V3700 V3701 V3702 V3703 V3704 V3705 V3706)
  (shen/let Throwcontrol
            (shen/shen\.catchpoint)
            (shen/shen\.cutpoint Throwcontrol
                                 (shen/let Case
                                           (shen/let V3256
                                                     (shen/shen\.lazyderef V3700 V3705)
                                                     (shen/if
                                                      (shen/internal/predicate->shen
                                                       (null V3256))
                                                      (shen/do
                                                       (shen/shen\.incinfs)
                                                       (shen/thaw V3706))
                                                      'false))
                                           (shen/if
                                            (shen/= Case 'false)
                                            (shen/let Case
                                                      (shen/let V3257
                                                                (shen/shen\.lazyderef V3700 V3705)
                                                                (shen/if
                                                                 (shen/cons\? V3257)
                                                                 (shen/let Rule
                                                                           (shen/hd V3257)
                                                                           (shen/let Rules
                                                                                     (nthcdr 1 V3257)
                                                                                     (shen/do
                                                                                      (shen/shen\.incinfs)
                                                                                      (shen/shen\.t*-rule
                                                                                       (shen/shen\.ue Rule)
                                                                                       V3701 V3704 V3705
                                                                                       (shen/freeze
                                                                                        (shen/cut Throwcontrol V3705
                                                                                                  (shen/freeze
                                                                                                   (shen/shen\.t*-rules Rules V3701
                                                                                                                        (1+ V3702)
                                                                                                                        V3703 V3704 V3705 V3706))))))))
                                                                 'false))
                                                      (shen/if
                                                       (shen/= Case 'false)
                                                       (shen/let Err
                                                                 (shen/shen\.newpv V3705)
                                                                 (shen/do
                                                                  (shen/shen\.incinfs)
                                                                  (shen/bind Err
                                                                             (shen/simple-error
                                                                              (shen/cn "type error in rule "
                                                                                       (shen/shen\.app
                                                                                        (shen/shen\.lazyderef V3702 V3705)
                                                                                        (shen/cn " of "
                                                                                                 (shen/shen\.app
                                                                                                  (shen/shen\.lazyderef V3703 V3705)
                                                                                                  "" 'shen\.a))
                                                                                        'shen\.a)))
                                                                             V3705 V3706)))
                                                       Case))
                                            Case)))))
(defun shen/shen\.t*-rule
    (V3712 V3713 V3714 V3715 V3716)
  (shen/let Throwcontrol
            (shen/shen\.catchpoint)
            (shen/shen\.cutpoint Throwcontrol
                                 (shen/let V3248
                                           (shen/shen\.lazyderef V3712 V3715)
                                           (shen/if
                                            (shen/cons\? V3248)
                                            (shen/let Patterns
                                                      (shen/hd V3248)
                                                      (shen/let V3249
                                                                (shen/shen\.lazyderef
                                                                 (nthcdr 1 V3248)
                                                                 V3715)
                                                                (shen/if
                                                                 (shen/cons\? V3249)
                                                                 (shen/let Action
                                                                           (shen/hd V3249)
                                                                           (shen/let V3250
                                                                                     (shen/shen\.lazyderef
                                                                                      (nthcdr 1 V3249)
                                                                                      V3715)
                                                                                     (shen/if
                                                                                      (shen/internal/predicate->shen
                                                                                       (null V3250))
                                                                                      (shen/let NewHyps
                                                                                                (shen/shen\.newpv V3715)
                                                                                                (shen/do
                                                                                                 (shen/shen\.incinfs)
                                                                                                 (shen/shen\.newhyps
                                                                                                  (shen/shen\.placeholders Patterns)
                                                                                                  V3714 NewHyps V3715
                                                                                                  (shen/freeze
                                                                                                   (shen/shen\.t*-patterns Patterns V3713 NewHyps V3715
                                                                                                                           (shen/freeze
                                                                                                                            (shen/cut Throwcontrol V3715
                                                                                                                                      (shen/freeze
                                                                                                                                       (shen/shen\.t*-action
                                                                                                                                        (shen/shen\.curry
                                                                                                                                         (shen/shen\.ue Action))
                                                                                                                                        (shen/shen\.result-type Patterns V3713)
                                                                                                                                        (shen/shen\.patthyps Patterns V3713 V3714)
                                                                                                                                        V3715 V3716)))))))))
                                                                                      'false)))
                                                                 'false)))
                                            'false)))))
(defun shen/shen\.placeholders
    (V3722)
  (shen/cond
   ((shen/shen\.ue\? V3722)
    (list V3722))
   ((shen/cons\? V3722)
    (shen/union
     (shen/shen\.placeholders
      (shen/hd V3722))
     (shen/shen\.placeholders
      (nthcdr 1 V3722))))
   (shen/true nil)))
(defun shen/shen\.newhyps
    (V3728 V3729 V3730 V3731 V3732)
  (shen/let Case
            (shen/let V3235
                      (shen/shen\.lazyderef V3728 V3731)
                      (shen/if
                       (shen/internal/predicate->shen
                        (null V3235))
                       (shen/do
                        (shen/shen\.incinfs)
                        (shen/unify! V3730 V3729 V3731 V3732))
                       'false))
            (shen/if
             (shen/= Case 'false)
             (shen/let V3236
                       (shen/shen\.lazyderef V3728 V3731)
                       (shen/if
                        (shen/cons\? V3236)
                        (shen/let V3231
                                  (shen/hd V3236)
                                  (shen/let Vs
                                            (nthcdr 1 V3236)
                                            (shen/let V3237
                                                      (shen/shen\.lazyderef V3730 V3731)
                                                      (shen/if
                                                       (shen/cons\? V3237)
                                                       (shen/let V3238
                                                                 (shen/shen\.lazyderef
                                                                  (shen/hd V3237)
                                                                  V3731)
                                                                 (shen/if
                                                                  (shen/cons\? V3238)
                                                                  (shen/let V
                                                                            (shen/hd V3238)
                                                                            (shen/let V3239
                                                                                      (shen/shen\.lazyderef
                                                                                       (nthcdr 1 V3238)
                                                                                       V3731)
                                                                                      (shen/if
                                                                                       (shen/cons\? V3239)
                                                                                       (shen/let V3240
                                                                                                 (shen/shen\.lazyderef
                                                                                                  (shen/hd V3239)
                                                                                                  V3731)
                                                                                                 (shen/if
                                                                                                  (shen/= ': V3240)
                                                                                                  (shen/let V3241
                                                                                                            (shen/shen\.lazyderef
                                                                                                             (nthcdr 1 V3239)
                                                                                                             V3731)
                                                                                                            (shen/if
                                                                                                             (shen/cons\? V3241)
                                                                                                             (shen/let A
                                                                                                                       (shen/hd V3241)
                                                                                                                       (shen/let V3242
                                                                                                                                 (shen/shen\.lazyderef
                                                                                                                                  (nthcdr 1 V3241)
                                                                                                                                  V3731)
                                                                                                                                 (shen/if
                                                                                                                                  (shen/internal/predicate->shen
                                                                                                                                   (null V3242))
                                                                                                                                  (shen/let NewHyp
                                                                                                                                            (nthcdr 1 V3237)
                                                                                                                                            (shen/do
                                                                                                                                             (shen/shen\.incinfs)
                                                                                                                                             (shen/unify! V V3231 V3731
                                                                                                                                                          (shen/freeze
                                                                                                                                                           (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                                                  (shen/if
                                                                                                                                   (shen/shen\.pvar\? V3242)
                                                                                                                                   (shen/do
                                                                                                                                    (shen/shen\.bindv V3242 nil V3731)
                                                                                                                                    (shen/let Result
                                                                                                                                              (shen/let NewHyp
                                                                                                                                                        (nthcdr 1 V3237)
                                                                                                                                                        (shen/do
                                                                                                                                                         (shen/shen\.incinfs)
                                                                                                                                                         (shen/unify! V V3231 V3731
                                                                                                                                                                      (shen/freeze
                                                                                                                                                                       (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                                                              (shen/do
                                                                                                                                               (shen/shen\.unbindv V3242 V3731)
                                                                                                                                               Result)))
                                                                                                                                   'false))))
                                                                                                             (shen/if
                                                                                                              (shen/shen\.pvar\? V3241)
                                                                                                              (shen/let A
                                                                                                                        (shen/shen\.newpv V3731)
                                                                                                                        (shen/do
                                                                                                                         (shen/shen\.bindv V3241
                                                                                                                                           (list A)
                                                                                                                                           V3731)
                                                                                                                         (shen/let Result
                                                                                                                                   (shen/let NewHyp
                                                                                                                                             (nthcdr 1 V3237)
                                                                                                                                             (shen/do
                                                                                                                                              (shen/shen\.incinfs)
                                                                                                                                              (shen/unify! V V3231 V3731
                                                                                                                                                           (shen/freeze
                                                                                                                                                            (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                                                   (shen/do
                                                                                                                                    (shen/shen\.unbindv V3241 V3731)
                                                                                                                                    Result))))
                                                                                                              'false)))
                                                                                                  (shen/if
                                                                                                   (shen/shen\.pvar\? V3240)
                                                                                                   (shen/do
                                                                                                    (shen/shen\.bindv V3240 ': V3731)
                                                                                                    (shen/let Result
                                                                                                              (shen/let V3243
                                                                                                                        (shen/shen\.lazyderef
                                                                                                                         (nthcdr 1 V3239)
                                                                                                                         V3731)
                                                                                                                        (shen/if
                                                                                                                         (shen/cons\? V3243)
                                                                                                                         (shen/let A
                                                                                                                                   (shen/hd V3243)
                                                                                                                                   (shen/let V3244
                                                                                                                                             (shen/shen\.lazyderef
                                                                                                                                              (nthcdr 1 V3243)
                                                                                                                                              V3731)
                                                                                                                                             (shen/if
                                                                                                                                              (shen/internal/predicate->shen
                                                                                                                                               (null V3244))
                                                                                                                                              (shen/let NewHyp
                                                                                                                                                        (nthcdr 1 V3237)
                                                                                                                                                        (shen/do
                                                                                                                                                         (shen/shen\.incinfs)
                                                                                                                                                         (shen/unify! V V3231 V3731
                                                                                                                                                                      (shen/freeze
                                                                                                                                                                       (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                                                              (shen/if
                                                                                                                                               (shen/shen\.pvar\? V3244)
                                                                                                                                               (shen/do
                                                                                                                                                (shen/shen\.bindv V3244 nil V3731)
                                                                                                                                                (shen/let Result
                                                                                                                                                          (shen/let NewHyp
                                                                                                                                                                    (nthcdr 1 V3237)
                                                                                                                                                                    (shen/do
                                                                                                                                                                     (shen/shen\.incinfs)
                                                                                                                                                                     (shen/unify! V V3231 V3731
                                                                                                                                                                                  (shen/freeze
                                                                                                                                                                                   (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                                                                          (shen/do
                                                                                                                                                           (shen/shen\.unbindv V3244 V3731)
                                                                                                                                                           Result)))
                                                                                                                                               'false))))
                                                                                                                         (shen/if
                                                                                                                          (shen/shen\.pvar\? V3243)
                                                                                                                          (shen/let A
                                                                                                                                    (shen/shen\.newpv V3731)
                                                                                                                                    (shen/do
                                                                                                                                     (shen/shen\.bindv V3243
                                                                                                                                                       (list A)
                                                                                                                                                       V3731)
                                                                                                                                     (shen/let Result
                                                                                                                                               (shen/let NewHyp
                                                                                                                                                         (nthcdr 1 V3237)
                                                                                                                                                         (shen/do
                                                                                                                                                          (shen/shen\.incinfs)
                                                                                                                                                          (shen/unify! V V3231 V3731
                                                                                                                                                                       (shen/freeze
                                                                                                                                                                        (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                                                               (shen/do
                                                                                                                                                (shen/shen\.unbindv V3243 V3731)
                                                                                                                                                Result))))
                                                                                                                          'false)))
                                                                                                              (shen/do
                                                                                                               (shen/shen\.unbindv V3240 V3731)
                                                                                                               Result)))
                                                                                                   'false)))
                                                                                       (shen/if
                                                                                        (shen/shen\.pvar\? V3239)
                                                                                        (shen/let A
                                                                                                  (shen/shen\.newpv V3731)
                                                                                                  (shen/do
                                                                                                   (shen/shen\.bindv V3239
                                                                                                                     (list ': A)
                                                                                                                     V3731)
                                                                                                   (shen/let Result
                                                                                                             (shen/let NewHyp
                                                                                                                       (nthcdr 1 V3237)
                                                                                                                       (shen/do
                                                                                                                        (shen/shen\.incinfs)
                                                                                                                        (shen/unify! V V3231 V3731
                                                                                                                                     (shen/freeze
                                                                                                                                      (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                             (shen/do
                                                                                                              (shen/shen\.unbindv V3239 V3731)
                                                                                                              Result))))
                                                                                        'false))))
                                                                  (shen/if
                                                                   (shen/shen\.pvar\? V3238)
                                                                   (shen/let V
                                                                             (shen/shen\.newpv V3731)
                                                                             (shen/let A
                                                                                       (shen/shen\.newpv V3731)
                                                                                       (shen/do
                                                                                        (shen/shen\.bindv V3238
                                                                                                          (list V ': A)
                                                                                                          V3731)
                                                                                        (shen/let Result
                                                                                                  (shen/let NewHyp
                                                                                                            (nthcdr 1 V3237)
                                                                                                            (shen/do
                                                                                                             (shen/shen\.incinfs)
                                                                                                             (shen/unify! V V3231 V3731
                                                                                                                          (shen/freeze
                                                                                                                           (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732)))))
                                                                                                  (shen/do
                                                                                                   (shen/shen\.unbindv V3238 V3731)
                                                                                                   Result)))))
                                                                   'false)))
                                                       (shen/if
                                                        (shen/shen\.pvar\? V3237)
                                                        (shen/let V
                                                                  (shen/shen\.newpv V3731)
                                                                  (shen/let A
                                                                            (shen/shen\.newpv V3731)
                                                                            (shen/let NewHyp
                                                                                      (shen/shen\.newpv V3731)
                                                                                      (shen/do
                                                                                       (shen/shen\.bindv V3237
                                                                                                         (append
                                                                                                          (list
                                                                                                           (list V ': A))
                                                                                                          NewHyp)
                                                                                                         V3731)
                                                                                       (shen/let Result
                                                                                                 (shen/do
                                                                                                  (shen/shen\.incinfs)
                                                                                                  (shen/unify! V V3231 V3731
                                                                                                               (shen/freeze
                                                                                                                (shen/shen\.newhyps Vs V3729 NewHyp V3731 V3732))))
                                                                                                 (shen/do
                                                                                                  (shen/shen\.unbindv V3237 V3731)
                                                                                                  Result))))))
                                                        'false)))))
                        'false))
             Case)))
(defun shen/shen\.patthyps
    (V3738 V3739 V3740)
  (shen/cond
   ((shen/internal/predicate->shen
     (null V3738))
    V3740)
   ((shen/and
     (shen/cons\? V3738)
     (shen/and
      (shen/cons\? V3739)
      (shen/and
       (shen/cons\?
        (nthcdr 1 V3739))
       (shen/and
        (shen/= '-->
                (shen/hd
                 (nthcdr 1 V3739)))
        (shen/and
         (shen/cons\?
          (nthcdr 2 V3739))
         (shen/internal/predicate->shen
          (null
           (nthcdr 3 V3739))))))))
    (shen/adjoin
     (list
      (shen/hd V3738)
      ':
      (shen/hd V3739))
     (shen/shen\.patthyps
      (nthcdr 1 V3738)
      (shen/hd
       (nthcdr 2 V3739))
      V3740)))
   (shen/true
    (shen/shen\.f_error 'shen\.patthyps))))
(defun shen/shen\.result-type
    (V3747 V3748)
  (cl-flet
      ((tail-trampoline
        (V3747 V3748)
        (shen/cond
         ((shen/and
           (shen/internal/predicate->shen
            (null V3747))
           (shen/and
            (shen/cons\? V3748)
            (shen/and
             (shen/= '-->
                     (shen/hd V3748))
             (shen/and
              (shen/cons\?
               (nthcdr 1 V3748))
              (shen/internal/predicate->shen
               (null
                (nthcdr 2 V3748)))))))
          (shen/hd
           (nthcdr 1 V3748)))
         ((shen/internal/predicate->shen
           (null V3747))
          V3748)
         ((shen/and
           (shen/cons\? V3747)
           (shen/and
            (shen/cons\? V3748)
            (shen/and
             (shen/cons\?
              (nthcdr 1 V3748))
             (shen/and
              (shen/= '-->
                      (shen/hd
                       (nthcdr 1 V3748)))
              (shen/and
               (shen/cons\?
                (nthcdr 2 V3748))
               (shen/internal/predicate->shen
                (null
                 (nthcdr 3 V3748))))))))
          (vector
           (list
            (nthcdr 1 V3747)
            (shen/hd
             (nthcdr 2 V3748)))))
         (shen/true
          (shen/shen\.f_error 'shen\.result-type)))))
    (let
        ((result
          (funcall #'tail-trampoline V3747 V3748)))
      (while
          (vectorp result)
        (setq result
              (apply #'tail-trampoline
                     (aref result 0))))
      result)))
(defun shen/shen\.t*-patterns
    (V3754 V3755 V3756 V3757 V3758)
  (shen/let Case
            (shen/let V3223
                      (shen/shen\.lazyderef V3754 V3757)
                      (shen/if
                       (shen/internal/predicate->shen
                        (null V3223))
                       (shen/do
                        (shen/shen\.incinfs)
                        (shen/thaw V3758))
                       'false))
            (shen/if
             (shen/= Case 'false)
             (shen/let V3224
                       (shen/shen\.lazyderef V3754 V3757)
                       (shen/if
                        (shen/cons\? V3224)
                        (shen/let Pattern
                                  (shen/hd V3224)
                                  (shen/let Patterns
                                            (nthcdr 1 V3224)
                                            (shen/let V3225
                                                      (shen/shen\.lazyderef V3755 V3757)
                                                      (shen/if
                                                       (shen/cons\? V3225)
                                                       (shen/let A
                                                                 (shen/hd V3225)
                                                                 (shen/let V3226
                                                                           (shen/shen\.lazyderef
                                                                            (nthcdr 1 V3225)
                                                                            V3757)
                                                                           (shen/if
                                                                            (shen/cons\? V3226)
                                                                            (shen/let V3227
                                                                                      (shen/shen\.lazyderef
                                                                                       (shen/hd V3226)
                                                                                       V3757)
                                                                                      (shen/if
                                                                                       (shen/= '--> V3227)
                                                                                       (shen/let V3228
                                                                                                 (shen/shen\.lazyderef
                                                                                                  (nthcdr 1 V3226)
                                                                                                  V3757)
                                                                                                 (shen/if
                                                                                                  (shen/cons\? V3228)
                                                                                                  (shen/let B
                                                                                                            (shen/hd V3228)
                                                                                                            (shen/let V3229
                                                                                                                      (shen/shen\.lazyderef
                                                                                                                       (nthcdr 1 V3228)
                                                                                                                       V3757)
                                                                                                                      (shen/if
                                                                                                                       (shen/internal/predicate->shen
                                                                                                                        (null V3229))
                                                                                                                       (shen/do
                                                                                                                        (shen/shen\.incinfs)
                                                                                                                        (shen/shen\.t*
                                                                                                                         (list Pattern ': A)
                                                                                                                         V3756 V3757
                                                                                                                         (shen/freeze
                                                                                                                          (shen/shen\.t*-patterns Patterns B V3756 V3757 V3758))))
                                                                                                                       'false)))
                                                                                                  'false))
                                                                                       'false))
                                                                            'false)))
                                                       'false))))
                        'false))
             Case)))
(defun shen/shen\.t*-action
    (V3764 V3765 V3766 V3767 V3768)
  (shen/let Throwcontrol
            (shen/shen\.catchpoint)
            (shen/shen\.cutpoint Throwcontrol
                                 (shen/let Case
                                           (shen/let V3200
                                                     (shen/shen\.lazyderef V3764 V3767)
                                                     (shen/if
                                                      (shen/cons\? V3200)
                                                      (shen/let V3201
                                                                (shen/shen\.lazyderef
                                                                 (shen/hd V3200)
                                                                 V3767)
                                                                (shen/if
                                                                 (shen/= 'where V3201)
                                                                 (shen/let V3202
                                                                           (shen/shen\.lazyderef
                                                                            (nthcdr 1 V3200)
                                                                            V3767)
                                                                           (shen/if
                                                                            (shen/cons\? V3202)
                                                                            (shen/let P
                                                                                      (shen/hd V3202)
                                                                                      (shen/let V3203
                                                                                                (shen/shen\.lazyderef
                                                                                                 (nthcdr 1 V3202)
                                                                                                 V3767)
                                                                                                (shen/if
                                                                                                 (shen/cons\? V3203)
                                                                                                 (shen/let Action
                                                                                                           (shen/hd V3203)
                                                                                                           (shen/let V3204
                                                                                                                     (shen/shen\.lazyderef
                                                                                                                      (nthcdr 1 V3203)
                                                                                                                      V3767)
                                                                                                                     (shen/if
                                                                                                                      (shen/internal/predicate->shen
                                                                                                                       (null V3204))
                                                                                                                      (shen/do
                                                                                                                       (shen/shen\.incinfs)
                                                                                                                       (shen/cut Throwcontrol V3767
                                                                                                                                 (shen/freeze
                                                                                                                                  (shen/shen\.t*
                                                                                                                                   (list P ': 'boolean)
                                                                                                                                   V3766 V3767
                                                                                                                                   (shen/freeze
                                                                                                                                    (shen/cut Throwcontrol V3767
                                                                                                                                              (shen/freeze
                                                                                                                                               (shen/shen\.t*-action Action V3765
                                                                                                                                                                     (append
                                                                                                                                                                      (list
                                                                                                                                                                       (list P ': 'verified))
                                                                                                                                                                      V3766)
                                                                                                                                                                     V3767 V3768))))))))
                                                                                                                      'false)))
                                                                                                 'false)))
                                                                            'false))
                                                                 'false))
                                                      'false))
                                           (shen/if
                                            (shen/= Case 'false)
                                            (shen/let Case
                                                      (shen/let V3205
                                                                (shen/shen\.lazyderef V3764 V3767)
                                                                (shen/if
                                                                 (shen/cons\? V3205)
                                                                 (shen/let V3206
                                                                           (shen/shen\.lazyderef
                                                                            (shen/hd V3205)
                                                                            V3767)
                                                                           (shen/if
                                                                            (shen/= 'shen\.choicepoint! V3206)
                                                                            (shen/let V3207
                                                                                      (shen/shen\.lazyderef
                                                                                       (nthcdr 1 V3205)
                                                                                       V3767)
                                                                                      (shen/if
                                                                                       (shen/cons\? V3207)
                                                                                       (shen/let V3208
                                                                                                 (shen/shen\.lazyderef
                                                                                                  (shen/hd V3207)
                                                                                                  V3767)
                                                                                                 (shen/if
                                                                                                  (shen/cons\? V3208)
                                                                                                  (shen/let V3209
                                                                                                            (shen/shen\.lazyderef
                                                                                                             (shen/hd V3208)
                                                                                                             V3767)
                                                                                                            (shen/if
                                                                                                             (shen/cons\? V3209)
                                                                                                             (shen/let V3210
                                                                                                                       (shen/shen\.lazyderef
                                                                                                                        (shen/hd V3209)
                                                                                                                        V3767)
                                                                                                                       (shen/if
                                                                                                                        (shen/= 'fail-if V3210)
                                                                                                                        (shen/let V3211
                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                   (nthcdr 1 V3209)
                                                                                                                                   V3767)
                                                                                                                                  (shen/if
                                                                                                                                   (shen/cons\? V3211)
                                                                                                                                   (shen/let F
                                                                                                                                             (shen/hd V3211)
                                                                                                                                             (shen/let V3212
                                                                                                                                                       (shen/shen\.lazyderef
                                                                                                                                                        (nthcdr 1 V3211)
                                                                                                                                                        V3767)
                                                                                                                                                       (shen/if
                                                                                                                                                        (shen/internal/predicate->shen
                                                                                                                                                         (null V3212))
                                                                                                                                                        (shen/let V3213
                                                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                                                   (nthcdr 1 V3208)
                                                                                                                                                                   V3767)
                                                                                                                                                                  (shen/if
                                                                                                                                                                   (shen/cons\? V3213)
                                                                                                                                                                   (shen/let Action
                                                                                                                                                                             (shen/hd V3213)
                                                                                                                                                                             (shen/let V3214
                                                                                                                                                                                       (shen/shen\.lazyderef
                                                                                                                                                                                        (nthcdr 1 V3213)
                                                                                                                                                                                        V3767)
                                                                                                                                                                                       (shen/if
                                                                                                                                                                                        (shen/internal/predicate->shen
                                                                                                                                                                                         (null V3214))
                                                                                                                                                                                        (shen/let V3215
                                                                                                                                                                                                  (shen/shen\.lazyderef
                                                                                                                                                                                                   (nthcdr 1 V3207)
                                                                                                                                                                                                   V3767)
                                                                                                                                                                                                  (shen/if
                                                                                                                                                                                                   (shen/internal/predicate->shen
                                                                                                                                                                                                    (null V3215))
                                                                                                                                                                                                   (shen/do
                                                                                                                                                                                                    (shen/shen\.incinfs)
                                                                                                                                                                                                    (shen/cut Throwcontrol V3767
                                                                                                                                                                                                              (shen/freeze
                                                                                                                                                                                                               (shen/shen\.t*-action
                                                                                                                                                                                                                (list 'where
                                                                                                                                                                                                                      (list 'not
                                                                                                                                                                                                                            (list F Action))
                                                                                                                                                                                                                      Action)
                                                                                                                                                                                                                V3765 V3766 V3767 V3768))))
                                                                                                                                                                                                   'false))
                                                                                                                                                                                        'false)))
                                                                                                                                                                   'false))
                                                                                                                                                        'false)))
                                                                                                                                   'false))
                                                                                                                        'false))
                                                                                                             'false))
                                                                                                  'false))
                                                                                       'false))
                                                                            'false))
                                                                 'false))
                                                      (shen/if
                                                       (shen/= Case 'false)
                                                       (shen/let Case
                                                                 (shen/let V3216
                                                                           (shen/shen\.lazyderef V3764 V3767)
                                                                           (shen/if
                                                                            (shen/cons\? V3216)
                                                                            (shen/let V3217
                                                                                      (shen/shen\.lazyderef
                                                                                       (shen/hd V3216)
                                                                                       V3767)
                                                                                      (shen/if
                                                                                       (shen/= 'shen\.choicepoint! V3217)
                                                                                       (shen/let V3218
                                                                                                 (shen/shen\.lazyderef
                                                                                                  (nthcdr 1 V3216)
                                                                                                  V3767)
                                                                                                 (shen/if
                                                                                                  (shen/cons\? V3218)
                                                                                                  (shen/let Action
                                                                                                            (shen/hd V3218)
                                                                                                            (shen/let V3219
                                                                                                                      (shen/shen\.lazyderef
                                                                                                                       (nthcdr 1 V3218)
                                                                                                                       V3767)
                                                                                                                      (shen/if
                                                                                                                       (shen/internal/predicate->shen
                                                                                                                        (null V3219))
                                                                                                                       (shen/do
                                                                                                                        (shen/shen\.incinfs)
                                                                                                                        (shen/cut Throwcontrol V3767
                                                                                                                                  (shen/freeze
                                                                                                                                   (shen/shen\.t*-action
                                                                                                                                    (list 'where
                                                                                                                                          (list 'not
                                                                                                                                                (list
                                                                                                                                                 (list '= Action)
                                                                                                                                                 (list 'fail)))
                                                                                                                                          Action)
                                                                                                                                    V3765 V3766 V3767 V3768))))
                                                                                                                       'false)))
                                                                                                  'false))
                                                                                       'false))
                                                                            'false))
                                                                 (shen/if
                                                                  (shen/= Case 'false)
                                                                  (shen/do
                                                                   (shen/shen\.incinfs)
                                                                   (shen/shen\.t*
                                                                    (list V3764 ': V3765)
                                                                    V3766 V3767 V3768))
                                                                  Case))
                                                       Case))
                                            Case)))))
(defun shen/findall
    (V3774 V3775 V3776 V3777 V3778)
  (shen/let B
            (shen/shen\.newpv V3777)
            (shen/let A
                      (shen/shen\.newpv V3777)
                      (shen/do
                       (shen/shen\.incinfs)
                       (shen/bind A
                                  (shen/gensym 'shen\.a)
                                  V3777
                                  (shen/freeze
                                   (shen/bind B
                                              (shen/set
                                               (shen/shen\.lazyderef A V3777)
                                               nil)
                                              V3777
                                              (shen/freeze
                                               (shen/shen\.findallhelp V3774 V3775 V3776 A V3777 V3778)))))))))
(defun shen/shen\.findallhelp
    (V3785 V3786 V3787 V3788 V3789 V3790)
  (shen/let Case
            (shen/do
             (shen/shen\.incinfs)
             (shen/call V3786 V3789
                        (shen/freeze
                         (shen/shen\.remember V3788 V3785 V3789
                                              (shen/freeze
                                               (shen/fwhen 'false V3789 V3790))))))
            (shen/if
             (shen/= Case 'false)
             (shen/do
              (shen/shen\.incinfs)
              (shen/bind V3787
                         (shen/value
                          (shen/shen\.lazyderef V3788 V3789))
                         V3789 V3790))
             Case)))
(defun shen/shen\.remember
    (V3795 V3796 V3797 V3798)
  (shen/let B
            (shen/shen\.newpv V3797)
            (shen/do
             (shen/shen\.incinfs)
             (shen/bind B
                        (shen/set
                         (shen/shen\.deref V3795 V3797)
                         (append
                          (list
                           (shen/shen\.deref V3796 V3797))
                          (shen/value
                           (shen/shen\.deref V3795 V3797))))
                        V3797 V3798))))
(provide 'shen-elisp)
